﻿using System;
using System.Collections.Generic;
using System.IO;
using Android.App;
using Android.Content.PM;
using Android.Runtime;
using Android.Views;
using Android.Provider;
using Environment = Android.OS.Environment;
using Xamarin.Forms;
using Android.Media;
using Android.Graphics;
using Android.Widget;
using Android.OS;
using Plugin.CurrentActivity;
using Plugin.Settings;
using Android.Locations;
using Android.Content;
using System.Threading.Tasks;
using Plugin.Permissions;
using Plugin.Permissions.Abstractions;

namespace AlignmentViewer.Droid
{



    //[Activity(Label = "Alignment Viewer", Icon = "@drawable/icon", Theme = "@style/MainTheme", MainLauncher = true, ConfigurationChanges = ConfigChanges.ScreenSize | ConfigChanges.Orientation, ScreenOrientation = ScreenOrientation.Portrait)]
    [Activity(Label = "Alignment Viewer", Icon = "@drawable/icon", Theme = "@style/splashScreen", MainLauncher = true, ConfigurationChanges = ConfigChanges.ScreenSize | ConfigChanges.Orientation)]
    public class MainActivity : global::Xamarin.Forms.Platform.Android.FormsAppCompatActivity, Android.App.Application.IActivityLifecycleCallbacks
    {
        protected override void OnCreate(Bundle bundle)
        {
            CrossCurrentActivity.Current.Activity = this;
            string path = Android.OS.Environment.ExternalStorageDirectory.AbsolutePath + "/NRG/";


            App.screenWidth = (int)(Resources.DisplayMetrics.WidthPixels / Resources.DisplayMetrics.Density);
            App.screenHeight = (int)(Resources.DisplayMetrics.HeightPixels / Resources.DisplayMetrics.Density);
            App.density = (int)Resources.DisplayMetrics.Density;
            this.Window.SetFlags(WindowManagerFlags.KeepScreenOn, WindowManagerFlags.KeepScreenOn);
            RequestWindowFeature(WindowFeatures.NoTitle);
            base.SetTheme(Resource.Style.MainTheme);
            base.OnCreate(bundle);

            App.Init(new PostCodeUtils(), new scanFiles(), new SetupSampleData());
            global::Xamarin.Forms.Forms.Init(this, bundle);
            Acr.UserDialogs.UserDialogs.Init(() => (Activity)Forms.Context);
            Xamarin.FormsGoogleMaps.Init(this, bundle);
            TabLayoutResource = Resource.Layout.Tabbar;

            ToolbarResource = Resource.Layout.Toolbar;

            LoadApplication(new App(path));
        }

        protected override async void OnResume()
        {
            base.OnResume();
        }


        protected override void OnActivityResult(int requestCode, Result resultCode, Intent data)
        {
            base.OnActivityResult(requestCode, resultCode, data);

            //Take Photo
            if(requestCode == 0 && resultCode == Result.Ok)
            {
                double lat = App.lat;
                double lng = App.lng;
                string ch = App.ch;
                string off = App.off;
                string alignment = App.alignment;

                Task.Run(() =>
                {
                    try
                    {
                        var documentsDirectory = Forms.Context.GetExternalFilesDir(Environment.DirectoryPictures);
                        string pngFilename = System.IO.Path.Combine(documentsDirectory.AbsolutePath + "/" + "TempPictureFile" + ".jpg");

                        if (System.IO.File.Exists(pngFilename))
                        {
                            DateTime localDate = DateTime.Now;
                            string picturename = App.photoName;

                            string folderPath = Environment.ExternalStorageDirectory + "/" + Environment.DirectoryPictures;
                            string newFilename = folderPath + "/" + picturename;

                            if (!System.IO.Directory.Exists(folderPath))
                                System.IO.Directory.CreateDirectory(folderPath);
                            
                            System.IO.File.Move(pngFilename, newFilename);

                            string timeTag = localDate.Year + ":" + SingleValueCheck(localDate.Month) + ":" + SingleValueCheck(localDate.Day) + " " + SingleValueCheck(localDate.Hour) + ":" + SingleValueCheck(localDate.Minute) + ":" + SingleValueCheck(localDate.Second);

                            var originalMetadata = new ExifInterface(newFilename);
                            var orientation = GetOrientation(originalMetadata);
                            originalMetadata.Dispose();
                            Java.IO.File file = new Java.IO.File(newFilename);
                            HandleBitmap(Android.Net.Uri.FromFile(file), orientation, newFilename, lat, lng, timeTag, ch, off, alignment);
                        }
                    }
                    catch(Exception e)
                    {
                        App.StackTrace = e.StackTrace;
                        MessagingCenter.Send<string>(e.Message, "PhotoFailed");
                    }
                });
            }
            else
            {
                if (App.IsPhotoNameIncrmented)
                    App.PhotoNameIncrement -= 1;
                MessagingCenter.Send<string>("Cancel", "PhotoFailed");
            }
        }
        
        public int GetOrientation(ExifInterface exif)
        {
            try
            {
                var orientest = exif.GetAttribute(ExifInterface.TagOrientation);
                var orientation = /*(Android.Media.Orientation)*/exif.GetAttributeInt(ExifInterface.TagOrientation, (int)Android.Media.Orientation.Normal);
                //switch(orientation)
                //{
                //    case Android.Media.Orientation.Rotate90:
                //        return 90;
                //    case Android.Media.Orientation.Rotate180:
                //        return 180;
                //    case Android.Media.Orientation.Rotate270:
                //        return 270;
                //    default:
                //        return 0;
                //}
                return 0;
            }
            catch(Exception e)
            {
                Console.WriteLine(e.Message);
                return 0;
            }
        }

        public override void OnRequestPermissionsResult(int requestCode, string[] permissions, [GeneratedEnum] Android.Content.PM.Permission[] grantResults)
        {
            base.OnRequestPermissionsResult(requestCode, permissions, grantResults);
            PermissionsImplementation.Current.OnRequestPermissionsResult(requestCode, permissions, grantResults);
        }

        public async Task HandleBitmap(Android.Net.Uri uri, int orientation, string imageId, double lat, double lng , string time, string ch, string off, string alignment)
        {
            try
            {
                BitmapFactory.Options bmap = new BitmapFactory.Options();

                Bitmap mBitmap = Android.Provider.MediaStore.Images.Media.GetBitmap(this.ContentResolver, uri);
                Bitmap myBitmap = null;

                if(mBitmap != null)
                {
                    Matrix matrix = new Matrix();
                    if(orientation != 0)
                    {
                        matrix.PreRotate(orientation);
                    }

                    myBitmap = Bitmap.CreateBitmap(mBitmap, 0, 0, mBitmap.Width, mBitmap.Height, matrix, false);

                    if (imageId != null && imageId != "")
                    {
                        var watermark = CreateWatermark(ch, off);
                        AddWatermark(imageId, myBitmap, watermark, lat, lng, time, ch, off, alignment);
                    }
                    GC.Collect();
                }
            }
            catch(Exception e)
            {
                App.StackTrace = e.StackTrace;
                MessagingCenter.Send<string>(e.Message, "PhotoFailed");
            }
        }

        public Bitmap CreateWatermark(string ch, string off)
        {
            Android.Content.Res.AssetManager assets = Assets;

            Paint paint = new Paint(PaintFlags.AntiAlias);
            paint.TextSize = 48f * App.watermarkSize;
           
            paint.SetTypeface(Typeface.Create(Typeface.Default, TypefaceStyle.Normal));
            paint.Color = Android.Graphics.Color.Black;
            paint.TextAlign = Paint.Align.Left;

            Paint smallText = new Paint(PaintFlags.AntiAlias);
            smallText.TextSize = 24f * App.watermarkSize;
            smallText.SetTypeface(Typeface.Create(Typeface.Default, TypefaceStyle.Normal));
            smallText.Color = Android.Graphics.Color.Black;
            smallText.TextAlign = Paint.Align.Left;

            float baseline = -paint.Ascent();
            var logoWidth = App.watermarkSize * 147;
            var logoHeight = App.watermarkSize * 100;

            var maxString = Math.Max(paint.MeasureText(ch), paint.MeasureText(off));
            float maxWidth = logoWidth + maxString;

            if (App.watermarkShowString && App.StringName != "")
                maxWidth = Math.Max(maxWidth, smallText.MeasureText(App.Acc));

            if (App.watermarkShowAcc)
                maxWidth = Math.Max(maxWidth, smallText.MeasureText(App.Acc));

            if (App.watermarkShowDate)
                maxWidth = Math.Max(maxWidth, smallText.MeasureText(App.Date));

            int width = (int)(maxWidth + 18);
            int height = (int)(logoHeight + 10);
            int stringOffset = 0;

            Rect chBounds = new Rect(), offBounds = new Rect();
            paint.GetTextBounds(ch, 0, ch.Length, chBounds);
            paint.GetTextBounds(off, 0, off.Length, offBounds);

            if (App.watermarkShowString && App.StringName != "")
                height += (int)smallText.TextSize + 5;

            stringOffset = 0;

            if (App.watermarkShowAcc)
                stringOffset += (int)smallText.TextSize + 5;

            if(App.watermarkShowDate)
                stringOffset += (int)smallText.TextSize + 5;

            height += stringOffset;

            Bitmap image = Bitmap.CreateBitmap(width, height, Bitmap.Config.Argb8888);
            Canvas canvas = new Canvas(image);

            //Draw background
            canvas.DrawRect(new Rect(0, 0, width, height), new Paint() { Color = Android.Graphics.Color.White });
            var borderPaint = new Paint() { Color = Android.Graphics.Color.Black };

            //Draw borders
            //      ______________
            //     |     |________|
            //     |_____|________|
            //     |______________|
            //     |______________|
            //     |______________|
            //
            canvas.DrawRect(new Rect(0, 0, width, 2), borderPaint);
            canvas.DrawRect(new Rect(0, logoHeight + 8, width, logoHeight + 10), borderPaint);
            canvas.DrawRect(new Rect(0, 0, 2, height), borderPaint);
            canvas.DrawRect(new Rect(width - 2, 0, width, height), borderPaint);
            canvas.DrawRect(new Rect(logoWidth + 8, 0, logoWidth + 10, logoHeight + 10), borderPaint);
            canvas.DrawRect(new Rect(logoWidth + 10, (logoHeight / 2) + 5, width, (logoHeight / 2) + 7), borderPaint);

            //Add final bottom lines
            if (App.watermarkShowString && App.StringName != "")
                canvas.DrawRect(new Rect(0, (height - 2) - stringOffset, width, height - stringOffset), borderPaint);
            if (App.watermarkShowDate && App.watermarkShowAcc)
            {
                canvas.DrawRect(new Rect(0, (height - 2) - stringOffset / 2, width, height - (stringOffset / 2)), borderPaint);
                canvas.DrawRect(new Rect(0, height - 2, width, height), borderPaint);
            }
            else if(App.watermarkShowDate || App.watermarkShowAcc)
                canvas.DrawRect(new Rect(0, height - 2, width, height), borderPaint);

            Bitmap logo = Bitmap.CreateBitmap(BitmapFactory.DecodeStream(assets.Open("NRGWatermark" + App.watermarkSize.ToString() + "x.png")));

            canvas.DrawBitmap(logo, 5, 5, null);
            var chShift = ((logoHeight / 2) - chBounds.Height()) / 2;
            var offShift = ((logoHeight / 2) - offBounds.Height()) / 2;
            canvas.DrawText(ch, logoWidth + 15, (logoHeight / 2) - (chShift +  5) + 5, paint);
            canvas.DrawText(off, logoWidth + 15, logoHeight - (offShift + 5) + 10, paint);

            if(App.watermarkShowString && App.StringName != "")
            {
                var stringBounds = new Rect();
                smallText.GetTextBounds(App.StringName, 0, App.StringName.Length, stringBounds);
                var stringShift = (smallText.TextSize - stringBounds.Height()) / 2;
                canvas.DrawText(App.StringName, 5, (height - (stringShift + 5 * App.watermarkSize)) - stringOffset, smallText);
            }

            if(App.watermarkShowAcc)
            {
                var accBounds = new Rect();
                smallText.GetTextBounds(App.Acc, 0, App.Acc.Length, accBounds);
                var accShift = (smallText.TextSize - accBounds.Height()) / 2;
                if (App.watermarkShowDate)
                    canvas.DrawText(App.Acc, 5, (height - (accShift + 5) - (stringOffset / 2)), smallText);
                else
                    canvas.DrawText(App.Acc, 5, (height - (accShift + 5)), smallText);
            }

            if(App.watermarkShowDate)
            {
                var dateBounds = new Rect();
                smallText.GetTextBounds(App.Date, 0, App.Date.Length, dateBounds);
                var dateShift = (smallText.TextSize - dateBounds.Height()) / 2;
                canvas.DrawText(App.Date, 5, height - (dateShift + 5), smallText);
            }

            return image;
        }

        public void AddWatermark(string filename, Bitmap image, Bitmap watermark, double lat, double lng, string time, string ch, string off, string alignment)
        {
            using (Bitmap newBitmap = image.Copy(image.GetConfig(), true))
            {
                using (var canvas = new Canvas(newBitmap))
                {
                    switch(App.watermarkPosition)
                    {
                        case 1:
                            canvas.DrawBitmap(watermark, image.Width - watermark.Width, image.Height - watermark.Height, null);
                            break;
                        case 2:
                            canvas.DrawBitmap(watermark, 0, image.Height - watermark.Height, null);
                            break;
                        case 3:
                            canvas.DrawBitmap(watermark, image.Width - watermark.Width, 0 , null);
                            break;
                        case 4:
                            canvas.DrawBitmap(watermark, 0, 0, null);
                            break;
                    }
                }
                using (var filestream = new System.IO.FileStream(filename, System.IO.FileMode.OpenOrCreate, System.IO.FileAccess.Write))
                {
                    newBitmap.Compress(Bitmap.CompressFormat.Jpeg, 100, filestream);
                }
            }
            GC.Collect();
            ExifInterface exif = new ExifInterface(filename);
            exif.SetAttribute(ExifInterface.TagDatetime, time);
            exif.SetAttribute(ExifInterface.TagDatetimeOriginal, time);
            exif.SetAttribute(ExifInterface.TagDatetimeDigitized, time);
            
            if(lat != 0 && lng != 0)
            {
                if (lat < 0)
                    exif.SetAttribute(ExifInterface.TagGpsLatitudeRef, "S");
                else
                    exif.SetAttribute(ExifInterface.TagGpsLatitudeRef, "N");

                if (lng < 0)
                    exif.SetAttribute(ExifInterface.TagGpsLongitudeRef, "W");
                else
                    exif.SetAttribute(ExifInterface.TagGpsLongitudeRef, "E");

                exif.SetAttribute(ExifInterface.TagGpsLatitude, DDtoDMS(lat));
                exif.SetAttribute(ExifInterface.TagGpsLongitude, DDtoDMS(lng));
            }

            //if (exif.GetAttribute(ExifInterface.TagUserComment) != null)
            //{
            //    string comment = exif.GetAttribute(ExifInterface.TagUserComment);
            //    if(comment.Contains("NRGComment::"))
            //    {
            //        //Needs some extra logic incase users mess with comments / add info after the NRG comment
            //        comment += "," + "Chainage=" + App.chValue + "," + "Offset=" + App.offValue;
            //    }
            //    else if(comment != null && comment != "")
            //    {
            //        comment += ",NRGComment::" + "Chainage=" + App.chValue + "," + "Offset=" + App.offValue;
            //    }
            //    else
            //    {
            //        comment = "NRGComment::" + "Chainage=" + App.chValue + "," + "Offset=" + App.offValue;
            //    }                
            //    exif.SetAttribute(ExifInterface.TagUserComment, comment);
            //}
            //else
            //    exif.SetAttribute(ExifInterface.TagUserComment, "NRGComment::" + "Chainage=" + App.chValue + "," + "Offset=" + App.offValue);

            exif.SaveAttributes();

            var returnValues = new List<string>();
            returnValues.Add(filename);
            returnValues.Add(System.IO.Path.GetFileName(filename));
            returnValues.Add(alignment);
            returnValues.Add(ch);
            returnValues.Add(off);
            returnValues.Add(lat.ToString("0.000000"));
            returnValues.Add(lng.ToString("0.000000"));
            MediaScannerConnection.ScanFile(ApplicationContext, new String[] { filename }, null, null);
            MessagingCenter.Send<List<string>>(returnValues, "PhotoTaken");
        }

        public string DDtoDMS(double dd)
        {
            string dms = "";
            dd = Math.Abs(dd);
            double d = Math.Floor(dd);
            double mTemp = ((dd - d) * 60);
            double m = Math.Floor(mTemp);
            double s = ((mTemp - m) * 60);

            dms = d + "/1," + m + "/1," + Math.Floor(s * 100) + "/100";

            return dms;
        }

        public void OnActivityCreated(Activity activity, Bundle savedInstanceState)
        {
            CrossCurrentActivity.Current.Activity = activity;

        }

        public void OnActivityDestroyed(Activity activity)
        {
        }

        public void OnActivityPaused(Activity activity)
        {
        }

        public void OnActivityResumed(Activity activity)
        {
            CrossCurrentActivity.Current.Activity = this;
        }
        public void OnActivitySaveInstanceState(Activity activity, Bundle outState)
        {
        }

        public void OnActivityStarted(Activity activity)
        {
            CrossCurrentActivity.Current.Activity = activity;
        }

        public void OnActivityStopped(Activity activity)
        {
        }

        private string SingleValueCheck(int value)
        {
            string result = "";

            if (value <= 9)
                result = "0" + value.ToString();
            else
                result = value.ToString();

            return result;
        }

    }
}

