﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using AVFoundation;
using Foundation;
using UIKit;
using Xamarin.Forms;
using CoreGraphics;
using ImageIO;
using Photos;
using PhotosUI;
using AlignmentViewer;
using Plugin.Permissions;

[assembly: Dependency(typeof(AlignmentViewer.iOS.CameraService))]
namespace AlignmentViewer.iOS
{
    public class CameraService : ICameraService
    {
        public async void takePhoto()
        {
            //Check for permission
            var authorizationStatus = AVCaptureDevice.GetAuthorizationStatus(AVMediaType.Video);

            //if no access
            if(authorizationStatus != AVAuthorizationStatus.Authorized)
            {
                var access = await AVCaptureDevice.RequestAccessForMediaTypeAsync(AVMediaType.Video);


                //if access granted
                if(access)
                {
                    GotAccessToCamera();
                }
            }
            else
            {
                GotAccessToCamera();
            }
        }

        private async void GotAccessToCamera()
        {
            //Create image picker object
            var imagePicker = new UIImagePickerController { SourceType = UIImagePickerControllerSourceType.Camera };

            //Find the top most view controlle to launch camera
            var window = UIApplication.SharedApplication.KeyWindow;
            var vc = window.RootViewController;

            while(vc.PresentedViewController != null)
            {
                vc = vc.PresentedViewController;
            }

            vc.PresentViewController(imagePicker, true, null);

            //Callback
            imagePicker.FinishedPickingMedia += async (sender, e) =>
            {
                //Grap the image
                UIImage image = (UIImage)e.Info.ObjectForKey(new NSString("UIImagePickerControllerOriginalImage"));
                var metadata = e.Info.ObjectForKey(new NSString("UIImagePickerControllerMediaMetadata")).MutableCopy() as NSMutableDictionary;



                //rotatedImage = rotatedImage.Scale(rotatedImage.Size, 1.0f);
                var watermarkedImage = AddWatermark(image, image.Orientation);
                image.Dispose();
                var rotatedImage = RotateImage(watermarkedImage, watermarkedImage.Orientation);
                watermarkedImage.Dispose();
                var jpegImage = rotatedImage.AsJPEG();
                rotatedImage.Dispose();
                SetUpMetadata(App.ch + "_" + App.off + ".jpg", ref metadata);
                CGDataProvider dataProvider = new CGDataProvider(jpegImage);
                CGImageSource imageRef = CGImageSource.FromDataProvider(dataProvider);
                NSMutableData newImageData = new NSMutableData();

                //Get access to library
                if(PHPhotoLibrary.AuthorizationStatus != PHAuthorizationStatus.Authorized)
                {
                    var request = await PHPhotoLibrary.RequestAuthorizationAsync();
                    if(request != PHAuthorizationStatus.Authorized)
                    {
                        MessagingCenter.Send<string>("Rejected", "PhotoFailed");
                        Device.BeginInvokeOnMainThread(() =>
                        {
                            vc.DismissViewController(true, null);
                        });
                        return;
                    }
                }

                using (var d = CGImageDestination.Create(newImageData, imageRef.TypeIdentifier, 1, new CGImageDestinationOptions()))
                {
                    d.AddImage(imageRef, (int)(imageRef.ImageCount - 1), metadata);
                    d.Close();
                }

                NSError writeError;
                var imageSaved = newImageData.Save("Testfile.jpg", NSDataWritingOptions.Atomic, out writeError);
                var library = PHPhotoLibrary.SharedPhotoLibrary;


                library.PerformChanges(() =>
                {
                    var options = new PHAssetResourceCreationOptions();
                    options.OriginalFilename = "filename.jpg";
                    options.UniformTypeIdentifier = MobileCoreServices.UTType.JPEG;

                    var createRequest = PHAssetCreationRequest.CreationRequestForAsset();
                    createRequest.AddResource(PHAssetResourceType.Photo, newImageData, options);
                    var test = createRequest.PlaceholderForCreatedAsset.LocalIdentifier;
                    test = test.Replace("/L0/001", "");
                    var path = "assets-library://asset/asset.JPG?id=" + test + "&ext=JPG";
                }, (ok, error) =>
                {
                    var list = new List<string>();
                    if(error != null)
                    {
                        MessagingCenter.Send<string>("Fail", "PhotoFailed");
                    }
                    if (ok)
                        MessagingCenter.Send<List<string>>(list, "PhotoTaken");
                });

                Device.BeginInvokeOnMainThread(() =>
                {
                    vc.DismissViewController(true, null);
                });

                
            };

            imagePicker.Canceled += (sender, e) =>
            {
                MessagingCenter.Send<string>("Fail", "PhotoFailed");
                vc.DismissViewController(true, null);
            };
        }

        private UIImage RotateImage(UIImage image, UIImageOrientation orientation)
        {

            UIImage res;

            using (CGImage imageRef = image.CGImage)
            {
                CGImageAlphaInfo alphaInfo = imageRef.AlphaInfo;
                CGColorSpace colorSpaceInfo = CGColorSpace.CreateDeviceRGB();

                if(alphaInfo == CGImageAlphaInfo.None)
                {
                    alphaInfo = CGImageAlphaInfo.NoneSkipLast;
                }

                int width, height;

                width = (int)imageRef.Width;
                height = (int)imageRef.Height;

                CGBitmapContext bitmap;
                
                if(image.Orientation == UIImageOrientation.Up || image.Orientation == UIImageOrientation.Down)
                {
                    bitmap = new CGBitmapContext(IntPtr.Zero, width, height, imageRef.BitsPerComponent, imageRef.BytesPerRow, colorSpaceInfo, alphaInfo);
                }
                else
                {
                    bitmap = new CGBitmapContext(IntPtr.Zero, height, width, imageRef.BitsPerComponent, imageRef.BytesPerRow, colorSpaceInfo, alphaInfo);
                }

                switch(image.Orientation)
                {
                    case UIImageOrientation.Left:
                        bitmap.RotateCTM((float)Math.PI / 2);
                        bitmap.TranslateCTM(0, -height);
                        break;
                    case UIImageOrientation.Right:
                        bitmap.RotateCTM(-((float)Math.PI / 2));
                        bitmap.TranslateCTM(-width, 0);
                        break;
                    case UIImageOrientation.Up:
                        break;
                    case UIImageOrientation.Down:
                        bitmap.TranslateCTM(width, height);
                        bitmap.RotateCTM(-(float)Math.PI);
                        break;
                }

                bitmap.DrawImage(new CGRect(0, 0, width, height), imageRef);

                res = UIImage.FromImage(bitmap.ToImage());
                bitmap = null;
            }

            return res;
        }

        private UIImage AddWatermark(UIImage image, UIImageOrientation orientation)
        {
            nfloat fwidth = image.Size.Width;
            nfloat fheight = image.Size.Height;

            CGColorSpace colorSpace = CGColorSpace.CreateDeviceRGB();

            UIGraphics.BeginImageContext(new CGSize(image.Size.Width, image.Size.Height));
            UIImage resultImage;
            using (CGContext context = UIGraphics.GetCurrentContext())
            {
                context.SaveState();
                var rect = new CGRect(0, 0, image.Size.Width, image.Size.Height);
                context.SetFillColor(UIColor.Black.CGColor);
                image.Draw(rect);

                var textSize = 48f * App.watermarkSize;
                var smallTextSize = 24f * App.watermarkSize;

                var logoWidth = App.watermarkSize * 147;
                var logoHeight = App.watermarkSize * 100;

                var maxString = (float)Math.Max(calculateWidth(App.ch, textSize), calculateWidth(App.off, textSize));
                float maxWidth = logoWidth + maxString + 18;

                if (App.watermarkShowString && App.StringName != "")
                    maxWidth = (float)Math.Max(maxWidth, calculateWidth(App.StringName, smallTextSize));

                if (App.watermarkShowAcc)
                    maxWidth = (float)Math.Max(maxWidth, calculateWidth(App.Acc, smallTextSize));

                if (App.watermarkShowDate)
                    maxWidth = (float)Math.Max(maxWidth, calculateWidth(App.Date, smallTextSize));

                int width = (int)(maxWidth);
                int height = (int)(logoHeight + 10);
                int stringOffset = 0;

                if (App.watermarkShowString && App.StringName != "")
                    height += (int)smallTextSize + 5;

                if (App.watermarkShowAcc)
                    stringOffset += (int)smallTextSize + 5;

                if (App.watermarkShowDate)
                    stringOffset += (int)smallTextSize + 5;

                height += stringOffset;

                CGRect logoRect;

                UIImage logo = new UIImage(NSBundle.MainBundle.ResourcePath.ToString() + "/NRGWatermark" + App.watermarkSize + "x.png");
                var chString = new NSAttributedString(App.ch, new CoreText.CTStringAttributes { ForegroundColorFromContext = true, Font = new CoreText.CTFont("Droid Sans", textSize), ParagraphStyle = new CoreText.CTParagraphStyle(new CoreText.CTParagraphStyleSettings { Alignment = CoreText.CTTextAlignment.Left }) });
                var offString = new NSAttributedString(App.off, new CoreText.CTStringAttributes { ForegroundColorFromContext = true, Font = new CoreText.CTFont("Droid Sans", textSize), ParagraphStyle = new CoreText.CTParagraphStyle(new CoreText.CTParagraphStyleSettings { Alignment = CoreText.CTTextAlignment.Left }) });
                var accString = new NSAttributedString(App.Acc, new CoreText.CTStringAttributes { ForegroundColorFromContext = true, Font = new CoreText.CTFont("Droid Sans", smallTextSize), ParagraphStyle = new CoreText.CTParagraphStyle(new CoreText.CTParagraphStyleSettings { Alignment = CoreText.CTTextAlignment.Left }) });
                var dateString = new NSAttributedString(App.Date, new CoreText.CTStringAttributes { ForegroundColorFromContext = true, Font = new CoreText.CTFont("Droid Sans", smallTextSize), ParagraphStyle = new CoreText.CTParagraphStyle(new CoreText.CTParagraphStyleSettings { Alignment = CoreText.CTTextAlignment.Left }) });
                var stringNameString = new NSAttributedString(App.StringName, new CoreText.CTStringAttributes { ForegroundColorFromContext = true, Font = new CoreText.CTFont("Droid Sans", smallTextSize), ParagraphStyle = new CoreText.CTParagraphStyle(new CoreText.CTParagraphStyleSettings { Alignment = CoreText.CTTextAlignment.Left }) });

                var chLine = new CoreText.CTLine(chString);
                var offLine = new CoreText.CTLine(offString);
                var accLine = new CoreText.CTLine(accString);
                var dateLine = new CoreText.CTLine(dateString);
                var stringNameLine = new CoreText.CTLine(stringNameString);

                nfloat startX = 0, startY = 0;

                switch(App.watermarkPosition)
                {
                    case 1:
                        startX = image.Size.Width - width;
                        startY = image.Size.Height - height;
                        break;
                    case 2:
                        startX = 0;
                        startY = image.Size.Height - height;
                        break;
                    case 3:
                        startX = image.Size.Width - width;
                        startY = 0;
                        break;
                    case 4:
                        startX = 0;
                        startY = 0;
                        break;
                }

                //Fill background
                context.SetFillColor(UIColor.White.CGColor);
                context.FillRect(new CGRect(startX, startY, width, height));

                //Draw borders
                //      ______________
                //     |     |________|
                //     |_____|________|
                //     |______________|
                //     |______________|
                //     |______________|
                //
                context.SetFillColor(UIColor.Black.CGColor);
                context.FillRect(new CGRect(startX, startY, width, 2));
                context.FillRect(new CGRect(startX, startY + logoHeight + 8, width, 2));
                context.FillRect(new CGRect(startX, startY, 2, height));
                context.FillRect(new CGRect(startX + width - 2, startY, 2, height));
                context.FillRect(new CGRect(startX + logoWidth + 8, startY, 2, logoHeight + 10));
                context.FillRect(new CGRect(startX + logoWidth + 10, startY + (logoHeight / 2) + 5, width - (logoWidth + 10), 2));

                //Add final bottom lines
                if (App.watermarkShowString && App.StringName != "")
                    context.FillRect(new CGRect(startX, startY + height - stringOffset - 2, width, 2));

                if (App.watermarkShowDate && App.watermarkShowAcc)
                {
                    context.FillRect(new CGRect(startX, startY + height - (stringOffset / 2) - 2, width, 2));
                    context.FillRect(new CGRect(startX, startY + height - 2, width, 2));
                }
                else if (App.watermarkShowDate || App.watermarkShowAcc)
                    context.FillRect(new CGRect(startX, startY + height - 2, width, 2));

                logoRect = new CGRect(startX + 5, startY + 5, logoWidth, logoHeight);
                logo.Draw(logoRect);

                //Get height of text
                var chShift = Math.Abs(((logoHeight / 2) - calculateHeight(App.ch, textSize)) / 2);
                var offShift = Math.Abs(((logoHeight / 2) - calculateHeight(App.off, textSize)) / 2);

                //Draw CH String
                context.ScaleCTM(1f, -1f);
                context.TranslateCTM(startX + logoWidth + 13, -(nfloat)(startY + (logoHeight / 2) - (chShift + 5) + 5));
                chLine.Draw(context);
                var b = chLine.GetBounds(CoreText.CTLineBoundsOptions.UseOpticalBounds).Width;
                context.RestoreState();
                context.SaveState();

                //Draw OFF String
                context.ScaleCTM(1f, -1f);
                context.TranslateCTM(startX + logoWidth + 13 - b, -(nfloat)(startY + logoHeight - (offShift + 5) + 10));
                offLine.Draw(context);
                b += offLine.GetBounds(CoreText.CTLineBoundsOptions.UseOpticalBounds).Width;
                context.RestoreState();
                context.SaveState();

                //Draw String Name
                if(App.watermarkShowString && App.StringName != "")
                {
                    var stringShift = Math.Abs((smallTextSize - calculateHeight(App.StringName, smallTextSize)) / 2);
                    context.ScaleCTM(1f, -1f);
                    context.TranslateCTM(startX + 5 - b, -(nfloat)(startY + (height - (stringShift + 5 * App.watermarkSize) - stringOffset)));
                    stringNameLine.Draw(context);
                    b += stringNameLine.GetBounds(CoreText.CTLineBoundsOptions.UseOpticalBounds).Width;
                    context.RestoreState();
                    context.SaveState();
                }

                //Draw Accuracy String
                if (App.watermarkShowAcc)
                {
                    var accShift = Math.Abs((smallTextSize - calculateHeight(App.Acc, smallTextSize)) / 2);
                    context.ScaleCTM(1f, -1f);

                    if (App.watermarkShowDate)
                        context.TranslateCTM(startX + 5 - b, -(nfloat)(startY + (height - (accShift + 5) - (stringOffset / 2))));
                    else
                        context.TranslateCTM(startX + 5 - b, -(nfloat)(startY + (height - (accShift + 5))));

                    accLine.Draw(context);
                    b += accLine.GetBounds(CoreText.CTLineBoundsOptions.UseOpticalBounds).Width;
                    context.RestoreState();
                    context.SaveState();
                }

                //Draw Date String
                if (App.watermarkShowDate)
                {
                    var dateShift = Math.Abs((smallTextSize - calculateHeight(App.Date, smallTextSize)) / 2);
                    context.ScaleCTM(1f, -1f);
                    context.TranslateCTM((nfloat)(startX + 5 - b), -(nfloat)(startY + (height - (dateShift + 5))));
                    dateLine.Draw(context);
                    context.RestoreState();
                    context.SaveState();
                }

                resultImage = UIGraphics.GetImageFromCurrentImageContext();
                UIGraphics.EndImageContext();   
            }
            return resultImage;
        }

        private double calculateWidth(string text, float fontSize)
        {
            var label = new UILabel();
            label.Text = text;
            //label.Font = label.Font.WithSize(fontSize);
            label.Font = UIFont.FromName("Droid Sans", fontSize);
            var length = label.Text.StringSize(label.Font);
            return length.Width;
        }

        private double calculateHeight(string text, float fontSize)
        {
            var label = new UILabel();
            label.Text = text;
            //label.Font = label.Font.WithSize(fontSize);
            
            label.Font = UIFont.FromName("Droid Sans", fontSize);
            var length = label.Text.StringSize(label.Font);
            return length.Height;
        }

        private double Radians(double degrees) { return degrees * Math.PI / 180; }

        private void SetUpMetadata(string filename, ref NSMutableDictionary metadata)
        {
            NSMutableDictionary gps;
            NSMutableDictionary exif;
            NSMutableDictionary iptc;

            if (metadata[ImageIO.CGImageProperties.GPSDictionary] != null)
                gps = metadata.ObjectForKey(ImageIO.CGImageProperties.GPSDictionary).MutableCopy() as NSMutableDictionary;
            else
                gps = metadata.ObjectForKey(ImageIO.CGImageProperties.GPSDictionary) as NSMutableDictionary;

            if (metadata[ImageIO.CGImageProperties.ExifDictionary] != null)
                exif = metadata.ObjectForKey(ImageIO.CGImageProperties.ExifDictionary).MutableCopy() as NSMutableDictionary;
            else
                exif = metadata.ObjectForKey(ImageIO.CGImageProperties.ExifDictionary) as NSMutableDictionary;

            if (metadata[ImageIO.CGImageProperties.IPTCDictionary] != null)
                iptc = metadata.ObjectForKey(ImageIO.CGImageProperties.IPTCDictionary).MutableCopy() as NSMutableDictionary;
            else
                iptc = metadata.ObjectForKey(ImageIO.CGImageProperties.IPTCDictionary) as NSMutableDictionary;

            if (gps == null)
                gps = new NSMutableDictionary();

            if (exif == null)
                exif = new NSMutableDictionary();

            if (iptc == null)
                iptc = new NSMutableDictionary();

            if (gps[ImageIO.CGImageProperties.GPSLatitude] != null)
                gps[ImageIO.CGImageProperties.GPSLatitude] = new NSNumber(Math.Abs(App.lat));
            else
                gps.Add(ImageIO.CGImageProperties.GPSLatitude, new NSNumber(Math.Abs(App.lat)));

            if(App.lat < 0)
            {
                if (gps[ImageIO.CGImageProperties.GPSLatitudeRef] != null)
                    gps[ImageIO.CGImageProperties.GPSLatitudeRef] = new NSString("S");
                else
                    gps.Add(ImageIO.CGImageProperties.GPSLatitudeRef, new NSString("S"));
            }
            else
            {
                if (gps[ImageIO.CGImageProperties.GPSLatitudeRef] != null)
                    gps[ImageIO.CGImageProperties.GPSLatitudeRef] = new NSString("N");
                else
                    gps.Add(ImageIO.CGImageProperties.GPSLatitudeRef, new NSString("N"));
            }

            if (gps[ImageIO.CGImageProperties.GPSLongitude] != null)
                gps[ImageIO.CGImageProperties.GPSLongitude] = new NSNumber(Math.Abs(App.lng));
            else
                gps.Add(ImageIO.CGImageProperties.GPSLongitude, new NSNumber(Math.Abs(App.lng)));

            if(App.lng < 0)
            {
                if (gps[ImageIO.CGImageProperties.GPSLongitudeRef] != null)
                    gps[ImageIO.CGImageProperties.GPSLongitudeRef] = new NSString("W");
                else
                    gps.Add(ImageIO.CGImageProperties.GPSLongitudeRef, new NSString("W"));
            }
            else
            {
                if (gps[ImageIO.CGImageProperties.GPSLongitudeRef] != null)
                    gps[ImageIO.CGImageProperties.GPSLongitudeRef] = new NSString("E");
                else
                    gps.Add(ImageIO.CGImageProperties.GPSLongitudeRef, new NSString("E"));
            }

            //if (exif[ImageIO.CGImageProperties.ExifUserComment] != null)
            //    exif[ImageIO.CGImageProperties.ExifUserComment] = new NSString("NRGComment::Chainage=" + App.chValue + "," + "Offset=" + App.offValue);
            //else
            //    exif.Add(ImageIO.CGImageProperties.ExifUserComment, new NSString("NRGComment::Chainage=" + App.chValue + "," + "Offset=" + App.offValue));

            if (iptc[ImageIO.CGImageProperties.IPTCObjectName] != null)
                iptc[ImageIO.CGImageProperties.IPTCObjectName] = new NSString(filename);
            else
                iptc.Add(ImageIO.CGImageProperties.IPTCObjectName, new NSString(filename));

            if (metadata[ImageIO.CGImageProperties.Orientation] != null)
                metadata[ImageIO.CGImageProperties.Orientation] = new NSNumber(1);
            else
                metadata.Add(ImageIO.CGImageProperties.Orientation, new NSNumber(1));
            

            metadata.SetValueForKey(iptc, ImageIO.CGImageProperties.IPTCDictionary);
            metadata.SetValueForKey(exif, ImageIO.CGImageProperties.ExifDictionary);
            metadata.SetValueForKey(gps, ImageIO.CGImageProperties.GPSDictionary);
        }
    }
}