﻿using System;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Linq;
using System.Xml.Serialization;
using System.Text;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Windows;
using System.Linq;
using System.Diagnostics;
//using nrgEngine;


namespace AlignmentViewer
{
    public class LandXmlParser
    {
        private string _theXmlFile = "";
        private string _xmlDocumentString = "";
        private XElement _xDoc = null;
        private List<LandXml_Alignment> _allAlignments;   //store the alignment nodes (replaces dimension 1 of vb6 double-dimension arrays
        private alignments.alignment[] _nrgAlignments;  //the NRG alignment objects
        int _numAlignments = -1;
        int _numNrgAlignments = 0;

        //constructors
        public LandXmlParser() { }

        public string xmlFileToParse
        {
            //get => _theXmlFile;
            set => _theXmlFile = value;
        }
        public string xmlDocumentAsString
        {
            set => _xmlDocumentString = value;
        }
        public int GetNrgAlignmentCount
        {
            get { return _numNrgAlignments; }
        }
        public alignments.alignment GetNrgAlignment(int index)
        {
             return _nrgAlignments[index]; 
        }
        //public int GetAlignmentCount
        //{
        //    get { return _allAlignments.Count(); }
        //}
        //public LandXml_Alignment GetAlignment(int index)
        //{
        //    return _allAlignments[index];
        //}


        /// <summary>
        /// The logic is built for general use. You can pass in the parameters or ignore these paramaters and use the public properties before calling the method.
        /// For Android, due to architecture restrictions, open and read the file and pass in the document string instead of filename.
        /// Builds the NRG Alignment objects, which you access through the public properties.
        /// </summary>
        /// <param name="xmlFileToParse"></param>
        /// <param name="readingSurfaces"></param>
        public void LoadLandXmlFile(string xmlFileToParse = "", string xmlDocumentString = "")
        {
            if (xmlFileToParse.Length > 0)
            {
                _theXmlFile = xmlFileToParse;
            }
            if (xmlDocumentString.Length > 0)
            {
                _xmlDocumentString = xmlDocumentString;
            }
            if (_theXmlFile.Length <= 0)
            {
                if (_xmlDocumentString.Length <= 0)
                {
                    return;
                }
                else
                {
                    try
                    {
                        _xDoc = XElement.Parse(_xmlDocumentString, LoadOptions.PreserveWhitespace);
                    }
                    catch (XmlException ex)
                    {
                        return;
                    }
                }
            }
            else
            {
                try
                {
                    if (nrgFileRead.reader.fileExists(_theXmlFile) == false) { return; }
                    _xDoc = XElement.Load(_theXmlFile);
                }
                catch (XmlException ex)
                {
                    return;
                }
            }


            ParseTheXml();
            if (_numAlignments > 0)
            {
                TransferTo_NRGAlignment();
            }
        }

        private void ParseTheXml()
        {
            int angleFormat = -1;
            int algIndex = -1;
            double curCh = 0;
            bool eleHasCh = false;
            string unitText = "";

            try
            {
                XNamespace ns = _xDoc.GetDefaultNamespace();
                IEnumerable<XElement> xAligns = from elem in _xDoc.Elements(ns + "Alignments") select elem;
                if (xAligns.Descendants(ns + "Alignment").Count() > 0) { _numAlignments = xAligns.Descendants(ns + "Alignment").Count(); }
                if (_numAlignments < 0)
                {
                    xAligns = from elem in _xDoc.Elements("Alignments") select elem;
                    if(xAligns.Descendants("Alignment").Count() > 0)
                    {
                        _numAlignments = xAligns.Descendants("Alignment").Count();
                    }

                    if (_numAlignments < 0)
                        return;

                    ns = xAligns.First().GetDefaultNamespace();
                }

                _allAlignments = new List<LandXml_Alignment>(_numAlignments);
                for (int n = 0; n < _numAlignments; n++)
                {
                    _allAlignments.Add(null);
                }

                //if someone can figure out how to do the next section in a single query - go for it!  the problem is that the TAG is either METRIC or IMPERIAL...
                IEnumerable<XElement> xMetric = from elem in _xDoc.Descendants(ns + "Metric") select elem;
                foreach (XElement el in xMetric)
                {
                    if (el.Attribute("directionUnit") != null)
                    {
                        unitText = (string)el.Attribute("directionUnit").Value;
                    }
                    else if (el.Attribute("angularUnit") != null)
                    {
                        unitText = (string)el.Attribute("angularUnit").Value;
                    }
                }
                IEnumerable<XElement> xImperial = from elem in _xDoc.Descendants(ns + "Imperial") select elem;
                foreach (XElement el in xImperial)
                {
                    if (el.Attribute("directionUnit") != null)
                    {
                        unitText = (string)el.Attribute("directionUnit").Value;
                    }
                    else if (el.Attribute("angularUnit") != null)
                    {
                        unitText = (string)el.Attribute("angularUnit").Value;
                    }
                }
                switch (unitText)
                {
                    case "":
                    case "radians":
                        angleFormat = 0;
                        break;
                    case "grads":
                        angleFormat = 1;
                        break;
                    case "decimal degrees":
                        angleFormat = 2;
                        break;
                    case "decimal dd.mm.ss":
                        angleFormat = 3;
                        break;
                }

                foreach (XElement el in xAligns.Descendants())
                {
                    //Debug.WriteLine(el.Name.ToString());
                    if (el.Name == ns + "Alignment")
                    {
                        if (el.Attribute("name").ToString().Length > 0)
                        {
                            algIndex += 1;

                            LandXml_Alignment _alignment = new LandXml_Alignment
                            {
                                AngleFormat = angleFormat,
                                UnitText = unitText,
                                alignmentName = el.Attribute("name").Value.ToString()
                            };

                            ///horiontal alignment parsing (also called coordgeom)
                            var cntLine = from geom in el.Elements(ns + "CoordGeom").Descendants(ns + "Line") select geom;
                            var cntSpiral = from geom in el.Elements(ns + "CoordGeom").Descendants(ns + "Spiral") select geom;
                            var cntCurve = from geom in el.Elements(ns + "CoordGeom").Descendants(ns + "Curve") select geom;
                            _alignment.numHzElements = (int)cntLine.Count() + (int)cntSpiral.Count() + (int)cntCurve.Count();
                            if (el.Attributes("staStart").Any())
                                eleHasCh = true;
                            curCh = Convert.ToDouble(el.Attribute("staStart").Value);

                            IEnumerable<XElement> xGeom = from geom in el.Elements(ns + "CoordGeom").Descendants() select geom;
                            foreach (XElement geomEl in xGeom)
                            {
                                //Debug.WriteLine(geomEl.Name.ToString());
                                XElement ele = new XElement(geomEl);
                                LandXml_HzElement hzEle = new LandXml_HzElement();
                                if (hzEle.ExtractElement(ref ele, ns, ref eleHasCh, angleFormat))
                                {
                                    if (eleHasCh) { hzEle.StartChainage = curCh; }
                                    curCh += hzEle.HzLength;
                                    _alignment.hzElems.Add(hzEle);
                                }
                            }


                            ///vertical alignment parsing (also called profile)
                            var cntCirc = from profile in el.Elements(ns + "Profile").Elements(ns + "ProfAlign").Descendants(ns + "CircCurve") select profile;
                            var cntPara = from profile in el.Elements(ns + "Profile").Elements(ns + "ProfAlign").Descendants(ns + "ParaCurve") select profile;
                            var cntPvi = from profile in el.Elements(ns + "Profile").Elements(ns + "ProfAlign").Descendants(ns + "PVI") select profile;
                            _alignment.numVtElements = (int)cntCirc.Count() + (int)cntPara.Count() + (int)cntPvi.Count();
                            IEnumerable<XElement> xProfile = from profile in el.Elements(ns + "Profile").Elements(ns + "ProfAlign").Descendants() select profile;
                            foreach (XElement profEl in xProfile)
                            {
                                //Debug.WriteLine(profEl.Name.ToString());
                                XElement ele = new XElement(profEl);
                                LandXml_VertElement vtEle = new LandXml_VertElement();
                                if (vtEle.ExtractElement(ref ele, ns))
                                {
                                    _alignment.vtElems.Add(vtEle);
                                }
                            }
                            _allAlignments[algIndex] = _alignment;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                return;
            }
        }

        private void TransferTo_NRGAlignment()
        {
            _nrgAlignments = new alignments.alignment[0];

            alignments.horizontalAlignment nrgHzAlign = new alignments.horizontalAlignment();
            alignments.verticalAlignment nrgVtAlign = new alignments.verticalAlignment();
            alignments.HorizontalElement thisHzEle = new alignments.HorizontalElement();
            alignments.HorizontalElement nextHzEle = new alignments.HorizontalElement();
            alignments.VerticalElement thisVtEle;
            LandXml_HzElement eleHz;
            LandXml_VertElement eleVt;

            foreach (LandXml_Alignment xmlAlignment in _allAlignments)
            {
                _numNrgAlignments++;
                Array.Resize<alignments.alignment>(ref _nrgAlignments, _numNrgAlignments);
                _nrgAlignments[_numNrgAlignments - 1] = new alignments.alignment();
                nrgHzAlign = new alignments.horizontalAlignment();
                nrgVtAlign = new alignments.verticalAlignment();

                ///convert the xml objects into NRG objects
                for (int hzCount = 0; hzCount < xmlAlignment.numHzElements; hzCount++)
                {
                    eleHz = new LandXml_HzElement();
                    thisHzEle = new alignments.HorizontalElement();
                    eleHz = xmlAlignment.hzElems[hzCount];
                    thisHzEle.East = eleHz.StartEast;
                    thisHzEle.North = eleHz.StartNorth;
                    thisHzEle.Radius = eleHz.StartRadius;
                    thisHzEle.Length = eleHz.HzLength;
                    thisHzEle.Chainage = eleHz.StartChainage;
                    switch (eleHz.EleType)
                    {
                        case 1:
                            thisHzEle.ElementType = helpers.HorizontalElementType.Arc;
                            break;
                        case 2:
                            thisHzEle.ElementType = helpers.HorizontalElementType.Spiral;
                            break;
                        default:
                            thisHzEle.ElementType = helpers.HorizontalElementType.Straight;
                            break;
                    }
                    nrgHzAlign.AddElement(thisHzEle);
                }
                //add final horizontal element
                eleHz = new LandXml_HzElement();
                eleHz = xmlAlignment.hzElems[xmlAlignment.numHzElements-1];
                thisHzEle = new alignments.HorizontalElement();
                if(eleHz.HzLength > 0 )
                {
                    thisHzEle.East = eleHz.EndEast;
                    thisHzEle.North = eleHz.EndNorth;
                    thisHzEle.Radius = 0;
                    thisHzEle.Length = eleHz.HzLength;
                    thisHzEle.Chainage = eleHz.StartChainage + eleHz.HzLength;
                    thisHzEle.ElementType = helpers.HorizontalElementType.Straight;
                    nrgHzAlign.AddElement(thisHzEle);
                }
                ///convert The vertical Elements into NRG Alignment
                Convert_IntersectTo_Tangent(xmlAlignment.vtElems, ref nrgVtAlign);

                //for (int vtCount = 0; vtCount < xmlAlignment.numVtElements; vtCount++)
                //{

                //    eleVt = new LandXml_VertElement();
                //    thisVtEle = new alignments.VerticalElement();
                //    eleVt = xmlAlignment.vtElems[vtCount];
                //    thisVtEle.Chainage = eleVt.EleChainage;
                //    thisVtEle.Level = eleVt.EleLevel;
                //    thisVtEle.Radius = eleVt.EleRadius;
                //    switch (eleVt.EleType)
                //    {
                //        case 1:
                //        case 2:
                //            thisVtEle.ElementType = helpers.verticalElementType.Parabola;
                //            break;
                //        default:
                //            thisVtEle.ElementType = helpers.verticalElementType.Straight;
                //            break;
                //    }
                //    nrgVtAlign.AddElement(thisVtEle);
                //}
                _nrgAlignments[_numNrgAlignments - 1].vertical = nrgVtAlign;

                ///convert the horizontal NRG objects into proper NRG Alignment
                nextHzEle = new alignments.HorizontalElement();
                if (nrgHzAlign.elements.Count > 1)
                {
                    for (int i = 0; i < nrgHzAlign.elements.Count -1; i++)
                    {
                        thisHzEle = nrgHzAlign.elements[i];
                        nextHzEle = nrgHzAlign.elements[i + 1];
                        nrgHzAlign.CalcOrigins(ref thisHzEle, nextHzEle);
                    }
                    nrgHzAlign.elements.Last().ChainageOrigin = nrgHzAlign.elements.Last().Chainage;
                    nrgHzAlign.elements.Last().RadiusOrigin = nrgHzAlign.elements[nrgHzAlign.elements.Count()-2].Radius;
                    var tang = 0D;
                    var r = 0D;
                    bool OK = false;
                    var outVec = new AlignmentViewer.workspace.dataTypes.nrgVector2();
                    if(nrgHzAlign.elements.Count <= 2)
                        nrgHzAlign.Coords(nrgHzAlign.elements.Last().Chainage - 0.001, nrgHzAlign.elements[nrgHzAlign.elements.Count() - 2], null, nrgHzAlign.elements.Last(), ref outVec, ref tang, ref r, ref OK);
                    else
                        nrgHzAlign.Coords(nrgHzAlign.elements.Last().Chainage - 0.001, nrgHzAlign.elements[nrgHzAlign.elements.Count() - 2], nrgHzAlign.elements[nrgHzAlign.elements.Count() - 3], nrgHzAlign.elements.Last(), ref outVec, ref tang, ref r, ref OK);
                    nrgHzAlign.elements.Last().BrgOrigin = tang;
                }
                _nrgAlignments[_numNrgAlignments - 1].horizontal = nrgHzAlign;
                _nrgAlignments[_numNrgAlignments - 1].stringName = xmlAlignment.alignmentName;
                //Debug.WriteLine("nrg num aligns = " + _numNrgAlignments);
                //Debug.WriteLine("nrg string name = " + _nrgAlignments[_numNrgAlignments - 1].stringName);
            }
        }

        private void Convert_IntersectTo_Tangent(List<LandXml_VertElement> xmlVerEles, ref alignments.verticalAlignment nrgVertEles)
        {
            for (int i = 0; i < xmlVerEles.Count(); i++)
            {
                var v = xmlVerEles[i];

                if (v.EleRadius != 0 && nrgVertEles.elements.Count() > 0)
                {
                    var ingrad = Math.Asin((v.EleLevel - nrgVertEles.elements.Last().Level) / (v.EleChainage - nrgVertEles.elements.Last().Chainage));
                    var outgrad = ingrad - (v.EleLength / v.EleRadius);

                    var da = (v.EleLength / v.EleRadius) / 2;
                    var tl = v.EleRadius * Math.Tan(da);

                    var nrgVert = new alignments.VerticalElement();
                    nrgVert.Chainage = v.EleChainage - (v.EleLength / 2);
                    nrgVert.Radius = v.EleRadius;
                    nrgVert.Level = v.EleLevel - (Math.Sin(ingrad) * tl);

                    if (v.EleLevel < nrgVert.Level)
                        nrgVert.Radius = Math.Abs(nrgVert.Radius);
                    else
                        nrgVert.Radius = -Math.Abs(nrgVert.Radius);

                    nrgVertEles.AddElement(nrgVert);

                    var nrgVert2 = new alignments.VerticalElement();
                    nrgVert2.Chainage = v.EleChainage + (v.EleLength / 2D);
                    nrgVert2.Radius = 0D;
                    nrgVert2.Level = v.EleLevel + (Math.Sin(outgrad) * tl);
                    nrgVertEles.AddElement(nrgVert2);
                }
                else if (v.EleType == 2 && nrgVertEles.elements.Count() > 0)
                {
                    var nrgVert = new alignments.VerticalElement();
                    nrgVert.Chainage = v.EleChainage - (v.EleLength / 2D);

                    var ingrad = Math.Asin((v.EleLevel - nrgVertEles.elements.Last().Level) / (v.EleChainage - nrgVertEles.elements.Last().Chainage));
                    var outgrad = Math.Asin((xmlVerEles[i + 1].EleLevel - v.EleLevel) / (xmlVerEles[i + 1].EleChainage - v.EleChainage));

                    var ig = ((xmlVerEles[i - 1].EleLevel - v.EleLevel) / (xmlVerEles[i - 1].EleChainage - v.EleChainage));
                    var og = ((xmlVerEles[i + 1].EleLevel - v.EleLevel) / (xmlVerEles[i + 1].EleChainage - v.EleChainage));

                    nrgVert.Radius = v.EleLength / (og - ig);

                    var da = (v.EleLength / nrgVert.Radius) / 2D;
                    var tl = nrgVert.Radius * Math.Tan(da);

                    nrgVert.Level = v.EleLevel - (Math.Sin(ig) * tl);

                    if (v.EleLevel < nrgVert.Level)
                        nrgVert.Level = Math.Abs(nrgVert.Level);
                    else
                        nrgVert.Level = -Math.Abs(nrgVert.Level);

                    nrgVertEles.AddElement(nrgVert);
                    var nrgVert2 = new alignments.VerticalElement();
                    nrgVert.Chainage = v.EleChainage + (v.EleLength / 2D);
                    nrgVert.Radius = 0D;
                    nrgVert.Level = v.EleLevel + (Math.Sin(og) * tl);
                    nrgVertEles.AddElement(nrgVert2);
                }
                else
                {
                    if (i < xmlVerEles.Count() || v.EleChainage != 0)
                    {
                        var nrgVert = new alignments.VerticalElement();
                        nrgVert.Chainage = v.EleChainage;
                        nrgVert.Level = v.EleLevel;
                        nrgVert.Radius = v.EleRadius;
                        nrgVertEles.AddElement(nrgVert);
                    }
                }
            }
        }
    }
}
