﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Linq;
using System.Linq;
using System.Diagnostics;

namespace AlignmentViewer
{
    public partial class LandXml_Alignment
    {
        //the parent holder, containing the hzElements[], vertElements[], chainage[] data
        //common variables for all alignments
        public string UnitText;
        public int AngleFormat;
        
        //for each unique alignment
        public string alignmentName;
        public int numHzElements;
        public int numVtElements;
        public List<LandXml_HzElement> hzElems = new List<LandXml_HzElement>();
        public List<LandXml_VertElement> vtElems = new List<LandXml_VertElement>();

    }

    public partial class LandXml_HzElement
    {
        public int EleType;
        public double StartChainage;
        public double StartRadius;
        public double HzLength;
        public double StartEast;
        public double StartNorth;
        public double EndEast;
        public double EndNorth;
        public double StartBearing;
        public double EndBearing;

        public bool ExtractElement(ref XElement section, XNamespace nmSpace, ref bool objectHasCh, int angleFormat)
        {
            if (section == null) { return false; }

            if (section.Name == nmSpace + "Line" || section.Name == nmSpace + "IrregularLine")
            {
                EleType = 0;
            }
            else if (section.Name == nmSpace + "Curve")
            {
                EleType = 1;
            }
            else if (section.Name == nmSpace + "Spiral")
            {
                EleType = 2;
            }
            else
            {
                return false;
            }

            if (section.Attribute("staStart") != null)
            {
                StartChainage = Convert.ToDouble(section.Attribute("staStart").Value);
                objectHasCh = true;
            }
            if (section.Attribute("dirStart") != null)
            {
                StartBearing = ParseAngle(angleFormat, section.Attribute("dirStart").Value);
            }
            if (section.Attribute("dirEnd") != null)
            {
                EndBearing = ParseAngle(angleFormat, section.Attribute("dirEnd").Value);
            }
            if (section.Attribute("radius") != null)
            {
                bool q = Double.TryParse(section.Attribute("radius").Value, out StartRadius);
            }
            else
            {
                if (section.Attribute("radiusStart") != null)
                {

                    bool q = Double.TryParse(section.Attribute("radiusStart").Value, out StartRadius);
                }
            }
            if (section.Attribute("rot") != null)
            {
                if (section.Attribute("rot").Value == "ccw")
                {
                    StartRadius = -Math.Abs(StartRadius);
                }
            }
            if (section.Attribute("length") != null)
            {
                HzLength = Convert.ToDouble(section.Attribute("length").Value);
            }
            if (section.HasElements)
            {
                IEnumerable<XElement> xSub = from el in section.Descendants() select el;
                foreach (XElement subEl in xSub)
                {
                    string contents = subEl.Value;
                    contents = contents.Replace("\n", " ");
                    contents = contents.Replace("\t", " ");
                    string[] subContents = contents.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                    if (subEl.Name == nmSpace + "Start")
                    {
                        StartEast = Convert.ToDouble(subContents[1]);
                        StartNorth = Convert.ToDouble(subContents[0]);
                    }
                    else if (subEl.Name == nmSpace + "End")
                    {
                        EndEast = Convert.ToDouble(subContents[1]);
                        EndNorth = Convert.ToDouble(subContents[0]);
                    }
                }
            }
            return true;

        }

        private double ParseAngle(int angleFormat, string angle)
        {
            double d = 0;
            double m = 0;
            double s = 0;
            double tempAngle = Convert.ToDouble(angle);
            double theAngle = 0;
            switch (angleFormat)
            {
                case 0:
                    theAngle = tempAngle;
                    break;
                case 1:
                    theAngle = tempAngle * Math.PI / 200;
                    break;
                case 2:
                    theAngle = tempAngle * Math.PI / 180;
                    break;
                case 3:
                    d = (int)tempAngle;
                    tempAngle = (tempAngle - d) * 100;
                    m = (int)tempAngle;
                    tempAngle = (tempAngle - m) * 100;
                    s = tempAngle;
                    theAngle = (d + (m / 60) + (s / 3600)) * Math.PI / 180;
                    break;
            }
            return theAngle;
        }

    }

    public partial class LandXml_VertElement
    {
        public int EleType;
        public double EleChainage;
        public double EleLevel;
        public double EleRadius;
        public double EleLength;

        public bool ExtractElement(ref XElement section, XNamespace nmSpace)
        {
            if (section.Name == nmSpace + "CircCurve")
            {
                EleType = 1;
                if (section.Attribute("radius") != null)
                {
                    bool q = Double.TryParse(section.Attribute("radius").Value, out EleRadius);
                }
                if (section.Attribute("length") != null)
                {
                    bool q = Double.TryParse(section.Attribute("length").Value, out EleLength);
                }
            }
            else if (section.Name == nmSpace + "ParaCurve")
            {
                EleType = 2;
                if (section.Attribute("length") != null)
                {
                    bool q = Double.TryParse(section.Attribute("length").Value, out EleLength);
                }
            }
            else if (section.Name == nmSpace + "PVI")
            {
                EleType = 0;
            }
            else
            {
                return false;
            }
            if (section.Value.ToString().Length > 0)
            {
                string contents = section.Value;
                contents = contents.Replace("\n", "");
                contents = contents.Replace("\t", " ");
                string[] subContents = contents.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                bool q = Double.TryParse(subContents[0], out EleChainage);
                q = Double.TryParse(subContents[1], out EleLevel);
            }
            return true;
        }
    }

}
