﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace AlignmentViewer
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class TransformationSearch : ContentPage
    {
        public List<Services.TransformationListItem> tList = new List<Services.TransformationListItem>();

        public TransformationSearch()
        {
            InitializeComponent();
            data();
            transformationList.ItemsSource = tList;
            if (Device.RuntimePlatform == Device.iOS)
            {
                searchBar.TextColor = Color.Black;
            }

        }

        public void data()
        {
            tList = new List<Services.TransformationListItem>();
            string transformations = DependencyService.Get<IGetTransformations>().getTransformations();
            string[] templist = transformations.Split(new string[] { "#" }, StringSplitOptions.None);
            foreach (string transformation in templist)
            {
                try
                {
                    string[] temp = transformation.Split('"');
                    if(temp.Length >= 2)
                    {
                        if(temp[0].Contains(',') && temp[1].Contains("+proj="))
                        {
                            //Valid file
                            string[] tempname = temp[0].Split(',');
                            if(tempname.Length >= 2)
                            {
                                var transform = new Services.TransformationListItem();
                                transform.Name = tempname[0].TrimStart(' ');
                                transform.EPSG = tempname[1];
                                transform.Transformation = temp[1].Replace("\"", "");
                                tList.Add(transform);
                            }
                        }
                    }
                }
                catch (Exception e)
                {

                }

            }
        }

        private void SearchBar_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (string.IsNullOrEmpty(e.NewTextValue))
            {
                transformationList.ItemsSource = tList;
            }
            else
            {
                transformationList.ItemsSource = tList.Where(x => x.Name.waContains(e.NewTextValue, StringComparison.CurrentCultureIgnoreCase) || x.EPSG.waContains(e.NewTextValue, StringComparison.CurrentCultureIgnoreCase));
            }
        }

        private void TextCell_Tapped(object sender, EventArgs e)
        {
            Services.TransformationListItem item = (Services.TransformationListItem)(((TextCell)sender).BindingContext);
            if (item.EPSG != App.transformationString)
            {
                App.transformationString = item.Name;
                App.transformationBody = item.Transformation;

                if (item.Transformation.Contains("+units=m"))
                    App.DefaultUnits = Unit.Metre;
                else if (item.Transformation.Contains("+units=ft"))
                    App.DefaultUnits = Unit.Foot;
                else if (item.Transformation.Contains("+units=us-ft"))
                    App.DefaultUnits = Unit.USFoot;
                else
                    App.DefaultUnits = Unit.None;

                App.SavedUnits = App.DefaultUnits;

                MessagingCenter.Send<string>("SwapTransformation", "SwapTransformation");
            }
            Navigation.PopAsync();
        }
    }

    public static class Extensions
    {
        public static bool waContains(this string source, string toCheck, StringComparison comp)
        {
            return source != null && toCheck != null && source.IndexOf(toCheck, comp) >= 0;
        }
    }
}