﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Threading.Tasks;
using System.IO;
using Plugin.Geolocator;
using Plugin.Geolocator.Abstractions;
using Plugin.Permissions;
using Plugin.Permissions.Abstractions;
using Plugin.Settings;
using Xamarin.Forms;
using Xamarin.Forms.GoogleMaps;

namespace AlignmentViewer
{


    public class workspace
    {
        public Xamarin.Forms.GoogleMaps.Position devicePosition;
        public dataTypes.nrgVector3Bounds workspaceBounds = new dataTypes.nrgVector3Bounds();
        public alignments.alignment Alignments = new alignments.alignment();
        public List<dataTypes.nrgLayer> LayerList = new List<dataTypes.nrgLayer>();
        public string datapath;
        public string currentFolder;
        public string currentAlignment;
        public string currentPins;
        public string currentLines;
        public string currentTransformation;
        public int decimalDisplay;
        public string fontSize;
        public int mapType;
        public bool scrollMap;
        public bool crosshairs;
        public string offsetDisplay;
        public string internationalDisplay;
        public double widthScale;
        public string pinType;
        public bool landscape;
        public bool crossSection;
        public string menuExpanded;
        public bool chainageFormat;
        public double distance;
        public bool scale;
        public double vertExag;
        public string countryCode;
        public bool scaleBar;
        public double elevationDistance;
        public double offsetCenter;
        public bool centeredByOffset;
        public bool locked;
        public double heading;
        public bool requestOnly;
        public double updateTime;
        public double updateDistance;
        public string watermarkPosition;
        public string watermarkSize;
        public string chainageFormatDesc;
        public bool alwaysUseGPS = false;

        //Photo stuff
        public string photoCoordName = "";
        public string photoIncrementName = "";

        /// <summary>
        /// Actually site scale
        /// </summary>
        public double scaleFactor;
        public double siteShiftE;
        public double siteShiftN;
        public string scaleFactorFormat;
        public transformationSettings transformation;
        public Xamarin.Forms.GoogleMaps.Position pos;
        public double radius;
        public List<string> listDirectory = new List<string>();
        public helpers helper = new helpers();


        public void startLocator()
        {
            var initMapResult = initLocator();
            //var initPermission = GetStoragePermission();
        }

        private async Task initLocator()
        {
            try
            {
                if (Device.RuntimePlatform == Device.iOS)
                {
                    var status = await CrossPermissions.Current.CheckPermissionStatusAsync(Permission.Location);
                    if (status != PermissionStatus.Granted)
                    {
                        await CrossPermissions.Current.RequestPermissionsAsync(Permission.Location);
                        status = await CrossPermissions.Current.CheckPermissionStatusAsync(Permission.Location);
                    }

                    if (CrossGeolocator.Current.IsListening)
                        await CrossGeolocator.Current.StopListeningAsync();

                    CrossGeolocator.Current.DesiredAccuracy = 100;
                    CrossGeolocator.Current.PositionError += (sender, args) =>
                    {

                    };

                    var settings = new ListenerSettings();
                    settings.AllowBackgroundUpdates = true;

                    if(CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                        await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromDays(3000), 1.0, true);
                }


                //if(Device.RuntimePlatform == Device.Android)
                //{
                //    var gpsPositionTask = locator.GetPositionAsync(TimeSpan.FromMilliseconds(25000));
                //    var gpsPosition = await gpsPositionTask;
                //    devicePosition = new Xamarin.Forms.GoogleMaps.Position(gpsPosition.Latitude, gpsPosition.Longitude);
                //}
                //else
                //{
                //    var gpsPositionTask = locator.GetPositionAsync(TimeSpan.FromMilliseconds(15000));
                //    var gpsPosition = await gpsPositionTask;
                //    devicePosition = new Xamarin.Forms.GoogleMaps.Position(gpsPosition.Latitude, gpsPosition.Longitude);
                //}

                //Plugin.Compass.CrossCompass.Current.Start();
                //Plugin.Compass.CrossCompass.Current.CompassChanged += (sender, args) =>
                //{
                //    heading = args.Heading;
                //};
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine("exc: " + ex.Message);

            }
        }

        private async Task GetStoragePermission()
        {
            var access = await Plugin.Permissions.CrossPermissions.Current.CheckPermissionStatusAsync(Plugin.Permissions.Abstractions.Permission.Storage);

            if (access != Plugin.Permissions.Abstractions.PermissionStatus.Granted)
            {
                //Ask for permission
                var test = await Plugin.Permissions.CrossPermissions.Current.RequestPermissionsAsync(Plugin.Permissions.Abstractions.Permission.Storage);
            }

            if (await Plugin.Permissions.CrossPermissions.Current.CheckPermissionStatusAsync(Plugin.Permissions.Abstractions.Permission.Storage) != Plugin.Permissions.Abstractions.PermissionStatus.Granted)
                return;

            nrgFileRead.reader.CreateFolder(datapath);
            nrgFileRead.reader.CreateFolder(datapath + "Transformation/");
            nrgFileRead.reader.CreateFile(datapath + "Transformation/setUp.tmp");

            return;
        }

        public class dataTypes
        {
            static public List<fieldCode> importFieldCodeStringToList(string inputCodeString)
            {
                List<string> switchIgnoreList = new List<string> { "+", "-" };

                List<fieldCode> outputCodeList = new List<fieldCode>();
                String majorCodeString = "";

                //Split coding up by space to get individual major coding
                string[] fieldCodeArray = inputCodeString.ToUpper().TrimEnd(' ').Split(new string[] { " " }, StringSplitOptions.None);

                foreach (string fieldCodeString in fieldCodeArray)
                {
                    bool isBreakLine = false;
                    //split each part to find major and command switch
                    string[] fieldCodeElementArray = fieldCodeString.Split(new string[] { "/" }, StringSplitOptions.None);
                    majorCodeString = fieldCodeElementArray[0].ToString();

                    //Check to see if breakline (from features ini)
                    isBreakLine = checkIfBreakLine(majorCodeString);
                    string newCode = "";
                    //check if any switches
                    List<codeSwitch> importCodeSwitchList = new List<codeSwitch>();
                    if (fieldCodeElementArray.GetLength(0) > 1)
                        if (fieldCodeElementArray[1].Length > 0)
                        {
                            for (int switchCount = 1; switchCount < fieldCodeElementArray.GetLength(0); switchCount++)
                            {
                                newCode = "";
                                //Do a pile of clean up and only import recognised switched
                                if (fieldCodeElementArray[switchCount].Contains("ST"))
                                    newCode = "ST";
                                if (fieldCodeElementArray[switchCount].Contains("STCV"))
                                    newCode = "";
                                if (fieldCodeElementArray[switchCount].Contains("RGB"))
                                    newCode = fieldCodeElementArray[switchCount];

                                if (fieldCodeElementArray[switchCount].Contains("CL"))
                                    newCode = "CL";
                                if (fieldCodeElementArray[switchCount].Contains("TR"))
                                    newCode = "";
                                if (fieldCodeElementArray[switchCount].Contains("TX"))
                                    newCode = "";
                                if (fieldCodeElementArray[switchCount].Contains("TY"))
                                    newCode = "";
                                if (fieldCodeElementArray[switchCount][0].Equals('-'))
                                    newCode = "-";
                                if (fieldCodeElementArray[switchCount][0].Equals('+'))
                                    newCode = "-";

                                if (newCode.Length > 0)
                                    importCodeSwitchList.Add(new codeSwitch { code = newCode });
                                //need to do some kind of check to split the value out here, only puting whole switch+value into code string at moment
                            }
                        }
                    outputCodeList.Add(new fieldCode { majorCode = majorCodeString, codeSwitchList = importCodeSwitchList, isBreakLine = isBreakLine });
                }
                return outputCodeList;
            }
            static public List<fieldCode> parseFieldCodeStringToList(string inputCodeString)
            {
                List<fieldCode> outputCodeList = new List<fieldCode>();
                String majorCodeString = "";

                //Split coding up by space to get individual major coding
                string[] fieldCodeArray = inputCodeString.ToUpper().TrimEnd(' ').Split(new string[] { " " }, StringSplitOptions.None);

                foreach (string fieldCodeString in fieldCodeArray)
                {
                    bool isBreakLine = false;
                    //split each part to find major and command switch
                    string[] fieldCodeElementArray = fieldCodeString.Split(new string[] { "/" }, StringSplitOptions.None);
                    majorCodeString = fieldCodeElementArray[0].ToString();

                    //Check to see if breakline (from features ini)
                    isBreakLine = checkIfBreakLine(majorCodeString);

                    //check if any switches
                    List<codeSwitch> importCodeSwitchList = new List<codeSwitch>();
                    if (fieldCodeElementArray.GetLength(0) > 1)
                    {
                        for (int switchCount = 1; switchCount < fieldCodeElementArray.GetLength(0); switchCount++)
                        {
                            //need to do some kind of check to split the value out here, only puting whole switch+value into code string at moment
                            importCodeSwitchList.Add(new codeSwitch { code = fieldCodeElementArray[switchCount].ToString() });
                        }

                    }
                    outputCodeList.Add(new fieldCode { majorCode = majorCodeString, codeSwitchList = importCodeSwitchList, isBreakLine = isBreakLine });
                }
                return outputCodeList;
            }
            static public string parseFieldCodeListToString(List<fieldCode> fieldCodeList)
            {
                string outputString = "";
                foreach (fieldCode fieldCode in fieldCodeList)
                {
                    outputString = outputString + fieldCode.majorCode;

                    foreach (codeSwitch switchCode in fieldCode.codeSwitchList)
                    {
                        outputString = outputString + "/";
                        outputString = outputString + switchCode.code;
                    }
                    outputString = outputString + " ";
                }

                return outputString;

            }
            static public bool checkIfBreakLine(string featureCodeString)
            {
                bool isBreakline = false;

                List<string> listOfStrings = new List<string> { "LS", "PK", "PZ", "PG", "PGA", "PGA", "PHS", "PW", "PGMC", "PI", "PH", "PP", "PP", "PR", "PS", "PD", "PB", "PIB", "NULL", "LE", "PC", "PCTV", "PFL", "PSP", "BOL", "PBA", "PBH", "PBIN", "PCO", "PCU", "PDE", "PEB", "PEP", "PER", "PES", "PFH", "PGR", "PGS", "PGL", "PGU", "PGV", "PHT", "PIC", "PINV", "PIR", "PIS", "PIT", "PL", "PLP", "PMA", "PMR", "PMS", "PMT", "PNT", "POB", "POS", "POST", "POT", "PPY", "PRE", "PRG", "PSC", "PSE", "PSL", "PSN", "PSS", "PSV", "PT", "PT1", "PT2", "PT3", "PTB", "PTM", "PTNL", "PTP", "PTR", "PTS", "PWM", "PWR", "PWS", "PRS", "PMK" };
                bool b = listOfStrings.Any((featureCodeString.ToUpper() + "        ").Substring(0, 4).Contains);

                if (b == false)
                    b = featureCodeString.Equals("L");

                isBreakline = !b;
                return isBreakline;

            }

            public class transformationData
            {
                public List<eastNorthTransform> pointList = new List<eastNorthTransform>();
                public double shiftX { get; set; }
                public double shiftY { get; set; }
                public double rotation { get; set; }
                public double scale { get; set; }

                public transformationData()
                {
                    pointList = new List<eastNorthTransform>();

                }

                public transformationData(List<eastNorthTransform> n_pointList, double n_shiftX, double n_shiftY, double n_rotation, double n_scale)
                {
                    pointList = new List<eastNorthTransform>(n_pointList);
                    shiftX = n_shiftX;
                    shiftY = n_shiftY;
                    rotation = n_rotation;
                    scale = n_scale;
                }


            }

            public class tunnelShape
            {
                //;LEN START    LEN ORIGIN     X              Y              BEARING        RADIUS            RL
                public float lenStart { get; set; }
                public float lenOrigin { get; set; }
                public float X { get; set; }
                public float Y { get; set; }
                public float Bearing { get; set; }
                public float Radius { get; set; }
                public float RL { get; set; }

            }

            public class nstHorizontalAlignment
            {
                public double CHSTART { get; set; }
                public double CHORIGIN { get; set; }
                public double EASTING { get; set; }
                public double NORTHING { get; set; }
                public double BEARING { get; set; }
                public double RADIUS { get; set; }
                public double RL { get; set; }

            }

            public class nstVerticalAlignment
            {
                public double CHSTART { get; set; }
                public double LEVEL { get; set; }
                public double RADIUS { get; set; }
            }

            public class nstCant
            {
                public double CHSTART { get; set; }
                public List<double> CANT { get; set; }
                public nstCant()
                {
                    CANT = new List<double>();
                }

            }


            public class crossSectionBounds
            {
                public double minOffset { get; set; }
                public double maxOffset { get; set; }
                public double minLevel { get; set; }
                public double maxLevel { get; set; }
            }



            public class chaOffLvl
            {
                public double chainage { get; set; }
                public double offset { get; set; }
                public double level { get; set; }
            }

            public class chainageObject
            {
                public List<int> pointIndexList { get; set; }
                public double chainage { get; set; }
                public double sweep { get; set; }
                public double offset { get; set; }
                public long pointCount
                {
                    get
                    {

                        return pointIndexList.Count;

                    }
                }


                public chainageObject()
                {

                    pointIndexList = new List<int>();
                }
            }

            public class crossSectionPoint
            {
                public nrgVector3 coord { get; set; }
                public double chainage { get; set; }
                public double offset { get; set; }
                public double level { get; set; }
                public double brg { get; set; }
                public double distance { get; set; }
                public nrgColor4 colour { get; set; }
                public crossSectionPoint()
                {
                    coord = new nrgVector3();
                    colour = new nrgColor4();
                }
            }

            public class gpfV2Point
            {
                public string pointNumber { get; set; }
                public string fieldCode { get; set; }
                public double X { get; set; }
                public double Y { get; set; }
                public double Z { get; set; }
                public string notes { get; set; }
            }

            public class eastNorthTransform
            {
                public double x1 { get; set; }
                public double y1 { get; set; }
                public double x2 { get; set; }
                public double y2 { get; set; }
            }

            public class nrgVector3Bounds
            {
                public double maxX { get; set; }
                public double minX { get; set; }
                public double maxY { get; set; }
                public double minY { get; set; }
                public double maxZ { get; set; }
                public double minZ { get; set; }

                public nrgVector3Bounds(nrgVector3Bounds newBounds)
                {
                    maxX = newBounds.maxX;
                    maxY = newBounds.maxY;
                    maxZ = newBounds.maxZ;
                    minX = newBounds.minX;
                    minY = newBounds.minY;
                    minZ = newBounds.minZ;

                }
                public nrgVector3Bounds()
                {
                }

                public void fix()
                {
                    if (minX > maxX)
                    {
                        var _minX = maxX;
                        var _maxX = minX;
                        minX = _maxX;
                        maxX = _minX;
                    }


                    if (minY > maxY)
                    {
                        var _minY = maxY;
                        var _maxY = minY;
                        minY = _minY;
                        maxY = _maxY;
                    }


                }

                public void setInitialBounds(nrgVector3 point)
                {
                    maxX = point.X;
                    minX = point.X;
                    maxY = point.Y;
                    minY = point.Y;
                    maxZ = point.Z;
                    minZ = point.Z;


                }

                public void updateBounds(nrgVector3 point)
                {
                    if (point.X > maxX)
                        maxX = point.X;
                    if (point.X < minX)
                        minX = point.X;
                    if (point.Y > maxY)
                        maxY = point.Y;
                    if (point.Y < minY)
                        minY = point.Y;
                    if (point.Z > maxZ)
                        maxZ = point.Z;
                    if (point.Z < minZ)
                        minZ = point.Z;

                }

            }

            public class nrgPolyBounds
            {
                public List<nrgVector3> pointList { get; set; }
                public bool inBox(nrgVector3 point)
                {
                    bool isIn = false;
                    if (helpers.inPoly2(pointList, point))
                        isIn = true;
                    return isIn;
                }
            }

            public class nrgRect
            {
                public nrgVector3 topleft { get; set; }
                public nrgVector3 bottomright { get; set; }
            }

            public class rawObs : HVD
            {
                public double targetHeight { get; set; }
            }

            public class HVD
            {
                public double horizontalAngle { get; set; }
                public double verticalAngle { get; set; }
                public double slopeDistance { get; set; }
            }

            public class pointInTri
            {
                public int pointIndex { get; set; }
                public int triangleIndex { get; set; }

            }

            public class RGBColor
            {
                public int r { get; set; }
                public int g { get; set; }
                public int b { get; set; }

                public RGBColor(int ir, int ig, int ib)
                {

                    r = ir;
                    g = ig;
                    b = ib;

                }
            }


            public class RGBColorFloat
            {
                public float r { get; set; }
                public float g { get; set; }
                public float b { get; set; }
            }

            public class nrgModel
            {
                public string modelName { get; set; }
                public List<nrgTriangle> triangles { get; set; }
                public List<nrgVertex> vertices { get; set; }
            }

            public class nrgVolumeModel
            {
                public string modelName { get; set; }
                public List<nrgTriangle> triangles { get; set; }
                public List<nrgVolumeVertex> vertices { get; set; }
            }

            public class intARGB
            {
                public int A { get; set; }
                public int R { get; set; }
                public int G { get; set; }
                public int B { get; set; }
            }



            public class intVertex
            {
                public int X { get; set; }
                public int Y { get; set; }
                public int Z { get; set; }
            }

            public class byteColor
            {
                public byte R { get; set; }
                public byte G { get; set; }
                public byte B { get; set; }
            }

            public class chainageSweepOffset
            {
                public double Chainage { get; set; }
                public double Sweep { get; set; }
                public double Offset { get; set; }
            }


            public class stringFileList
            {

                public string stringName { get; set; }
                public string stringFile { get; set; }

            }


            public class nrgVector3 : nrgVector2
            {
                public double Z { get; set; }

                public nrgVector3() { }


                public nrgVector3(double x, double y, double z)
                {
                    X = x;
                    Y = y;
                    Z = z;
                }

                public nrgVector2 offset(nrgVector2 input)
                {

                    return new nrgVector2 { X = -Y + input.X, Y = X + input.Y };

                }



                public nrgVector2 xyPerpCW
                {
                    get
                    {
                        return new nrgVector2 { X = -Y, Y = X };
                    }
                }

                public nrgVector2 xyPerpCCW
                {
                    get
                    {
                        return new nrgVector2 { X = Y, Y = -X };
                    }
                }
            }


            public class nrgVector2
            {
                public double X { get; set; }
                public double Y { get; set; }
                public nrgVector3 toVector3()
                {
                    return new nrgVector3(X, Y, 0);
                }
            }

            public class angleDMS
            {

                public double degrees { get; set; }
                public double minutes { get; set; }
                public double seconds { get; set; }


            }


            public class nrgAlignmentVector5
            {
                public double chainage { get; set; }
                public double offset { get; set; }
                public double bearing { get; set; }
                public double radius { get; set; }
                public Int32 element { get; set; }
            }

            public class nrgVolumeVertex : nrgVector3
            {

                public double Z2 { get; set; }
            }

            public class nrgVertex
            {
                public double X { get; set; }
                public double Y { get; set; }
                public double Z { get; set; }
            }

            public class sourceFile
            {
                public String filename { get; set; }
            }

            //Main Layer Object, it is king

            public class nrgLayer
            {
                public string layerName { get; set; } //Free Text
                public bool fileLoaded { get; set; }
                public int layerType { get; set; } //0=empty 1=GPFV2 2=PointCLoud
                public string sourceFileName { get; set; }  //Filename + Path
                public nrgVector3Bounds nrgVector3Bounds { get; set; } //Max nrgVector3  This is max bounds for real coordinates
                public double coordOffsetX { get; set; } //Offset for coord to screen calc
                public double coordOffsetY { get; set; } //Offset for coord to screen calc
                public double coordOffsetZ { get; set; } //Offset for coord to screen calc
                public List<nrgPoint> nrgPointsList { get; set; }
                public nrgModel nrgModel { get; set; }  //Nrg model object that contains real world scale triangles.



                public List<alignments.alignment> alignments { get; set; }

                public long pointCloudCount;
                public long pointCount
                {
                    get
                    {
                        long outputCount = 0;
                        if (layerType == 1) outputCount = nrgPointsList.Count;
                        if (layerType == 3 || layerType == 2) outputCount = pointCloudCount;
                        return outputCount;
                    }
                }

                public void calculateChainage(int alignmentNumber)
                {
                    if (alignments != null && alignments.Count >= 0)
                    {
                        foreach (nrgPoint point in nrgPointsList)
                        {
                            nrgAlignmentVector5 alignmentVec = new nrgAlignmentVector5();
                            var gotAlignment = alignments[alignmentNumber].horizontal.getChainageAndOffset(
                                new nrgVector2 { X = point.coord.X, Y = point.coord.Y },
                                ref alignmentVec
                                );
                            if (gotAlignment)
                            {
                                point.chainage = alignmentVec.chainage;
                                point.offset = alignmentVec.offset;
                            }
                        }
                    }
                    else
                    {
                        Debug.WriteLine("No Alignments in Model");
                    }
                }

            }

            public class pointCloudPoint
            {
                public nrgVector3 coord { get; set; }
                public RGBColor CodedColour { get; set; }
                public int triangleIndex { get; set; }
                public bool exportPoint { get; set; }
            }

            public class features
            {
                public string Code { get; set; }
                public int Colour { get; set; }
                public RGBColorFloat CodedColour { get; set; }
                public int Pen { get; set; }
                public int Use { get; set; }
                public int LayerNumber { get; set; }
                public float lineThickness { get; set; }
                public float lineSmoothness { get; set; }
                public float pointThickness { get; set; }
            }

            public class lineFeatureListItem
            {
                public int pointNumber { get; set; }
                public string fieldCode { get; set; }
                public bool isStart { get; set; }
                public int lineCommand { get; set; }
            }

            public class codeSwitch
            {
                public string code { get; set; }
                public double value { get; set; }
            }

            public class fieldCode
            {
                public string majorCode { get; set; }
                public List<codeSwitch> codeSwitchList { get; set; }
                public bool isBreakLine { get; set; }

            }

            public class nrgPoint
            {

                public nrgPoint()
                {
                    coord = new nrgVector3();
                    CodedColour = new RGBColor(0, 0, 0);
                    fieldCodeList = new List<dataTypes.fieldCode>();
                    notesList = new List<string>();
                    pointColor = new nrgColor4();
                }

                public string pointNumber { get; set; }
                public nrgVector3 coord { get; set; }
                public RGBColor CodedColour { get; set; }
                public List<fieldCode> fieldCodeList { get; set; }
                public List<string> notesList { get; set; }
                public chaOffLvl hAlignmentData { get; set; }
                public string notes0
                {
                    get
                    {
                        string note = "";
                        if (notesList.Count > 0)
                            note = notesList[0];
                        return note;
                    }
                }
                public string notes1
                {
                    get
                    {
                        string note = "";
                        if (notesList.Count > 1)
                            note = notesList[1];
                        return note;
                    }
                }
                public string notes2
                {
                    get
                    {
                        string note = "";
                        if (notesList.Count > 2)
                            note = notesList[2];
                        return note;
                    }
                }
                public string notes3
                {
                    get
                    {
                        string note = "";
                        if (notesList.Count > 3)
                            note = notesList[3];
                        return note;
                    }
                }




                public nrgColor4 pointColor { get; set; }
                public string fieldCode
                {
                    set
                    {
                        fieldCodeList = helpers.parseFieldCodeStringToList(value);
                    }
                    get
                    {
                        return helpers.parseFieldCodeListToString(fieldCodeList);
                    }
                }
                public bool isLineStart
                {
                    get
                    {
                        bool isStart = false;
                        if (fieldCodeList.Count > 0)
                            if (fieldCodeList[0].codeSwitchList.Count > 0)
                                if (fieldCodeList[0].codeSwitchList[0].code.ToUpper().Contains("ST"))
                                    isStart = true;
                        return isStart;
                    }

                }
                public string majorCode
                {
                    get
                    {
                        string outText = "";
                        if (fieldCodeList.Count > 0)
                            outText = fieldCodeList[0].majorCode.ToString().ToUpper();
                        return outText;
                    }

                }
                //Additional Calculated Fields
                public float verticalDifferenceToBackGround { get; set; }
                public double chainage { get; set; }
                public double offset { get; set; }
            }

            public class nrgSimplePoint
            {

                public nrgSimplePoint()
                {
                    coord = new nrgVector3();
                    CodedColour = new RGBColor(0, 0, 0);
                }

                public nrgSimplePoint(nrgSimplePoint point)
                {
                    coord = new nrgVector3 { X = point.coord.X, Y = point.coord.Y, Z = point.coord.Z };
                    CodedColour = new RGBColor(point.CodedColour.r, point.CodedColour.g, point.CodedColour.b);
                }

                public string pointNumber { get; set; }
                public nrgVector3 coord { get; set; }
                public RGBColor CodedColour { get; set; }
                public int intColor
                {
                    get
                    {
                        return helpers.RGBtoInt(CodedColour.r, CodedColour.g, CodedColour.b);
                    }
                }
                public string label { get; set; }
                public string notes { get; set; }


                public bool isStart()
                {
                    bool start = false;
                    if (label.ToUpper().Contains(@"/ST"))
                        start = true;
                    return start;
                }

                public bool isClose()
                {
                    bool start = false;
                    if (label.ToUpper().Contains(@"/CL"))
                        start = true;
                    return start;
                }

            }

            public class nrgColor4
            {
                public float Alpha { get; set; }
                public float Red { get; set; }
                public float Green { get; set; }
                public float Blue { get; set; }
                /*
                        public System.Windows.Media.Color asMediaColor()
                        {
                            return new System.Windows.Media.Color { R = Convert.ToByte(Red * 255), G = Convert.ToByte(Green * 255), B = Convert.ToByte(Blue * 255), A = Convert.ToByte(Alpha * 255) };

                        }
                        */
            }


            public class volumeReport
            {
                public int surfaceID { get; set; }
                public string surfaceName { get; set; }
                public double fillVolume { get; set; }
                public double cutVolume { get; set; }
                public double surfaceArea { get; set; }
                public double planArea { get; set; }
            }

            //tofo
            public class shapePoint
            {
                public nrgVector2 coord { get; set; }
                public double radius { get; set; }

            }



            public class nrgTriangle
            {

                public int N0 { get; set; }
                public int N1 { get; set; }
                public int N2 { get; set; }


                public int P0 { get; set; }
                public int P1 { get; set; }
                public int P2 { get; set; }

                public int surfaceIndex { get; set; }
                public bool isContourable { get; set; }
                public double planArea { get; set; }
                public double surfaceArea { get; set; }

                public int GetVertex(int input)
                {
                    {
                        int output = 0;

                        if (input == 1)
                            output = N2;
                        if (input == 2)
                            output = N1;
                        if (input == 3)
                            output = N0;

                        return output;
                    }
                }

                public int GetAdjacent(int input)
                {
                    {
                        int output = 0;

                        if (input == 1)
                            output = P2;
                        if (input == 2)
                            output = P1;
                        if (input == 3)
                            output = P0;

                        return output;
                    }
                }

            }








            public class hiLowTriPoints
            {
                public int triangleIndex { get; set; }
                public int highPoint { get; set; }
                public int lowPoint { get; set; }
                public bool hasValue { get; set; }
            }

            public class pointCloud
            {

                public List<pointCloudPoint> points { get; set; }
                public nrgVector3Bounds doubleBounds { get; set; }
                public nrgVector3Bounds doubleDrawBounds { get; set; }
                public nrgVector3 singleOffset { get; set; }
                public List<nrgVector3> triMeshPoints { get; set; }
                public double levelOfDetail { get; set; }
                public nrgModel triModel { get; set; }

                public List<pointCloudPoint> triAveragePoint { get; set; }
                public List<hiLowTriPoints> triHighLowPoints { get; set; } //algo1

                public List<int> outputPointIndexList { get; set; }



                public pointCloud()
                {

                    points = new List<pointCloudPoint>();
                    doubleBounds = new nrgVector3Bounds();
                    doubleDrawBounds = new nrgVector3Bounds();
                    singleOffset = new nrgVector3();
                    levelOfDetail = new double();
                    triMeshPoints = new List<nrgVector3>();
                    triHighLowPoints = new List<hiLowTriPoints>();
                    outputPointIndexList = new List<int>();
                    triAveragePoint = new List<pointCloudPoint>();
                }



                public void outputHiLowTriPoints()
                {


                    foreach (hiLowTriPoints triPoints in triHighLowPoints)
                    {
                        outputPointIndexList.Add(triPoints.highPoint);
                        outputPointIndexList.Add(triPoints.lowPoint);
                    }
                }

                public void outputHiPoints()
                {


                    foreach (hiLowTriPoints triPoints in triHighLowPoints)
                    {
                        outputPointIndexList.Add(triPoints.highPoint);
                    }
                }

                public void outputLowPoints()
                {


                    foreach (hiLowTriPoints triPoints in triHighLowPoints)
                    {
                        outputPointIndexList.Add(triPoints.lowPoint);
                    }
                }

                public void outputRoughTriPoints()
                {
                    triAveragePoint.Clear();

                    foreach (hiLowTriPoints triPoints in triHighLowPoints)
                    {
                        outputPointIndexList.Add(triPoints.highPoint);
                        outputPointIndexList.Add(triPoints.lowPoint);

                        if (triPoints.highPoint != -1)
                        { 
                            dataTypes.nrgVector3 newPoint = new nrgVector3();
                            newPoint.X = (points[triPoints.highPoint].coord.X + points[triPoints.lowPoint].coord.X) / 2;
                            newPoint.Y = (points[triPoints.highPoint].coord.Y + points[triPoints.lowPoint].coord.Y) / 2;
                            newPoint.Z = (points[triPoints.highPoint].coord.Z + points[triPoints.lowPoint].coord.Z) / 2;

                            RGBColor newCol = new dataTypes.RGBColor(0, 0, 0);
                            newCol.r = (points[triPoints.highPoint].CodedColour.r + points[triPoints.lowPoint].CodedColour.r) / 2;
                            newCol.g = (points[triPoints.highPoint].CodedColour.g + points[triPoints.lowPoint].CodedColour.g) / 2;
                            newCol.b = (points[triPoints.highPoint].CodedColour.b + points[triPoints.lowPoint].CodedColour.b) / 2;

                            triAveragePoint.Add(new pointCloudPoint { coord = newPoint, CodedColour = newCol });
                        }
                    }
                }



            }

            public class Coord
            {
                public double x { get; set; }
                public double y { get; set; }
                public double z { get; set; }
            }

            public class Line
            {
                public string desc { get; set; }
                public LineStyle style { get; set; }
                public List<Coord> coords { get; set; }
                public double brg { get; set; }
            }

            public class LineStyle
            {
                public string id { get; set; }
                public Xamarin.Forms.Color col { get; set; }
                public float width { get; set; }
            }

            public class Scale
            {
                public double scaleX { get; set; }
                public double scaleY { get; set; }
                public double transX { get; set; }
                public double transY { get; set; }
                public double pivotX { get; set; }
                public double pivotY { get; set; }
            }

            public class minMaxH
            {
                public double minH { get; set; }
                public double maxH { get; set; }
            }

            public class minMaxW
            {
                public double minW { get; set; }
                public double maxW { get; set; }
            }


            public class CrossSection
            {
                public float ppw { get; set; }
                public float w { get; set; }
                public float h { get; set; }
                public float minH { get; set; }
                public float maxH { get; set; }
                public float minW { get; set; }
                public float maxW { get; set; }
                public float wd { get; set; }
                public float hd { get; set; }
            }

        }





    }
}

