﻿using Plugin.Geolocator;
using Plugin.Geolocator.Abstractions;
using Plugin.Settings;
using SkiaSharp;
using SkiaSharp.Views.Forms;
using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Linq;
using Xamarin.Forms;
using Xamarin.Forms.GoogleMaps;
using Xamarin.Forms.Xaml;
using Xamarin.Forms.PlatformConfiguration.iOSSpecific;
using Plugin.FilePicker;
using static AlignmentViewer.workspace;

namespace AlignmentViewer
{
    [XamlCompilation (XamlCompilationOptions.Compile)]
    public partial class MainPage : ContentPage
    {
        public int currentTagAmount;
        public static dataTypes.Scale scale = new dataTypes.Scale();
        public static dataTypes.minMaxH mmH = new dataTypes.minMaxH();
        public static dataTypes.minMaxW mmW = new dataTypes.minMaxW();
        dataTypes.CrossSection crossSection = new dataTypes.CrossSection();
        public static bool toUpdate = true;
        double lat;
        double lng;
        double x, y, lX, lY, startLX, startLY, LCurrentScale, LStartScale;
        int currentPinId = 0;
        int curLineID;
        double curWidth=0, curHeight=0;
        bool canDraw = false;
        double brgCount = 0;
        bool isData = false;
        double lastCh = 0;
        bool firstload = false;
        DotSpatial.Projections.ProjectionInfo projSource = DotSpatial.Projections.ProjectionInfo.FromProj4String("+proj=longlat +datum=WGS84 +no_defs");
        DotSpatial.Projections.ProjectionInfo projTarget;
        bool elevationFlag = false;
        int lastline = 0;
        Map mapObject = new Map();
        public double positions = 0;
        List<string> linesToRemove = new List<string>();
        List<string> crossSectionToRemove = new List<string>();
        List<string> mapLineList = new List<string>();
        List<string> crossSectionLineList = new List<string>();
        public double curOffset = 0;
        private bool NewTransformation; //Set this in load project settings
        private double startScale, currentScale, xOffset, yOffset, startX, startY;
        List<Grid> listLineGrid = new List<Grid>();
        bool xrepaint = false;
        bool yrepaint = false;
        string pinSaveAs = "gpf";
        double offsetXToCenterTo = 0, offsetYToCenterTo = 0;
        Dictionary<string, List<dataTypes.nrgVector2>> dictionaryIntersectPoints = new Dictionary<string, List<dataTypes.nrgVector2>>();
        Dictionary<string, List<dataTypes.Line>> dictionaryLines = new Dictionary<string, List<dataTypes.Line>>();
        Dictionary<string, KeyValuePair<int, int>> dictionaryLoadedLines = new Dictionary<string, KeyValuePair<int, int>>();

        workspace alignmentWorkspace;
        List<MapType> mapTypeValues = new List<MapType>();

        bool firstRun = true;

        int selectedMapType = 0;

        public MainPage()
        {
            InitializeComponent();
        }

        public MainPage(ref workspace currentWorkspace)
        {
            InitializeComponent();
            scale.scaleX = 1;
            scale.scaleY = 1;
            scale.transX = 0;
            scale.transY = 0;
          
            mapObject.BackgroundColor = Color.FromHex("#FAFAFA");
            mapObject.VerticalOptions = LayoutOptions.FillAndExpand;

            if(Device.RuntimePlatform == Device.iOS)
            {
                On<Xamarin.Forms.PlatformConfiguration.iOS>().SetUseSafeArea(true);
                //fullGrid.Padding = new Thickness(0, 20, 0, 0);
                gridMapPanel.Children.Clear();
                gridMapPanel.Children.Add(gridCrossection, 0, 1);
                gridMapPanel.Children.Add(mapObject, 0, 0);
                gridMapPanel.Children.Add(mapBoxView, 0, 1);
                gridMapPanel.Children.Add(crossHairs, 0, 0);
                gridMapPanel.Children.Add(buttonEnabledCrossSection, 0, 0);
                gridMapPanel.Children.Add(buttonAutoScroll, 0, 0);
            }
            alignmentWorkspace = currentWorkspace;

            Xamarin.Forms.NavigationPage.SetHasNavigationBar(this, false);

            foreach (var mapType in Enum.GetValues(typeof(MapType)))
            {
                mapTypeValues.Add((MapType)mapType);
            }

            MessagingCenter.Subscribe<List<string>>(this, "PhotoTaken", (args) =>
            {
                Device.BeginInvokeOnMainThread(() =>
                {
                    buttonCHPhoto.IsEnabled = true;
                    buttonLandscapePhoto.IsEnabled = true;

                    if (Device.RuntimePlatform == Device.Android)
                    {
                        Acr.UserDialogs.UserDialogs.Instance.Toast("Saved " + App.photoName + ".", TimeSpan.FromSeconds(1));
                    }
                    else if (Device.RuntimePlatform == Device.iOS)
                    {
                        Acr.UserDialogs.UserDialogs.Instance.Toast("Saved Photo", TimeSpan.FromSeconds(1));
                    }
                    else
                    {
                        Acr.UserDialogs.UserDialogs.Instance.Toast("Device Not Recognised", TimeSpan.FromSeconds(1));
                    }
                });
            });

            MessagingCenter.Subscribe<string>(this, "PhotoFailed", (args) =>
            {
                Device.BeginInvokeOnMainThread( async () =>
                {
                    if(args == "Cancel")
                    {
                        Acr.UserDialogs.UserDialogs.Instance.Toast("Action was cancelled by the user", TimeSpan.FromSeconds(1));
                    }
                    else if(args == "Permissions")
                    {
                        DisplayAlert("Notice", "Permission to access the photo libray is required in order to save a photo", "Ok");
                    }
                    else
                    {
                        if(Device.RuntimePlatform == Device.Android)
                        {
                            var result = await DisplayActionSheet(args + "\n" + App.StackTrace, "OK", "Save Log");

                            if (result == "Save Log")
                            {
                                try
                                {
                                    //Save the error in a text file in nrg directory and set toats to saving it
                                    var dateString = DateTime.Now.ToString().Replace("/", "_") + ".log";
                                    var errorLogPath = alignmentWorkspace.datapath + dateString;

                                    if (nrgFileRead.reader.fileExists(errorLogPath))
                                        nrgFileRead.reader.deleteFile(errorLogPath);

                                    nrgFileRead.reader.stringToStream(errorLogPath, args + "\n\n" + App.StackTrace);

                                    App.ScanFiles.scanFile(errorLogPath);

                                    Acr.UserDialogs.UserDialogs.Instance.Toast("Saved log to: " + "NRG/" + dateString);
                                }
                                catch (Exception e)
                                {
                                    DisplayAlert("Error", e.Message, "OK");
                                    DisplayAlert("Error", e.StackTrace, "OK");
                                }
                            }
                        }
                        else if(Device.RuntimePlatform == Device.iOS)
                        {

                        }
                        else
                        {
                            await DisplayAlert("Error", "Unrecognised device" + Device.RuntimePlatform, "OK");
                        }
                    }
                    
                    buttonCHPhoto.IsEnabled = true;
                    buttonLandscapePhoto.IsEnabled = true;
                });
            });

            DotSpatial.Projections.GridShift.ThrowGridShiftMissingExceptions = false;
            //mapObject.IsShowingUser = true;
            mapObject.MyLocationEnabled = true;
            mapObject.UiSettings.MyLocationButtonEnabled = true;

            alignmentWorkspace.widthScale = 100;
            alignmentWorkspace.transformation = alignmentWorkspace.Alignments.createOSTrans();

            mapObject.CameraMoving += (sender, e) =>
            {
                if (gridChainagePanel.IsVisible || gridLandscapeChainage.IsVisible)
                    UpdateChOffDisplay(e.Position.Target.Latitude, e.Position.Target.Longitude);
                if (gridENPanel.IsVisible || gridLandscapeEN.IsVisible)
                    UpdateENDisplay(e.Position.Target.Latitude, e.Position.Target.Longitude);
                if (gridLatLongPanel.IsVisible || gridLandscapeLL.IsVisible)
                    UpdateLLDisplay(e.Position.Target.Latitude, e.Position.Target.Longitude);
            };
            
            mapObject.CameraChanged += (sender, e) =>
            {

                CrossSettings.Current.AddOrUpdateValue("curLat", e.Position.Target.Latitude);
                CrossSettings.Current.AddOrUpdateValue("curLng", e.Position.Target.Longitude);
                CrossSettings.Current.AddOrUpdateValue("curRadius", mapObject.VisibleRegion.Radius.Meters);
                getRadius();

                alignmentWorkspace.devicePosition = new Xamarin.Forms.GoogleMaps.Position(e.Position.Target.Latitude, e.Position.Target.Longitude);
                mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPositionZoom(alignmentWorkspace.devicePosition, e.Position.Zoom);

                if (alignmentWorkspace.crossSection == true && gridCrossection.IsVisible == true)
                    createIntersection();
            };

            mapObject.MapLongClicked += (sender, e) =>
            {
                selectedMapType++;
                if (selectedMapType > 2)
                    selectedMapType = 0;



                alignmentWorkspace.mapType = selectedMapType;

                switch (alignmentWorkspace.mapType)
                {
                    case 0:
                        labelMapType.Text = "Street Map";
                        break;
                    case 1:
                        labelMapType.Text = "Satellite Map";
                        break;
                    case 2:
                        labelMapType.Text = "Hybrid Map";
                        break;
                }

                mapObject.MapType = mapTypeValues[selectedMapType];
                if (mapTypeValues[selectedMapType] == 0)
                {
                    if (alignmentWorkspace.scrollMap == true)
                        buttonAutoScroll.Image = "ScrollBlack.png";
                    else
                        buttonAutoScroll.Image = "NoScrollBlack.png";
                    labelScale.TextColor = Color.Black;
                    frameScale.BackgroundColor = Color.Black;
                    frameScaleLeft.BackgroundColor = Color.Black;
                    frameScaleRight.BackgroundColor = Color.Black;
                    frameScaleCenter.BackgroundColor = Color.Black;
                }
                else
                {
                    if (alignmentWorkspace.scrollMap == true)
                        buttonAutoScroll.Image = "ScrollBlack.png";
                    else
                        buttonAutoScroll.Image = "NoScrollBlack.png";
                    labelScale.TextColor = Color.White;
                    frameScale.BackgroundColor = Color.White;
                    frameScaleLeft.BackgroundColor = Color.White;
                    frameScaleRight.BackgroundColor = Color.White;
                    frameScaleCenter.BackgroundColor = Color.White;
                }
            };           

            mapObject.MyLocationButtonClicked += async(sender, e) =>
            {
                if(alignmentWorkspace.requestOnly == true)
                {
                    try
                    {
                        if (!CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                        {
                            await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(1), 0.1, true);
                            var pos = await CrossGeolocator.Current.GetPositionAsync(TimeSpan.FromSeconds(1));
                            if (pos != null)
                                mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(new Xamarin.Forms.GoogleMaps.Position(pos.Latitude, pos.Longitude), mapObject.VisibleRegion.Radius));
                        }
                        else if (CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                        {
                            var pos = await CrossGeolocator.Current.GetPositionAsync(TimeSpan.FromSeconds(1));
                            if (pos != null)
                                mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(new Xamarin.Forms.GoogleMaps.Position(pos.Latitude, pos.Longitude), mapObject.VisibleRegion.Radius));
                        }

                        await CrossGeolocator.Current.StopListeningAsync();
                    }
                    catch
                    {
                        await CrossGeolocator.Current.StopListeningAsync();
                    }
                }
                else
                {
                    try
                    {
                        if (!CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                        {
                            await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true, new ListenerSettings() { AllowBackgroundUpdates = true });
                            var pos = await CrossGeolocator.Current.GetPositionAsync(TimeSpan.FromSeconds(1));
                            if(pos != null)
                                mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(new Xamarin.Forms.GoogleMaps.Position(pos.Latitude, pos.Longitude), mapObject.VisibleRegion.Radius));
                        }
                        else if(CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                        {
                            var pos = await CrossGeolocator.Current.GetPositionAsync(TimeSpan.FromSeconds(1));
                            if (pos != null)
                                mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(new Xamarin.Forms.GoogleMaps.Position(pos.Latitude, pos.Longitude), mapObject.VisibleRegion.Radius));
                        }
                    }
                    catch
                    {

                    }
                }
            };

            buttonSaveGpf.Clicked += (sender, args) =>
            {
                if (alignmentWorkspace.currentFolder == null || alignmentWorkspace.currentFolder == "None Selected" || alignmentWorkspace.currentFolder == "None Found")
                {
                    DisplayAlert("Error", "Please select a project folder before saving pins", "Ok");
                    return;
                }
                else if (currentPinId == 0)
                {
                    DisplayAlert("Notice", "There are no pins to save", "Ok");
                    return;
                }

                gridPinButtonPanel.IsVisible = false;
                buttonPinDropDown.Image = "DropDownSmall.png";
                pickerPinSaveOptions.Focus();
            };

            buttonDeletePin.Clicked += (sender, args) =>
            {
                if (mapObject.SelectedPin == null)
                {
                    DisplayAlert("Error", "There are no pins selected", "Ok");
                }
                else
                {
                    mapObject.Pins.Remove(mapObject.SelectedPin);
                    currentPinId -= 1;
                }
            };

            buttonClearPins.Clicked += (sender, args) =>
            {
                mapObject.Pins.Clear();
                currentPinId = 0;
                if (labelCurPins.Text != "None Found")
                    labelCurPins.Text = "None Selected";
                alignmentWorkspace.currentPins = "None Found";
                CrossSettings.Current.AddOrUpdateValue("currentPins", "None Found");
            };

            buttonClearLines.Clicked += (sender, args) =>
            {
                if(lastline <= mapObject.Polylines.Count)
                {
                    for (int i = 0; i < mapObject.Polylines.Count;)
                    {
                        if (mapObject.Polylines[i].Tag == null)
                            mapObject.Polylines.Remove(mapObject.Polylines[i]);
                        else
                            i++;
                    }
                    alignmentWorkspace.currentLines = "None Found";
                    CrossSettings.Current.AddOrUpdateValue("currentLines", "None Found");
                    pickerLines.SelectedIndex = -1;
                    labelCurLines.Text = "None Selected";


                    foreach (string item in mapLineList)
                    {
                        if (crossSectionLineList.Contains(item) == false)
                            dictionaryLines.Remove(item);
                        dictionaryLoadedLines.Remove(item);
                    }

                    mapLineList.Clear();
                    CrossSettings.Current.AddOrUpdateValue("mapLineString", "none");

                    for (int i = 0; i < listLineGrid.Count(); i++)
                    {
                        var grid = listLineGrid[i];
                        var button = (Button)grid.Children[1];
                        button.Image = "EmptyBox.png";
                    }

                    lastline = mapObject.Polylines.Count();
                }
                else
                {
                    DisplayAlert("Notice", "Please wait for the current lines to finish loading", "Ok");
                }
            };

            buttonFolderHelp.Clicked += async (sender, args) =>
            {
                switch(Device.RuntimePlatform)
                {
                    case Device.Android:
                        var action = await DisplayActionSheet("Project folders should be copied to internalstorage/NRG/", "More Info", "Ok");
                        if(action == "More Info")
                            Device.OpenUri(new Uri("http://www.nrgsurveys.co.uk/downloads/alignmentviewer.pdf#page=7"));
                        break;
                    case Device.iOS:
                        var action2 = await DisplayActionSheet("Project folders should be added to Alignment Viewer documents through iTunes", "More Info", "Ok");
                        if (action2 == "More Info")
                            Device.OpenUri(new Uri("http://www.nrgsurveys.co.uk/downloads/alignmentviewer.pdf#page=7"));
                        break;
                }
            };

            buttonTransformationHelp.Clicked += (sender, args) =>
            {
                switch(Device.RuntimePlatform)
                {
                    case Device.Android:
                        DisplayAlert("Notice", "Transformation .dat files should be copied to internalstorage/NRG/Transformation/", "Ok");
                        break;
                    case Device.iOS:
                        DisplayAlert("Notice", "Transformation .dat files should be placed in the Alignment Viewer documents folder through iTunes", "Ok");
                        break;
                }
            };

            buttonImportHelp.Clicked += (sender, args) =>
            { 
                if (alignmentWorkspace.currentFolder == "None Selected" || alignmentWorkspace.currentFolder == null || alignmentWorkspace.currentFolder == "None Found")
                {
                    switch(Device.RuntimePlatform)
                    {
                        case Device.Android:
                            DisplayAlert("Notice", "LandXML files should be stored in the current project folder", "Ok");
                            break;
                        case Device.iOS:
                            DisplayAlert("Notice", "LandXML files should be stored in the current project folder", "Ok");
                            break;
                    }
                }
                else
                {
                    switch(Device.RuntimePlatform)
                    {
                        case Device.Android:
                            DisplayAlert("Notice", "LandXML files should be stored in the current project folder at internalstorage/NRG/" + alignmentWorkspace.currentFolder + "/", "Ok");
                            break;
                        case Device.iOS:
                            DisplayAlert("Notice", "LandXML files should be stored in the current project folder " + alignmentWorkspace.currentFolder, "Ok");
                            break;
                    }
                }
            };

            buttonAlignmentHelp.Clicked += (sender, args) =>
            {
                switch(Device.RuntimePlatform)
                {
                    case Device.Android:
                        DisplayAlert("Notice", "Alignment files should be stored in the current project folder at internalstorage/NRG/" + alignmentWorkspace.currentFolder + "/", "Ok");
                        break;
                    case Device.iOS:
                        DisplayAlert("Notice", "Alignment files should be stored in the current project folder " + alignmentWorkspace.currentFolder, "Ok");
                        break;
                }
            };

            buttonPinsHelp.Clicked += (sender, args) =>
            {
                switch(Device.RuntimePlatform)
                {
                    case Device.Android:
                        DisplayAlert("Notice", "Pin files should be stored in the current project folder at internalstorage/NRG/" + alignmentWorkspace.currentFolder + "/", "Ok");
                        break;
                    case Device.iOS:
                        DisplayAlert("Notice", "Pin files should be stored in the current project folder " + alignmentWorkspace.currentFolder, "Ok");
                        break;
                }
            };

            buttonLinesHelp.Clicked += (sender, args) =>
            {
                switch(Device.RuntimePlatform)
                {
                    case Device.Android:
                        DisplayAlert("Notice", "Line files should be stored in the current project folder at internalstorage/NRG/" + alignmentWorkspace.currentFolder + "/", "Ok");
                        break;
                    case Device.iOS:
                        DisplayAlert("Notice", "Line files should be stored in the current project folder " + alignmentWorkspace.currentFolder, "Ok");
                        break;
                }
            };

            pickerFolder.SelectedIndexChanged += async (sender, args) =>
            {
                if (pickerFolder.SelectedIndex != -1)
                {
                    try
                    {
                        if(pickerFolder.SelectedItem.ToString() == "New Project")
                        {
                            //Ask the user to enter a name for the new project.
                            var result = await Acr.UserDialogs.UserDialogs.Instance.PromptAsync("Enter a name for the new project", "New Project", "OK", "Cancel");
                            if(!result.Ok)
                            {
                                //Return without doing anything as the user has cancelled
                                return;
                            }

                            if(result.Value == "New Project")
                            {
                                await DisplayAlert("Warning", "A project cannot be called New Project", "OK");
                                return;
                            }

                            var folderPath = alignmentWorkspace.datapath + result.Value + "/";

                            //Check the name has not been used before
                            if (nrgFileRead.reader.FolderExits(folderPath))
                            {
                                await DisplayAlert("Warning", "A project with this name already exists", "OK");
                                return;
                            }

                            //Create the project and set the currentFolder to it
                            nrgFileRead.reader.CreateFolder(folderPath);
                            alignmentWorkspace.currentFolder = result.Value;
                            CrossSettings.Current.AddOrUpdateValue("currentFolder", alignmentWorkspace.currentFolder);
                            var listFiles = nrgFileRead.reader.listFiles(folderPath);
                            mapLineList.Clear();
                            crossSectionLineList.Clear();
                            labelCurProject.Text = result.Value;

                            //There will always be 0 files so we can avoid displaying the error;
                            dictionaryIntersectPoints.Clear();
                            dictionaryLines.Clear();
                            dictionaryLoadedLines.Clear();
                            setupAlignment(listFiles, false);
                            setupPins(listFiles, false);
                            setupLines(listFiles, false);
                            if (labelCurTransformation.Text != "Default (OSGB36)")
                                setupTransformation(true);
                            else
                                setupTransformation(false);
                            setupLinesList(listFiles);

                            if (nrgFileRead.reader.fileExists(folderPath + "TransformationSettings.DAT"))
                                ReadTransformation(folderPath + "TransformationSettings.DAT");

                            return;
                        }
                        else if (pickerFolder.SelectedItem.ToString() != alignmentWorkspace.currentFolder)
                        { 
                            alignmentWorkspace.currentFolder = pickerFolder.SelectedItem.ToString();
                            CrossSettings.Current.AddOrUpdateValue("currentFolder", alignmentWorkspace.currentFolder);
                            List<string> listFiles = new List<string>();
                            listFiles = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + pickerFolder.SelectedItem + "/");
                            mapLineList.Clear();
                            crossSectionLineList.Clear();
                            labelCurProject.Text = pickerFolder.SelectedItem.ToString();
                            if (listFiles.Count() == 0)
                                await DisplayAlert("Notice", "The selected project does not contain any files.", "Ok");
                            dictionaryIntersectPoints.Clear();
                            dictionaryLines.Clear();
                            dictionaryLoadedLines.Clear();
                            mapLineList.Clear();
                            crossSectionLineList.Clear();
                            setupAlignment(listFiles, false);
                            setupPins(listFiles, false);
                            setupLines(listFiles, false);
                            if (labelCurTransformation.Text != "Default (OSGB36)")
                                setupTransformation(true);
                            else
                                setupTransformation(false);
                            setupLinesList(listFiles);

                            if (nrgFileRead.reader.fileExists(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + "TransformationSettings.DAT"))
                                ReadTransformation(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + "TransformationSettings.DAT");

                            return;
                        }
                    }
                    catch(Exception e)
                    {
                        DisplayAlert("Error", e.Message, "Ok");
                        labelCurProject.Text = "None Selected";
                        return;
                    }
                }
            };

            pickerNst.SelectedIndexChanged += (sender, args) =>
            {
                if (pickerNst.SelectedIndex != -1)
                {
                    try
                    {
                        alignmentWorkspace.Alignments = new alignments.alignment();
                        alignmentWorkspace.currentAlignment = pickerNst.SelectedItem.ToString();
                        CrossSettings.Current.AddOrUpdateValue("currentAlignment", alignmentWorkspace.currentAlignment);
                        if (pickerNst.SelectedItem.ToString().ToUpper().EndsWith(".NST"))
                            alignmentWorkspace.Alignments.loadNST(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + pickerNst.Items[pickerNst.SelectedIndex]);
                        else if (pickerNst.SelectedItem.ToString().ToUpper().EndsWith(".NAF"))
                            alignmentWorkspace.Alignments.loadFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + pickerNst.Items[pickerNst.SelectedIndex]);
                        alignmentWorkspace.workspaceBounds = alignmentWorkspace.Alignments.CalcHzExtents();
                        double maxChain = alignmentWorkspace.Alignments.horizontal.elements.OrderBy(x => x.Chainage).Last().Chainage;
                        double minChain = alignmentWorkspace.Alignments.horizontal.elements.OrderBy(x => x.Chainage).First().Chainage;

                        int chainInt = (int)((maxChain - minChain) / 500);

                        if (chainInt < 1)
                            chainInt = 1;

                        alignmentWorkspace.Alignments.calcDrawCoords(alignmentWorkspace.Alignments.coords, chainInt);
                        Polyline polyline2 = null;
                        polyline2 = new Polyline();

                        foreach (workspace.dataTypes.nrgVector3 point in alignmentWorkspace.Alignments.coords)
                        {
                            var latLng = GetLLFromEN(point.X, point.Y);
                            polyline2.Positions.Add(new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y));
                        }

                        for (int i = 0; i < mapObject.Polylines.Count; i++)
                        {
                            if (mapObject.Polylines[i].Tag != null)
                            {
                                mapObject.Polylines.Remove(mapObject.Polylines[i]);
                            }
                        }

                        polyline2.IsClickable = true;
                        polyline2.StrokeColor = Color.Blue;
                        polyline2.StrokeWidth = 5f;
                        polyline2.Tag = "Alignment";
                        mapObject.Polylines.Add(polyline2);
                        labelCurAlignment.Text = pickerNst.Items[pickerNst.SelectedIndex];
                        lastline = mapObject.Polylines.Count();
                        updateChainageAndOffset();
                    }
                    catch
                    {
                        DisplayAlert("Error", "Failed to load the selected file", "Ok");
                    }
                   
                }
            };

            pickerTransform.SelectedIndexChanged += (sender, args) =>
            {
                if (pickerTransform.SelectedIndex != -1)
                {
                    try
                    {
                        alignmentWorkspace.currentTransformation = pickerTransform.SelectedItem.ToString();
                        CrossSettings.Current.AddOrUpdateValue("currentTransformation", alignmentWorkspace.currentTransformation);
                        if (pickerTransform.SelectedItem.ToString() == "Default (OSGB36)")
                        {
                            alignmentWorkspace.transformation = alignmentWorkspace.Alignments.createOSTrans();
                            UpdateSiteShiftScale(0, 0, 1.0);
                            labelCurTransformation.Text = "Default (OSGB36)";
                            NewTransformation = false;
                            updateToNewTransformation();
                            if (alignmentWorkspace.currentFolder != null)
                                if (alignmentWorkspace.currentFolder != "None Found" && alignmentWorkspace.currentFolder != "None Selected")
                                    SaveTransformationFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/TransformationSettings.DAT");

                            App.DefaultUnits = Unit.None;
                            App.SavedUnits = Unit.None;
                            UpdateTransformUnits();
                        }
                        else if (pickerTransform.SelectedItem.ToString() == "More..." || pickerTransform.SelectedItem.ToString() == CrossSettings.Current.GetValueOrDefault("transformationString", "Nothing"))
                        {
                            MessagingCenter.Subscribe<string>(this, "SwapTransformation", (arguments) =>
                            {
                                Device.BeginInvokeOnMainThread(() =>
                                {
                                    MessagingCenter.Unsubscribe<string>(this, "SwapTransformation");
                                    //MessagingCenter.Unsubscribe<string>(this, "SwapTransformation");
                                    CrossSettings.Current.AddOrUpdateValue("transformationString", App.transformationString);
                                    CrossSettings.Current.AddOrUpdateValue("transformationBody", App.transformationBody);
                                    LoadNewTransformation();
                                    labelCurTransformation.Text = App.transformationString;
                                    NewTransformation = true;
                                    updateToNewTransformation();
                                    if (alignmentWorkspace.currentFolder != null)
                                        if (alignmentWorkspace.currentFolder != "None Found" && alignmentWorkspace.currentFolder != "None Selected")
                                            SaveTransformationFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/TransformationSettings.DAT");

                                    UpdateTransformUnits();
                                });
                            });
                            Navigation.PushAsync(new TransformationSearch());
                           
                        }
                        else
                        {
                            if(Device.RuntimePlatform == Device.Android)
                                alignmentWorkspace.transformation = alignmentWorkspace.Alignments.ReadTransformation(alignmentWorkspace.datapath + "Transformation" + "/" + pickerTransform.Items[pickerTransform.SelectedIndex]);
                            if (Device.RuntimePlatform == Device.iOS)
                                alignmentWorkspace.transformation = alignmentWorkspace.Alignments.ReadTransformation(alignmentWorkspace.datapath + pickerTransform.SelectedItem.ToString());

                            UpdateSiteShiftScale(alignmentWorkspace.transformation.siteShiftE, alignmentWorkspace.transformation.siteShiftN, alignmentWorkspace.transformation.siteScale);
                            NewTransformation = false;
                            updateToNewTransformation();
                            labelCurTransformation.Text = pickerTransform.Items[pickerTransform.SelectedIndex];
                            if (alignmentWorkspace.currentFolder != null)
                                if (alignmentWorkspace.currentFolder != "None Found" && alignmentWorkspace.currentFolder != "None Selected")
                                    SaveTransformationFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/TransformationSettings.DAT");


                        }
                    }
                    catch(Exception error)
                    {
                        DisplayAlert("Error", "Failed to load the selected file", "Ok");
                    }
                }
            };

            pickerPins.SelectedIndexChanged += async (sender, args) =>
            {
                if (pickerPins.SelectedIndex != -1)
                {
                    if(nrgFileRead.reader.fileExists(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + pickerPins.SelectedItem.ToString()))
                    {
                        if(pickerPins.SelectedItem.ToString().ToUpper().EndsWith(".GPF"))
                        {
                            try
                            {
                                List<string> gpfString = alignmentWorkspace.Alignments.loadGPF(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + pickerPins.SelectedItem.ToString());
                                alignmentWorkspace.currentPins = pickerPins.SelectedItem.ToString();
                                CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                mapObject.Pins.Clear();
                                currentPinId = 0;

                                currentTagAmount = gpfString.First().Split(',').Count() - 5;

                                for (int i = 0; i < gpfString.Count; i++)
                                {
                                    string[] tmpValues = gpfString[i].Split(',');
                                    if (tmpValues.Count() >= 5)
                                    {
                                        string tag = "";
                                        for (int j = 4; j < tmpValues.Count(); j++)
                                        {
                                            tag += "," + tmpValues[j];
                                        }
                                        addPin(tmpValues[0], tmpValues[1], tmpValues[2], tmpValues[3], tag);
                                        currentTagAmount = (tmpValues.Count() - 5);
                                    }
                                    else if (tmpValues.Count() == 4)
                                    {
                                        addPin(tmpValues[0], "", tmpValues[1], tmpValues[2], ("," + tmpValues[3]));

                                    }
                                    pickerPins.SelectedIndex = -1;
                                }
                                labelCurPins.Text = pickerPins.SelectedItem.ToString();
                            }
                            catch
                            {
                                DisplayAlert("Error", "Failed to load the selected file", "Ok");
                            }
                        }
                        else if(pickerPins.SelectedItem.ToString().ToUpper().EndsWith(".KML"))
                        {
                            try
                            {

                                alignmentWorkspace.currentPins = pickerPins.SelectedItem.ToString();
                                CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                labelCurPins.Text = pickerPins.SelectedItem.ToString();
                                mapObject.Pins.Clear();
                                currentPinId = 0;
                                currentTagAmount = 0;
                                await loadKmlPins(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + pickerPins.SelectedItem.ToString());
                            }
                            catch
                            {
                                DisplayAlert("Error", "Failed to load the pin file", "Ok");
                            }
                        }
                        else if(pickerPins.SelectedItem.ToString().ToUpper().EndsWith(".TXT"))
                        {
                            try
                            {
                                loadAsciiPins(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + pickerPins.SelectedItem.ToString());
                                alignmentWorkspace.currentPins = pickerPins.SelectedItem.ToString();
                                CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                labelCurPins.Text = pickerPins.SelectedItem.ToString();
                                currentTagAmount = 0;
                            }
                            catch
                            {
                                DisplayAlert("Error", "Failed to load the pin file", "Ok");
                            }
                        }
                    }
                    else
                    {
                        DisplayAlert("Error", "Failed to load the selected file", "Ok");
                    }
                }
            };

            pickerOffset.SelectedIndexChanged += (sender, args) =>
            {
                if (pickerOffset.SelectedIndex != -1)
                {
                    alignmentWorkspace.offsetDisplay = pickerOffset.SelectedItem.ToString();
                    CrossSettings.Current.AddOrUpdateValue("offsetDisplay", alignmentWorkspace.offsetDisplay);

                    var lat = mapObject.CameraPosition.Target.Latitude;
                    var lng = mapObject.CameraPosition.Target.Longitude;

                    var en = GetENFromLL(lat, lng);

                    var invec = new dataTypes.nrgVector2(){ X = en.X, Y = en.Y };
                    var outvec = new dataTypes.nrgAlignmentVector5();

                    if (alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(invec, ref outvec))
                    {
                        if (outvec.offset < 0)
                            labelOff.Text = Math.Abs(outvec.offset) + " Left";
                        else
                            labelOff.Text = Math.Abs(outvec.offset) + " Right";
                    }
                    else
                    {
                        labelCh.Text = "????";
                        labelOff.Text = "????";
                    }

                    labelLandCh.Text = labelCh.Text;
                    labelLandOff.Text = labelOff.Text;


                    labelOffsetDisplay.Text = pickerOffset.SelectedItem.ToString();
                }
            };

            pickerInternational.SelectedIndexChanged += (sender, args) =>
            {
                if (pickerInternational.SelectedIndex != -1)
                {
                    alignmentWorkspace.internationalDisplay = pickerInternational.SelectedItem.ToString();
                    CrossSettings.Current.AddOrUpdateValue("internationalDisplay", alignmentWorkspace.internationalDisplay);
                    labelInternational.Text = pickerInternational.SelectedItem.ToString();

                    switch (alignmentWorkspace.internationalDisplay)
                    {
                        case "Chainage":
                            labelChStart.Text = "CH";
                            if (labelCurPinType.Text == "Station" || labelCurPinType.Text == "Meterage")
                            {
                                titlePinX.Text = "Chainage";
                                entryPinX.Placeholder = "Enter Chainage";
                            }
                            break;
                        case "Station":
                            labelChStart.Text = "ST";
                            if (labelCurPinType.Text == "Chainage" || labelCurPinType.Text == "Meterage")
                            {
                                titlePinX.Text = "Station";
                                entryPinX.Placeholder = "Enter Station";
                            }
                            break;
                        case "Meterage":
                            labelChStart.Text = "M";
                            if (labelCurPinType.Text == "Chainage" || labelCurPinType.Text == "Station")
                            {
                                titlePinX.Text = "Meterage";
                                entryPinX.Placeholder = "Meterage";
                            }
                            break;
                    }
                    labelLandChStart.Text = labelChStart.Text;
                    getRadius();
                }
            };

            pickerDecimalPlace.SelectedIndexChanged += async (sender, args) =>
            {
                if (pickerDecimalPlace.SelectedIndex != -1)
                {
                    int.TryParse(pickerDecimalPlace.Items[pickerDecimalPlace.SelectedIndex], out alignmentWorkspace.decimalDisplay);
                    CrossSettings.Current.AddOrUpdateValue("decimalDisplay", alignmentWorkspace.decimalDisplay);
                    labelDecimalPlaces.Text = alignmentWorkspace.decimalDisplay.ToString();

                    var lat = mapObject.CameraPosition.Target.Latitude;
                    var lng = mapObject.CameraPosition.Target.Longitude;

                    var en = GetENFromLL(lat, lng);
                    var invec = new dataTypes.nrgVector2(){ X = en.X, Y = en.Y };
                    var outvec = new dataTypes.nrgAlignmentVector5();

                    if(alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(invec, ref outvec))
                    {
                        if (alignmentWorkspace.chainageFormat == true)
                        {
                            if (alignmentWorkspace.chainageFormatDesc == "Format1")
                                labelCh.Text = setChainage(getDecimalDisplay(outvec.chainage));
                            else
                                labelCh.Text = setChainageFormat2(getDecimalDisplay(outvec.chainage));
                        }
                        else
                            labelCh.Text = getDecimalDisplay(outvec.chainage);
                        labelOff.Text = getDecimalDisplay(outvec.chainage);
                    }
                    else
                    {
                        labelCh.Text = "????";
                        labelOff.Text = "????";
                    }

                    labelEast.Text = getDecimalDisplay(en.X);
                    labelNorth.Text = getDecimalDisplay(en.Y);

                    labelLandCh.Text = labelCh.Text;
                    labelLandOff.Text = labelOff.Text;
                    labelLandEast.Text = labelEast.Text;
                    labelLandNorth.Text = labelNorth.Text;

                    if(mapObject.SelectedPin != null)
                    {
                        if (gridChainagePanel.IsVisible == true || gridLandscapeChainage.IsVisible == true)
                            await changePinAddres("Chainage");
                        else if (gridENPanel.IsVisible == true || gridLandscapeEN.IsVisible == true)
                            await changePinAddres("Easting Northing");
                        else if (gridLatLongPanel.IsVisible == true || gridLandscapeLL.IsVisible == true || gridPcodePanel.IsVisible == true || gridLandscapePcode.IsVisible == true)
                            await changePinAddres("Latitude Longitude");
                        else
                            await changePinAddres("Latitude Longitude");
                    }

                    if(labelCurScaleUnits.Text == "Metric")
                    {
                        //Update accuracy
                        if(App.Accuracy <= App.AccuracyThreshold)
                        {
                            //Its good
                            spanAccQuality.Text = "Good ";
                            spanAccQuality.ForegroundColor = Color.Green;
                            spanAccValue.Text = getDecimalDisplay(App.Accuracy) + "m";

                            spanLandAccQuality.Text = "Good ";
                            spanLandAccQuality.ForegroundColor = Color.Green;
                            spanLandAccValue.Text = getDecimalDisplay(App.Accuracy) + "m";
                        }
                        else
                        {
                            //Its poor
                            spanAccQuality.Text = "Poor ";
                            spanAccQuality.ForegroundColor = Color.Red;
                            spanAccValue.Text = getDecimalDisplay(App.Accuracy) + "m";

                            spanLandAccQuality.Text = "Poor ";
                            spanLandAccQuality.ForegroundColor = Color.Green;
                            spanLandAccValue.Text = getDecimalDisplay(App.Accuracy) + "m";
                        }
                    }
                    else if(labelCurScaleUnits.Text == "Imperial")
                    {
                        //Update accuracy
                        if(App.Accuracy * 3.28084 <= App.AccuracyThreshold)
                        {
                            //Its good
                            spanAccQuality.Text = "Good ";
                            spanAccQuality.ForegroundColor = Color.Green;
                            spanAccValue.Text = getDecimalDisplay(App.Accuracy * 3.28084) + "ft";

                            spanLandAccQuality.Text = "Good ";
                            spanLandAccQuality.ForegroundColor = Color.Green;
                            spanLandAccValue.Text = getDecimalDisplay(App.Accuracy * 3.28084) + "ft";
                        }
                        else
                        {
                            //Its poor
                            spanAccQuality.Text = "Poor ";
                            spanAccQuality.ForegroundColor = Color.Red;
                            spanAccValue.Text = getDecimalDisplay(App.Accuracy * 3.28084) + "ft";

                            spanLandAccQuality.Text = "Poor ";
                            spanLandAccQuality.ForegroundColor = Color.Green;
                            spanLandAccValue.Text = getDecimalDisplay(App.Accuracy * 3.28084) + "ft";
                        }
                    }
                }
            };

            pickerMapType.SelectedIndexChanged += (sender, e) =>
            {
                if (pickerMapType.SelectedIndex != -1)
                {
                    labelMapType.Text = pickerMapType.SelectedItem.ToString();

                    switch (pickerMapType.SelectedItem.ToString())
                    {
                        case "Street Map":
                            alignmentWorkspace.mapType = 0;
                            //mapObject.MapType = Xamarin.Forms.GoogleMaps.MapType.Street;
                            selectedMapType = 0;
                            break;
                        case "Satellite Map":
                            alignmentWorkspace.mapType = 1;
                            //mapObject.MapType = Xamarin.Forms.GoogleMaps.MapType.Satellite;
                            selectedMapType = 1;
                            break;
                        case "Hybrid Map":
                            alignmentWorkspace.mapType = 2;
                            //mapObject.MapType = Xamarin.Forms.GoogleMaps.MapType.Hybrid;
                            selectedMapType = 2;
                            break;
                    }
                    CrossSettings.Current.AddOrUpdateValue("mapType", alignmentWorkspace.mapType);
                    mapObject.MapType = mapTypeValues[alignmentWorkspace.mapType];
                    if (alignmentWorkspace.mapType == 0)
                    {
                        if (alignmentWorkspace.scrollMap == true)
                            buttonAutoScroll.Image = "ScrollBlack.png";
                        else
                            buttonAutoScroll.Image = "NoScrollBlack.png";
                        labelScale.TextColor = Color.Black;
                        frameScale.BackgroundColor = Color.Black;
                        frameScaleLeft.BackgroundColor = Color.Black;
                        frameScaleRight.BackgroundColor = Color.Black;
                        frameScaleCenter.BackgroundColor = Color.Black;
                    }
                    else
                    {
                        if (alignmentWorkspace.scrollMap == true)
                            buttonAutoScroll.Image = "ScrollBlack.png";
                        else
                            buttonAutoScroll.Image = "NoScrollBlack.png";
                        labelScale.TextColor = Color.White; 
                        frameScale.BackgroundColor = Color.White;
                        frameScaleLeft.BackgroundColor = Color.White;
                        frameScaleRight.BackgroundColor = Color.White;
                        frameScaleCenter.BackgroundColor = Color.White;
                    }
                }
            };

            pickerFontSize.SelectedIndexChanged += (sender, args) =>
            {
                if (pickerFontSize.SelectedIndex != -1)
                {
                    alignmentWorkspace.fontSize = pickerFontSize.SelectedItem.ToString();
                    CrossSettings.Current.AddOrUpdateValue("fontSize", alignmentWorkspace.fontSize);

                    switch (alignmentWorkspace.fontSize)
                    {
                        case "Huge":
                            changeFontSize(50, 46, 42, 38);
                            break;
                        case "Very Large":
                            changeFontSize(37, 34, 31, 27);
                            break;
                        case "Large":
                            changeFontSize(25, 23, 21, 19);
                            break;
                        case "Medium":
                            changeFontSize(23, 21, 19, 17);
                            break;
                        case "Small":
                            changeFontSize(21, 19, 17, 15);
                            break;
                        case "Micro":
                            changeFontSize(19, 17, 15, 13);
                            break;
                    }

                    labelFontSize.Text = alignmentWorkspace.fontSize;
                }
            };

            pickerImport.SelectedIndexChanged += async (sender, args) =>
            {
                if (pickerImport.SelectedIndex != -1)
                {
                    if(pickerImport.SelectedItem.ToString() == "Browse...")
                    {
                        //Allow the user to pick a file then try to load it as a LandXML file
                        try
                        {
                            var fileData = await CrossFilePicker.Current.PickFile();
                           
                            //If user cancels selection
                            if (fileData == null)
                                return;

                            //Check the file is xml
                            string fileName = fileData.FileName;
                            if(!fileName.ToUpper().EndsWith(".XML"))
                            {
                                await DisplayAlert("Import Failed", "The selected file is not a valid LandXML File.", "OK");
                                return;
                            }

                            //Read and attempt to parse the file
                            string contents = new StreamReader(fileData.GetStream()).ReadToEnd();

                            Device.BeginInvokeOnMainThread( async () =>
                            {

                                Acr.UserDialogs.UserDialogs.Instance.ShowLoading("Importing Alignments...");

                               // Task.Run(async () =>
                               //{
                                   await Task.Delay(300);
                                   var parse = new LandXmlParser();
                                   parse.LoadLandXmlFile("", contents);
                                   var count = parse.GetNrgAlignmentCount;
                                   for (int i = 0; i < count; i++)
                                   {
                                       var alignment = parse.GetNrgAlignment(i);
                                       FinalizeAlignment(alignment);
                                   }

                                   Device.BeginInvokeOnMainThread(() =>
                                   {
                                       Acr.UserDialogs.UserDialogs.Instance.HideLoading();

                                       if (count <= 0)
                                           DisplayAlert("Import Failed", "No valid alignments found in the selected file.", "OK");
                                       else
                                           DisplayAlert("Import Successful", "Imported " + count + " alignment(s).", "OK");
                                   });
                               //});
                            });
                        }
                        catch(Exception e)
                        {
                            Acr.UserDialogs.UserDialogs.Instance.HideLoading();
                            await DisplayAlert("Import Failed", e.Message, "OK");
                        }
                    }
                    else
                    {
                        //Try to load the file directly
                        try
                        {
                            var parse = new LandXmlParser();
                            parse.LoadLandXmlFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + pickerImport.SelectedItem.ToString());
                            var count = parse.GetNrgAlignmentCount;
                            for (int i = 0; i < count; i++)
                            {
                                var alignment = parse.GetNrgAlignment(i);
                                FinalizeAlignment(alignment);
                            }

                            if (count <= 0)
                                await DisplayAlert("Import Failed", "No valid alignments found in the selected file.", "OK");
                            else
                                await DisplayAlert("Import Successful", "Imported " + count + " alignments(s).", "OK");
                        }
                        catch(Exception e)
                        {
                            await DisplayAlert("Import Failed", e.Message, "OK");
                        }
                    }
                }
                pickerImport.SelectedIndex = -1;
            };

            pickerUnits.SelectedIndexChanged += (sender, args) =>
            {
                if (pickerUnits.SelectedIndex != -1)
                {
                    if (pickerUnits.SelectedItem.ToString() == "Metric")
                    {
                        CrossSettings.Current.AddOrUpdateValue("mapScaleSetting", "Metric");
                        labelCurScaleUnits.Text = "Metric";

                        //Change acc to metric
                        if(App.Accuracy / 3.28084 <= App.AccuracyThreshold)
                        {
                            //Its good
                            spanAccQuality.Text = "Good ";
                            spanAccQuality.ForegroundColor = Color.Green;
                            spanAccValue.Text = getDecimalDisplay(App.Accuracy) + "m";

                            spanLandAccQuality.Text = "Good ";
                            spanLandAccQuality.ForegroundColor = Color.Green;
                            spanLandAccValue.Text = getDecimalDisplay(App.Accuracy) + "m";
                        }
                        else
                        {
                            //Its poor
                            spanAccQuality.Text = "Poor ";
                            spanAccQuality.ForegroundColor = Color.Red;
                            spanAccValue.Text = getDecimalDisplay(App.Accuracy) + "m";

                            spanLandAccQuality.Text = "Poor ";
                            spanLandAccQuality.ForegroundColor = Color.Red;
                            spanLandAccValue.Text = getDecimalDisplay(App.Accuracy) + "m";
                        }

                        double accuracy = 0;
                        titleAccuracyThreshold.Text = "Accuracy Threshold (Meters)";
                        if(double.TryParse(entryAccuracyThreshold.Text, out accuracy))
                        {
                            App.AccuracyThreshold = accuracy / 3.28084;
                            entryAccuracyThreshold.Text = getDecimalDisplay(App.AccuracyThreshold);
                        }
                    }
                    else if (pickerUnits.SelectedItem.ToString() == "Imperial")
                    {
                        CrossSettings.Current.AddOrUpdateValue("mapScaleSetting", "Imperial");
                        labelCurScaleUnits.Text = "Imperial";

                        //Change acc to ft
                        if(App.Accuracy <= App.AccuracyThreshold)
                        {
                            //Its good
                            spanAccQuality.Text = "Good ";
                            spanAccQuality.ForegroundColor = Color.Green;
                            spanAccValue.Text = getDecimalDisplay(App.Accuracy * 3.28084) + "ft";

                            spanLandAccQuality.Text = "Good ";
                            spanLandAccQuality.ForegroundColor = Color.Green;
                            spanLandAccValue.Text = getDecimalDisplay(App.Accuracy * 3.28084) + "ft";
                        }
                        else
                        {
                            //Its poor
                            spanAccQuality.Text = "Poor ";
                            spanAccQuality.ForegroundColor = Color.Red;
                            spanAccValue.Text = getDecimalDisplay(App.Accuracy * 328084) + "ft";

                            spanLandAccQuality.Text = "Poor ";
                            spanLandAccQuality.ForegroundColor = Color.Red;
                            spanLandAccValue.Text = getDecimalDisplay(App.Accuracy * 3.28084) + "ft";
                        }

                        titleAccuracyThreshold.Text = "Accuracy Threshold (Feet)";

                        double accuracy = 0;
                        if (double.TryParse(entryAccuracyThreshold.Text, out accuracy))
                        {
                            App.AccuracyThreshold = accuracy * 3.28084;
                            entryAccuracyThreshold.Text = getDecimalDisplay(App.AccuracyThreshold);
                        }
                    }

                    getRadius();
                }
            };

            pickerPinSaveOptions.SelectedIndexChanged += async (sender, args) =>
            {
                if (pickerPinSaveOptions.SelectedIndex != -1 && alignmentWorkspace.currentFolder != null && alignmentWorkspace.currentFolder != "NoneFound" && alignmentWorkspace.currentFolder != "NoneSelected")
                {
                    if (pickerPinSaveOptions.SelectedItem.ToString() == ".GPF")
                        pinSaveAs = "gpf";
                    else if (pickerPinSaveOptions.SelectedItem.ToString() == ".KML")
                        pinSaveAs = "kml";
                    else if (pickerPinSaveOptions.SelectedItem.ToString() == "ASCII")
                        pinSaveAs = "ascii";

                    var result = await Acr.UserDialogs.UserDialogs.Instance.PromptAsync("Enter a name for the file", "Save Pins", "Save", "Cancel");
                    pickerPinSaveOptions.Unfocus();
                    if(result.Ok)
                    {
                        if (result.Text == "" || result.Text == null)
                        {
                            await DisplayAlert("Notice", "Please enter a name for the pin file", "Ok");
                        }
                        else
                        {
                            if (pinSaveAs == "gpf")
                            {
                                List<string> listGpf = new List<string>();
                                string filename = result.Text;
                                if (filename == null || filename == "")
                                    await DisplayAlert("Error", "You must enter a file name first", "Ok");
                                else
                                {
                                    listGpf = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");

                                    if (listGpf.Contains(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + filename + ".gpf"))
                                    {
                                        var action = await DisplayActionSheet("Warning file with that name already exists", "Cancel", "Overwrite");
                                        switch (action)
                                        {
                                            case "Overwrite":
                                                nrgFileRead.reader.deleteFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + filename + ".gpf");
                                                string newPath = @alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + filename + ".gpf";
                                                savePins(newPath);
                                                labelCurPins.Text = filename + ".gpf";
                                                alignmentWorkspace.currentPins = labelCurPins.Text;
                                                CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                                break;
                                        }
                                    }
                                    else
                                    {
                                        string newPath = @alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + filename + ".gpf";
                                        savePins(newPath);
                                        labelCurPins.Text = result.Text + ".gpf";
                                        alignmentWorkspace.currentPins = labelCurPins.Text;
                                        CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                    }
                                }
                            }
                            else if (pinSaveAs == "kml")
                            {
                                List<string> listKml = new List<string>();
                                var file = result.Text.ToString();
                                if (file == null || file == "")
                                    await DisplayAlert("Error", "You must enter a file name first", "Ok");
                                else
                                {
                                    listKml = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");

                                    if (listKml.Contains(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + file + ".kml"))
                                    {
                                        var action = await DisplayActionSheet("Warning file with that name already exists", "Cancel", "Overwrite");
                                        switch (action)
                                        {
                                            case "Overwrite":
                                                nrgFileRead.reader.deleteFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + file + ".kml");
                                                pinsToKml(file);
                                                labelCurPins.Text = file + ".kml";
                                                alignmentWorkspace.currentPins = labelCurPins.Text;
                                                CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                                break;

                                        }
                                    }
                                    else
                                    {
                                        pinsToKml(file);
                                        labelCurPins.Text = result.Text + ".kml";
                                        alignmentWorkspace.currentPins = labelCurPins.Text;
                                        CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                    }
                                }
                            }
                            else if (pinSaveAs == "ascii")
                            {
                                List<string> listAscii = new List<string>();
                                var file = result.Text.ToString();
                                if (file == null || file == "")
                                    await DisplayAlert("Error", "You must enter a file name first", "Ok");
                                else
                                {
                                    listAscii = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");

                                    if (listAscii.Contains(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + file + ".txt"))
                                    {
                                        var action = await DisplayActionSheet("Warning file with that name already exists", "Canel", "Overwrite");
                                        switch (action)
                                        {
                                            case "Overwrite":
                                                nrgFileRead.reader.deleteFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + file + ".txt");
                                                savePinAscii(file);
                                                labelCurPins.Text = file + ".txt";
                                                alignmentWorkspace.currentPins = labelCurPins.Text;
                                                CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                                break;
                                        }
                                    }
                                    else
                                    {
                                        savePinAscii(file);

                                        labelCurPins.Text = file + ".txt";
                                        alignmentWorkspace.currentPins = labelCurPins.Text;
                                        CrossSettings.Current.AddOrUpdateValue("currentPins", alignmentWorkspace.currentPins);
                                    }
                                }
                            }
                        }
                    }
                    pickerPinSaveOptions.SelectedIndex = -1;
                }
            };

            pickerDateFormat.SelectedIndexChanged += (sender, args) =>
            {
                if(pickerDateFormat.SelectedIndex != -1)
                {
                    App.watermarkDateFormat = pickerDateFormat.SelectedIndex;
                    labelDateFormat.Text = pickerDateFormat.SelectedItem.ToString();
                    CrossSettings.Current.AddOrUpdateValue("dateFormat", App.watermarkDateFormat);
                }
            };

            pickerTimeFormat.SelectedIndexChanged += (sender, args) =>
            {
                if(pickerTimeFormat.SelectedIndex != -1)
                {
                    App.watermarkTimeFormat = pickerTimeFormat.SelectedIndex;
                    labelTimeFormat.Text = pickerTimeFormat.SelectedItem.ToString();
                    CrossSettings.Current.AddOrUpdateValue("timeFormat", App.watermarkTimeFormat);
                }
            };

            pickerTransformUnits.SelectedIndexChanged += (sender, args) =>
            {
                if (pickerTransformUnits.SelectedIndex != -1)
                {
                    try
                    {
                        var newUnits = (Unit)pickerTransformUnits.SelectedIndex + 1;
                        if (App.SavedUnits != newUnits)
                        {
                            //Update all
                            App.SavedUnits = newUnits;
                            UpdateTransformUnits();
                            RedrawAlignment();

                            //Save transformation
                            if (alignmentWorkspace.currentFolder != null)
                                if (alignmentWorkspace.currentFolder != "None Found" && alignmentWorkspace.currentFolder != "None Selected")
                                    SaveTransformationFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/TransformationSettings.DAT");
                        }
                    }
                    catch
                    {
                        UpdateTransformUnits();
                    }
                }
            };

            buttonTransformUnits.Clicked += (sender, args) =>
            {
                pickerTransformUnits.Focus();
            };

            buttonShowAcc.Clicked += (sender, args) =>
            {
                if(App.watermarkShowAcc)
                {
                    App.watermarkShowAcc = false;
                    labelShowAcc.Text = "No";
                }
                else
                {
                    App.watermarkShowAcc = true;
                    labelShowAcc.Text = "Yes";
                }

                CrossSettings.Current.AddOrUpdateValue("showAcc", App.watermarkShowAcc);
            };

            buttonShowDate.Clicked += (sender, args) =>
            {
                if(App.watermarkShowDate)
                {
                    App.watermarkShowDate = false;
                    labelShowDate.Text = "No";
                }
                else
                {
                    App.watermarkShowDate = true;
                    labelShowDate.Text = "Yes";
                }

                CrossSettings.Current.AddOrUpdateValue("showDate", App.watermarkShowDate);
            };

            buttonShowStringName.Clicked += (sender, args) =>
            {
                if(App.watermarkShowString)
                {
                    App.watermarkShowString = false;
                    labelShowStringName.Text = "No";
                }
                else
                {
                    App.watermarkShowString = true;
                    labelShowStringName.Text = "Yes";
                }

                CrossSettings.Current.AddOrUpdateValue("showString", App.watermarkShowString);
            };

            buttonDateFormat.Clicked += (sender, args) =>
            {
                pickerDateFormat.Focus();
            };

            buttonTimeFormat.Clicked += (sender, args) =>
            {
                pickerTimeFormat.Focus();
            };

            buttonImport.Clicked += (sender, args) =>
            {
                if (labelCurProject.Text == "None Selected")
                    DisplayAlert("Error", "You must select a project first", "Ok");
                else if (labelCurProject.Text == "None Found")
                    DisplayAlert("Error", "You must select a project first", "Ok");
                else
                {
                    var list = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");
                    pickerImport.Items.Clear();
                    foreach (string item in list)
                    {
                        if (item.ToUpper().EndsWith(".XML"))
                            pickerImport.Items.Add(Path.GetFileName(item));
                    }

                    pickerImport.Items.Add("Browse...");

                    if (pickerImport.Items.Count >= 1)
                    {
                        labelImportExists.Text = "Some Found";
                        pickerImport.Focus();
                    }
                    else
                    {
                        labelImportExists.Text = "None Found";
                        DisplayAlert("Error", "There are no XML files to import for this project", "Ok");
                    }
                }             
            };

            buttonProjectPick.Clicked += (sender, args) =>
            {
                var list = nrgFileRead.reader.listFolders(alignmentWorkspace.datapath);
                //if (list.Contains(alignmentWorkspace.datapath + "Transformation") && list.Count <= 1)
                //{
                //    //labelCurProject.Text = "None Found";
                //    //if (Device.RuntimePlatform == Device.iOS)
                //    //    DisplayAlert("Error", "There are no project folders found", "Ok");
                //    //else
                //    //    DisplayAlert("Error", "There are no project folders found within the NRG folder", "Ok");
                //}
                //else if (list.Count <= 0)
                //{
                //    //labelCurProject.Text = "None Found";
                //    //if (Device.RuntimePlatform == Device.iOS)
                //    //    DisplayAlert("Error", "There are no project folders found", "Ok");
                //    //else
                //    //    DisplayAlert("Error", "There are no project folders found within the NRG folder", "Ok");
                //}
                //else
                //{
                    pickerFolder.Items.Clear();
                    foreach(string item in list)
                    {
                        if (item != alignmentWorkspace.datapath + "Transformation")
                            pickerFolder.Items.Add(Path.GetFileName(item));
                    }

                    pickerFolder.Items.Add("New Project");

                    if(pickerFolder.Items.Count >= 1)
                    {
                        pickerFolder.Focus();
                    }
                    else
                    {
                        if (Device.RuntimePlatform == Device.iOS)
                            DisplayAlert("Error", "There are no project folders found", "Ok");
                        else
                            DisplayAlert("Error", "There are no project folders found within the NRG folder", "Ok");
                    }
                //}
      
            };

            buttonAlignmentPick.Clicked += (sender, args) =>
            {
                var list = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");
                if(list.Count <= 0)
                {
                    labelCurAlignment.Text = "None Found";
                    DisplayAlert("Error", "There are no alignment files to load", "Ok");
                }
                else
                {
                    pickerNst.Items.Clear();
                    foreach(string item in list)
                    {
                        if (item.ToUpper().EndsWith(".NST") || item.ToUpper().EndsWith(".NAF"))
                            pickerNst.Items.Add(Path.GetFileName(item));
                    }
                    if(pickerNst.Items.Count >= 1)
                    {
                        pickerNst.Focus();
                    }
                    else
                    {
                        DisplayAlert("Error", "There are no alignment files to load", "Ok");
                    }
                }
                    
            };

            buttonScaleFactor.Clicked += (sender, args) =>
            {
                entryScaleFactor.Focus();
                buttonScaleFactor.InputTransparent = true;
                buttonScaleFactor.IsEnabled = false;
            };

            buttonShiftE.Clicked += (sender, args) =>
            {
                entryShiftE.Focus();
                buttonShiftE.InputTransparent = true;
                buttonShiftE.IsEnabled = false;
            };

            buttonShiftN.Clicked += (sender, args) =>
            {
                entryShiftN.Focus();
                buttonShiftN.InputTransparent = true;
                buttonShiftN.IsEnabled = false;
            };

            buttonPinsPick.Clicked += (sender, args) =>
            {
                var list = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");
                if(list.Count <= 0)
                {
                    labelCurPins.Text = "None Found";
                    DisplayAlert("Error", "There are no pin files to load", "Ok");
                }
                else
                {
                    pickerPins.Items.Clear();
                    foreach(string item in list)
                    {
                        if (item.ToUpper().EndsWith(".GPF") || item.ToUpper().EndsWith(".KML") || item.ToUpper().EndsWith(".TXT"))
                            pickerPins.Items.Add(Path.GetFileName(item));
                    }
                    if(pickerPins.Items.Count >= 1)
                    {
                        pickerPins.Focus();
                    }
                    else
                    {
                        DisplayAlert("Error", "There are no pin files to load", "Ok");
                    }

                }
            };

            buttonLinesPick.Clicked += (sender, args) =>
            {
                var list = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");
                if (list.Count <= 0)
                {
                    DisplayAlert("Error", "There are no line files to load", "Ok");
                }
                else
                {
                    if(lastline <= mapObject.Polylines.Count())
                    {
                        setupLinesList(list);
                        for (int i = 1; i < listLineGrid.Count(); i++)
                        {
                            var grid = listLineGrid[i];
                            var label = (Label)grid.Children[0];
                            var button = (Button)grid.Children[1];
                            var button2 = (Button)grid.Children[2];
                            var button3 = (Button)grid.Children[3];

                            if (mapLineList.Contains(label.Text) == false)
                                button.Image = "EmptyBox.png";
                            else
                                button.Image = "checkMarkSmall.png";

                            if (crossSectionLineList.Contains(label.Text) == false)
                                button2.Image = "EmptyBox.png";
                            else
                                button2.Image = "checkMarkSmall.png";

                            button3.BackgroundColor = Color.FromHex(CrossSettings.Current.GetValueOrDefault("LineColor" + label.Text, "#000000"));
                            grid.IsVisible = true;
                            
                        }
                        gridLinesList.IsVisible = true;
                        gridLinesSelector.IsVisible = true;
                    }
                    else
                    {
                        DisplayAlert("Notice", "Please wait for the current lines to finish loading", "Ok");
                    }
                }
            };

            buttonTransformationPick.Clicked += (sender, args) =>
            {
                var list = new List<string>();

                if (Device.RuntimePlatform == Device.Android)
                    list = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + "Transformation/");
                else if (Device.RuntimePlatform == Device.iOS)
                    list = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath);

                if(list == null || list.Count <= 0)
                {
                    pickerTransform.Items.Clear();
                    pickerTransform.Items.Add("Default (OSGB36)");
                    if (alignmentWorkspace.currentTransformation == "More...")
                        pickerTransform.Items.Add(App.transformationString);
                    pickerTransform.Items.Add("More...");
                    pickerTransform.Focus();
                }
                else
                {
                    pickerTransform.Items.Clear();
                    pickerTransform.Items.Add("Default (OSGB36)");
                    foreach (string item in list)
                    {
                        if (item.ToUpper().EndsWith(".DAT"))
                            pickerTransform.Items.Add(Path.GetFileName(item));
                    }
                    if (alignmentWorkspace.currentTransformation == "More...")
                        pickerTransform.Items.Add(App.transformationString);
                    pickerTransform.Items.Add("More...");
                    if(pickerTransform.Items.Count >= 1)
                    {
                        pickerTransform.Focus();
                    }
                    else
                    {
                        DisplayAlert("Error", "There are no transformation files to load", "Ok");
                    }          
                }
            };

            buttonScaleMethod.Clicked += (sender, args) =>
            {
                if (App.ScaleFactorMethod == ScaleFactor.Multiply)
                {
                    App.ScaleFactorMethod = ScaleFactor.Divide;
                    labelScaleMethod.Text = "Divide";
                    RedrawAlignment();

                }
                else if (App.ScaleFactorMethod == ScaleFactor.Divide)
                {
                    App.ScaleFactorMethod = ScaleFactor.Multiply;
                    labelScaleMethod.Text = "Multiply";
                    RedrawAlignment();
                }
            };

            buttonDecimalPlaces.Clicked += (sender, args) =>
            {
                pickerDecimalPlace.Focus();
            };

            buttonOffsetDisplay.Clicked += (sender, args) =>
            {
                pickerOffset.Focus();
            };

            buttonInternational.Clicked += (sender, args) =>
            {
                pickerInternational.Focus();
            };

            buttonFontSize.Clicked += (sender, args) =>
            {
                pickerFontSize.Focus();
            };

            buttonMapType.Clicked += (sender, args) =>
            {
                pickerMapType.Focus();
            };

            buttonCrossHair.Clicked += (sender, args) =>
            {
                if (labelCrossHair.Text == "Enabled")
                {
                    alignmentWorkspace.crosshairs = false;
                    CrossSettings.Current.AddOrUpdateValue("crosshairs", alignmentWorkspace.crosshairs);
                    labelCrossHair.Text = "Disabled";
                    crossHairs.Source = "";
                }
                else if (labelCrossHair.Text == "Disabled")
                {
                    alignmentWorkspace.crosshairs = true;
                    CrossSettings.Current.AddOrUpdateValue("crosshairs", alignmentWorkspace.crosshairs);
                    labelCrossHair.Text = "Enabled";
                    crossHairs.Source = "redCross.png";
                }
            };

            buttonCountryCode.Clicked += (sender, args) =>
            {
                entryCountryCode.Focus();
            };

            entryCountryCode.Focused += (sender, args) =>
            {
                buttonCountryCode.IsEnabled = false;
            };

            entryCountryCode.Unfocused += (sender, args) =>
            {
                buttonCountryCode.IsEnabled = true;
                alignmentWorkspace.countryCode = entryCountryCode.Text;
                CrossSettings.Current.AddOrUpdateValue("countryCode", alignmentWorkspace.countryCode);
            };

            buttonUpdateTime.Clicked += (sender, args) =>
            {
                entryUpdateTime.Focus();
            };

            entryUpdateTime.Focused += (sender, args) =>
            {
                buttonUpdateTime.IsEnabled = false;
            };

            entryScaleFactor.Unfocused += (sender, args) =>
            {
                //If value is not a number -- return it to 1.0
                double val = 1.0;
                if(!double.TryParse(entryScaleFactor.Text, out val))
                {
                    //Reset value to 1
                    entryScaleFactor.Text = "1.0";
                }
                if (val == 0)
                    val = 1;
                alignmentWorkspace.scaleFactor = val;
                buttonScaleFactor.InputTransparent = false;
                buttonScaleFactor.IsEnabled = true;
                CrossSettings.Current.AddOrUpdateValue("scaleFactor", val);
                updateToNewTransformation();
                if (alignmentWorkspace.currentFolder != null)
                    if (alignmentWorkspace.currentFolder != "None Selected" && alignmentWorkspace.currentFolder != "None Found")
                        SaveTransformationFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/TransformationSettings.DAT");

                RedrawAlignment();
            };

            entryShiftE.Unfocused += (sender, args) =>
            {
                //If value is not a number -- return it to 0.0
                double val = 0.0;
                if(!double.TryParse(entryShiftE.Text, out val))
                {
                    //Reset value to 0
                    entryShiftE.Text = "0";
                }
                alignmentWorkspace.siteShiftE = val;
                buttonShiftE.InputTransparent = false;
                buttonShiftE.IsEnabled = true;
                CrossSettings.Current.AddOrUpdateValue("siteShiftE", val);
                updateToNewTransformation();

                if (alignmentWorkspace.currentFolder != null)
                    if (alignmentWorkspace.currentFolder != "None Selected" && alignmentWorkspace.currentFolder != "None Found")
                        SaveTransformationFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/TransformationSettings.DAT");

                RedrawAlignment();
            };

            entryShiftN.Unfocused += (sender, args) =>
            {
                //If value is not a number -- return it to 0.0
                double val = 0.0;
                if (!double.TryParse(entryShiftN.Text, out val))
                {
                    //Reset value to 0
                    entryShiftN.Text = "0";
                }
                alignmentWorkspace.siteShiftN = val;
                buttonShiftN.InputTransparent = false;
                buttonShiftN.IsEnabled = true;
                CrossSettings.Current.AddOrUpdateValue("siteShiftN", val);
                updateToNewTransformation();

                if (alignmentWorkspace.currentFolder != null)
                    if (alignmentWorkspace.currentFolder != "None Selected" && alignmentWorkspace.currentFolder != "None Found")
                        SaveTransformationFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/TransformationSettings.DAT");

                RedrawAlignment();
            };

            entryUpdateTime.Unfocused += async (sender, args) =>
            {
                double val;
                if (!double.TryParse(entryUpdateTime.Text, out val))
                {
                    await DisplayAlert("Error", "Invalid number entered", "OK");
                    entryUpdateTime.Focus();
                }
                else if (val < 0.5)
                {
                    await DisplayAlert("Error", "Number must be higher than 0.5", "OK");
                    entryUpdateTime.Focus();
                }
                else
                {
                    try
                    {
                        alignmentWorkspace.updateTime = val;
                        CrossSettings.Current.AddOrUpdateValue("updateTime", val);
                        await CrossGeolocator.Current.StopListeningAsync();

                        if (!CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                            await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true);
                    }
                    catch
                    {
                        await DisplayAlert("Error", "Failed to apply the selected time. Reverting to default", "Ok");
                        alignmentWorkspace.updateTime = 3;
                        entryUpdateTime.Text = "3";
                        CrossSettings.Current.AddOrUpdateValue("updateTime", 3.0);
                        await CrossGeolocator.Current.StopListeningAsync();

                        if (!CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                            await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true);
                    }

                    if (alignmentWorkspace.requestOnly == true)
                        await CrossGeolocator.Current.StopListeningAsync();

                    buttonUpdateTime.IsEnabled = true;
                }        
            };

            buttonAccuracyThreshold.Clicked += (sender, args) =>
            {
                entryAccuracyThreshold.Focus();
            };

            entryAccuracyThreshold.Focused += (sender, args) =>
            {
                buttonAccuracyThreshold.IsEnabled = false;
            };

            entryAccuracyThreshold.Unfocused += async (sender, args) =>
            {
                double val;
                if(!double.TryParse(entryAccuracyThreshold.Text, out val))
                {
                    await DisplayAlert("Error", "Invalid number entered", "OK");
                    entryAccuracyThreshold.Focus();
                }
                else if(val <= 0)
                {
                    await DisplayAlert("Error", "Number must be higher than 0", "OK");
                    entryAccuracyThreshold.Focus();
                }
                else
                {
                    App.AccuracyThreshold = val;
                    CrossSettings.Current.AddOrUpdateValue("accuracyThreshold", App.AccuracyThreshold);
                    buttonAccuracyThreshold.IsEnabled = true;
                }
            };
            

            buttonUpdateDistance.Clicked += (sender, args) =>
            {
                entryUpdateDistance.Focus();
            };

            entryUpdateDistance.Focused += (sender, args) =>
            {
                buttonUpdateDistance.IsEnabled = false;
            };

            entryUpdateDistance.Unfocused += async (sender, args) =>
            {
                double val;
                if(!double.TryParse(entryUpdateDistance.Text, out val))
                {
                    await DisplayAlert("Error", "Invalid number entered", "Ok");
                    entryUpdateDistance.Focus();
                }
                else if(val < 0.1)
                {
                    await DisplayAlert("Error", "Number must be higher than 0.1", "Ok");
                    entryUpdateDistance.Focus();
                }
                else
                {
                    try
                    {
                        alignmentWorkspace.updateDistance = val;
                        CrossSettings.Current.AddOrUpdateValue("updateDistance", val);
                        await CrossGeolocator.Current.StopListeningAsync();

                        if (!CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                            await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true);
                    }
                    catch
                    {
                        await DisplayAlert("Error", "Failed to apply the selected distance. Reverting to default", "Ok");
                        alignmentWorkspace.updateDistance = 0.1;
                        entryUpdateDistance.Text = "0.1";
                        CrossSettings.Current.AddOrUpdateValue("updateDistance", 0.1);
                        await CrossGeolocator.Current.StopListeningAsync();

                        if (!CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                            await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true);
                    }
                    if (alignmentWorkspace.requestOnly == true)
                        await CrossGeolocator.Current.StopListeningAsync();
                    buttonUpdateDistance.IsEnabled = true;
                }
            };

            buttonFile.Clicked += (sender, args) =>
            {
                if (gridSearchPanel.IsVisible == true)
                {
                    gridSearchPanel.IsVisible = false;
                    buttonSearchPanel.Image = "SearchIcon.png";
                }
                selectFolder();
                if (gridFileContent.IsVisible == false)
                {
                    buttonFile.Image = "Back.png";
                    buttonNRGIcon.IsVisible = false;
                    buttonPinDropDown.IsVisible = false;
                    buttonCHPhoto.IsVisible = false;
                    gridMain.IsVisible = false;
                    gridFileContent.IsVisible = true;
                    gridPinButtonPanel.IsVisible = false;
                    labelAccuracyMain.IsVisible = false;
                }
                else if(gridFileContent.IsVisible == true)
                {
                    if (gridLinesSelector.IsVisible == true)
                    {
                        gridLinesSelector.IsVisible = false;
                        gridColorPicker.IsVisible = false;
                        gridSimpleColorPicker.IsVisible = false;
                        setLines();
                    }
                    buttonFile.Image = "SettingsIcon.png";
                    buttonNRGIcon.IsVisible = true;
                    buttonCHPhoto.IsVisible = true;
                    buttonPinDropDown.IsVisible = true;
                    gridMain.IsVisible = true;
                    gridEditPinPanel.IsVisible = false;
                    gridFileContent.IsVisible = false;
                    labelAccuracyMain.IsVisible = true;
                    buttonPinDropDown.Image = "DropDownSmall.png";
                    hidePickers();
                }
            };

            buttonChOff.Clicked += async (sender, args) =>
            {
                if(gridChainagePanel.IsVisible == false)
                {
                    if (labelSearchX.Text != "CH" || labelSearchX.Text != "ST" || labelSearchX.Text != "M" || gridSearchPanel.IsVisible == false)
                    {
                        switch (alignmentWorkspace.internationalDisplay)
                        {
                            case "Chainage":
                                labelSearchX.Text = "CH";
                                break;
                            case "Station":
                                labelSearchX.Text = "ST";
                                break;
                            case "Meterage":
                                labelSearchX.Text = "M";
                                break;
                        }

                        labelSearchY.Text = "OFF";
                        entrySearchX.Text = "";
                        entrySearchY.Text = "";
                        entrySearchX.InputTransparent = true;
                        labelSearchX.IsVisible = true;
                        labelSearchY.IsVisible = true;
                        entrySearchX.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.Keyboard = Keyboard.Numeric;
                    }

                    //Calc ch value
                    UpdateChOffDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);

                    setUpChOffPin();
                    if (mapObject.SelectedPin != null)
                        await changePinAddres("Chainage");
                    gridENPanel.IsVisible = false;
                    gridLatLongPanel.IsVisible = false;
                    gridPcodePanel.IsVisible = false;
                    gridChainagePanel.IsVisible = true;
                    buttonChOff.Opacity = 1;
                    buttonEN.Opacity = 0.5;
                    buttonLatLong.Opacity = 0.5;
                    buttonPcode.Opacity = 0.5;
                }
            };

            buttonEN.Clicked += async (sender, args) =>
            {
                if(gridENPanel.IsVisible == false)
                {
                    if (labelSearchX.Text != "E" || gridSearchPanel.IsVisible == false)
                    {
                        labelSearchX.Text = "E";
                        labelSearchY.Text = "N";
                        entrySearchX.Text = "";
                        entrySearchY.Text = "";
                        entrySearchX.InputTransparent = true;
                        labelSearchX.IsVisible = true;
                        labelSearchY.IsVisible = true;
                        entrySearchX.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.Keyboard = Keyboard.Numeric;
                    }

                    //Calc en value
                    UpdateENDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);

                    setUpENPin();
                    if (mapObject.SelectedPin != null)
                        await changePinAddres("Easting Northing");
                    gridChainagePanel.IsVisible = false;
                    gridLatLongPanel.IsVisible = false;
                    gridPcodePanel.IsVisible = false;
                    gridENPanel.IsVisible = true;
                    buttonEN.Opacity = 1;
                    buttonChOff.Opacity = 0.5;
                    buttonLatLong.Opacity = 0.5;
                    buttonPcode.Opacity = 0.5;
                }
            };

            buttonLatLong.Clicked += async(sender, args) =>
            {
                if (gridLatLongPanel.IsVisible == false)
                {
                    if (labelSearchX.Text != "LAT" || gridSearchPanel.IsVisible == false)
                    {
                        labelSearchX.Text = "LAT";
                        labelSearchY.Text = "LNG";
                        entrySearchX.Text = "";
                        entrySearchY.Text = "";
                        entrySearchX.InputTransparent = false;
                        labelSearchX.IsVisible = true;
                        labelSearchY.IsVisible = true;
                        entrySearchX.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.Keyboard = Keyboard.Numeric;
                    }

                    //Calc new ll value
                    UpdateLLDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);

                    setUpLLPin();
                    if (mapObject.SelectedPin != null)
                        await changePinAddres("Latitude Longitude");
                    gridChainagePanel.IsVisible = false;
                    gridENPanel.IsVisible = false;
                    gridPcodePanel.IsVisible = false;
                    gridLatLongPanel.IsVisible = true;
                    buttonLatLong.Opacity = 1;
                    buttonChOff.Opacity = 0.5;
                    buttonEN.Opacity = 0.5;
                    buttonPcode.Opacity = 0.5;
                }
            };

            buttonPcode.Clicked +=  async (sender, args) =>
            {
                if (entrySearchX.Text != "POSTCODE" || gridSearchPanel.IsVisible == false)
                {
                    entrySearchX.Text = "POSTCODE";
                    entrySearchY.Text = "";
                    labelSearchX.Text = "";
                    labelSearchY.Text = "";
                    entrySearchX.InputTransparent = true;
                    labelSearchX.IsVisible = false;
                    labelSearchY.IsVisible = false;
                    entrySearchX.HorizontalTextAlignment = TextAlignment.Center;
                    entrySearchY.HorizontalTextAlignment = TextAlignment.Center;
                    entrySearchY.Keyboard = Keyboard.Default;

                }
                setUpLLPin();
                if (mapObject.SelectedPin != null)
                    await changePinAddres("Latitude Longitude");
                try
                {
                    gridChainagePanel.IsVisible = false;
                    gridENPanel.IsVisible = false;
                    gridLatLongPanel.IsVisible = false;
                    gridPcodePanel.IsVisible = true;
                    buttonPcode.Opacity = 1;
                    buttonChOff.Opacity = 0.5;
                    buttonEN.Opacity = 0.5;
                    buttonLatLong.Opacity = 0.5;
                    double lat, lng;
                    double.TryParse(labelLat.Text, out lat);
                    double.TryParse(labelLong.Text, out lng);
                    var address = await App.PostCodeSearch.getPostCode(lat, lng);
                    labelPcode.Text = address;
                    labelLandPcode.Text = address;
                }
                catch(Exception e)
                {
                    DisplayAlert("Error", "Failed to retrieve postcode", "Ok");
                }
            };

            buttonSearchPanel.Clicked += (sender, args) =>
            {
                if (gridPinButtonPanel.IsVisible == true)
                {
                    gridPinButtonPanel.IsVisible = false;
                }

                if (gridSearchPanel.IsVisible == true)
                {
                    buttonSearchPanel.Image = "SearchIcon.png";
                    gridSearchPanel.IsVisible = false;
                }
                else
                {
                    if(gridChainagePanel.IsVisible == true)
                    {
                        switch(alignmentWorkspace.internationalDisplay)
                        {
                            case "Chainage":
                                labelSearchX.Text = "CH";
                                break;
                            case "Station":
                                labelSearchX.Text = "ST";
                                break;
                            case "Meterage":
                                labelSearchX.Text = "M";
                                break;
                        }
                        labelSearchY.Text = "OFF";
                        entrySearchX.Text = "";
                        entrySearchY.Text = "";
                        entrySearchX.InputTransparent = false;
                        labelSearchX.IsVisible = true;
                        labelSearchY.IsVisible = true;
                        entrySearchX.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.Keyboard = Keyboard.Numeric;
                    }
                    else if(gridENPanel.IsVisible == true)
                    {
                        labelSearchX.Text = "E";
                        labelSearchY.Text = "N";
                        entrySearchX.Text = "";
                        entrySearchY.Text = "";
                        entrySearchX.InputTransparent = false;
                        labelSearchX.IsVisible = true;
                        labelSearchY.IsVisible = true;
                        entrySearchX.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.Keyboard = Keyboard.Numeric;
                    }
                    else if(gridLatLongPanel.IsVisible == true)
                    {
                        labelSearchX.Text = "LAT";
                        labelSearchY.Text = "LNG";
                        entrySearchX.Text = "";
                        entrySearchY.Text = "";
                        entrySearchX.InputTransparent = false;
                        labelSearchX.IsVisible = true;
                        labelSearchY.IsVisible = true;
                        entrySearchY.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchX.HorizontalTextAlignment = TextAlignment.Start;
                        entrySearchY.Keyboard = Keyboard.Numeric;
                    }
                    else if(gridPcodePanel.IsVisible == true)
                    {
                        entrySearchX.Text = "POSTCODE";
                        entrySearchY.Text = "";
                        entrySearchX.InputTransparent = true;
                        labelSearchX.IsVisible = false;
                        labelSearchY.IsVisible = false;
                        entrySearchX.HorizontalTextAlignment = TextAlignment.Center;
                        entrySearchY.HorizontalTextAlignment = TextAlignment.Center;
                        entrySearchY.Keyboard = Keyboard.Default;
                    }
                    entrySearchX.Placeholder = "Enter a value";
                    entrySearchY.Placeholder = "Enter a value";
                    gridSearchPanel.IsVisible = true;
                    buttonSearchPanel.Image = "SearchExit.png";
                }
            };

            buttonSearchFind.Clicked += (sender, args) =>
            {
                if (labelSearchY.Text == "OFF")
                    moveToChOff(entrySearchX.Text, entrySearchY.Text);
                else if (labelSearchX.Text == "E")
                    moveToEN(entrySearchX.Text, entrySearchY.Text);
                else if (labelSearchX.Text == "LAT")
                    moveToLatLng(entrySearchX.Text, entrySearchY.Text);
                else if (entrySearchX.Text == "POSTCODE" && alignmentWorkspace.countryCode != "")
                    moveToPCode(entrySearchX.Text);
                else if (entrySearchX.Text == "POSTCODE" && alignmentWorkspace.countryCode == "")
                    DisplayAlert(null, "Please Enter a Country Code in the settings menu first", "Ok");
                gridSearchPanel.IsVisible = false;
                buttonSearchPanel.Image = "SearchIcon.png";
            };

            buttonMapTraffic.Clicked += (sender, args) =>
            {
                if(labelMapTraffic.Text == "Disabled")
                {
                    mapObject.IsTrafficEnabled = true;
                    labelMapTraffic.Text = "Enabled";
                    CrossSettings.Current.AddOrUpdateValue("mapTraffic", true);
                }
                else if(labelMapTraffic.Text == "Enabled")
                {
                    mapObject.IsTrafficEnabled = false;
                    labelMapTraffic.Text = "Disabled";
                    CrossSettings.Current.AddOrUpdateValue("mapTraffic", false);
                }
            };

            buttonRequestOnly.Clicked += async (sender, args) =>
            {
                if(labelRequestOnly.Text == "Disabled")
                {
                    if (CrossGeolocator.Current.IsListening)
                        await CrossGeolocator.Current.StopListeningAsync();
                    labelRequestOnly.Text = "Enabled";
                    alignmentWorkspace.requestOnly = true;
                    CrossSettings.Current.AddOrUpdateValue("requestOnly", true);
                }
                else if(labelRequestOnly.Text == "Enabled")
                {
                    labelRequestOnly.Text = "Disabled";
                    alignmentWorkspace.requestOnly = false;
                    CrossSettings.Current.AddOrUpdateValue("requestOnly", false);
                    if (!CrossGeolocator.Current.IsListening)
                    {
                        if (CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                            await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true);
                    }
                    else
                    {
                        await CrossGeolocator.Current.StopListeningAsync();
                        if (!CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                            await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true);
                    }
                }
            };

            buttonCreatePin.Clicked += (sender, args) =>
            {
                if (buttonCreatePin.Text == "Create")
                {
                    switch (labelCurPinType.Text)
                    {
                        case "Chainage":
                                createInternationalPin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, "");
                            break;
                        case "Station":
                            createInternationalPin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, "");
                            break;
                        case "Meterage":
                            createInternationalPin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, "");
                            break;
                        case "Easting Northing":
                            createEastingNorthingPin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, "");
                            break;
                        case "Latitude Longitude":
                            createLatitudeLongitudePin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, "");
                            break;
                    }
                    gridEditPinPanel.IsVisible = false;
                    gridMapPanel.IsVisible = true;
                    buttonFile.IsVisible = true;
                    buttonNRGIcon.IsVisible = true;
                    buttonCHPhoto.IsVisible = true;
                    buttonPinDropDown.IsVisible = true;
                    buttonPinDropDown.Image = "DropDownSmall.png";
                }
                else if (buttonCreatePin.Text == "Save")
                {
                    switch (labelCurPinType.Text)
                    {
                        case "Chainage":
                            createInternationalPin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, mapObject.SelectedPin.Tag.ToString());
                            break;
                        case "Station":
                            createInternationalPin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, mapObject.SelectedPin.Tag.ToString());
                            break;
                        case "Meterage":
                            createInternationalPin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, mapObject.SelectedPin.Tag.ToString());
                            break;
                        case "Easting Northing":
                            createEastingNorthingPin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, mapObject.SelectedPin.Tag.ToString());
                            break;
                        case "Latitude Longitude":
                            createLatitudeLongitudePin(entryPinID.Text, entryPinNote.Text, entryPinX.Text, entryPinY.Text, mapObject.SelectedPin.Tag.ToString());
                            break;
                    }
                    mapObject.Pins.Remove(mapObject.SelectedPin);
                    currentPinId--;

                    gridEditPinPanel.IsVisible = false;
                    gridMapPanel.IsVisible = true;
                    buttonFile.IsVisible = true;
                    buttonNRGIcon.IsVisible = true;
                    buttonCHPhoto.IsVisible = true;
                    buttonPinDropDown.IsVisible = true;
                    mapObject.SelectedPin = null;
                    buttonPinDropDown.Image = "DropDownSmall.png";
                }
            };

            buttonCancelPin.Clicked += (sender, args) =>
            {
                gridEditPinPanel.IsVisible = false;
                gridPinButtonPanel.IsVisible = false;
                buttonPinDropDown.Image = "DropDownSmall.png";
            };

            buttonPinID.Clicked += (sender, args) =>
            {
                entryPinID.Focus();
            };

            entryPinID.Focused += (sender, args) =>
            {
                buttonPinID.IsVisible = false;
            };

            entryPinID.Unfocused += (sender, args) =>
            {
                buttonPinID.IsVisible = true;
            };

            buttonPinX.Clicked += (sender, args) =>
            {
                entryPinX.Focus();
            };

            entryPinX.Focused += (sender, args) =>
            {
                buttonPinX.IsVisible = false;
            };

            entryPinX.Unfocused += (sender, args) =>
            {
                buttonPinX.IsVisible = true;
            };

            buttonPinY.Clicked += (sender, args) =>
            {
                entryPinY.Focus();
            };

            entryPinY.Focused += (sender, args) =>
            {
                buttonPinY.IsVisible = false;
            };

            entryPinY.Unfocused += (sender, args) =>
            {
                buttonPinY.IsVisible = true;
            };

            buttonPinNote.Clicked += (sender, args) =>
            {
                entryPinNote.Focus();
            };

            entryPinNote.Focused += (sender, args) =>
            {
                buttonPinNote.IsVisible = false;
            };

            entryPinNote.Unfocused += (sender, args) =>
            {
                buttonPinNote.IsVisible = true;
            };

            buttonPlacePin.Clicked += (sender, args) =>
            {
                if (gridEditPinPanel.IsVisible == false)
                {
                    if (gridChainagePanel.IsVisible == true)
                    {
                        if (alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Selected" && alignmentWorkspace.currentAlignment != "None Found")
                            alignmentWorkspace.pinType = alignmentWorkspace.internationalDisplay;
                        else
                        {
                            alignmentWorkspace.pinType = "Latitude Longitude";
                                gridChainagePanel.IsVisible = false;
                                gridLatLongPanel.IsVisible = true;
                                buttonChOff.Opacity = 0.5;
                                buttonLatLong.Opacity = 1;
                        }
                    }

                    else if (gridENPanel.IsVisible == true)
                        alignmentWorkspace.pinType = "Easting Northing";
                    else if(gridLatLongPanel.IsVisible == true || gridPcodePanel.IsVisible == true)
                        alignmentWorkspace.pinType = "Latitude Longitude";
                    var estNrth = GetENFromLL(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                    Xamarin.Forms.GoogleMaps.Position pos = new Xamarin.Forms.GoogleMaps.Position(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                    entryPinNote.Text = "";
                    entryPinID.Text = "";
                    switch (alignmentWorkspace.pinType)
                    {
                        case "Chainage":
                            var inVec = new workspace.dataTypes.nrgVector2 { X = estNrth.X, Y = estNrth.Y };
                            var outVec = new workspace.dataTypes.nrgAlignmentVector5();
                            alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(inVec, ref outVec);
                            entryPinX.Text = outVec.chainage.ToString("0.000000");
                            entryPinY.Text = outVec.offset.ToString("0.000000");
                            labelCurPinType.Text = "Chainage";
                            titlePinX.Text = "Chainage";
                            titlePinY.Text = "Offset";
                            break;
                        case "Station":
                            var outVec2 = new dataTypes.nrgAlignmentVector5();
                            alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(new dataTypes.nrgVector2() { X = estNrth.X, Y = estNrth.Y }, ref outVec2);
                            entryPinX.Text = outVec2.chainage.ToString("0.000000");
                            entryPinY.Text = outVec2.offset.ToString("0.000000");
                            labelCurPinType.Text = "Station";
                            titlePinX.Text = "Station";
                            titlePinY.Text = "Offset";
                            break;
                        case "Meterage":
                            var outVec3 = new dataTypes.nrgAlignmentVector5();
                            alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(new dataTypes.nrgVector2() { X = estNrth.X, Y = estNrth.Y }, ref outVec3);
                            entryPinX.Text = outVec3.chainage.ToString("0.000000");
                            entryPinY.Text = outVec3.offset.ToString("0.000000");
                            labelCurPinType.Text = "Meterage";
                            titlePinX.Text = "Meterage";
                            titlePinY.Text = "Offset";
                            break;
                        case "Easting Northing":
                            entryPinX.Text = estNrth.X.ToString("0.000000");
                            entryPinY.Text = estNrth.Y.ToString("0.000000");
                            labelCurPinType.Text = "Easting Northing";
                            titlePinX.Text = "Easting";
                            titlePinY.Text = "Northing ";
                            break;
                        case "Latitude Longitude":
                            entryPinX.Text = pos.Latitude.ToString("0.000000");
                            entryPinY.Text = pos.Longitude.ToString("0.000000");
                            labelCurPinType.Text = "Latitude Longitude";
                            titlePinX.Text = "Latitude";
                            titlePinY.Text = "Longitude";
                            break;
                    }
                    buttonCreatePin.Text = "Create";
                    gridPinButtonPanel.IsVisible = false;
                    gridEditPinPanel.IsVisible = true;
                }
                else if (gridEditPinPanel.IsVisible == true)
                {
                    gridMapPanel.IsVisible = true;
                    gridEditPinPanel.IsVisible = false;
                }
            };

            mapObject.PinClicked += (sender, e) =>
            {
                buttonEditPin.IsVisible = true;
            };

            buttonEditPin.Clicked += (sender, args) =>
            {
                if (gridEditPinPanel.IsVisible == true)
                {
                    gridEditPinPanel.IsVisible = false;
                    gridMapPanel.IsVisible = true;
                    buttonPinDropDown.IsVisible = true;
                    buttonCreatePin.Text = "Create";
                }
                else if (gridEditPinPanel.IsVisible == false)
                {

                    try
                    {
                        if (mapObject.SelectedPin != null && mapObject.Pins.Count > 0)
                        {
                            var pos = mapObject.SelectedPin.Position;
                            
                            if(gridChainagePanel.IsVisible == true)
                            {
                                if(alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Selected" && alignmentWorkspace.currentAlignment != "None Found")
                                {
                                    var estNrth = GetENFromLL(pos.Latitude, pos.Longitude);
                                    var outVec = new dataTypes.nrgAlignmentVector5();
                                    alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(new dataTypes.nrgVector2() { X = estNrth.X, Y = estNrth.Y }, ref outVec);
                                    entryPinX.Text = outVec.chainage.ToString("0.000000");
                                    entryPinY.Text = outVec.offset.ToString("0.000000");
                                }
                                else
                                {
                                    alignmentWorkspace.pinType = "Latitude Longitude";
                                    gridChainagePanel.IsVisible = false;
                                    gridLatLongPanel.IsVisible = true;
                                    entryPinX.Text = pos.Latitude.ToString("0.000000");
                                    entryPinY.Text = pos.Longitude.ToString("0.000000");
                                    buttonChOff.Opacity = 0.5;
                                    buttonLatLong.Opacity = 1;
                                }

                            }
                            else if(gridENPanel.IsVisible == true)
                            {
                                var eastNrth = GetENFromLL(pos.Latitude, pos.Longitude);
                                entryPinX.Text = eastNrth.X.ToString("0.000000");
                                entryPinY.Text = eastNrth.Y.ToString("0.000000");
                            }
                            else if(gridLatLongPanel.IsVisible == true || gridPcodePanel.IsVisible == true)
                            {
                                entryPinX.Text = pos.Latitude.ToString("0.000000");
                                entryPinY.Text = pos.Longitude.ToString("0.000000");
                            }

                            if (mapObject.SelectedPin.Label.Contains(','))
                            {
                                string[] tempData = mapObject.SelectedPin.Label.ToString().Split(',');
                                entryPinID.Text = tempData[0];
                                entryPinNote.Text = tempData[1];
                            }
                            else
                            {
                                entryPinID.Text = mapObject.SelectedPin.Label.ToString();
                                entryPinNote.Text = "Note";
                            }


                            buttonCreatePin.Text = "Save";
                            gridPinButtonPanel.IsVisible = false;
                            gridEditPinPanel.IsVisible = true;
                        }
                        else if (mapObject.SelectedPin == null && mapObject.Pins.Count > 0)
                        {
                            DisplayAlert("Error", "There are no pins selected", "Ok");
                        }
                        else if (mapObject.Pins.Count <= 0)
                        {
                            DisplayAlert("Error", "There are no pins to edit", "Ok");
                        }
                    }
                    catch
                    {
                        DisplayAlert("Error", "error code: 1", "Ok");
                    }
                    try
                    {

                    }
                    catch
                    {
                        gridEditPinPanel.IsVisible = false;
                        DisplayAlert("Error", "There are currently no pins to edit", "Ok");
                    }


                }
            };

            buttonNRGIcon.Clicked += (sender, args) =>
            {
                Device.OpenUri(new Uri("http://www.nrgsurveys.co.uk/"));
            };

            buttonPinDropDown.Clicked += (sender, args) =>
            {
                if (gridSearchPanel.IsVisible == true)
                {
                    gridSearchPanel.IsVisible = false;
                    buttonSearchPanel.Image = "SearchIcon.png";
                }
                if (gridPinButtonPanel.IsVisible == false)
                {
                    buttonFile.IsVisible = true;
                    gridMapPanel.IsVisible = true;
                    gridPinButtonPanel.IsVisible = true;
                    gridEditPinPanel.IsVisible = false;
                    buttonPinDropDown.Image = "DropUpSmall.png";

                }
                else
                {
                    buttonFile.IsVisible = true;
                    gridPinButtonPanel.IsVisible = false;
                    gridMapPanel.IsVisible = true;
                    buttonPinDropDown.Image = "DropDownSmall.png";
                }
            };

            gridPinButtonPanel.Unfocused += (sender, args) =>
            {
                gridPinButtonPanel.IsVisible = false;
            };

            buttonOpenMaps.Clicked += (sender, args) =>
            {
                try
                {
                    switch(Device.RuntimePlatform)
                    {
                        case Device.Android:
                            string url;
                            url = String.Format("google.navigation:q" + mapObject.CameraPosition.Target.Latitude.ToString("0.000000") + "," + mapObject.CameraPosition.Target.Longitude.ToString("0.000000"));
                            Device.OpenUri(new Uri(url));
                            break;
                        case Device.iOS:
                            string uri;
                            uri = string.Format("http://maps.apple.com/?daddr=" + mapObject.CameraPosition.Target.Latitude.ToString("0.000000") + "," + mapObject.CameraPosition.Target.Longitude.ToString("0.000000"));
                            Device.OpenUri(new Uri(uri));
                            break;
                    }
                }
                catch
                {
                    DisplayAlert("Error", "Failed to launch Google Maps", "Ok");
                }
            };

            buttonRefreshAlignment.Clicked += (sender, args) =>
            {
                RedrawAlignment();
            };

            buttonLoadManual.Clicked += (sender, args) =>
            {
                string url;
                url = String.Format("http://www.nrgsurveys.co.uk/downloads/alignmentviewer.pdf");
                Device.OpenUri(new Uri(url));
            };

            buttonLandscapeFile.Clicked += (sender, args) =>
            {
                if (gridFileContent.IsVisible == true)
                {
                    if (gridLinesSelector.IsVisible == true)
                    {
                        gridLinesSelector.IsVisible = false;
                        gridColorPicker.IsVisible = false;
                        gridSimpleColorPicker.IsVisible = false;
                        setLines();
                    }

                    if (gridMain.Children.Contains(gridLandscapeChainage))
                        gridLandscapeChainage.IsVisible = true;
                    else if (gridMain.Children.Contains(gridLandscapeEN))
                        gridLandscapeEN.IsVisible = true;
                    else if (gridMain.Children.Contains(gridLandscapeLL))
                        gridLandscapeLL.IsVisible = true;
                    else if (gridMain.Children.Contains(gridLandscapePcode))
                        gridLandscapePcode.IsVisible = true;;
                    gridFileContent.IsVisible = false;
                    buttonLandscapeChOff.IsVisible = true;
                    buttonLandscapeEN.IsVisible = true;
                    buttonLandscapeFile.Image = "SettingsIcon.png";
                    buttonLandscapeLL.IsVisible = true;
                    buttonLandscapePcode.IsVisible = true;
                    buttonLandscapePhoto.IsVisible = true;
                    hidePickers();
                    labelAccuracyMain.IsVisible = true;
                    labelLandAccuracy.IsVisible = true;
                }
                else
                {
                    gridLandscapeChainage.IsVisible = false;
                    gridLandscapeEN.IsVisible = false;
                    gridLandscapeLL.IsVisible = false;
                    gridLandscapePcode.IsVisible = false;
                    gridFileContent.IsVisible = true;
                    buttonLandscapeChOff.IsVisible = false;
                    buttonLandscapeEN.IsVisible = false;
                    buttonLandscapeFile.Image = "Back.png";
                    buttonLandscapeLL.IsVisible = false;
                    buttonLandscapePcode.IsVisible = false;
                    buttonLandscapePhoto.IsVisible = false;
                    labelAccuracyMain.IsVisible = false;
                    labelLandAccuracy.IsVisible = false;
                }
            };

            buttonLandscapeChOff.Clicked += async (sender, args) =>
            {
                if (gridLandscapeChainage.IsVisible == false)
                {
                    if (gridLandscapeEN.IsVisible == true)
                    {
                        gridLandscapeEN.IsVisible = false;
                        buttonLandscapeEN.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeEN);
                    }
                    else if (gridLandscapeLL.IsVisible == true)
                    {
                        gridLandscapeLL.IsVisible = false;
                        buttonLandscapeLL.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeLL);
                    }
                    else if (gridLandscapePcode.IsVisible == true)
                    {
                        gridLandscapePcode.IsVisible = false;
                        buttonLandscapePcode.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapePcode);
                    }
                    gridMain.Children.Add(gridLandscapeChainage, 1, 0);
                    gridLandscapeChainage.IsVisible = true;
                    buttonLandscapeChOff.Opacity = 1;
                    UpdateChOffDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                    setUpChOffPin();
                    if (mapObject.SelectedPin != null)
                        await changePinAddres("Chainage");
                }
                else
                {
                    gridMain.Children.Remove(gridLandscapeChainage);
                    gridLandscapeChainage.IsVisible = false;
                    buttonLandscapeChOff.Opacity = 0.5;
                }
            };

            buttonLandscapeEN.Clicked += async(sender, args) =>
            {
                if (gridLandscapeEN.IsVisible == false)
                {
                    if (gridLandscapeChainage.IsVisible == true)
                    {
                        gridLandscapeChainage.IsVisible = false;
                        buttonLandscapeChOff.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeChainage);
                    }
                    else if (gridLandscapeLL.IsVisible == true)
                    {
                        gridLandscapeLL.IsVisible = false;
                        buttonLandscapeLL.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeLL);
                    }
                    else if (gridLandscapePcode.IsVisible == true)
                    {
                        gridLandscapePcode.IsVisible = false;
                        buttonLandscapePcode.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapePcode);
                    }
                    gridMain.Children.Add(gridLandscapeEN, 1, 0);
                    gridLandscapeEN.IsVisible = true;
                    buttonLandscapeEN.Opacity = 1;
                    UpdateENDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                    setUpENPin();
                    if (mapObject.SelectedPin != null)
                        await changePinAddres("Easting Northing");
                }
                else
                {
                    gridMain.Children.Remove(gridLandscapeEN);
                    gridLandscapeEN.IsVisible = false;
                    buttonLandscapeEN.Opacity = 0.5;
                }
            };

            buttonLandscapeLL.Clicked += async(sender, args) =>
            {
                if (gridLandscapeLL.IsVisible == false)
                {
                    if (gridLandscapeChainage.IsVisible == true)
                    {
                        gridLandscapeChainage.IsVisible = false;
                        buttonLandscapeChOff.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeChainage);
                    }
                    else if (gridLandscapeEN.IsVisible == true)
                    {
                        gridLandscapeEN.IsVisible = false;
                        buttonLandscapeEN.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeEN);
                    }
                    else if (gridLandscapePcode.IsVisible == true)
                    {
                        gridLandscapePcode.IsVisible = false;
                        buttonLandscapePcode.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapePcode);
                    }
                    gridMain.Children.Add(gridLandscapeLL, 1, 0);
                    gridLandscapeLL.IsVisible = true;
                    buttonLandscapeLL.Opacity = 1;
                    UpdateLLDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                    setUpLLPin();
                    if (mapObject.SelectedPin != null)
                        await changePinAddres("Latitude Longitude");
                }
                else
                {
                    gridMain.Children.Remove(gridLandscapeLL);
                    gridLandscapeLL.IsVisible = false;
                    buttonLandscapeLL.Opacity = 0.5;
                }
            };

            buttonLandscapePcode.Clicked += async (sender, args) =>
            {
                if (gridLandscapePcode.IsVisible == false)
                {
                    if (gridLandscapeChainage.IsVisible == true)
                    {
                        gridLandscapeChainage.IsVisible = false;
                        buttonLandscapeChOff.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeChainage);
                    }
                    else if (gridLandscapeEN.IsVisible == true)
                    {
                        gridLandscapeEN.IsVisible = false;
                        buttonLandscapeEN.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeEN);
                    }
                    else if (gridLandscapeLL.IsVisible == true)
                    {
                        gridLandscapeLL.IsVisible = false;
                        buttonLandscapeLL.Opacity = 0.5;
                        gridMain.Children.Remove(gridLandscapeLL);
                    }
                    gridMain.Children.Add(gridLandscapePcode, 1, 0);
                    gridLandscapePcode.IsVisible = true;
                    buttonLandscapePcode.Opacity = 1;
                    setUpLLPin();
                    if (mapObject.SelectedPin != null)
                        await changePinAddres("Latitude Longitude");
                    double lat, lng;
                    double.TryParse(labelLat.Text, out lat);
                    double.TryParse(labelLong.Text, out lng);
                    try
                    {
                        var address = await App.PostCodeSearch.getPostCode(lat, lng);
                        labelPcode.Text = address;
                        labelLandPcode.Text = address;
                    }
                    catch
                    {
                        DisplayAlert("Error", "Failed to retrieve postcode", "Ok");
                    }
                }
                else
                {
                    gridMain.Children.Remove(gridLandscapePcode);
                    gridLandscapePcode.IsVisible = false;
                    buttonLandscapePcode.Opacity = 0.5;
                }
            };

            buttonCrossSection.Clicked += (sender, args) =>
            {
                if (labelCrossSection.Text == "Enabled")
                {
                    alignmentWorkspace.crossSection = false;
                    labelCrossSection.Text = "Disabled";
                    removeCrossSection();
                    CrossSettings.Current.AddOrUpdateValue("crossSection", alignmentWorkspace.crossSection);
                    if (mapObject.IsVisible == false && alignmentWorkspace.landscape == false)
                    {
                        labelCurMapStatus.Text = "Enabled";
                        mapObject.IsVisible = true;
                        gridMapPanel.RowDefinitions.Clear();
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                        getRadius();
                        mapBoxView.IsVisible = true;
                        frameScale.IsVisible = true;
                        labelScale.IsVisible = true;
                        frameScaleLeft.IsVisible = true;
                        frameScaleRight.IsVisible = true;
                        frameScaleCenter.IsVisible = true;
                    }
                    else if(mapObject.IsVisible == false && alignmentWorkspace.landscape == true)
                    {
                        labelCurMapStatus.Text = "Disabled";
                        mapObject.IsVisible = false;
                        gridMapPanel.RowDefinitions.Clear();
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                        mapBoxView.IsVisible = false;
                        frameScale.IsVisible = false;
                        labelScale.IsVisible = false;
                        frameScaleLeft.IsVisible = false;
                        frameScaleRight.IsVisible = false;
                        frameScaleCenter.IsVisible = false;
                    }
                }
                else if (labelCrossSection.Text == "Disabled")
                {
                    alignmentWorkspace.crossSection = true;
                    labelCrossSection.Text = "Enabled";
                    createIntersection();
                    createCrossSection();
                    CrossSettings.Current.AddOrUpdateValue("crossSection", alignmentWorkspace.crossSection);
                    if(mapObject.IsVisible == false && alignmentWorkspace.landscape == true)
                    {
                        labelCurMapStatus.Text = "Disabled";
                        mapObject.IsVisible = false;
                        gridMapPanel.RowDefinitions.Clear();
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                        mapBoxView.IsVisible = false;
                        frameScale.IsVisible = false;
                        labelScale.IsVisible = false;
                        frameScaleLeft.IsVisible = false;
                        frameScaleRight.IsVisible = false;
                        frameScaleCenter.IsVisible = false;
                    }
                }
            };

            canvasView.PaintSurface += OnCanvasViewPaintSurface;

            var tapGesture = new TapGestureRecognizer();
            tapGesture.NumberOfTapsRequired = 2;
            canvasView.GestureRecognizers.Add(tapGesture);
            tapGesture.Tapped += (sender, args) =>
            {
                if(alignmentWorkspace.locked == false)
                {
                    scale.scaleX = 1;
                    scale.scaleY = 1;
                    scale.transX = 0;
                    scale.transY = 0;
                    x = 0;
                    y = 0;
                    xOffset = 0;
                    yOffset = 0;
                    currentScale = 1;
                    redrawCanvas();
                }
            };

            var tapGesture1 = new TapGestureRecognizer();
            tapGesture1.NumberOfTapsRequired = 1;
            tapGesture1.Tapped += (sender, args) =>
            {
                var stack = (StackLayout)sender;
                var curstack = (StackLayout)stack.Parent.Parent;
                var button = (Button)stack.Children[1];

                if (button.Image == "DropUp.png")
                {
                    if (curstack.Children[1].ClassId == "file")
                        CrossSettings.Current.AddOrUpdateValue("fileExpanded", "0");
                    else if (curstack.Children[1].ClassId == "settings")
                        CrossSettings.Current.AddOrUpdateValue("settingsExpanded", "0");
                    else if (curstack.Children[1].ClassId == "map")
                        CrossSettings.Current.AddOrUpdateValue("mapExpanded", "0");
                    else if (curstack.Children[1].ClassId == "crossSection")
                        CrossSettings.Current.AddOrUpdateValue("crossSectionExpanded", "0");
                    curstack.Children[1].IsVisible = false;

                    button.Image = "DropDown.png";
                }
                else if (button.Image == "DropDown.png")
                {
                    if (curstack.Children[1].ClassId == "file")
                        CrossSettings.Current.AddOrUpdateValue("fileExpanded", "1");
                    else if (curstack.Children[1].ClassId == "settings")
                        CrossSettings.Current.AddOrUpdateValue("settingsExpanded", "1");
                    else if (curstack.Children[1].ClassId == "map")
                        CrossSettings.Current.AddOrUpdateValue("mapExpanded", "1");
                    else if (curstack.Children[1].ClassId == "crossSection")
                        CrossSettings.Current.AddOrUpdateValue("crossSectionExpanded", "1");
                    curstack.Children[1].IsVisible = true;
                    button.Image = "DropUp.png";
                }
            };
            menu.GestureRecognizers.Add(tapGesture1);
            menu1.GestureRecognizers.Add(tapGesture1);
            menu2.GestureRecognizers.Add(tapGesture1);
            menu3.GestureRecognizers.Add(tapGesture1);

            var panGesture = new PanGestureRecognizer();
            panGesture.PanUpdated += PanGesture_PanUpdated;
            canvasView.GestureRecognizers.Add(panGesture);

            var pinchGesture = new PinchGestureRecognizer();
            pinchGesture.PinchUpdated += PinchGesture_PinchUpdated;
            canvasView.GestureRecognizers.Add(pinchGesture);

            gridLandscapeChainage.GestureRecognizers.Clear();
            gridLandscapeEN.GestureRecognizers.Clear();
            gridLandscapeLL.GestureRecognizers.Clear();
            gridLandscapePcode.GestureRecognizers.Clear();

            var tapLandGesture = new TapGestureRecognizer();
            tapLandGesture.NumberOfTapsRequired = 2;
            tapLandGesture.Tapped += (sender, args) =>
            {
                var grid = (Grid)sender;

                if(grid.Scale != 1)
                {
                    grid.Scale = 1;
                    grid.TranslationX = 0;
                    grid.TranslationY = 0;
                    grid.AnchorX = 0;
                    grid.AnchorY = 0;
                }
                else if(grid.Scale == 1)
                {
                    grid.Scale = (gridMapPanel.Width / grid.Width);
                    if (grid.Scale * grid.Height > gridMapPanel.Height)
                        grid.Scale = (gridMapPanel.Height / grid.Height);
                    grid.TranslationX = 0;
                    grid.TranslationY = 0;
                    grid.AnchorX = 0.5;
                    grid.AnchorY = 0.5;
                }

            };

            gridLandscapeChainage.GestureRecognizers.Add(tapLandGesture);
            gridLandscapeEN.GestureRecognizers.Add(tapLandGesture);
            gridLandscapeLL.GestureRecognizers.Add(tapLandGesture);
            gridLandscapePcode.GestureRecognizers.Add(tapLandGesture);

            var panLandGesture = new PanGestureRecognizer();
            panLandGesture.PanUpdated += PanLandGesture_PanUpdated;
            panLandGesture.TouchPoints = 1;
            gridLandscapeChainage.GestureRecognizers.Add(panLandGesture);
            gridLandscapeEN.GestureRecognizers.Add(panLandGesture);
            gridLandscapeLL.GestureRecognizers.Add(panLandGesture);
            gridLandscapePcode.GestureRecognizers.Add(panLandGesture);

            sliderR.ValueChanged += (sender, args) =>
            {
                entryR.Text = Math.Round(sliderR.Value).ToString();
                var r = Convert.ToDouble(entryR.Text);
                var g = Convert.ToDouble(entryG.Text);
                var b = Convert.ToDouble(entryB.Text);
                colorFrame.BackgroundColor = Color.FromRgb(r / 255, g / 255, b / 255);
                entryHex.Text = rgbToHex(r, g, b);
            };

            sliderG.ValueChanged += (sender, args) =>
            {
                entryG.Text = Math.Round(sliderG.Value).ToString();
                var r = Convert.ToDouble(entryR.Text);
                var g = Convert.ToDouble(entryG.Text);
                var b = Convert.ToDouble(entryB.Text);
                colorFrame.BackgroundColor = Color.FromRgb(r / 255, g / 255, b / 255);
                entryHex.Text = rgbToHex(r, g, b);
            };

            sliderB.ValueChanged += (sender, args) =>
            {
                entryB.Text = Math.Round(sliderB.Value).ToString();
                var r = Convert.ToDouble(entryR.Text);
                var g = Convert.ToDouble(entryG.Text);
                var b = Convert.ToDouble(entryB.Text);
                colorFrame.BackgroundColor = Color.FromRgb(r / 255, g / 255, b / 255);
                entryHex.Text = rgbToHex(r, g, b);
            };

            entryR.Unfocused += (sender, args) =>
            {
                double r = -999;
                double.TryParse(entryR.Text, out r);
                if (r < 0)
                    r = 0;
                else if (r > 255)
                    r = 255;

                sliderR.Value = r;
                entryR.Text = r.ToString();
                var g = Convert.ToDouble(entryG.Text);
                var b = Convert.ToDouble(entryB.Text);
                colorFrame.BackgroundColor = Color.FromRgb(r / 255, g / 255, b / 255);
                entryHex.Text = rgbToHex(r, g, b);

            };

            entryG.Unfocused += (sender, args) =>
            {
                double g = -999;
                double.TryParse(entryG.Text, out g);
                if (g < 0)
                    g = 0;
                else if (g > 255)
                    g = 255;

                sliderG.Value = g;
                entryG.Text = g.ToString();
                var r = Convert.ToDouble(entryR.Text);
                var b = Convert.ToDouble(entryB.Text);
                colorFrame.BackgroundColor = Color.FromRgb(r / 255, g / 255, b / 255);
                entryHex.Text = rgbToHex(r, g, b);
            };

            entryB.Unfocused += (sender, args) =>
            {
                double b = -999;
                double.TryParse(entryB.Text, out b);
                if (b < 0)
                    b = 0;
                else if (b > 255)
                    b = 255;

                sliderB.Value = b;
                entryB.Text = b.ToString();
                var r = Convert.ToDouble(entryR.Text);
                var g = Convert.ToDouble(entryG.Text);
                colorFrame.BackgroundColor = Color.FromRgb(r / 255, g / 255, b / 255);
                entryHex.Text = rgbToHex(r, g, b);
            };

            buttonR.Clicked += (sender, args) =>
            {
                entryR.Focus();
            };

            buttonG.Clicked += (sender, args) =>
            {
                entryG.Focus();
            };

            buttonB.Clicked += (sender, args) =>
            {
                entryB.Focus();
            };

            buttonColorSelect.Clicked += (sender, args) =>
            {
                gridColorPicker.IsVisible = false;
                gridLinesList.IsVisible = true;
                if(((Label)listLineGrid[curLineID].Children[0]).Text == "Google Section")
                    listLineGrid[curLineID].Children[1].BackgroundColor = Color.FromHex(entryHex.Text);
                else
                    listLineGrid[curLineID].Children[3].BackgroundColor = Color.FromHex(entryHex.Text);
                Label label = (Label)listLineGrid[curLineID].Children[0];
            };

            buttonConfirmLine.Clicked += (sender, args) =>
            {
                setLines();
            };

            buttonMoreColors.Clicked += (sender, args) =>
            {
                gridSimpleColorPicker.IsVisible = false;
                gridColorPicker.IsVisible = true;
            };

            colorPickerBack.Clicked += (sender, args) =>
            {
                gridColorPicker.IsVisible = false;
                gridLinesList.IsVisible = true;
            };

            colorSimplePickerBack.Clicked += (sender, args) =>
            {
                gridSimpleColorPicker.IsVisible = false;
                gridLinesList.IsVisible = true;
            };

            buttonCrossSectionCenter.Clicked += (sender, args) =>
             {
                 if(alignmentWorkspace.locked == false)
                 {
                     scale.scaleX = 1;
                     scale.scaleY = 1;
                     scale.transX = 0;
                     scale.transY = 0;
                     x = 0;
                     y = 0;
                     xOffset = 0;
                     yOffset = 0;
                     currentScale = 1;
                     redrawCanvas();
                 }
                 else
                     DisplayAlert("Error", "The cross-section cannot be reset when it is locked", "Ok");
             };

            buttonInternationalFormat.Clicked += (sender, args) =>
            {
                pickerFormat.Focus();
            };

            pickerFormat.SelectedIndexChanged += (sender, args) =>
            {
                if (pickerFormat.SelectedIndex != -1)
                {
                    if (pickerFormat.SelectedItem.ToString() == "10000")
                    {
                        var lat = mapObject.CameraPosition.Target.Latitude;
                        var lng = mapObject.CameraPosition.Target.Longitude;

                        var en = GetENFromLL(lat, lng);

                        var invec = new dataTypes.nrgVector2() { X = en.X, Y = en.Y };
                        var outvec = new dataTypes.nrgAlignmentVector5();

                        if (alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(invec, ref outvec))
                        {
                            labelCh.Text = getDecimalDisplay(outvec.chainage);
                            labelOff.Text = getDecimalDisplay(outvec.offset);
                        }
                        else
                        {
                            labelCh.Text = "????";
                            labelOff.Text = "????";
                        }

                        labelCurFormat.Text = "10000";
                        alignmentWorkspace.chainageFormat = false;
                    }
                    else if (pickerFormat.SelectedItem.ToString() == "10+000")
                    {
                        alignmentWorkspace.chainageFormatDesc = "Format1";
                        CrossSettings.Current.AddOrUpdateValue("internationalFormatDesc", alignmentWorkspace.chainageFormatDesc);
                        var lat = mapObject.CameraPosition.Target.Latitude;
                        var lng = mapObject.CameraPosition.Target.Longitude;

                        var en = GetENFromLL(lat, lng);

                        var invec = new dataTypes.nrgVector2() { X = en.X, Y = en.Y };
                        var outvec = new dataTypes.nrgAlignmentVector5();

                        if(alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(invec, ref outvec))
                        {
                            labelCh.Text = setChainage(getDecimalDisplay(outvec.chainage));
                            labelOff.Text = getDecimalDisplay(outvec.offset);
                        }
                        else
                        {
                            labelCh.Text = "????";
                            labelOff.Text = "????";
                        }

                        labelCurFormat.Text = "10+000";
                        alignmentWorkspace.chainageFormat = true;
                    }
                    else if(pickerFormat.SelectedItem.ToString() == "100+00")
                    {
                        alignmentWorkspace.chainageFormatDesc = "Format2";
                        CrossSettings.Current.AddOrUpdateValue("internationalFormatDesc", alignmentWorkspace.chainageFormatDesc);
                        var lat = mapObject.CameraPosition.Target.Latitude;
                        var lng = mapObject.CameraPosition.Target.Longitude;

                        var en = GetENFromLL(lat, lng);
                        var invec = new dataTypes.nrgVector2() { X = en.X, Y = en.Y };
                        var outvec = new dataTypes.nrgAlignmentVector5();
                        if(alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(invec, ref outvec))
                        {
                            labelCh.Text = setChainageFormat2(getDecimalDisplay(outvec.chainage));
                            labelOff.Text = getDecimalDisplay(outvec.offset);
                        }
                        else
                        {
                            labelCh.Text = "????";
                            labelOff.Text = "????";
                        }
                        labelCurFormat.Text = "100+00";
                        alignmentWorkspace.chainageFormat = true;
                    }
                    labelLandCh.Text = labelCh.Text;
                    labelLandOff.Text = labelOff.Text;
                    CrossSettings.Current.AddOrUpdateValue("internationalFormat", alignmentWorkspace.chainageFormat);
                }
            };

            buttonScale.Clicked += (sender, args) =>
            {
                if (labelCurScale.Text == "Enabled")
                {
                    labelCurScale.Text = "Disabled";
                    alignmentWorkspace.scale = false;
                    CrossSettings.Current.AddOrUpdateValue("scale", false);
                    disableScale();
                }
                else if (labelCurScale.Text == "Disabled")
                {
                    labelCurScale.Text = "Enabled";
                    alignmentWorkspace.scale = true;
                    CrossSettings.Current.AddOrUpdateValue("scale", true);
                    enableScale();
                }
            };

            buttonVertExag.Clicked += (sender, args) =>
            {
                entryVertExag.Focus();
            };

            buttonDistance.Clicked += (sender, args) =>
            {
                entryDistance.Focus();
            };

            buttonMapStatus.Clicked += (sender, args) =>
            {
                if (alignmentWorkspace.landscape == false && gridCrossection.IsVisible == false && mapObject.IsVisible == true)
                    DisplayAlert("Notice", "The map can only be disabled when landscape mode or cross-section view are enabled", "Ok");
                else
                {
                    if (labelCurMapStatus.Text == "Enabled")
                    {
                        labelCurMapStatus.Text = "Disabled";
                        mapObject.IsVisible = false;
                        gridMapPanel.RowDefinitions.Clear();
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                        mapBoxView.IsVisible = false;
                        frameScale.IsVisible = false;
                        labelScale.IsVisible = false;
                        frameScaleLeft.IsVisible = false;
                        frameScaleRight.IsVisible = false;
                        frameScaleCenter.IsVisible = false;

                    }
                    else if (labelCurMapStatus.Text == "Disabled" && gridCrossection.IsVisible == false)
                    {
                        labelCurMapStatus.Text = "Enabled";
                        mapObject.IsVisible = true;
                        gridMapPanel.RowDefinitions.Clear();
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                        getRadius();
                        mapBoxView.IsVisible = true;
                        frameScale.IsVisible = true;
                        labelScale.IsVisible = true;
                        frameScaleLeft.IsVisible = true;
                        frameScaleRight.IsVisible = true;
                        frameScaleCenter.IsVisible = true;
                    }
                    else if (labelCurMapStatus.Text == "Disabled" && gridCrossection.IsVisible == true)
                    {
                        labelCurMapStatus.Text = "Enabled";
                        mapObject.IsVisible = true;
                        gridMapPanel.RowDefinitions.Clear();
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0.5, GridUnitType.Star) });
                        gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0.5, GridUnitType.Star) });
                        getRadius();
                        mapBoxView.IsVisible = true;
                        frameScale.IsVisible = true;
                        labelScale.IsVisible = true;
                        frameScaleLeft.IsVisible = true;
                        frameScaleRight.IsVisible = true;
                        frameScaleCenter.IsVisible = true;
                    }
                }
            };

            entryVertExag.Focused += (sender, args) =>
            {
                buttonVertExag.IsEnabled = false;
            };

            entryVertExag.Unfocused += (sender, args) =>
            {
                double d = 1;
                if (double.TryParse(entryVertExag.Text, out d) == false)
                    entryVertExag.Text = d.ToString();
                alignmentWorkspace.vertExag = d;
                CrossSettings.Current.AddOrUpdateValue("vertExag", alignmentWorkspace.vertExag);
                buttonVertExag.IsEnabled = true;
                if (gridCrossection.IsVisible)
                    createIntersection();
            };

            entryDistance.Focused += (sender, args) =>
            {
                buttonDistance.IsEnabled = false;
            };

            entryDistance.Unfocused += (sender, args) =>
            {
                double d = 0;
                if (double.TryParse(entryDistance.Text, out d) == false)
                    d = 0;
                else if (d < 0)
                    d = 0;
                alignmentWorkspace.distance = d;
                CrossSettings.Current.AddOrUpdateValue("distance", alignmentWorkspace.distance);
                buttonDistance.IsEnabled = true;
            };

            buttonScaleBar.Clicked += (sender, args) =>
            {
                if (labelCurScaleBar.Text == "Enabled")
                {
                    labelCurScaleBar.Text = "Disabled";
                    alignmentWorkspace.scaleBar = false;
                    CrossSettings.Current.AddOrUpdateValue("scaleBar", false);
                    getRadius();
                }
                else if (labelCurScaleBar.Text == "Disabled")
                {
                    labelCurScaleBar.Text = "Enabled";
                    alignmentWorkspace.scaleBar = true;
                    CrossSettings.Current.AddOrUpdateValue("scaleBar", true);
                    getRadius();
                }
            };

            buttonScaleUnits.Clicked += (sender, args) =>
            {
                pickerUnits.Focus();
            };

            buttonEnabledCrossSection.Clicked += (sender, args) =>
            {
                if(alignmentWorkspace.crossSection == false || gridCrossection.IsVisible == false)
                {
                    labelCrossSection.Text = "Enabled";
                    alignmentWorkspace.crossSection = true;
                    createCrossSection();
                }
                else if(alignmentWorkspace.crossSection == true)
                {
                    labelCrossSection.Text = "Disabled";
                    alignmentWorkspace.crossSection = false;
                    removeCrossSection();
                }
            };

            buttonElevation.Clicked += (sender, args) =>
            {
                entryElevation.Focus();
                buttonElevation.InputTransparent = true;
            };

            entryElevation.Unfocused += (sender, args) =>
            {
                buttonElevation.InputTransparent = false;
                double elevation = 1;

                if (double.TryParse(entryElevation.Text, out elevation) == false)
                {
                    DisplayAlert("Notice", "That distance was not recognised", "Ok");
                    entryElevation.Text = "1";
                    alignmentWorkspace.elevationDistance = elevation;
                    CrossSettings.Current.AddOrUpdateValue("elevationSegment", elevation);
                }
                else
                { 
                    DisplayAlert("Notice", "Elevation will try to get as close to " + entryElevation.Text + " segment size as possible", "Ok");
                    alignmentWorkspace.elevationDistance = elevation;
                    CrossSettings.Current.AddOrUpdateValue("elevationSegment", elevation);
                }
            };

            buttonCrossSectionLock.Clicked += (sender, args) =>
            {
                if(buttonCrossSectionLock.Image == "Unlock.png")
                {
                    alignmentWorkspace.locked = true;
                    buttonCrossSectionLock.Image = "Lock.png";
                    getOldScale();

                }
                else if(buttonCrossSectionLock.Image == "Lock.png")
                {
                    alignmentWorkspace.locked = false;
                    buttonCrossSectionLock.Image = "Unlock.png";
                }
            };

            buttonCenterSelection.Clicked += (sender, args) =>
            {
                pickerCenter.Focus();
            };

            buttonCrossSectionPause.Clicked += (sender, args) =>
            {
                if (buttonCrossSectionPause.Image == "Pause.png")
                {
                    toUpdate = false;
                    buttonCrossSectionPause.Image = "Play.png";
                } 
                else if (buttonCrossSectionPause.Image == "Play.png")
                {
                    toUpdate = true;
                    buttonCrossSectionPause.Image = "Pause.png";
                }              
            };

            pickerCenter.SelectedIndexChanged += (sender, args) =>
            {
                if(pickerCenter.SelectedIndex != -1)
                {
                    if(pickerCenter.SelectedItem.ToString() == "Centre by Mid-point")
                    {
                        labelCurCrossSectionCenter.Text = "Centre by Mid-point";
                        gridOffsetCenter.IsVisible = false;
                        alignmentWorkspace.centeredByOffset = false;
                        CrossSettings.Current.AddOrUpdateValue("centeredByOffset", false);
                    }
                    else if(pickerCenter.SelectedItem.ToString() == "Centre by Offset")
                    {
                        labelCurCrossSectionCenter.Text = "Centre by Offset";
                        gridOffsetCenter.IsVisible = true;
                        alignmentWorkspace.centeredByOffset = true;
                        CrossSettings.Current.AddOrUpdateValue("centeredByOffset", true);
                    }
                }
            };

            buttonOffsetCenter.Clicked += (sender, args) =>
            {
                entryOffsetCenter.Focus();
                buttonOffsetCenter.InputTransparent = true;
            };

            entryOffsetCenter.Unfocused += (sender, args) =>
            {
                buttonOffsetCenter.InputTransparent = false;
                double off = 0;
                if(double.TryParse(entryOffsetCenter.Text, out off) == false)
                {
                    DisplayAlert("Error", "Unrecognized value", "Ok");
                    entryOffsetCenter.Text = alignmentWorkspace.offsetCenter.ToString();
                }
                else
                {
                    alignmentWorkspace.offsetCenter = off;
                    CrossSettings.Current.AddOrUpdateValue("offsetCenter", alignmentWorkspace.offsetCenter);
                }
            };

            buttonCrossSectionElevation.Clicked += (sender, args) =>
            {
                elevationFlag = true;
                createIntersection();
            };

            mapObject.SelectedPinChanged += (sender, args) =>
            {
                if(args.SelectedPin != null)
                {
                    if (gridChainagePanel.IsVisible == true || (gridLandscapeChainage.IsVisible == true) && alignmentWorkspace.landscape == true)
                    {
                        var estNrth = GetENFromLL(args.SelectedPin.Position.Latitude, args.SelectedPin.Position.Longitude);
                        var outVec = new dataTypes.nrgAlignmentVector5();
                        if (alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(new dataTypes.nrgVector2() { X = estNrth.X, Y = estNrth.Y }, ref outVec) && (alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Selected" && alignmentWorkspace.currentAlignment != "None Found"))
                            args.SelectedPin.Address = getDecimalDisplay(outVec.chainage) + "," + getDecimalDisplay(outVec.offset);
                        else
                            args.SelectedPin.Address = "????,????";
                    }
                    else if (gridENPanel.IsVisible == true || gridLandscapeEN.IsVisible == true)
                    {
                        var estNrth = GetENFromLL(args.SelectedPin.Position.Latitude, args.SelectedPin.Position.Longitude);
                        args.SelectedPin.Address = getDecimalDisplay(estNrth.X) + "," + getDecimalDisplay(estNrth.Y);
                    }
                    else if (gridLatLongPanel.IsVisible == true || gridPcodePanel.IsVisible == true || gridLandscapePcode.IsVisible == true || gridLandscapeLL.IsVisible == true)
                    {
                        args.SelectedPin.Address = args.SelectedPin.Position.Latitude.ToString("0.000000") + "," + args.SelectedPin.Position.Longitude.ToString("0.000000");
                    }
                } 
            };

            gridScaleBar.Margin = new Thickness(0, 20 * App.density, 0, 0);

            if (CrossGeolocator.Current != null)
                CrossGeolocator.Current.PositionChanged += UpdatePosition;

            gridLandscapeChainage.SizeChanged += (sender, args) =>
            {
                var grid = (Grid)sender;
                if (grid.Scale * grid.Width >= gridMapPanel.Width)
                    grid.Scale = (gridMapPanel.Width / grid.Width);
                if (grid.Scale * grid.Height >= gridMapPanel.Height)
                    grid.Scale = (gridMapPanel.Height / grid.Height);
            };

            gridLandscapeEN.SizeChanged += (sender, args) =>
            {
                var grid = (Grid)sender;
                if (grid.Scale * grid.Width >= gridMapPanel.Width)
                    grid.Scale = (gridMapPanel.Width / grid.Width);
                if (grid.Scale * grid.Height >= gridMapPanel.Height)
                    grid.Scale = (gridMapPanel.Height / grid.Height);
            };

            gridLandscapeLL.SizeChanged += (sender, args) =>
            {
                var grid = (Grid)sender;
                if (grid.Scale * grid.Width >= gridMapPanel.Width)
                    grid.Scale = (gridMapPanel.Width / grid.Width);
                if (grid.Scale * grid.Height >= gridMapPanel.Height)
                    grid.Scale = (gridMapPanel.Height / grid.Height);
            };

            gridLandscapePcode.SizeChanged += (sender, args) =>
            {
                var grid = (Grid)sender;
                if (grid.Scale * grid.Width >= gridMapPanel.Width)
                    grid.Scale = (gridMapPanel.Width / grid.Width);
                if (grid.Scale * grid.Height >= gridMapPanel.Height)
                    grid.Scale = (gridMapPanel.Height / grid.Height);
                
            };

            CollapseFile.Clicked += (sender, args) =>
            {
                if (buttonCollapseFile.Image == "DropDown.png")
                {
                    buttonCollapseFile.Image = "DropUp.png";
                    CrossSettings.Current.AddOrUpdateValue("fileExpanded", "1");
                    fileExpander.IsVisible = true;
                }
                else if(buttonCollapseFile.Image == "DropUp.png")
                {
                    buttonCollapseFile.Image = "DropDown.png";
                    CrossSettings.Current.AddOrUpdateValue("fileExpanded", "0");
                    fileExpander.IsVisible = false;
                }
            };

            CollapseSettings.Clicked += (sender, args) =>
            {
                if (buttonCollapseSettings.Image == "DropDown.png")
                {
                    buttonCollapseSettings.Image = "DropUp.png";
                    CrossSettings.Current.AddOrUpdateValue("settingsExpanded", "1");
                    settingsExpander.IsVisible = true;
                }
                else if (buttonCollapseSettings.Image == "DropUp.png")
                {
                    buttonCollapseSettings.Image = "DropDown.png";
                    CrossSettings.Current.AddOrUpdateValue("settingsExpanded", "0");
                    settingsExpander.IsVisible = false;
                }
            };

            CollapseMap.Clicked += (sender, args) =>
            {
                if(buttonCollapseMap.Image == "DropDown.png")
                {
                    buttonCollapseMap.Image = "DropUp.png";
                    CrossSettings.Current.AddOrUpdateValue("mapExpanded", "1");
                    mapExpander.IsVisible = true;
                }
                else if(buttonCollapseMap.Image == "DropUp.png")
                {
                    buttonCollapseMap.Image = "DropDown.png";
                    CrossSettings.Current.AddOrUpdateValue("mapExpanded", "0");
                    mapExpander.IsVisible = false;
                }
            };

            CollapseCrossSection.Clicked += (sender, args) =>
            {
                if(buttonCollapseCrossSection.Image == "DropDown.png")
                {
                    buttonCollapseCrossSection.Image = "DropUp.png";
                    CrossSettings.Current.AddOrUpdateValue("crossSectionExpanded", "1");
                    crossSectionExpander.IsVisible = true;
                }
                else if(buttonCollapseCrossSection.Image == "DropUp.png")
                {
                    buttonCollapseCrossSection.Image = "DropDown.png";
                    CrossSettings.Current.AddOrUpdateValue("crossSectionExpanded", "0");
                    crossSectionExpander.IsVisible = false;
                }
            };

            CollapseLocation.Clicked += (sender, args) =>
            {
                if(buttonCollapseLocation.Image == "DropDown.png")
                {
                    buttonCollapseLocation.Image = "DropUp.png";
                    CrossSettings.Current.AddOrUpdateValue("locationExpanded", "1");
                    locationExpander.IsVisible = true;
                }
                else if(buttonCollapseLocation.Image == "DropUp.png")
                {
                    buttonCollapseLocation.Image = "DropDown.png";
                    CrossSettings.Current.AddOrUpdateValue("locationExpanded", "0");
                    locationExpander.IsVisible = false;
                }
            };

            CollapsePhoto.Clicked += (sender, args) =>
            {
                if(buttonCollapsePhoto.Image == "DropDown.png")
                {
                    buttonCollapsePhoto.Image = "DropUp.png";
                    CrossSettings.Current.AddOrUpdateValue("photoExpanded", "1");
                    photoSelectionExpander.IsVisible = true;
                }
                else if(buttonCollapsePhoto.Image == "DropUp.png")
                {
                    buttonCollapsePhoto.Image = "DropDown.png";
                    CrossSettings.Current.AddOrUpdateValue("photoExpanded", "0");
                    photoSelectionExpander.IsVisible = false;
                }
            };

            buttonWatermarkPosition.Clicked += (sender, args) =>
            {
                pickerWatermarkPosition.Focus();
            };

            buttonWatermarkSize.Clicked += (sender, args) =>
            {
                pickerWatermarkSize.Focus();
            };

            pickerWatermarkPosition.SelectedIndexChanged += (sender, args) =>
            {
                if(pickerWatermarkPosition.SelectedIndex != -1)
                {
                    alignmentWorkspace.watermarkPosition = pickerWatermarkPosition.SelectedItem.ToString();
                    labelCurWatermarkPosition.Text = alignmentWorkspace.watermarkPosition;
                    CrossSettings.Current.AddOrUpdateValue("watermarkPosition", alignmentWorkspace.watermarkPosition);
                    switch(alignmentWorkspace.watermarkPosition)
                    {
                        case "Bottom Right":
                            App.watermarkPosition = 1;
                            break;
                        case "Bottom Left":
                            App.watermarkPosition = 2;
                            break;
                        case "Top Right":
                            App.watermarkPosition = 3;
                            break;
                        case "Top Left":
                            App.watermarkPosition = 4;
                            break;
                    }
                }
            };
        
            pickerWatermarkSize.SelectedIndexChanged += (sender, args) =>
            {
                if(pickerWatermarkSize.SelectedIndex != -1)
                {
                    alignmentWorkspace.watermarkSize = pickerWatermarkSize.SelectedItem.ToString();
                    labelCurWatermarkSize.Text = alignmentWorkspace.watermarkSize;
                    CrossSettings.Current.AddOrUpdateValue("watermarkSize", alignmentWorkspace.watermarkSize);
                    switch(alignmentWorkspace.watermarkSize)
                    {
                        case "Small":
                            App.watermarkSize = 1;
                            break;
                        case "Medium":
                            App.watermarkSize = 2;
                            break;
                        case "Large":
                            App.watermarkSize = 3;
                            break;
                    }
                }
            };

            buttonAutoScroll.Clicked += (sender, args) =>
            {
                if (alignmentWorkspace.scrollMap == true)
                {
                    alignmentWorkspace.scrollMap = false;
                    CrossSettings.Current.AddOrUpdateValue("scrollMap", false);
                    if (buttonAutoScroll.Image == "ScrollBlack.png")
                        buttonAutoScroll.Image = "NoScrollBlack.png";
                }
                else
                {
                    if (alignmentWorkspace.requestOnly == true)
                    {
                        alignmentWorkspace.requestOnly = false;
                        labelRequestOnly.Text = "Disabled";
                        CrossSettings.Current.AddOrUpdateValue("requestOnly", false);
                        if (CrossGeolocator.Current.IsListening == false)
                        {
                            CrossGeolocator.Current.DesiredAccuracy = 100;
                            CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true);
                        }
                    }

                    alignmentWorkspace.scrollMap = true;
                    CrossSettings.Current.AddOrUpdateValue("scrollMap", true);
                    if (buttonAutoScroll.Image == "NoScrollBlack.png")
                        buttonAutoScroll.Image = "ScrollBlack.png";
                }
            };

            buttonCHPhoto.Clicked += TakePhoto;
            buttonLandscapePhoto.Clicked += TakePhoto;

            buttonPhotoSave.Clicked += async (sender, args) =>
            {
                var photoName = "";
                var incremented = false;
                var newIncrmeent = false;
                var nameWithoutIncrement = "";
                if(switchIncrementPhotoName.IsToggled)
                {
                    nameWithoutIncrement = Path.GetFileNameWithoutExtension(entryPhotoName.Text);
                    if(nameWithoutIncrement.EndsWith((App.PhotoNameIncrement + 1).ToString()))
                    {
                        //Does appear to be incremented correctly 
                        nameWithoutIncrement = nameWithoutIncrement.Substring(0, nameWithoutIncrement.Length - (App.PhotoNameIncrement + 1).ToString().Length);

                        if(nameWithoutIncrement == alignmentWorkspace.photoIncrementName)
                        {
                            //Its the same name
                            incremented = true;
                            photoName = entryPhotoName.Text;
                        }
                        else
                        {
                            //Its a new increment
                            newIncrmeent = true;
                            photoName = entryPhotoName.Text;
                        }
                    }
                    else
                    {
                        //Its a new increment
                        newIncrmeent = true;
                        photoName = entryPhotoName.Text;
                    }
                    App.IsPhotoNameIncrmented = true;
                }
                else
                {
                    App.IsPhotoNameIncrmented = false;
                    photoName = entryPhotoName.Text;
                }

                if (!photoName.ToUpper().EndsWith(".JPG") && !photoName.ToUpper().EndsWith(".JPEG"))
                    photoName += ".jpg";

                photoName = photoName.Replace("/", "");

                App.photoName = photoName;

                if (nrgFileRead.reader.fileExists(alignmentWorkspace.datapath.Remove(alignmentWorkspace.datapath.Count() - 5, 4) + "Pictures" + "/" + App.photoName))
                {
                    Device.BeginInvokeOnMainThread(async () =>
                    {
                        var an = await DisplayActionSheet("A file with this name already exists", "Cancel", "Overwrite");
                        if (an == "Overwrite")
                        {
                            nrgFileRead.reader.deleteFile(alignmentWorkspace.datapath.Remove(alignmentWorkspace.datapath.Count() - 5, 4) + "Pictures" + "/" + App.photoName);
                            DependencyService.Get<ICameraService>().takePhoto();
                        }
                        else
                            return;
                    });
                }
                else
                    DependencyService.Get<ICameraService>().takePhoto();

                if(newIncrmeent)
                {
                    alignmentWorkspace.photoIncrementName = nameWithoutIncrement;
                    App.PhotoNameIncrement = 1;
                }

                if(incremented)
                {
                    App.PhotoNameIncrement += 1;
                }

                gridTakePhotoSave.IsVisible = false;
            };

            buttonPhotoCancel.Clicked += (sender, args) =>
            {
                gridTakePhotoSave.IsVisible = false;
            };

            switchIncrementPhotoName.Toggled += (sender, args) =>
            {
                if(switchIncrementPhotoName.IsToggled)
                {
                    if(entryPhotoName.Text == alignmentWorkspace.photoCoordName)
                    {
                        //Setup to increment
                        if (alignmentWorkspace.photoIncrementName == "")
                        {
                            //Setup new increment

                            //Do nothing
                        }
                        else
                        {
                            //Setup old increment

                            entryPhotoName.Text = alignmentWorkspace.photoIncrementName + (App.PhotoNameIncrement + 1).ToString();
                        }
                    }
                }
                else
                {
                    //Setup coord name
                    //entryPhotoName.Text = alignmentWorkspace.photoCoordName;
                }
            };

            buttonLoadLastProjectYes.Clicked += async (sender, args) =>
            {
                gridLoadProject.IsVisible = false;

                await Task.Run(() =>
                {
                    Device.BeginInvokeOnMainThread(() =>
                    {
                        alignmentWorkspace.currentFolder = CrossSettings.Current.GetValueOrDefault("currentFolder", "None Found");
                        loadCurProject();
                    });
                });
            };

            buttonLoadLastProjectNo.Clicked += (sender, args) =>
            {
                gridLoadProject.IsVisible = false;
                alignmentWorkspace.currentFolder = "None Selected";
                //CrossSettings.Current.AddOrUpdateValue("currentFolder", "None Found");
                gridAlignmentSelection.IsVisible = false;
                gridPinsSelection.IsVisible = false;
                gridLinesSelection.IsVisible = false;
                labelCurProject.Text = "None Selected";
                alignmentWorkspace.currentAlignment = "None Selected";
                alignmentWorkspace.currentPins = "None Selected";
                alignmentWorkspace.currentLines = "None Selected";
                //CrossSettings.Current.AddOrUpdateValue("currentLines", "None Found");
                //CrossSettings.Current.AddOrUpdateValue("currentPins", "None Found");
                //CrossSettings.Current.AddOrUpdateValue("currentAlignment", "None Found");
                //CrossSettings.Current.AddOrUpdateValue("mapLineString", "none");
                //CrossSettings.Current.AddOrUpdateValue("crossLineString", "none");

            };

            entryPhotoName.Focused += (sender, args) =>
            {
                if (entryPhotoName.Text == alignmentWorkspace.photoCoordName)
                    entryPhotoName.Text = "";
            };

            entryPhotoName.Unfocused += (sender, args) =>
            {
                if (entryPhotoName.Text == "")
                    entryPhotoName.Text = alignmentWorkspace.photoCoordName;
            };

            buttonUseGPS.Clicked += (sender, args) =>
            {
                if(labelUseGPS.Text == "Disabled")
                {
                    labelUseGPS.Text = "Enabled";
                    alignmentWorkspace.alwaysUseGPS = true;
                    buttonAutoScroll.IsVisible = false;
                    alignmentWorkspace.scrollMap = true;
                    buttonAutoScroll.Image = "ScrollBlack.png";
                    CrossSettings.Current.AddOrUpdateValue("scrollMap", alignmentWorkspace.scrollMap);
                }
                else if(labelUseGPS.Text == "Enabled")
                {
                    labelUseGPS.Text = "Disabled";
                    alignmentWorkspace.alwaysUseGPS = false;
                    buttonAutoScroll.IsVisible = true;
                }

                CrossSettings.Current.AddOrUpdateValue("alwaysUseGPS", alignmentWorkspace.alwaysUseGPS);
            };

            drawTimer();
        }

        private async void ShowTerms()
        {
            await DisplayAlert("Disclaimer", "IN NO EVENT SHALL NRG SURVEYS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF NRG SURVEYS HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE." +
            "\r\nNRG SURVEYS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED" +
            "AS IS .NRG SURVEYS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.", "Accept");

            CrossSettings.Current.AddOrUpdateValue("licence", true);
        }

        protected override async void OnAppearing()
        {
            if (firstload == false)
            {
                if (!CrossSettings.Current.GetValueOrDefault("licence", false))
                    ShowTerms();

                //Get permissions
                firstload = true;
                if (Device.RuntimePlatform == Device.Android)
                {
                    if (await Plugin.Permissions.CrossPermissions.Current.CheckPermissionStatusAsync(Plugin.Permissions.Abstractions.Permission.Storage) != Plugin.Permissions.Abstractions.PermissionStatus.Granted)
                    {
                        try
                        {
                            var permission = await Plugin.Permissions.CrossPermissions.Current.RequestPermissionsAsync(Plugin.Permissions.Abstractions.Permission.Storage);
                            if (permission[Plugin.Permissions.Abstractions.Permission.Storage] != Plugin.Permissions.Abstractions.PermissionStatus.Granted)
                            {
                                await DisplayAlert("Error", "This app needs access to your storage in order to open project files.", "Ok");
                                await DisplayAlert("Notice", "This app will now close.", "Ok");
                                DependencyService.Get<ICloseApplication>().CloseApplication();
                            }
                        }
                        catch (Exception e)
                        {

                        }
                    }

                    if(await Plugin.Permissions.CrossPermissions.Current.CheckPermissionStatusAsync(Plugin.Permissions.Abstractions.Permission.Location) != Plugin.Permissions.Abstractions.PermissionStatus.Granted)
                    {
                        try
                        {
                            var permission = await Plugin.Permissions.CrossPermissions.Current.RequestPermissionsAsync(Plugin.Permissions.Abstractions.Permission.Location);
                            if(permission[Plugin.Permissions.Abstractions.Permission.Location] != Plugin.Permissions.Abstractions.PermissionStatus.Granted)
                            {
                                await DisplayAlert("Error", "This app needs access to your location to display your position.", "Ok");
                                await DisplayAlert("Notice", "This app will now close.", "Ok");
                                DependencyService.Get<ICloseApplication>().CloseApplication();
                            }
                        }
                        catch(Exception e)
                        {

                        }
                    }
                    gridMapPanel.Children.Clear();
                    gridMapPanel.Children.Add(gridCrossection, 0, 1);
                    gridMapPanel.Children.Add(mapObject, 0, 0);
                    gridMapPanel.Children.Add(mapBoxView, 0, 1);
                    gridMapPanel.Children.Add(crossHairs, 0, 0);
                    gridMapPanel.Children.Add(buttonEnabledCrossSection, 0, 0);
                    gridMapPanel.Children.Add(buttonAutoScroll, 0, 0);

                    if (CrossGeolocator.Current.IsListening)
                        await CrossGeolocator.Current.StopListeningAsync();

                    var settings = new ListenerSettings();
                    settings.AllowBackgroundUpdates = true;

                    await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromMilliseconds(3000), 1.0, true);
                    CrossGeolocator.Current.DesiredAccuracy = 100;
                    CrossGeolocator.Current.PositionError += (sender, args) =>
                    {

                    };


                    //Creates the default directory if it doesn't already exits
                    nrgFileRead.reader.folderSetup(alignmentWorkspace.datapath);

                    if (CrossSettings.Current.GetValueOrDefault("firstRun", false) == false)
                    {
                        App.SetupFiles.setupData();
                        manualSelection();
                    }

                    loadSettings();
                    listAllFolders();

                    CrossSettings.Current.AddOrUpdateValue("firstRun", true);
                }
                else if(Device.RuntimePlatform == Device.iOS)
                {
                    if (CrossGeolocator.Current.IsListening)
                        await CrossGeolocator.Current.StopListeningAsync();

                    var settings = new ListenerSettings();
                    settings.AllowBackgroundUpdates = true;

                    await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromMilliseconds(3000), 1.0, true);
                    CrossGeolocator.Current.DesiredAccuracy = 100;
                    CrossGeolocator.Current.PositionError += (sender, args) =>
                    {

                    };


                    if (CrossSettings.Current.GetValueOrDefault("firstRun", false) == false)
                    {
                        App.SetupFiles.setupData();
                        manualSelection();
                    }

                    loadSettings();
                    listAllFolders();

                    CrossSettings.Current.AddOrUpdateValue("firstRun", true);
                }
            }

            base.OnAppearing();
        }

        private async void GetStoragePermission()
        {
            var access = await Plugin.Permissions.CrossPermissions.Current.CheckPermissionStatusAsync(Plugin.Permissions.Abstractions.Permission.Storage);

            if (access != Plugin.Permissions.Abstractions.PermissionStatus.Granted)
            {
                //Ask for permission
                var test = Plugin.Permissions.CrossPermissions.Current.RequestPermissionsAsync(Plugin.Permissions.Abstractions.Permission.Storage);
            }

            if (await Plugin.Permissions.CrossPermissions.Current.CheckPermissionStatusAsync(Plugin.Permissions.Abstractions.Permission.Storage) != Plugin.Permissions.Abstractions.PermissionStatus.Granted)
                return;

            nrgFileRead.reader.CreateFolder(alignmentWorkspace.datapath);
            nrgFileRead.reader.CreateFolder(alignmentWorkspace.datapath + "Transformation/");
            nrgFileRead.reader.CreateFile(alignmentWorkspace.datapath + "Transformation/setUp.tmp");

            return;
        }

        private void LoadNewTransformation()
        {
            projTarget = DotSpatial.Projections.ProjectionInfo.FromProj4String(App.transformationBody);
            NewTransformation = true;
        }

        private async void TakePhoto(object sender, EventArgs e)
        {
            //Get coords
            //Ch/off = 0 e/n = 1 l/l = 2
            var choice = 0;




            if ((gridPcodePanel.IsVisible == true && alignmentWorkspace.landscape == false) || gridLandscapePcode.IsVisible == true || gridLandscapeLL.IsVisible == true || (gridLatLongPanel.IsVisible == true && alignmentWorkspace.landscape == false))
                choice = 2;
            else if ((gridENPanel.IsVisible == true && alignmentWorkspace.landscape == false) || gridLandscapeEN.IsVisible == true)
                choice = 1;
            else
            {
                if (alignmentWorkspace.currentAlignment == null || alignmentWorkspace.currentAlignment == "None Selected" || alignmentWorkspace.currentAlignment == "None Found")
                {
                    var ans = await DisplayActionSheet("No alignment exists, select a watermark type.", "East/North", "Lat/Long");
                    if (ans == "East/North")
                        choice = 1;
                    else if (ans == "Lat/Long")
                        choice = 2;
                    else
                        return;
                }
                else
                    choice = 0;
            }

            App.StringName = "";
            var now = DateTime.Now;
            string day = now.Day.ToString(), month = now.Month.ToString(), year = now.Year.ToString();
            int hour = now.Hour, minute = now.Minute;
            string minString = minute.ToString();

            while(minString.Length < 2)
            {
                minString = "0" + minString;
            }

            switch(App.watermarkTimeFormat)
            {
                //24 Hour
                case 0:
                    var hourString = now.Hour.ToString();
                    while (hourString.Length < 2)
                        hourString = "0" + hourString;

                    App.Time = hourString + ":" + minString;
                    break;
                case 1:
                    if(hour > 12)
                        App.Time = now.Hour - 12 + ":" + minString + " pm";
                    else
                        App.Time = now.Hour + ":" + minString + " am";

                    break;
            }

            switch(App.watermarkDateFormat)
            {
                //      DD/MM/YYYY
                case 0:
                    while (month.Length <= 1)
                        month = "0" + month;

                    App.Date = "Date: " + day + "/" + month + "/" + year + "  " + App.Time;
                    break;
                //      MM/DD/YYYY
                case 1:
                    while (day.Length <= 1)
                        day = "0" + day;

                    App.Date = "Date: " + month + "/" + day + "/" + year + "  " + App.Time;
                    break;
                case 2:
                    while (month.Length <= 1)
                        month = "0" + month;

                    while (day.Length <= 1)
                        day = "0" + day;

                    App.Date = "Date: " + year + "/" + month + "/" + day + "  " + App.Time;
                    break;
            }
           
            //Always take device position, if position is not available, show N/A
            if (alignmentWorkspace.requestOnly == true || !CrossGeolocator.Current.IsListening)
            {
                Plugin.Geolocator.Abstractions.Position position = null;

                if (CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                    position = await CrossGeolocator.Current.GetPositionAsync(TimeSpan.FromMilliseconds(3000));
                else if(CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                {
                    CrossGeolocator.Current.DesiredAccuracy = 100;
                    await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromMilliseconds(3000), 0.1, true);

                    //Get location
                    position = await CrossGeolocator.Current.GetPositionAsync(TimeSpan.FromMilliseconds(3000));
                }

                //Stop listening
                await CrossGeolocator.Current.StopListeningAsync();

                if (position != null)
                {
                    if (labelCurScaleUnits.Text == "Metric")
                    {
                        if (position.Accuracy > App.AccuracyThreshold)
                            await DisplayAlert("Warning", "The current position accuracy is of poor quality.", "OK");
                    }
                    else if(labelCurScaleUnits.Text == "Imperial")
                    {
                        if (position.Accuracy * 3.28084 > App.AccuracyThreshold)
                            await DisplayAlert("Warning", "The current position accuracy is of poor quality.", "OK");
                    }

                    //We have a value
                    App.lat = Math.Round(position.Latitude, 6);
                    App.lng = Math.Round(position.Longitude, 6);

                    if (labelCurScaleUnits.Text == "Metric")
                        App.Acc = "Acc: " + getDecimalDisplay(position.Accuracy) + "m";
                    else if(labelCurScaleUnits.Text == "Imperial")
                        App.Acc = "Acc: " + getDecimalDisplay(position.Accuracy * 3.28084) + "ft";

                    var en = GetENFromLL(position.Latitude, position.Longitude);

                    switch (choice)
                    {
                        case 0:
                            if (alignmentWorkspace.Alignments == null)
                            {
                                App.ch = "CH:  N/A";
                                App.off = "OFF: N/A";
                                App.StringName = "";
                            }
                            else
                                App.StringName = "String: " + alignmentWorkspace.Alignments.stringName;

                            var outVec = new dataTypes.nrgAlignmentVector5();
                            if (alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(en, ref outVec))
                                SetupChOffSetPhotoLabels(outVec.chainage, outVec.offset);
                            else
                            {
                                App.ch = "CH:  N/A";
                                App.off = "OFF: N/A";
                            }
                            break;
                        case 1:
                            App.ch = "E: " + getDecimalDisplay(en.X);
                            App.off = "N: " + getDecimalDisplay(en.Y);
                            break;
                        case 2:
                            App.ch = "LAT: " + App.lat;
                            App.off = "LNG: " + App.lng;
                            break;
                    }
                }
                else
                {
                    //Set value to N/A
                    App.Acc = "Acc: N/A";
                    switch (choice)
                    {
                        case 0:
                            App.ch = "CH:  N/A";
                            App.off = "OFF: N/A";
                            App.StringName = "";
                            break;
                        case 1:
                            App.ch = "E: N/A";
                            App.off = "N: N/A";
                            break;
                        case 2:
                            App.ch = "LAT: N/A";
                            App.off = "LNG: N/A";
                            break;
                    }
                }

                if (Device.RuntimePlatform == Device.Android)
                {
                    var filename = App.ch + "_" + App.off + ".jpg";
                    filename = filename.Replace("/", "");
                    ShowPhotoSaveDialog(filename);
                }
                else
                    DependencyService.Get<ICameraService>().takePhoto();
            }
            else if(CrossGeolocator.Current.IsListening)
            {
                //Get current position
                Plugin.Geolocator.Abstractions.Position position = null;

                if(CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                    position = await CrossGeolocator.Current.GetPositionAsync(TimeSpan.FromMilliseconds(3000));

                if(position != null)
                {
                    //We have a value
                    App.lat = Math.Round(position.Latitude, 6);
                    App.lng = Math.Round(position.Longitude, 6);

                    if (labelCurScaleUnits.Text == "Metric")
                    {
                        if (position.Accuracy > App.AccuracyThreshold)
                            await DisplayAlert("Warning", "The current position accuracy is of poor quality.", "OK");
                    }
                    else if (labelCurScaleUnits.Text == "Imperial")
                    {
                        if (position.Accuracy * 3.28084 > App.AccuracyThreshold)
                            await DisplayAlert("Warning", "The current position accuracy is of poor quality.", "OK");
                    }

                    if (labelCurScaleUnits.Text == "Metric")
                        App.Acc = "Acc: " + getDecimalDisplay(position.Accuracy) + "m";
                    else if (labelCurScaleUnits.Text == "Imperial")
                        App.Acc = "Acc: " + getDecimalDisplay(position.Accuracy * 3.28084) + "ft";

                    var en = GetENFromLL(position.Latitude, position.Longitude);

                    switch (choice)
                    {
                        case 0:
                            if (alignmentWorkspace.Alignments == null)
                            {
                                App.ch = "CH:  N/A";
                                App.off = "OFF: N/A";
                                App.StringName = "";
                            }
                            else
                                App.StringName = "String: " + alignmentWorkspace.Alignments.stringName;

                            var outVec = new dataTypes.nrgAlignmentVector5();
                            if (alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(en, ref outVec))
                                SetupChOffSetPhotoLabels(outVec.chainage, outVec.offset);
                            else
                            {
                                App.ch = "CH:  N/A";
                                App.off = "OFF: N/A";
                            }
                            break;
                        case 1:
                            App.ch = "E: " + getDecimalDisplay(en.X);
                            App.off = "N: " + getDecimalDisplay(en.Y);
                            break;
                        case 2:
                            App.ch = "LAT: " + App.lat;
                            App.off = "LNG: " + App.lng;
                            break;
                    }
                }
                else
                {
                    //Set value to N/A
                    App.Acc = "Acc: N/A"; 
                    switch(choice)
                    {
                        case 0:
                            App.ch = "CH:  N/A";
                            App.off = "OFF: N/A";
                            App.StringName = "";
                            break;
                        case 1:
                            App.ch = "E: N/A";
                            App.off = "N: N/A";
                            break;
                        case 2:
                            App.ch = "LAT: N/A";
                            App.off = "LNG: N/A";
                            break;
                    }
                }

                if (Device.RuntimePlatform == Device.Android)
                {
                    var filename = App.ch + "_" + App.off + ".jpg";
                    filename = filename.Replace("/", "");
                    ShowPhotoSaveDialog(filename);
                }
                else
                    DependencyService.Get<ICameraService>().takePhoto();
            }
        }

        private void ShowPhotoSaveDialog(string photoName)
        {
            gridTakePhotoSave.IsVisible = true;
            alignmentWorkspace.photoCoordName = photoName;

            if(switchIncrementPhotoName.IsToggled)
            {
                //Increment previous name
                entryPhotoName.Text = alignmentWorkspace.photoIncrementName + (App.PhotoNameIncrement + 1).ToString();
            }
            else
            {
                entryPhotoName.Text = photoName;
            }
        }

        private void SetChOffWatermarkString()
        {
            UpdateChOffDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
            switch(alignmentWorkspace.internationalDisplay)
            {
                case "Chainage":
                    App.ch = "CH:  " + labelCh.Text;
                    break;
                case "Station":
                    App.ch = "ST:  " + labelCh.Text;
                    break;
                case "Meterage":
                    App.ch = "M:   " + labelCh.Text;
                    break;
            }
            App.off = "OFF: " + labelOff.Text;
        }

        private void SetEastNorthWatermarkString()
        {
            UpdateENDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
            App.ch = "E: " + labelEast.Text;
            App.off = "N: " + labelNorth.Text;
        }

        private void SetLatLongWatermarkString()
        {
            UpdateLLDisplay(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
            App.ch = "LAT:  " + labelLat.Text;
            App.off = "LNG: " + labelLong.Text;
        }

        private void GetChAndOffsetValue(string ch, string off)
        {
            try
            {
                if (alignmentWorkspace.chainageFormat)
                    ch = ch.Replace("+", "");

                if (alignmentWorkspace.offsetDisplay != "+/-")
                {
                    if (off.Contains(" Right"))
                    {
                        off = off.Replace(" Right", "");
                        App.offValue = (Convert.ToDouble(off) * -1);
                    }
                    else if (off.Contains(" Left"))
                    {
                        off = off.Replace(" Left", "");
                        App.offValue = (Convert.ToDouble(off));
                    }
                }
                else
                    App.offValue = Convert.ToDouble(off);


                App.chValue = Convert.ToDouble(ch);
            }
            catch
            {
                App.chValue = -999;
                App.offValue = -999;
            }

        }

        private void locationTime()
        {
            Device.StartTimer(TimeSpan.FromSeconds(30), () =>
            {
                Task.Factory.StartNew(async () =>
                {
                    if(!CrossGeolocator.Current.IsListening && CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled && !alignmentWorkspace.requestOnly)
                        await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.distance, true);
                });
                return false;
            });
        }

        private void drawTimer()
        {
            Device.StartTimer(TimeSpan.FromSeconds(1), () =>
            {
                Task.Factory.StartNew(async () =>
                {
                    canDraw = true;
                    Device.BeginInvokeOnMainThread(() =>
                    {
                        drawTimer();
                    });
                });
                return false;
            });
        }

        private void SetupChOffSetPhotoLabels(double ch, double off)
        {
            string chainage = "";
            string offset = "";

            offset = "OFF: ";
            switch (alignmentWorkspace.internationalDisplay)
            {
                case "Chainage":
                    chainage += "CH:  ";
                    break;
                case "Station":
                    chainage += "ST:  ";
                    break;
                case "Meterage":
                    chainage += "M:   ";
                    break;
            }
            if (alignmentWorkspace.chainageFormat == true)
            {
                if (alignmentWorkspace.chainageFormatDesc == "Format1")
                    chainage += setChainage(getDecimalDisplay(ch));
                else
                    chainage += setChainageFormat2(getDecimalDisplay(ch));
            }
            else
            {
                chainage += getDecimalDisplay(ch);
            }

            if (alignmentWorkspace.offsetDisplay == "-/+" && off < 0)
            {
                offset += getDecimalDisplay(off);
            }
            else if (alignmentWorkspace.offsetDisplay == "-/+" && off >= 0)
            {
                offset += " " + getDecimalDisplay(off);
            }
            else if (off < 0)
            {
                offset += " " + getDecimalDisplay(Math.Abs(off)) + " LEFT";
            }
            else if (off > 0)
            {
                offset += " " + getDecimalDisplay(Math.Abs(off)) + " RIGHT";
            }

            App.ch = chainage;
            App.off = offset;
        }

        private async void manualSelection()
        {
            var action = await DisplayActionSheet("Would you like to view the manual?", "No", "Yes");

            if(action == "Yes")
            {
                string url;
                url = String.Format("http://www.nrgsurveys.co.uk/downloads/alignmentviewer.pdf");
                Device.OpenUri(new Uri(url));
            }
            else if(action == "No")
            {
                await DisplayAlert("Notice", "You can view the manual at any time from the menu", "Ok");
            }

        }

        private void PinchGesture_PinchUpdated(object sender, PinchGestureUpdatedEventArgs e)
        {
            if(alignmentWorkspace.locked == false)
            {
                if (e.Status == GestureStatus.Started)
                {
                    startScale = scale.scaleX;
                    Content.AnchorX = 0;
                    Content.AnchorY = 0;
                    scale.pivotX = Content.Width;
                    scale.pivotY = Content.Height;
                }
                if (e.Status == GestureStatus.Running)
                {
                    currentScale += (e.Scale - 1) * startScale;
                    currentScale = Math.Max(1, currentScale);

                    scale.scaleX = currentScale;
                    scale.scaleY = currentScale;

                    redrawCanvas();
                }
                if (e.Status == GestureStatus.Completed)
                {
                    redrawCanvas();
                }
            }            
        }

        private void PanGesture_PanUpdated(object sender, PanUpdatedEventArgs e)
        {
            if(alignmentWorkspace.locked == false)
            { 
                switch (e.StatusType)
                {
                    case GestureStatus.Started:
                        startX = e.TotalX;
                        startY = e.TotalY;
                        break;
                    case GestureStatus.Running:
                        var maxTranslationX = (scale.scaleX * ((App.density * Content.Width) * 0.9));
                        scale.transX = Math.Min((scale.scaleX * ((App.density * Content.Width) * 0.9)), Math.Max(-maxTranslationX, x + (e.TotalX/scale.scaleX) - startX));
                        var maxTranslationY = (scale.scaleY * ((App.density * Content.Height) * 0.9));
                        scale.transY = Math.Min((scale.scaleY * ((App.density * Content.Height) * 0.9)), Math.Max(-maxTranslationY, y + (e.TotalY/scale.scaleY) - startY));
                        redrawCanvas();
                        break;
                    case GestureStatus.Completed:
                        x = scale.transX;
                        y = scale.transY;
                        redrawCanvas();
                        break;
                }
            }
            
        }

        private void PanLandGesture_PanUpdated(object sender, PanUpdatedEventArgs e)
        {
            var grid = (Grid)sender;
            switch(e.StatusType)
            {
                case GestureStatus.Started:
                    startLX = e.TotalX;
                    startLY = e.TotalY;
                    lX = grid.TranslationX;
                    lY = grid.TranslationY;
                    grid.AnchorX = 0.5;
                    grid.AnchorY = 0.5;
                    break;
                case GestureStatus.Running:
                    var maxTranslationX = (gridMapPanel.Width / 2) - ((grid.Width /2) * grid.Scale);
                    var TranslationX = Math.Min((gridMapPanel.Width / 2) - ((grid.Width / 2) * (grid.Scale)), Math.Max(-maxTranslationX, lX + (e.TotalX/grid.Scale) - startLX));
                    var maxTranslationY = (gridMapPanel.Height / 2) - ((grid.Height / 2) * grid.Scale);
                    var TranslationY = Math.Min((gridMapPanel.Height / 2) - ((grid.Height / 2) * (grid.Scale)), Math.Max(-maxTranslationY, lY + (e.TotalY/grid.Scale) - startLY));
                    grid.TranslateTo(Math.Ceiling(TranslationX), Math.Ceiling(TranslationY), 50, null);
                    lX = grid.TranslationX;
                    lY = grid.TranslationY;
                    break;
                case GestureStatus.Completed:

                    break;
            }
        }

        private void PinchLandGesture_PinchUpdated(object sender, PinchGestureUpdatedEventArgs e)
        {
            var grid = (Grid)sender;
            var count = 0;
            switch(e.Status)
            {
                case GestureStatus.Started:

                    LStartScale = grid.Scale;
                    LCurrentScale = grid.Scale;
                    grid.AnchorX = 0.5;
                    grid.AnchorY = 0.5;
                    count = 0;
                    break;
                case GestureStatus.Running:

                    if(count == 1)
                    {
                        count = 0;
                    }
                    else if(e.Scale >= 0)
                    {
                        LCurrentScale += (e.Scale - 1) * LStartScale;
                        LCurrentScale = Math.Max(0.5, LCurrentScale);
                        if (grid.Width * LCurrentScale <= gridMapPanel.Width)
                            grid.Scale = LCurrentScale;
                        else
                        {
                            LCurrentScale = (gridMapPanel.Width / grid.Width);
                            grid.Scale = LCurrentScale;
                        }

                        var maxTranslationX = (gridMapPanel.Width / 2) - ((grid.Width / 2) * grid.Scale);
                        var maxTranslationY = (gridMapPanel.Height / 2) - ((grid.Height / 2) * grid.Scale);

                        if (grid.TranslationX >= maxTranslationX)
                            grid.TranslationX = maxTranslationX;
                        else if (grid.TranslationX <= -maxTranslationX)
                            grid.TranslationX = -maxTranslationX;

                        if (grid.TranslationY >= maxTranslationY)
                            grid.TranslationY = maxTranslationY;
                        else if (grid.TranslationY <= -maxTranslationY)
                            grid.TranslationY = -maxTranslationY;
                        count += 1;
                    }
                    break;
                case GestureStatus.Completed:
                    break;
            }
        }

        private void listAllFolders()
        {
            alignmentWorkspace.listDirectory = nrgFileRead.reader.listFolders(alignmentWorkspace.datapath);

            if (alignmentWorkspace.listDirectory.Count() == 0)
            {
                labelCurProject.Text = "None Found";
            }
            else if (labelCurProject.Text == "None Found" && alignmentWorkspace.listDirectory.Count() > 0)
            {
                labelCurProject.Text = "None Selected";
            }

            pickerFolder.Items.Clear();
            foreach (string item in alignmentWorkspace.listDirectory)
            {
                pickerFolder.Items.Add(Path.GetFileName(item));
            }

            for (int i = 0; i < 4; i++)
            {
                pickerDecimalPlace.Items.Add(i.ToString());
            }

            pickerOffset.Items.Add("-/+");
            pickerOffset.Items.Add("Left/Right");

            pickerInternational.Items.Add("Chainage");
            pickerInternational.Items.Add("Station");
            pickerInternational.Items.Add("Meterage");

            pickerFontSize.Items.Add("Micro");
            pickerFontSize.Items.Add("Small");
            pickerFontSize.Items.Add("Medium");
            pickerFontSize.Items.Add("Large");
            pickerFontSize.Items.Add("Very Large");
            pickerFontSize.Items.Add("Huge");

            pickerMapType.Items.Add("Street Map");
            pickerMapType.Items.Add("Satellite Map");
            pickerMapType.Items.Add("Hybrid Map");

            pickerFormat.Items.Add("10000");
            pickerFormat.Items.Add("10+000");
            pickerFormat.Items.Add("100+00");

            pickerUnits.Items.Add("Metric");
            pickerUnits.Items.Add("Imperial");

            pickerCenter.Items.Add("Centre by Mid-point");
            pickerCenter.Items.Add("Centre by Offset");

            pickerPinSaveOptions.Items.Add(".GPF");
            pickerPinSaveOptions.Items.Add(".KML");
            pickerPinSaveOptions.Items.Add("ASCII");

            pickerWatermarkPosition.Items.Add("Bottom Right");
            pickerWatermarkPosition.Items.Add("Bottom Left");
            pickerWatermarkPosition.Items.Add("Top Right");
            pickerWatermarkPosition.Items.Add("Top Left");

            pickerWatermarkSize.Items.Add("Small");
            pickerWatermarkSize.Items.Add("Medium");
            pickerWatermarkSize.Items.Add("Large");

            PickerScaleFactor.Items.Add("Normal");
            PickerScaleFactor.Items.Add("Reverse");

            pickerDateFormat.Items.Add("DD/MM/YYYY");
            pickerDateFormat.Items.Add("MM/DD/YYYY");
            pickerDateFormat.Items.Add("YYYY/MM/DD");

            pickerTransformUnits.Items.Add("Meters");
            pickerTransformUnits.Items.Add("Feet");
            pickerTransformUnits.Items.Add("US Survey Feet");

            pickerTimeFormat.Items.Add("24 Hour (13:00)");
            pickerTimeFormat.Items.Add("12 Hour (1:00 pm)");
        }

        private void selectFolder()
        {
            alignmentWorkspace.listDirectory = nrgFileRead.reader.listFolders(alignmentWorkspace.datapath);

            if (alignmentWorkspace.listDirectory.Count > 0)
            {
                pickerFolder.Items.Clear();
                for (int i = 0; i < alignmentWorkspace.listDirectory.Count; i++)
                {
                    pickerFolder.Items.Add(Path.GetFileName(alignmentWorkspace.listDirectory[i]));
                }
                pickerFolder.Items.Add("New Project");

                try
                {
                    pickerFolder.Items.Remove("Transformation");
                }
                catch
                {

                }
                pickerFolder.Title = "Select a Project";
            }
            else
                labelCurProject.Text = "None Found";

        }

        private void setupLinesList(List<string> files)
        {
            listLineGrid.Clear();
            for (int i = 0; i < gridLinesList.Children.Count();)
                gridLinesList.Children.RemoveAt(i);

            var gEleGrid  = new Grid
            {
                HorizontalOptions = LayoutOptions.Fill,
                VerticalOptions = LayoutOptions.Fill,
                ClassId = gridLinesList.Children.Count().ToString()
            };

            var gEleLabel = new Label
            {
                Text = "Google Section",
                LineBreakMode = LineBreakMode.WordWrap,
                FontSize = 18,
                HorizontalOptions = LayoutOptions.Center,
                VerticalOptions = LayoutOptions.Center
            };

            var gEleButton = new Button
            {
                Image = "",
                VerticalOptions = LayoutOptions.Center,
                HorizontalOptions = LayoutOptions.Center,
                BackgroundColor = Color.FromHex(CrossSettings.Current.GetValueOrDefault("LineColor" + gEleLabel.Text, "#000000")),
                BorderColor = Color.Black,
                BorderWidth = 1,
                BorderRadius = 1
            };

            gEleGrid.ColumnDefinitions.Clear();
            gEleGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.55, GridUnitType.Star) });
            gEleGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.15, GridUnitType.Star) });
            gEleGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.15, GridUnitType.Star) });
            gEleGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.15, GridUnitType.Star) });
            gEleGrid.RowDefinitions.Clear();
            gEleGrid.RowDefinitions.Add(new RowDefinition { Height = new GridLength(42, GridUnitType.Absolute) });
            gEleGrid.Children.Add(gEleLabel, 0, 0);
            gEleButton.Clicked += (sender, args) =>
            {
                curLineID = Convert.ToInt16(gEleGrid.ClassId);
                setupColorPicker(gEleButton.BackgroundColor);
                colorFrame2.BackgroundColor = gEleButton.BackgroundColor;
                gridSimpleColorPicker.IsVisible = true;
                gridLinesList.IsVisible = false;
            };

            gEleGrid.Children.Add(gEleButton, 3, 0);
            listLineGrid.Add(gEleGrid);
            gridLinesList.Children.Add(gEleGrid);

            for(int i = 0; i<files.Count(); i++)
            {
                if (files[i].ToString().ToUpper().EndsWith(".KML"))
                {
                    var grid = new Grid
                    {
                        HorizontalOptions = LayoutOptions.Fill,
                        VerticalOptions = LayoutOptions.Fill, 
                        ClassId = gridLinesList.Children.Count().ToString()
                    };
                    var label = new Label
                    {
                        Text = Path.GetFileName(files[i]).Replace(".KML", "").Replace(".kml", ""),
                        LineBreakMode = LineBreakMode.WordWrap,
                        FontSize = 18,
                        HorizontalOptions = LayoutOptions.Center,
                        VerticalOptions = LayoutOptions.Center
                    };
                    var button = new Button
                    {
                        Image = "EmptyBox.png",
                        VerticalOptions = LayoutOptions.Center,
                        HorizontalOptions = LayoutOptions.Center,
                        BackgroundColor = Color.Transparent,
                        BorderColor = Color.Black,
                        BorderWidth = 1,
                        BorderRadius = 1
                    };
                    var button2 = new Button
                    {
                        Image = "EmptyBox.png",
                        VerticalOptions = LayoutOptions.Center,
                        HorizontalOptions = LayoutOptions.Center,
                        BackgroundColor = Color.Transparent,
                        BorderColor = Color.Black,
                        BorderWidth = 1,
                        BorderRadius = 1
                    };
                    var button3 = new Button
                    {
                        Image = "",
                        VerticalOptions = LayoutOptions.Center,
                        HorizontalOptions = LayoutOptions.Center,
                        BackgroundColor = Color.FromHex(CrossSettings.Current.GetValueOrDefault("LineColor" + label.Text, "#000000")),
                        BorderColor = Color.Black,
                        BorderWidth = 1,
                        BorderRadius = 1
                    };

                    button3.MinimumWidthRequest = button2.Width;
                    button3.MinimumHeightRequest = button2.Height;
                    gEleButton.MinimumWidthRequest = button2.Width;
                    gEleButton.MinimumHeightRequest = button.Height;

                    grid.ColumnDefinitions.Clear();
                    grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.55, GridUnitType.Star) });
                    grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.15, GridUnitType.Star) });
                    grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.15, GridUnitType.Star) });
                    grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.15, GridUnitType.Star) });
                    grid.RowDefinitions.Clear();
                    grid.RowDefinitions.Add(new RowDefinition { Height = new GridLength(42, GridUnitType.Absolute) });
                    grid.Children.Add(label, 0, 0);
                    button.Clicked += (sender, args) =>
                    {
                            if (button.Image == "EmptyBox.png")
                            {
                                button.Image = "checkMarkSmall.png";
                                if (linesToRemove.Contains(label.Text))
                                    linesToRemove.Remove(label.Text);
                            }
                            else if (button.Image == "checkMarkSmall.png")
                            {
                                button.Image = "EmptyBox.png";
                                if (dictionaryLoadedLines.ContainsKey(label.Text))
                                    linesToRemove.Add(label.Text);
                            }                         
                    };

                    button2.Clicked += (sender, args) =>
                    {
                        if(button2.Image == "EmptyBox.png")
                        {
                            button2.Image = "checkMarkSmall.png";
                            if (crossSectionToRemove.Contains(label.Text))
                                crossSectionToRemove.Remove(label.Text);
                        }
                        else if(button2.Image == "checkMarkSmall.png")
                        {
                            button2.Image = "EmptyBox.png";
                            if(dictionaryLines.ContainsKey(label.Text))
                                crossSectionToRemove.Add(label.Text);
                        }
                    };

                    button3.Clicked += (sender, args) =>
                    {
                        curLineID = Convert.ToInt16(grid.ClassId);
                        setupColorPicker(button3.BackgroundColor);
                        colorFrame2.BackgroundColor = button3.BackgroundColor;
                        gridSimpleColorPicker.IsVisible = true;
                        gridLinesList.IsVisible = false;
                    };

                    grid.Children.Add(button, 1, 0);
                    grid.Children.Add(button2, 2, 0);
                    grid.Children.Add(button3, 3, 0);
                    listLineGrid.Add(grid);
                    gridLinesList.Children.Add(grid);
                }
            }
        }

        private void setupAlignment(List<string> files, bool alignmentExists)
        {
            gridAlignmentSelection.IsVisible = true;
            pickerNst.Items.Clear();

            foreach (string item in files)
            {
                if (item.ToUpper().EndsWith(".NST") || item.ToUpper().EndsWith(".NAF"))
                {
                    pickerNst.Items.Add(Path.GetFileName(item));
                }
            }

            if (pickerNst.Items.Count <= 0)
            {
                for(int i = 0; i<mapObject.Polylines.Count();)
                {
                    if (mapObject.Polylines[i].Tag.ToString() == "Alignment")
                        mapObject.Polylines.Remove(mapObject.Polylines[i]);
                    else
                        i++;
                }
                alignmentWorkspace.Alignments = new alignments.alignment();
                CrossSettings.Current.AddOrUpdateValue("currentAlignment", "None Found");
                alignmentWorkspace.currentAlignment = null;
                updateChainageAndOffset();
                labelCurAlignment.Text = "None Found";
            }
            else
            {
                alignmentWorkspace.Alignments = new alignments.alignment();
                if (pickerNst.Items.Count > 0)
                {
                    if (alignmentExists == true && pickerNst.Items.Contains(alignmentWorkspace.currentAlignment))
                    {
                        labelCurAlignment.Text = alignmentWorkspace.currentAlignment;
                        if (alignmentWorkspace.currentAlignment.ToUpper().EndsWith(".NST"))
                            alignmentWorkspace.Alignments.loadNST(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + alignmentWorkspace.currentAlignment);
                        else if (alignmentWorkspace.currentAlignment.ToUpper().EndsWith(".NAF"))
                            alignmentWorkspace.Alignments.loadFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + alignmentWorkspace.currentAlignment);

                    }
                    else
                    {
                        alignmentWorkspace.currentAlignment = pickerNst.Items[0];
                        CrossSettings.Current.AddOrUpdateValue("currentAlignment", alignmentWorkspace.currentAlignment);
                        labelCurAlignment.Text = pickerNst.Items[0];
                        alignmentWorkspace.Alignments.loadNST(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + pickerNst.Items[0]);
                    }

                    alignmentWorkspace.workspaceBounds = alignmentWorkspace.Alignments.CalcHzExtents();
                    double maxChain = alignmentWorkspace.Alignments.horizontal.elements.OrderBy(x => x.Chainage).Last().Chainage;
                    double minChain = alignmentWorkspace.Alignments.horizontal.elements.OrderBy(x => x.Chainage).First().Chainage;

                    int chainInt = (int)((maxChain - minChain) / 500);

                    if (chainInt < 1)
                        chainInt = 1;

                    alignmentWorkspace.Alignments.calcDrawCoords(alignmentWorkspace.Alignments.coords, chainInt);


                    Polyline polyline = null;
                    polyline = new Polyline();

                    foreach (workspace.dataTypes.nrgVector3 point in alignmentWorkspace.Alignments.coords)
                    {
                        var latLng = GetLLFromEN(point.X, point.Y);
                        polyline.Positions.Add(new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y));
                    }

                    mapObject.Polylines.Clear();
                    polyline.IsClickable = true;
                    polyline.StrokeColor = Color.Blue;
                    polyline.StrokeWidth = 5f;
                    polyline.Tag = "Alignment";
                    mapObject.Polylines.Add(polyline);
                    lastline = mapObject.Polylines.Count();

                    updateChainageAndOffset();
                }
            }

        }

        private async void setupPins(List<string> files, bool pinsExist)
        {
            gridPinsSelection.IsVisible = true;
            pickerPins.Items.Clear();

            foreach (string item in files)
            {
                if (item.ToUpper().EndsWith(".GPF") || item.ToUpper().EndsWith(".KML") || item.ToUpper().EndsWith(".TXT"))
                    pickerPins.Items.Add(Path.GetFileName(item));
                   
            }

            if (pickerPins.Items.Count <= 0)
            {
                mapObject.Pins.Clear();
                currentPinId = 0;
                CrossSettings.Current.AddOrUpdateValue("currentPins", "None Found");
                alignmentWorkspace.currentPins = null;
                labelCurPins.Text = "None Found";
            }
            else
            {
                labelCurPins.Text = "None Selected";
                if (pinsExist && pickerPins.Items.Contains(alignmentWorkspace.currentPins))
                {
                    labelCurPins.Text = alignmentWorkspace.currentPins;
                    if (alignmentWorkspace.currentPins.ToUpper().EndsWith(".GPF"))
                    {
                        List<string> gpfString = alignmentWorkspace.Alignments.loadGPF(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + alignmentWorkspace.currentPins);

                        mapObject.Pins.Clear();
                        currentPinId = 0;

                        for (int i = 0; i < gpfString.Count; i++)
                        {
                            string[] tmpValues = gpfString[i].Split(',');
                            if (tmpValues.Count() >= 5)
                            {
                                string tag = "";
                                for (int j = 4; j < tmpValues.Count(); j++)
                                {
                                    tag += "," + tmpValues[j];
                                }
                                addPin(tmpValues[0], tmpValues[1], tmpValues[2], tmpValues[3], tag);
                                currentTagAmount = (tmpValues.Count() - 5);
                            }
                            else if (tmpValues.Count() == 4)
                            {
                                addPin(tmpValues[0], "", tmpValues[1], tmpValues[2], ("," + tmpValues[3]));
                            }
                        }
                    }
                    else if (alignmentWorkspace.currentPins.ToUpper().EndsWith(".KML"))
                    {
                        mapObject.Pins.Clear();
                        currentPinId = 0;
                        await loadKmlPins(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + alignmentWorkspace.currentPins);
                    }
                    else if (alignmentWorkspace.currentPins.ToUpper().EndsWith(".TXT"))
                    {
                        mapObject.Pins.Clear();
                        currentPinId = 0;
                        loadAsciiPins(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + alignmentWorkspace.currentPins);
                    }
                    
                }
            }
        }

        private async void setupLines(List<string> files, bool linesExist)
        {
            gridLinesSelection.IsVisible = true;
            List<string> listCutLines = new List<string>();
            foreach (string item in files)
            {
                if (item.ToUpper().EndsWith(".KML"))
                {
                    listCutLines.Add(Path.GetFileName(item));
                }
            }

            if (listCutLines.Count <= 0)
            {
                labelCurLines.Text = "None Found";
                for (int i = 0; i < mapObject.Polylines.Count();)
                {
                    if (mapObject.Polylines[i].Tag != null)
                        i++;
                    else
                        mapObject.Polylines.Remove(mapObject.Polylines[i]);
                }
                CrossSettings.Current.AddOrUpdateValue("currentLines", "None Found");
                CrossSettings.Current.AddOrUpdateValue("mapLineString", "none");
                CrossSettings.Current.AddOrUpdateValue("crossLineString", "none");
                mapLineList.Clear();
                crossSectionLineList.Clear();
                linesToRemove.Clear();
                crossSectionToRemove.Clear();
                dictionaryIntersectPoints.Clear();
                dictionaryLines.Clear();
                dictionaryLoadedLines.Clear();
                alignmentWorkspace.currentLines = null;
            }
            else
            {
                labelCurLines.Text = "None Selected";
                if (linesExist == true)
                {
                    if (alignmentWorkspace.currentLines.Contains(',') == true)
                    {
                        string[] lines = alignmentWorkspace.currentLines.Split(',');
                        labelCurLines.Text = "Multiple Selected";
                        foreach (string item in lines)
                        {
                            if (listCutLines.Contains(item) == true)
                                await loadKmlFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + item);
                        }
                    }
                    else
                    {
                        labelCurLines.Text = alignmentWorkspace.currentLines;
                        await loadKmlFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + alignmentWorkspace.currentLines);
                    }
                }
            }
        }

        private void setupTransformation(bool transformationExists)
        {
            pickerTransform.Items.Clear();

            if(Device.RuntimePlatform == Device.Android && nrgFileRead.reader.listFolders(alignmentWorkspace.datapath).Contains(alignmentWorkspace.datapath + "Transformation") == false)
            {
                gridTransformationSelection.IsVisible = true;
                pickerTransform.Items.Add("Default (OSGB36)");
                if (alignmentWorkspace.currentTransformation == "More...")
                {
                    pickerTransform.Items.Add(App.transformationString);
                }
                pickerTransform.Items.Add("More...");
                labelCurTransformation.Text = "Default (OSGB36)";
            }
            else
            {

                gridTransformationSelection.IsVisible = true;
                var listFiles = new List<string>();
                if (Device.RuntimePlatform == Device.Android)
                    listFiles = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + "Transformation" + "/");
                else if (Device.RuntimePlatform == Device.iOS)
                    listFiles = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath);

                pickerTransform.Items.Add("Default (OSGB36)");

                if (listFiles.Count <= 0)
                {
                    labelCurTransformation.Text = "Default (OSGB36)";
                    NewTransformation = false;

                }
                else
                {
                    foreach (string item in listFiles)
                    {
                        if (item.ToUpper().EndsWith(".DAT"))
                            pickerTransform.Items.Add(Path.GetFileName(item));
                    }
                    labelCurTransformation.Text = "Default (OSGB36)";
                }
                if (alignmentWorkspace.currentTransformation == "More...")
                {
                    pickerTransform.Items.Add(App.transformationString);
                    NewTransformation = true;
                }
                else
                    NewTransformation = false;
                pickerTransform.Items.Add("More...");

                if(transformationExists == true && alignmentWorkspace.currentTransformation == "More...")
                {
                    labelCurTransformation.Text = App.transformationString;
                    LoadNewTransformation();
                    updateToNewTransformation();
                }
                else if (transformationExists == true && pickerTransform.Items.Contains(alignmentWorkspace.currentTransformation))
                {
                    labelCurTransformation.Text = alignmentWorkspace.currentTransformation;
                    if (Device.RuntimePlatform == Device.Android)
                        alignmentWorkspace.transformation = alignmentWorkspace.Alignments.ReadTransformation(alignmentWorkspace.datapath + "Transformation" + "/" + alignmentWorkspace.currentTransformation);
                    else if (Device.RuntimePlatform == Device.iOS)
                        alignmentWorkspace.transformation = alignmentWorkspace.Alignments.ReadTransformation(alignmentWorkspace.datapath + alignmentWorkspace.currentTransformation);

                    updateToNewTransformation();
                }
            }
        }

        private void setupImport(List<string> files)
        {
            pickerImport.Items.Clear();
            foreach (string item in files)
            {
                if (item.ToUpper().EndsWith(".XML"))
                {
                    pickerImport.Items.Add(Path.GetFileName(item));
                }
            }

            if (pickerImport.Items.Count() >= 1)
            {
                labelImportExists.Text = "Some Found";
            }

            else
                labelImportExists.Text = "None Found";
        }

        private async void UpdatePosition(object sender, PositionEventArgs e)
        {
            if (alignmentWorkspace.scrollMap == true && alignmentWorkspace.requestOnly == false)
            {
                var devicePosition = new Xamarin.Forms.GoogleMaps.Position(e.Position.Latitude, e.Position.Longitude);
                //var t = await alignmentWorkspace.locator.GetPositionAsync();

                lat = devicePosition.Latitude;
                lng = devicePosition.Longitude;
                var estNrth = GetENFromLL(lat, lng);
                App.lat = lat;
                App.lng = lng;
                App.Accuracy = e.Position.Accuracy;


                double acc = 0;
                string format = "";

                if(labelCurScaleUnits.Text == "Metric")
                {
                    acc = e.Position.Accuracy;
                    format = "m";
                }
                else if(labelCurScaleUnits.Text == "Imperial")
                {
                    acc = e.Position.Accuracy * 3.28084;
                    format = "ft";
                }

                if(acc <= App.AccuracyThreshold)
                {
                    //Its good
                    spanAccQuality.Text = "Good ";
                    spanAccQuality.ForegroundColor = Color.Green;
                    spanAccValue.Text = getDecimalDisplay(acc) + format;

                    spanLandAccQuality.Text = "Good ";
                    spanLandAccQuality.ForegroundColor = Color.Green;
                    spanLandAccValue.Text = getDecimalDisplay(acc) + format;
                }
                else
                {
                    //Its poor
                    spanAccQuality.Text = "Poor ";
                    spanAccQuality.ForegroundColor = Color.Red;
                    spanAccValue.Text = getDecimalDisplay(acc) + format;

                    spanLandAccQuality.Text = "Poor ";
                    spanLandAccQuality.ForegroundColor = Color.Red;
                    spanLandAccValue.Text = getDecimalDisplay(acc) + format;
                }

                labelEast.Text = getDecimalDisplay(estNrth.X);
                labelNorth.Text = getDecimalDisplay(estNrth.Y);

                labelLandEast.Text = labelEast.Text;
                labelLandNorth.Text = labelNorth.Text;

                labelLat.Text = lat.ToString("0.000000");
                labelLong.Text = lng.ToString("0.000000");
                labelLandLat.Text = labelLat.Text;
                LabelLandLng.Text = labelLong.Text;
                Xamarin.Forms.GoogleMaps.Position psn = new Xamarin.Forms.GoogleMaps.Position(lat, lng);

                if (mapObject.VisibleRegion != null)
                {
                    if (Device.RuntimePlatform == Device.iOS)
                        mapObject.MoveCamera(CameraUpdateFactory.NewPosition(psn));
                    else if (Device.RuntimePlatform == Device.Android)
                        mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(psn, mapObject.VisibleRegion.Radius));
                        //mapObject.MoveCamera(CameraUpdateFactory.NewPosition(psn));

                    alignmentWorkspace.devicePosition = psn;
                    mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPosition(alignmentWorkspace.devicePosition);
                    getRadius();
                }

                if (gridCrossection.IsVisible == true && canDraw == true)
                {
                    try
                    {
                        createIntersection();
                    }
                    catch
                    {
                    }
                }
                updateChainageAndOffset();

                if (firstRun)
                {
                    labelEast.Text = getDecimalDisplay(estNrth.X);
                    labelNorth.Text = getDecimalDisplay(estNrth.Y);
                    labelLandEast.Text = labelEast.Text;
                    labelLandNorth.Text = labelNorth.Text;
                    labelLat.Text = lat.ToString("0.000000");
                    labelLong.Text = lng.ToString("0.000000");
                    labelLandLat.Text = labelLat.Text;
                    LabelLandLng.Text = labelLong.Text;
                    if (alignmentWorkspace.scrollMap == true)
                    {
                        Xamarin.Forms.GoogleMaps.Position pos = new Xamarin.Forms.GoogleMaps.Position(lat, lng);
                        //mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(pos, Distance.FromMeters(200)));
                        mapObject.MoveCamera(CameraUpdateFactory.NewPosition(pos));
                        alignmentWorkspace.devicePosition = pos;
                        mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPositionZoom(alignmentWorkspace.devicePosition, 15D);
                    }
                    firstRun = false;
                }
            }
        }

        private async void updateToNewTransformation()
        {         
            if(labelLat.Text.ToString() != "Waiting for GPS")
            {
                double lat;
                double lng;

                double.TryParse(labelLat.Text.ToString(), out lat);
                double.TryParse(labelLong.Text.ToString(), out lng);

                var estNrth = GetENFromLL(lat, lng);

                var estNrthSearch = GetENFromLL(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);

                labelLandEast.Text = getDecimalDisplay(estNrth.X);
                labelLandNorth.Text = getDecimalDisplay(estNrth.Y);
                labelEast.Text = getDecimalDisplay(estNrth.X);
                labelNorth.Text = getDecimalDisplay(estNrth.Y);

                if (mapObject.SelectedPin != null && (gridENPanel.IsVisible == true || gridLandscapeEN.IsVisible == true))
                    await changePinAddres("Easting Northing");

                updateChainageAndOffset();
            }        
        }

        private void addPin(string name, string note, string x, string y, string tag)
        {

            double est; double nrth;
            double.TryParse(x, out est);
            double.TryParse(y, out nrth);
            string label = "";

            var latLng = GetLLFromEN(est, nrth);

            if (name == "" && note == "")
                label = "\"\",\"\"";
            else if (name == "")
                label = "\"\"" + "," + note;
            else if (note == "")
                label = name + "," + "\"\"";
            else
                label = name + "," + note;

            string address = latLng.X.ToString("0.000000") + "," + latLng.Y.ToString("0.000000");

            var newPin = new Pin()
            {
                Type = PinType.Generic,
                Label = label,
                Address = address,
                Position = new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y),
                Tag = tag,

            };

            mapObject.Pins.Add(newPin);

            currentPinId++;
        }

        private void createInternationalPin(string name, string note, string x, string y, string tag)
        {
            if (alignmentWorkspace.currentAlignment == null || alignmentWorkspace.currentAlignment == "None Selected" || alignmentWorkspace.currentAlignment == "None Found") 
                DisplayAlert("Error", "There are no alignment strings loaded", "Ok");
            else
            {
                double ch; double off;

                double.TryParse(x, out ch);
                double.TryParse(y, out off);
                var estNrth = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(ch, off);

                name = name.Trim(',');
                note = note.Trim(',');
                string pinTag = "";
                if (tag == "")
                    pinTag = ",-999";
                string label = "";
                if (name == "" && note == "")
                    label = "\"\",\"\"";
                else if (name == "")
                    label = "\"\"" + "," + note;
                else if (note == "")
                    label = name + "," + "\"\"";
                else
                    label = name + "," + note;


                var latLng = GetLLFromEN(estNrth.X, estNrth.Y);
                string address = latLng.X.ToString("0.000000") + "," + latLng.Y.ToString("0.000000");
                Xamarin.Forms.GoogleMaps.Position pos = new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y);

                var newPin = new Pin()
                {
                    Type = PinType.Generic,
                    Label = label,
                    Address = address,
                    Position = pos,
                    Tag = pinTag
                };

                mapObject.Pins.Add(newPin);
                currentPinId++;
            } 
        }

        private void createEastingNorthingPin(string name, string note, string x, string y, string tag)
        {
            double est; double nrth;
            double.TryParse(x, out est);
            double.TryParse(y, out nrth);

            name = name.Trim(',');
            note = note.Trim(',');
            var latLng = GetLLFromEN(est, nrth);

            string pinTag = "";
            if (tag == "")
                pinTag = ",-999";
            string label = "";
            if (name == "" && note == "")
                label = "\"\",\"\"";
            else if (name == "")
                label = "\"\"" + "," + note;
            else if (note == "")
                label = name + "," + "\"\"";
            else
                label = name + "," + note;

            Xamarin.Forms.GoogleMaps.Position pos = new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y);
            string address = latLng.X.ToString("0.000000") + "," + latLng.Y.ToString("0.000000");

            var newPin = new Pin()
            {
                Type = PinType.Generic,
                Label = label,
                Address = address,
                Position = pos,
                Tag = pinTag
            };

            mapObject.Pins.Add(newPin);
            currentPinId++;
        }

        private void createLatitudeLongitudePin(string name, string note, string x, string y, string tag)
        {
            double lat; double lng;
            double.TryParse(x, out lat);
            double.TryParse(y, out lng);

            string pinTag = "";
            if (tag == "")
                pinTag = ",-999";
            string label = "";
            name = name.Trim(',');
            note = note.Trim(',');

            if (name == "" && note == "")
                label = "\"\",\"\"";
            else if (name == "")
                label = "\"\"" + "," + note;
            else if (note == "")
                label = name + "," + "\"\"";
            else
                label = name + "," + note;

            Xamarin.Forms.GoogleMaps.Position pos = new Xamarin.Forms.GoogleMaps.Position(lat, lng);
            string address = lat.ToString("0.000000") + "," + lng.ToString("0.000000");
            var newPin = new Pin()
            {
                Type = PinType.Generic,
                Label = label,
                Address = address,
                Position = pos,
                Tag = pinTag
            };

            mapObject.Pins.Add(newPin);
            currentPinId++;
        }

        void updateChainageAndOffset()
        {
            if(alignmentWorkspace.scrollMap == true)
            {
                double lat;
                double lng;

                double.TryParse(labelLat.Text, out lat);
                double.TryParse(labelLong.Text, out lng);

                var estNrth = GetENFromLL(lat, lng);

                var inVec = new workspace.dataTypes.nrgVector2 { X = estNrth.X, Y = estNrth.Y };
                var outVec = new workspace.dataTypes.nrgAlignmentVector5();
                alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(inVec, ref outVec);
                if (alignmentWorkspace.currentAlignment == null || alignmentWorkspace.currentAlignment == "None Selected" || alignmentWorkspace.currentAlignment == "None Found")
                {
                    labelCh.Text = "????";
                    labelOff.Text = "????";
                }
                else if (outVec.chainage == 0 && outVec.offset == 0)
                {
                    try
                    {
                        if (alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(inVec, ref outVec) == false)
                        {
                            labelCh.Text = "????";
                            labelOff.Text = "????";
                        }
                        else
                        {
                            if (outVec.chainage >= 1000 && alignmentWorkspace.chainageFormat == true)
                            {
                                string ch = getDecimalDisplay(outVec.chainage);
                                if (alignmentWorkspace.chainageFormatDesc == "Format1")
                                    labelCh.Text = setChainage(ch);
                                else
                                    labelCh.Text = setChainageFormat2(ch);
                            }
                            else
                                labelCh.Text = getDecimalDisplay(outVec.chainage);
                            if (alignmentWorkspace.offsetDisplay == "-/+")
                                labelOff.Text = getDecimalDisplay(outVec.offset);
                            else if (outVec.offset < 0)
                                labelOff.Text = getDecimalDisplay(Math.Abs(outVec.offset)) + " Left";
                            else
                                labelOff.Text = getDecimalDisplay(Math.Abs(outVec.offset)) + " Right";
                        }
                    }
                    catch
                    {
                        labelCh.Text = "????";
                        labelOff.Text = "????";
                    }


                }
                else
                {
                    if (alignmentWorkspace.chainageFormat == true)
                    {
                        string ch = getDecimalDisplay(outVec.chainage);
                        if (alignmentWorkspace.chainageFormatDesc == "Format1")
                            labelCh.Text = setChainage(ch);
                        else
                            labelCh.Text = setChainageFormat2(ch);
                    }
                    else
                        labelCh.Text = getDecimalDisplay(outVec.chainage);
                    if (alignmentWorkspace.offsetDisplay == "-/+")
                        labelOff.Text = getDecimalDisplay(outVec.offset);
                    else if (outVec.offset < 0)
                        labelOff.Text = getDecimalDisplay(Math.Abs(outVec.offset)) + " Left";
                    else
                        labelOff.Text = getDecimalDisplay(Math.Abs(outVec.offset)) + " Right";
                }

                labelLandCh.Text = labelCh.Text;
                labelLandOff.Text = labelOff.Text;
                labelLandChStart.Text = labelChStart.Text;
                labelLandOffStart.Text = labelOffStart.Text;
            }          
        }
        
        string setChainageFormat2(string ch)
        {
            string chainage = "";
            if(ch.Contains("."))
            {
                string[] temp = ch.Split('.');
                if(temp[0].Count() > 2)
                {
                    string m = "" + temp[0].ElementAt(temp[0].Count() - 2) + temp[0].ElementAt(temp[0].Count() - 1);
                    string km = "";
                    for (int i = 0; i < temp[0].Count() - 2; i++)
                        km += temp[0].ElementAt(i);

                    chainage = km + "+" + m + "." + temp[1];
                }
                else if(temp[0].Count() <= 2)
                {
                    string m = "" + temp[0];
                    string km = "0";

                    chainage = km + "+" + m + "." + temp[1];
                }                
            }
            else if(ch.Count() > 2)
            {
                string m = "" + ch.ElementAt(ch.Count() - 2) + ch.ElementAt(ch.Count() - 1);
                string km = "";
                for (int i = 0; i < ch.Count() - 2; i++)
                    km += ch.ElementAt(i);

                chainage = km + "+" + m;
            }
            else if(ch.Count() <= 2)
            {
                string m = "" + ch;
                string km = "0";

                chainage = km + "+" + m;
            }
            return chainage;
        }

        string setChainage(string ch)
        {
            string chainage = "";

            if(ch.Contains("."))
            {
                string[] temp = ch.Split('.');
                if(temp[0].Count() > 3)
                {
                    string m = "" + temp[0].ElementAt(temp[0].Count() - 3) + temp[0].ElementAt(temp[0].Count() - 2) + temp[0].ElementAt(temp[0].Count() - 1);
                    string km = "";
                    for (int i = 0; i < temp[0].Count() - 3; i++)
                        km += temp[0].ElementAt(i);

                    chainage = km + "+" + m + "." + temp[1];
                }
                else if(temp[0].Count() <= 3)
                {
                    string m = "" + temp[0];
                    string km = "0";

                    chainage = km + "+" + m + "." + temp[1];
                }
            }
            else if (ch.Count() > 3)
            {
                string m = "" + ch.ElementAt(ch.Count() - 3) + ch.ElementAt(ch.Count() - 2) + ch.ElementAt(ch.Count() - 1);
                string km = "";
                for (int i = 0; i < ch.Count() - 3; i++)
                    km += ch.ElementAt(i);

                chainage = km + "+" + m;
            }
            else if(ch.Count() <= 3)
            {
                string m = "" + ch;
                string km = "0";

                chainage = km + "+" + m;
            }
            return chainage;
        }

        void changeFontSize(int largeSize, int mediumSize, int smallSize, int microSize)
        {
            if(Device.RuntimePlatform == Device.iOS && Device.Idiom == TargetIdiom.Tablet)
            {
                largeSize = largeSize * 3;
                mediumSize = mediumSize * 3;
                smallSize = smallSize * 3;
                microSize = microSize * 3;
            }

            labelCh.FontSize = largeSize;
            labelOff.FontSize = largeSize;
            labelEast.FontSize = largeSize;
            labelNorth.FontSize = largeSize;
            labelLat.FontSize = largeSize;
            labelLong.FontSize = largeSize;
            labelPcode.FontSize = largeSize;
            entrySearchX.FontSize = largeSize;
            entrySearchY.FontSize = largeSize;

            titleProject.FontSize = mediumSize;
            labelCurProject.FontSize = mediumSize;
            titleAlignment.FontSize = mediumSize;
            labelCurAlignment.FontSize = mediumSize;
            titlePins.FontSize = mediumSize;
            labelCurPins.FontSize = mediumSize;
            titleLines.FontSize = mediumSize;
            labelCurLines.FontSize = mediumSize;
            titleTransformation.FontSize = mediumSize;
            labelCurTransformation.FontSize = mediumSize;
            titleOffset.FontSize = mediumSize;
            labelOffsetDisplay.FontSize = mediumSize;
            titleInternational.FontSize = mediumSize;
            labelInternational.FontSize = mediumSize;
            titleDecimal.FontSize = mediumSize;
            labelDecimalPlaces.FontSize = mediumSize;
            titleText.FontSize = mediumSize;
            labelFontSize.FontSize = mediumSize;
            titleMapType.FontSize = mediumSize;
            labelMapType.FontSize = mediumSize;
            labelPostcodeStart.FontSize = mediumSize;
            labelCrossHair.FontSize = mediumSize;
            titleCrossHair.FontSize = mediumSize;
            labelCrossSection.FontSize = mediumSize;
            titleCrossSection.FontSize = mediumSize;
            labelImportExists.FontSize = mediumSize;
            titleImport.FontSize = mediumSize;
            titleDistance.FontSize = mediumSize;
            titleFormat.FontSize = mediumSize;
            entryDistance.FontSize = mediumSize;
            labelCurFormat.FontSize = mediumSize;
            titleVertExag.FontSize = mediumSize;
            entryVertExag.FontSize = mediumSize;
            titleScale.FontSize = mediumSize;
            labelCurScale.FontSize = mediumSize;
            labelCurMapStatus.FontSize = mediumSize;
            titleMapStatus.FontSize = mediumSize;
            titleScaleBar.FontSize = mediumSize;
            labelCurScaleBar.FontSize = mediumSize;
            titleElevaton.FontSize = mediumSize;
            entryElevation.FontSize = mediumSize;
            titleOffsetCenter.FontSize = mediumSize;
            entryOffsetCenter.FontSize = mediumSize;
            entryCountryCode.FontSize = mediumSize;
            titleCountryCode.FontSize = mediumSize;
            titleScaleUnits.FontSize = mediumSize;
            labelCurScaleUnits.FontSize = mediumSize;
            labelCurCrossSectionCenter.FontSize = mediumSize;
            titleCrossSectionCenter.FontSize = mediumSize;
            titleMapTraffic.FontSize = mediumSize;
            labelMapTraffic.FontSize = mediumSize;
            buttonCreatePin.FontSize = mediumSize;
            buttonCancelPin.FontSize = mediumSize;
            titleUpdateDistance.FontSize = mediumSize;
            titleUpdateTime.FontSize = mediumSize;
            titleRequestOnly.FontSize = mediumSize;
            entryUpdateTime.FontSize = mediumSize;
            entryUpdateDistance.FontSize = mediumSize;
            labelRequestOnly.FontSize = mediumSize;
            titleWatermarkPosition.FontSize = mediumSize;
            titleWatermarkSize.FontSize = mediumSize;
            labelCurWatermarkPosition.FontSize = mediumSize;
            labelCurWatermarkSize.FontSize = mediumSize;
            titleScaleFactor.FontSize = mediumSize;
            entryScaleFactor.FontSize = mediumSize;
            titleShiftE.FontSize = mediumSize;
            titleShiftN.FontSize = mediumSize;
            entryShiftE.FontSize = mediumSize;
            entryShiftN.FontSize = mediumSize;
            titleUseGPS.FontSize = mediumSize;
            labelUseGPS.FontSize = mediumSize;
            titleShowAcc.FontSize = mediumSize;
            labelShowAcc.FontSize = mediumSize;
            titleShowDate.FontSize = mediumSize;
            labelShowDate.FontSize = mediumSize;
            titleDateFormat.FontSize = mediumSize;
            labelDateFormat.FontSize = mediumSize;
            titleShowStringName.FontSize = mediumSize;
            labelShowStringName.FontSize = mediumSize;
            titleTransformUnits.FontSize = mediumSize;
            labelTransformUnits.FontSize = mediumSize;
            titleScaleMethod.FontSize = mediumSize;
            labelScaleMethod.FontSize = mediumSize;
            titleAccuracyThreshold.FontSize = mediumSize;
            entryAccuracyThreshold.FontSize = mediumSize;
            titleTimeFormat.FontSize = mediumSize;
            labelTimeFormat.FontSize = mediumSize;

            dividerPhoto.FontSize = smallSize;
            dividerFile.FontSize = smallSize;
            dividerSettings.FontSize = smallSize;
            dividerMap.FontSize = smallSize;
            dividerCrossSection.FontSize = smallSize;
            dividerManual.FontSize = smallSize;
            labelChStart.FontSize = smallSize;
            labelOffStart.FontSize = smallSize;
            labelEastStart.FontSize = smallSize;
            labelNorthStart.FontSize = smallSize;
            labelLatStart.FontSize = smallSize;
            labelLongStart.FontSize = smallSize;
            buttonClearPins.FontSize = smallSize;
            labelCurPinType.FontSize = smallSize;
            entryPinID.FontSize = smallSize;
            entryPinNote.FontSize = smallSize;
            entryPinX.FontSize = smallSize;
            entryPinY.FontSize = smallSize;
            buttonSearchFind.FontSize = smallSize;
            dividerLocation.FontSize = smallSize;
            labelSearchX.FontSize = smallSize;
            labelSearchY.FontSize = smallSize;

            labelAccuracyMain.FontSize = microSize -2;
            labelLandAccuracy.FontSize = microSize - 2;
            spanAccStart.FontSize = microSize - 2;
            spanAccQuality.FontSize = microSize - 2;
            spanAccValue.FontSize = microSize - 2;
            spanLandAccStart.FontSize = microSize -2;
            spanLandAccQuality.FontSize = microSize - 2;
            spanLandAccValue.FontSize = microSize - 2;

            titleClearPins.FontSize = microSize;
            titleClearLines.FontSize = microSize;
            titleEditPin.FontSize = microSize;
            titlePlacePin.FontSize = microSize;
            titleSavePin.FontSize = microSize;
            titledeletePin.FontSize = microSize;
            titleOpenMaps.FontSize = microSize;
            titlePinID.FontSize = microSize;
            titleCurPinType.FontSize = microSize;
            titlePinNote.FontSize = microSize;
            titlePinX.FontSize = microSize;
            titlePinY.FontSize = microSize;
            titleRefreshAlignment.FontSize = microSize;

            ///Maybe need adjusting for ipads
            labelLandCh.FontSize = 50;
            labelLandOff.FontSize = 50;
            labelLandChStart.FontSize = 42;
            labelLandOffStart.FontSize = 42;
            labelLandEast.FontSize = 50;
            labelLandNorth.FontSize = 50;
            labelLandEastStart.FontSize = 42;
            labelLandNorthStart.FontSize = 42;
            labelLandLat.FontSize = 50;
            LabelLandLng.FontSize = 50;
            labelLandLatStart.FontSize = 42;
            labelLandLngStart.FontSize = 42;
            labelLandPcode.FontSize = 50;
            labelLandPcodeStart.FontSize = 50;
        }

        void getRadius()
        {
            try
            {
                if (alignmentWorkspace.scaleBar == true && mapObject.IsVisible == true)
                {
                    frameScale.IsVisible = true;
                    labelScale.IsVisible = true;
                    frameScaleLeft.IsVisible = true;
                    frameScaleRight.IsVisible = true;
                    frameScaleCenter.IsVisible = true;
                    frameScale.WidthRequest = alignmentWorkspace.widthScale;
                    double mapRadius = mapObject.VisibleRegion.Radius.Meters;
                    double lattest = mapObject.VisibleRegion.LatitudeDegrees;
                    double lngtest = mapObject.VisibleRegion.LongitudeDegrees;
                    double unRoundedDistance;
                    double roundeDistance;
                    double screenWidth = App.screenWidth;

                    var r = Math.Sqrt((mapObject.Height * mapObject.Height) + (mapObject.Width * mapObject.Width)) / 2;



                    if (labelCurScaleUnits.Text == "Metric")
                    {

                        var dist = mapObject.VisibleRegion.Radius.Meters;

                        unRoundedDistance = (dist / r) * 100;

                        if (unRoundedDistance >= 1000000)
                            roundeDistance = Math.Round(unRoundedDistance / 1000000.0, 0) * 1000000;
                        else if (unRoundedDistance >= 100000)
                        {
                            roundeDistance = Math.Round(unRoundedDistance / 100000.0, 0) * 100000;
                        }
                        else if (unRoundedDistance >= 10000)
                        {
                            roundeDistance = Math.Round(unRoundedDistance / 10000.0, 0) * 10000;
                        }
                        else if (unRoundedDistance >= 1000)
                        {
                            roundeDistance = Math.Round(unRoundedDistance / 1000.0, 0) * 1000;
                        }
                        else if (unRoundedDistance >= 100)
                        {
                            roundeDistance = Math.Round(unRoundedDistance / 100.0, 0) * 100;
                        }
                        else if (unRoundedDistance >= 10)
                        {
                            roundeDistance = Math.Round(unRoundedDistance / 10.0, 0) * 10;
                        }
                        else
                        {
                            roundeDistance = Math.Round(unRoundedDistance / 1.0, 0) * 1;
                        }

                        double scaleDiff = roundeDistance / unRoundedDistance;

                        frameScale.WidthRequest = alignmentWorkspace.widthScale * scaleDiff;

                        if (roundeDistance >= 1000)
                            labelScale.Text = (roundeDistance / 1000.0).ToString() + "km";
                        else
                            labelScale.Text = roundeDistance.ToString() + "m";
                    }
                    else if (labelCurScaleUnits.Text == "Imperial")
                    {
                        var dist = (mapObject.VisibleRegion.Radius.Miles / r) * 100;

                        if (dist < 1)
                        {
                            unRoundedDistance = dist * 5280;
                            if (unRoundedDistance >= 5000)
                                roundeDistance = Math.Round(unRoundedDistance / 5000.0, 0) * 5000;
                            else if (unRoundedDistance >= 1000)
                                roundeDistance = Math.Round(unRoundedDistance / 1000.0, 0) * 1000;
                            else if (unRoundedDistance >= 100)
                                roundeDistance = Math.Round(unRoundedDistance / 100.0, 0) * 100;
                            else if (unRoundedDistance >= 10)
                                roundeDistance = Math.Round(unRoundedDistance / 10.0, 0) * 10;
                            else
                                roundeDistance = Math.Round(unRoundedDistance / 1.0, 0) * 1;

                            double scaleDiff = roundeDistance / unRoundedDistance;
                            frameScale.WidthRequest = alignmentWorkspace.widthScale * scaleDiff;
                            labelScale.Text = (roundeDistance.ToString() + "ft");
                        }
                        else if (dist >= 1)
                        {
                            unRoundedDistance = dist;
                            if (unRoundedDistance >= 1000)
                                roundeDistance = Math.Round(unRoundedDistance / 1000.0, 0) * 1000;
                            else if (unRoundedDistance >= 100)
                                roundeDistance = Math.Round(unRoundedDistance / 100.0, 0) * 100;
                            else if (unRoundedDistance >= 10)
                                roundeDistance = Math.Round(unRoundedDistance / 10.0, 0) * 10;
                            else
                                roundeDistance = Math.Round(unRoundedDistance / 1.0, 0) * 1;

                            double scaleDiff = roundeDistance / unRoundedDistance;
                            frameScale.WidthRequest = alignmentWorkspace.widthScale * scaleDiff;
                            labelScale.Text = (roundeDistance.ToString() + "mi");
                        }

                        unRoundedDistance = (dist / r) * 100;
                    }
                }
                else
                {
                    frameScale.IsVisible = false;
                    labelScale.IsVisible = false;
                    frameScaleLeft.IsVisible = false;
                    frameScaleRight.IsVisible = false;
                    frameScaleCenter.IsVisible = false;
                }
            }
            catch
            {

            }     
        }

        void loadCurProject()
        {
            gridLoadProject.IsVisible = false;
            if (nrgFileRead.reader.fileExists(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + "TransformationSettings.DAT"))
            {
                //Read the file and set the transformation from it
                ReadTransformation(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + "TransformationSettings.DAT");
            }

            setupImport(nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder));
            List<string> files = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");
            labelCurProject.Text = alignmentWorkspace.currentFolder;
            alignmentWorkspace.currentAlignment = CrossSettings.Current.GetValueOrDefault("currentAlignment", "None Found");
            if (alignmentWorkspace.currentAlignment == "None Found")
                setupAlignment(files, false);
            else
            {
                try
                {
                    setupAlignment(files, true);
                }
                catch(Exception e)
                {
                    Device.BeginInvokeOnMainThread(() =>
                    {
                        DisplayAlert("Error", "Failed to load alignment: " + alignmentWorkspace.currentAlignment, "Ok");
                    });

                    alignmentWorkspace.currentAlignment = "None Found";
                    CrossSettings.Current.AddOrUpdateValue("currentAlignment", "None Found");
                }
            }

            alignmentWorkspace.currentPins = CrossSettings.Current.GetValueOrDefault("currentPins", "None Found");
            if (alignmentWorkspace.currentPins == "None Found")
                setupPins(files, false);
            else
            {
                try
                {
                    setupPins(files, true);
                }
                catch
                {
                    DisplayAlert("Error", "Failed to load pin file: " + alignmentWorkspace.currentPins, "Ok");
                    alignmentWorkspace.currentPins = "None Found";
                    CrossSettings.Current.AddOrUpdateValue("currentPins", "None Found");
                }
            }

            alignmentWorkspace.currentLines = CrossSettings.Current.GetValueOrDefault("currentLines", "None Found");
            if (alignmentWorkspace.currentLines == "None Found")
            {
                setupLines(files, false);
                setupLinesList(files);
            }
            else
            {
                try
                {
                    setupLines(files, true);
                }
                catch
                {
                    DisplayAlert("Error", "Failed to load line files", "Ok");
                    alignmentWorkspace.currentLines = "None Found";
                    CrossSettings.Current.AddOrUpdateValue("currentLines", "None Found");
                }
                setupLines(files, true);
                setupLinesList(files);
            }

            string mapSave = CrossSettings.Current.GetValueOrDefault("mapLineString", "none");
            string crossSave = CrossSettings.Current.GetValueOrDefault("crossLineString", "none");

            if (mapSave != "none")
            {
                string[] temp = mapSave.Split(',');
                for (int i = 0; i < temp.Count() - 1; i++)
                    mapLineList.Add(temp[i]);
                initialMapLines();
            }
            if (crossSave != "none")
            {
                string[] temp = crossSave.Split(',');
                for (int i = 0; i < temp.Count() - 1; i++)
                    crossSectionLineList.Add(temp[i]);
                initialCrossLines();
            }

            if(alignmentWorkspace.crossSection == true && crossSectionLineList.Count() >= 1)
            {
                createIntersection();
                createCrossSection();
            }


        }

        async void loadSettings()
        {
            List<string> files = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");

            alignmentWorkspace.currentTransformation = CrossSettings.Current.GetValueOrDefault("currentTransformation", "(Default OSGB36)");
            if (alignmentWorkspace.currentTransformation == "(Default OSGB36)")
                setupTransformation(false);
            else if(alignmentWorkspace.currentTransformation == "More...")
            {
                App.transformationString = CrossSettings.Current.GetValueOrDefault("transformationString", "Nothing");
                App.transformationBody = CrossSettings.Current.GetValueOrDefault("transformationBody", "Nothing");
                if(App.transformationBody.Contains("WGS84_A"))
                {
                    //Its an old transformation, reset and prompt to select again.
                    await DisplayAlert("Transformation Update", "The transformation format has been changed, your previously selected transformation has been reset. Please select it again from the transformation menu \r\n\r\n\r\n The previous transformation was: " + App.transformationString, "Ok");
                    App.transformationString = "Nothing";
                    App.transformationBody = "Nothing";
                    CrossSettings.Current.AddOrUpdateValue("transformationString", "Nothing");
                    CrossSettings.Current.AddOrUpdateValue("transformationBody", "Nothing");
                    alignmentWorkspace.currentTransformation = "(Default OSGB36)";
                    CrossSettings.Current.AddOrUpdateValue("currentTransformation", "(Default OSGB36)");
                    setupTransformation(false);
                }
                else
                    setupTransformation(true);
            }
            else
            {
                try
                {
                    setupTransformation(true);
                }
                catch
                {
                    DisplayAlert("Error", "Failed to load transformation: " + alignmentWorkspace.currentTransformation, "Ok");
                    alignmentWorkspace.currentTransformation = "(Default OSGB36)";
                    CrossSettings.Current.AddOrUpdateValue("currentTransformation", "(Default OSGB36)");
                }
            }

            alignmentWorkspace.offsetDisplay = CrossSettings.Current.GetValueOrDefault("offsetDisplay", "-/+");
            labelOffsetDisplay.Text = alignmentWorkspace.offsetDisplay;

            alignmentWorkspace.internationalDisplay = CrossSettings.Current.GetValueOrDefault("internationalDisplay", "Chainage");
            labelInternational.Text = alignmentWorkspace.internationalDisplay;
            switch (alignmentWorkspace.internationalDisplay)
            {
                case "Chainage":
                    labelChStart.Text = "CH";
                    break;
                case "Station":
                    labelChStart.Text = "ST";
                    break;
                case "Meterage":
                    labelChStart.Text = "M";
                    break;
            }

            labelCurScaleUnits.Text = CrossSettings.Current.GetValueOrDefault("mapScaleSetting", "Metric");

            alignmentWorkspace.decimalDisplay = CrossSettings.Current.GetValueOrDefault("decimalDisplay", 0);
            labelDecimalPlaces.Text = alignmentWorkspace.decimalDisplay.ToString();

            alignmentWorkspace.fontSize = CrossSettings.Current.GetValueOrDefault("fontSize", "Medium");
            labelFontSize.Text = alignmentWorkspace.fontSize;

            switch (alignmentWorkspace.fontSize)
            {
                case "Huge":
                    changeFontSize(50, 46, 42, 38);
                    break;
                case "Very Large":
                    changeFontSize(37, 34, 31, 27);
                    break;
                case "Large":
                    changeFontSize(25, 23, 21, 19);
                    break;
                case "Medium":
                    changeFontSize(23, 21, 19, 17);
                    break;
                case "Small":
                    changeFontSize(21, 19, 17, 15);
                    break;
                case "Micro":
                    changeFontSize(19, 17, 15, 13);
                    break;
            }

            alignmentWorkspace.scrollMap = CrossSettings.Current.GetValueOrDefault("scrollMap", true);

            alignmentWorkspace.alwaysUseGPS = CrossSettings.Current.GetValueOrDefault("alwaysUseGPS", false);
            if (alignmentWorkspace.alwaysUseGPS)
            {
                buttonAutoScroll.IsVisible = false;
                alignmentWorkspace.scrollMap = true;
                CrossSettings.Current.AddOrUpdateValue("scrollMap", true);
                labelUseGPS.Text = "Enabled";
            }
            else
            {
                buttonAutoScroll.IsVisible = true;
                labelUseGPS.Text = "Disabled";
            }


            alignmentWorkspace.mapType = CrossSettings.Current.GetValueOrDefault("mapType", 0);
            mapObject.MapType = mapTypeValues[alignmentWorkspace.mapType];
            switch (alignmentWorkspace.mapType)
            {
                case 0:
                    labelMapType.Text = "Street Map";
                    break;
                case 1:
                    labelMapType.Text = "Satellite Map";
                    break;
                case 2:
                    labelMapType.Text = "Hybrid Map";
                    break;
            }
            if (alignmentWorkspace.mapType == 0)
            {
                if (alignmentWorkspace.scrollMap == true)
                    buttonAutoScroll.Image = "ScrollBlack.png";
                else
                    buttonAutoScroll.Image = "NoScrollBlack.png";
                labelScale.TextColor = Color.Black;
                frameScale.BackgroundColor = Color.Black;
            }
            else
            {
                if (alignmentWorkspace.scrollMap == true)
                    buttonAutoScroll.Image = "ScrollBlack.png";
                else
                    buttonAutoScroll.Image = "NoScrollBlack.png";
                labelScale.TextColor = Color.White;
                frameScale.BackgroundColor = Color.White;
            }



            alignmentWorkspace.crosshairs = CrossSettings.Current.GetValueOrDefault("crosshairs", true);
            if (alignmentWorkspace.crosshairs == true)
            {
                labelCrossHair.Text = "Enabled";
                crossHairs.Source = "redCross.png";
            }
            else
            {
                crossHairs.Source = "";
                labelCrossHair.Text = "Disabled";
            }

            alignmentWorkspace.pinType = CrossSettings.Current.GetValueOrDefault("pinType", "international");
            switch (alignmentWorkspace.pinType)
            {
                case "international":
                    labelCurPinType.Text = alignmentWorkspace.internationalDisplay;
                    titlePinX.Text = alignmentWorkspace.internationalDisplay;
                    titlePinY.Text = "Offset";
                    entryPinX.Placeholder = "Enter " + alignmentWorkspace.internationalDisplay;
                    entryPinY.Placeholder = "Enter Offset";
                    break;
                case "eastingNorthing":
                    labelCurPinType.Text = "Easting Northing";
                    titlePinX.Text = "Easting";
                    titlePinY.Text = "Northing";
                    entryPinX.Placeholder = "Enter Easting";
                    entryPinY.Placeholder = "Enter Northing";
                    break;
                case "latitudeLongitude":
                    labelCurPinType.Text = "Latitude Longitude";
                    titlePinX.Text = "Latitude";
                    titlePinY.Text = "Longitude";
                    entryPinX.Placeholder = "Enter Latitude";
                    entryPinY.Placeholder = "Enter Longitude";

                    break;
            }

            alignmentWorkspace.crossSection = CrossSettings.Current.GetValueOrDefault("crossSection", false);
            if (alignmentWorkspace.crossSection == true)
                labelCrossSection.Text = "Enabled";
            else
                labelCrossSection.Text = "Disabled";


            alignmentWorkspace.pos = new Xamarin.Forms.GoogleMaps.Position(CrossSettings.Current.GetValueOrDefault("curLat", 53.258332), CrossSettings.Current.GetValueOrDefault("curLng", -3.980234));
            mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(alignmentWorkspace.pos, Distance.FromMeters(150)));
            mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPositionZoom(alignmentWorkspace.pos, 15D);

            alignmentWorkspace.menuExpanded = CrossSettings.Current.GetValueOrDefault("fileExpanded", "0") + CrossSettings.Current.GetValueOrDefault("settingsExpanded", "0") + CrossSettings.Current.GetValueOrDefault("mapExpanded", "0") + CrossSettings.Current.GetValueOrDefault("crossSectionExpanded", "0") + CrossSettings.Current.GetValueOrDefault("locationExpanded", "0") + CrossSettings.Current.GetValueOrDefault("photoExpanded", "0");
            if (alignmentWorkspace.menuExpanded.ElementAt(0) == '1')
            {
                buttonCollapseFile.Image = "DropUp.png";
                fileExpander.IsVisible = true;
            }

            if (alignmentWorkspace.menuExpanded.ElementAt(1) == '1')
            {
                buttonCollapseSettings.Image = "DropUp.png";
                settingsExpander.IsVisible = true;
            }

            if (alignmentWorkspace.menuExpanded.ElementAt(2) == '1')
            {
                buttonCollapseMap.Image = "DropUp.png";
                mapExpander.IsVisible = true;
            }

            if (alignmentWorkspace.menuExpanded.ElementAt(3) == '1')
            {
                buttonCollapseCrossSection.Image = "DropUp.png";
                crossSectionExpander.IsVisible = true;
            }

            if(alignmentWorkspace.menuExpanded.ElementAt(4) == '1')
            {
                buttonCollapseLocation.Image = "DropUp.png";
                locationExpander.IsVisible = true;
            }

            if(alignmentWorkspace.menuExpanded.ElementAt(5) == '1')
            {
                buttonCollapsePhoto.Image = "DropUp.png";
                photoSelectionExpander.IsVisible = true;
            }

            alignmentWorkspace.watermarkPosition = CrossSettings.Current.GetValueOrDefault("watermarkPosition", "Top Left");
            labelCurWatermarkPosition.Text = alignmentWorkspace.watermarkPosition;

            switch (alignmentWorkspace.watermarkPosition)
            {
                case "Bottom Right":
                    App.watermarkPosition = 1;
                    break;
                case "Bottom Left":
                    App.watermarkPosition = 2;
                    break;
                case "Top Right":
                    App.watermarkPosition = 3;
                    break;
                case "Top Left":
                    App.watermarkPosition = 4;
                    break;
            }

            alignmentWorkspace.watermarkSize = CrossSettings.Current.GetValueOrDefault("watermarkSize", "Large");
            labelCurWatermarkSize.Text = alignmentWorkspace.watermarkSize;

            switch (alignmentWorkspace.watermarkSize)
            {
                case "Small":
                    App.watermarkSize = 1;
                    break;
                case "Medium":
                    App.watermarkSize = 2;
                    break;
                case "Large":
                    App.watermarkSize = 3;
                    break;
            }

            var test = CrossSettings.Current.GetValueOrDefault("distance", 50D);
            alignmentWorkspace.distance = CrossSettings.Current.GetValueOrDefault("distance", 50D);
            entryDistance.Text = alignmentWorkspace.distance.ToString();

            alignmentWorkspace.vertExag = CrossSettings.Current.GetValueOrDefault("vertExag", 1D);
            entryVertExag.Text = alignmentWorkspace.vertExag.ToString();

            alignmentWorkspace.scale = CrossSettings.Current.GetValueOrDefault("scaleEnabled", true);
            if (alignmentWorkspace.scale == true)
                labelCurScale.Text = "Enabled";
            else
                labelCurScale.Text = "Disabled";

            alignmentWorkspace.chainageFormat = CrossSettings.Current.GetValueOrDefault("internationalFormat", false);
            alignmentWorkspace.chainageFormatDesc = CrossSettings.Current.GetValueOrDefault("internationalFormatDesc", "Format1");
            if (alignmentWorkspace.chainageFormat == true)
            {
                if (alignmentWorkspace.chainageFormatDesc == "Format1")
                    labelCurFormat.Text = "10+000";
                else
                    labelCurFormat.Text = "100+00";
            }
            else
                labelCurFormat.Text = "10000";

            alignmentWorkspace.countryCode = CrossSettings.Current.GetValueOrDefault("countryCode", "");
            entryCountryCode.Text = alignmentWorkspace.countryCode;

            alignmentWorkspace.scaleBar = CrossSettings.Current.GetValueOrDefault("scaleBar", true);

            alignmentWorkspace.offsetCenter = CrossSettings.Current.GetValueOrDefault("offsetCenter", 0D);
            entryOffsetCenter.Text = alignmentWorkspace.offsetCenter.ToString();

            alignmentWorkspace.centeredByOffset = CrossSettings.Current.GetValueOrDefault("centeredByOffset", false);
            if (alignmentWorkspace.centeredByOffset == false)
            {
                labelCurCrossSectionCenter.Text = "Centre by Mid-point";
                gridOffsetCenter.IsVisible = false;
            }
            else if (alignmentWorkspace.centeredByOffset == true)
            {
                labelCurCrossSectionCenter.Text = "Centre by Offset";
                gridOffsetCenter.IsVisible = true;
            }

            alignmentWorkspace.currentFolder = CrossSettings.Current.GetValueOrDefault("currentFolder", "None Found");
            if (alignmentWorkspace.currentFolder == "None Found" || nrgFileRead.reader.listFolders(alignmentWorkspace.datapath).Contains(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder) == false)
            {
                CrossSettings.Current.AddOrUpdateValue("currentFolder", "None Found");
                gridAlignmentSelection.IsVisible = false;
                gridPinsSelection.IsVisible = false;
                gridLinesSelection.IsVisible = false;
                labelCurProject.Text = "None Found";
            }
            else
            {
                labelLoadLastProjectName.Text = alignmentWorkspace.currentFolder;
                gridLoadProject.IsVisible = true;
                alignmentWorkspace.currentFolder = "None Selected";
            }

            if (CrossSettings.Current.GetValueOrDefault("mapTraffic", false) == true)
            {
                labelMapTraffic.Text = "Enabled";
                mapObject.IsTrafficEnabled = true;
            }
            else
            {
                labelMapTraffic.Text = "Disabled";
                mapObject.IsTrafficEnabled = false;
            }

            alignmentWorkspace.requestOnly = CrossSettings.Current.GetValueOrDefault("requestOnly", false);
            alignmentWorkspace.updateTime = CrossSettings.Current.GetValueOrDefault("updateTime", 0.5);
            alignmentWorkspace.updateDistance = CrossSettings.Current.GetValueOrDefault("updateDistance", 0.1);
            if (alignmentWorkspace.requestOnly == false)
            {
                labelRequestOnly.Text = "Disabled";

                if (CrossGeolocator.Current.IsListening)
                    await CrossGeolocator.Current.StopListeningAsync();

                CrossGeolocator.Current.DesiredAccuracy = 100;
                CrossGeolocator.Current.PositionError += (sender, args) =>
                {

                };

                if (CrossGeolocator.IsSupported && CrossGeolocator.Current.IsGeolocationAvailable && CrossGeolocator.Current.IsGeolocationEnabled)
                    await CrossGeolocator.Current.StartListeningAsync(TimeSpan.FromSeconds(alignmentWorkspace.updateTime), alignmentWorkspace.updateDistance, true);
            }
            else
                labelRequestOnly.Text = "Enabled";

            entryUpdateTime.Text = alignmentWorkspace.updateTime.ToString();
            entryUpdateDistance.Text = alignmentWorkspace.updateDistance.ToString();

            alignmentWorkspace.siteShiftE = CrossSettings.Current.GetValueOrDefault("siteShiftE", 0.0);
            entryShiftE.Text = alignmentWorkspace.siteShiftE.ToString();

            alignmentWorkspace.siteShiftN = CrossSettings.Current.GetValueOrDefault("siteShiftN", 0.0);
            entryShiftN.Text = alignmentWorkspace.siteShiftN.ToString();

            alignmentWorkspace.scaleFactor = CrossSettings.Current.GetValueOrDefault("scaleFactor", 1.0D);
            entryScaleFactor.Text = alignmentWorkspace.scaleFactor.ToString("0.00000000");


            App.watermarkShowAcc = CrossSettings.Current.GetValueOrDefault("showAcc", false);
            labelShowAcc.Text = App.watermarkShowAcc ? "Yes" : "No";

            App.watermarkShowDate = CrossSettings.Current.GetValueOrDefault("showDate", false);
            labelShowDate.Text = App.watermarkShowDate ? "Yes" : "No";

            App.watermarkDateFormat = CrossSettings.Current.GetValueOrDefault("dateFormat", 0);
            switch(App.watermarkDateFormat)
            {
                case 0:
                    labelDateFormat.Text = "DD/MM/YYYY";
                    break;
                case 1:
                    labelDateFormat.Text = "MM/DD/YYYY";
                    break;
                case 2:
                    labelDateFormat.Text = "YYYY/MM/DD";
                    break;
            }

            App.watermarkTimeFormat = CrossSettings.Current.GetValueOrDefault("timeFormat", 0);
            switch(App.watermarkTimeFormat)
            {
                case 0:
                    labelTimeFormat.Text = "24 Hour (13:00)";
                    break;
                case 1:
                    labelTimeFormat.Text = "12 Hour (1:00 pm)";
                    break;
            }

            App.watermarkShowString = CrossSettings.Current.GetValueOrDefault("showString", false);
            labelShowStringName.Text = App.watermarkShowString ? "Yes" : "No";


            var threshold = CrossSettings.Current.GetValueOrDefault("accuracyThreshold", -1D);

            if(labelCurScaleUnits.Text == "Metric")
            {
                titleAccuracyThreshold.Text = "Accuracy Threshold (Meters)";
                if (threshold == -1)
                    App.AccuracyThreshold = 5;
                else
                    App.AccuracyThreshold = threshold;
            }
            else if(labelCurScaleUnits.Text == "Imperial")
            {
                titleAccuracyThreshold.Text = "Accuracy Threshold (Feet)";
                if (threshold == -1)
                    App.AccuracyThreshold = 15;
                else
                    App.AccuracyThreshold = threshold;
            }

            entryAccuracyThreshold.Text = getDecimalDisplay(App.AccuracyThreshold);
            locationTime();
        }

        private async void initialMapLines()
        {
            for(int i = 0; i<mapLineList.Count(); i++)
            {
                if (nrgFileRead.reader.fileExists(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + mapLineList[i] + ".kml"))
                    await loadKmlFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + mapLineList[i] + ".kml");
            }
            if (mapLineList.Count() >= 2)
                labelCurLines.Text = "Multiple Selected";
            else if (mapLineList.Count == 1)
                labelCurLines.Text = mapLineList[0];
        }

        private async void initialCrossLines()
        {
            for(int i = 0; i<crossSectionLineList.Count();)
            {
                //Check if line exists then attempt to load if it does
                if (nrgFileRead.reader.fileExists(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + crossSectionLineList[i] + ".kml"))
                {
                    await loadKmlFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + crossSectionLineList[i] + ".kml");
                    i++;
                }
                else
                {
                    //Remove line from list if it doesn't exist anymore
                    crossSectionLineList.RemoveAt(i);
                }
            }
        }

        private void savePins(string filename)
        {
            List<string> dataList = new List<string>();
            dataList.Add(";NRG GROUND PLOT FILE VERSION 2 \r\n");

            foreach (Pin item in mapObject.Pins)
            {
                var estNrth = GetENFromLL(item.Position.Latitude, item.Position.Longitude);
                string toAdd = item.Label + "," + estNrth.X.ToString("0.0000") + "," + estNrth.Y.ToString("0.0000") + item.Tag;
                if ((item.Tag.ToString().Split(',').Count() - 2) < currentTagAmount)
                {
                    for (int i = 0; i < currentTagAmount - (item.Tag.ToString().Split(',').Count() - 2); i++)
                    {
                        toAdd += ",\"\"";
                    }
                }
                toAdd += "\r\n";
                dataList.Add(toAdd);
            }
            nrgFileRead.reader.listToStream(filename, dataList);


            switch(Device.RuntimePlatform)
            {
                case Device.Android:
                    App.ScanFiles.scanFile(filename);
                    Acr.UserDialogs.UserDialogs.Instance.Toast("Saved " + Path.GetFileName(filename) + ".", TimeSpan.FromSeconds(1));
                    break;
                case Device.iOS:
                    Acr.UserDialogs.UserDialogs.Instance.Toast("Saved " + Path.GetFileName(filename) + ".", TimeSpan.FromSeconds(1));
                    break;
            }
        }

        private void savePinAscii(string filename)
        {
            try
            {
                List<string> dataList = new List<string>();
                foreach (Pin pin in mapObject.Pins)
                {
                    var estNrth = GetENFromLL(pin.Position.Latitude, pin.Position.Longitude);
                    string[] tmpData = pin.Label.Replace("\"", "").Split(',');
                    string toAdd = tmpData[0].ToString() + "," + estNrth.X.ToString("0.000") + "," + estNrth.Y.ToString("0.000") + ",";
                    string[] tempData = pin.Tag.ToString().Replace("\"", "").Split(',');
                    double z = Convert.ToDouble(tempData[1]);
                    if (z == 0)
                        z = -999;
                    toAdd += z.ToString("0.000") + "," + tmpData[1] + "\r\n";
                    dataList.Add(toAdd);
                }
                nrgFileRead.reader.listToStream(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + filename + ".txt", dataList);
                
                switch(Device.RuntimePlatform)
                {
                    case Device.Android:
                        App.ScanFiles.scanFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + filename + ".txt");
                        Acr.UserDialogs.UserDialogs.Instance.Toast("Saved " + Path.GetFileName(filename) + ".", TimeSpan.FromSeconds(1));
                        break;
                    case Device.iOS:
                        Acr.UserDialogs.UserDialogs.Instance.Toast("Saved " + Path.GetFileName(filename) + ".", TimeSpan.FromSeconds(1));
                        break;
                }
            }
            catch(Exception e)
            {
                DisplayAlert("Error", e.Message, "Ok");
            }
        }

        public async Task loadKmlFile(string filename)
        {
            //var ns = XNamespace.Get("http://www.opengis.net/kml/2.2");
            //var ns = XNamespace.Get("http://earth.google.com/kml/2.1");
            XNamespace ns = "";
            try
            {
                XmlNamespaceManager manager = new XmlNamespaceManager(new NameTable());
                manager.AddNamespace("", "http://www.opengis.net/kml/2.2");
                manager.AddNamespace("gx", "http://www.google.com/kml/ext/2.2");
                manager.AddNamespace("kml", "http://www.opengis.net/kml/2.2");
                manager.AddNamespace("atom", "http://www.w3.org/2005/Atom");
                manager.AddNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
                var fileStream = nrgFileRead.reader.ReadFileAsStream(filename);
                XmlParserContext parserContext = new XmlParserContext(null, manager, null, XmlSpace.None);
                var textReader = XmlReader.Create(fileStream, new XmlReaderSettings() { }, parserContext);

                var doc = XDocument.Load(textReader);

                //var doc = XDocument.Load(filename);
                try
                {
                    ns = doc.Root.Name.Namespace;
                }
                catch
                {
                    //No namespace
                }
                var name = Path.GetFileName(filename).Replace(".kml", "");
                var styles = new List<dataTypes.LineStyle>();
                var defaultStyle = new dataTypes.LineStyle();
                defaultStyle.col = Color.Black;
                defaultStyle.width = 1f;
                styles.Add(defaultStyle);
                foreach (XElement item in doc.Descendants(ns + "Style"))
                {
                    dataTypes.LineStyle style = new dataTypes.LineStyle();

                    try
                    {
                        if(item.FirstAttribute != null)
                        {
                            style.id = item.FirstAttribute.Value;
                        }
                        else
                        {

                        }
                        string col = "";
                        if(item.Elements(ns + "LineStyle").Any())
                        {
                            if (item.Element(ns + "LineStyle").Elements(ns + "color").Any())
                            {
                                if (item.Element(ns + "LineStyle").Element(ns + "color").Value.Length <= 6)
                                    col = item.Element(ns + "LineStyle").Element(ns + "color").Value.ToString();
                                else if (item.Element(ns + "LineStyle").Element(ns + "color").Value.Length == 8)
                                    col = item.Element(ns + "LineStyle").Element(ns + "color").Value.Substring(2);
                                else if (item.Element(ns + "LineStyle").Element(ns + "color").Value.Length == 7)
                                    col = item.Element(ns + "LineStyle").Element(ns + "color").Value.Substring(1);
                            }
                            else
                                col = "FFFFFF";
                        }
                        else
                            col = "FFFFFF";

                        while (col.Length < 6)
                            col = "0" + col;

                        string r = col.ElementAt(4).ToString() + col.ElementAt(5).ToString();
                        string g = col.ElementAt(2).ToString() + col.ElementAt(3).ToString();
                        string b = col.ElementAt(0).ToString() + col.ElementAt(1).ToString();

                        string revCol = r + g + b;
                        style.col = Color.FromHex(revCol);
                    }
                    catch
                    {
                        style.col = Color.Black;
                    }

                    try
                    {
                        float width;
                        float.TryParse(item.Element(ns + "LineStyle").Element(ns + "width").Value, out width);
                        style.width = width;
                    }
                    catch
                    {
                        style.width = 1f;
                    }
                    styles.Add(style);
                }
                List<dataTypes.Line> lineList = new List<dataTypes.Line>();
                foreach (XElement item in doc.Descendants(ns + "Placemark"))
                {
                    try
                    {
                        var tempstyle = new dataTypes.LineStyle();
                        if (item.Elements(ns + "Style").Any())
                        {
                            try
                            {
                                var col = "";
                                if (item.Element(ns + "Style").Elements(ns + "LineStyle").Any())
                                {
                                    if (item.Element(ns + "Style").Element(ns + "LineStyle").Elements(ns + "color").Any())
                                    {
                                        if (item.Element(ns + "Style").Element(ns + "LineStyle").Element(ns + "color").Value.Length <= 6)
                                            col = item.Element(ns + "Style").Element(ns + "LineStyle").Element(ns + "color").Value.ToString();
                                        else if (item.Element(ns + "Style").Element(ns + "LineStyle").Element(ns + "color").Value.Length == 8)
                                            col = item.Element(ns + "Style").Element(ns + "LineStyle").Element(ns + "color").Value.Substring(2);
                                        else if (item.Element(ns + "Style").Element(ns + "LineStyle").Element(ns + "color").Value.Length == 7)
                                            col = item.Element(ns + "Style").Element(ns + "LineStyle").Element(ns + "color").Value.Substring(1);
                                    }
                                    else
                                        col = "FFFFFF";
                                }
                                else
                                    col = "FFFFFF";

                                while (col.Length < 6)
                                    col = "0" + col;

                                string r = col.ElementAt(4).ToString() + col.ElementAt(5).ToString();
                                string g = col.ElementAt(2).ToString() + col.ElementAt(3).ToString();
                                string b = col.ElementAt(0).ToString() + col.ElementAt(1).ToString();

                                string revCol = r + g + b;
                                tempstyle.col = Color.FromHex(revCol);
                            }
                            catch
                            {
                                tempstyle.col = Color.Black;
                            }
                            try
                            {
                                if (item.Element(ns + "Style").Elements(ns + "LineStyle").Any())
                                {
                                    if (item.Element(ns + "Style").Element(ns + "LineStyle").Elements(ns + "width").Any())
                                    {
                                        float width;
                                        float.TryParse(item.Element(ns + "Style").Element(ns + "LineStyle").Element(ns + "width").Value.ToString(), out width);
                                        tempstyle.width = width;
                                    }
                                    else
                                        tempstyle.width = 1f;
                                }
                                else
                                    tempstyle.width = 1f;
                            }
                            catch
                            {
                                tempstyle.width = 1f;
                            }
                        }
                        if (item.Elements(ns + "LineString").Any())
                        {

                            dataTypes.Line line = new dataTypes.Line();
                            List<dataTypes.Coord> coordinates = new List<dataTypes.Coord>();
                            string[] coords = item.Element(ns + "LineString").Element(ns + "coordinates").Value.Replace("\r\n", "").Replace("\r", "").Replace("\n", "").Replace("\t", "").Split(' ');
                            foreach (string coord in coords)
                            {
                                try
                                {
                                    dataTypes.Coord c = new dataTypes.Coord();
                                    string[] tmpCoords = coord.Split(',');
                                    double lat, lng, alt;
                                    double.TryParse(tmpCoords[0], out lng);
                                    double.TryParse(tmpCoords[1], out lat);
                                    double.TryParse(tmpCoords[2], out alt);

                                    c.x = lat;
                                    c.y = lng;
                                    c.z = alt;

                                    coordinates.Add(c);
                                }
                                catch
                                {

                                }
                            }
                            if (coordinates.Count() >= 2)
                            {
                                line.coords = coordinates;
                                if (item.Elements(ns + "description").Any())
                                {
                                    if(item.Elements(ns + "name").Any())
                                    {
                                        if (item.Element(ns + "name").Value.ToString().Contains("NRGalignment:"))
                                            line.desc = "NRGalignment:";
                                        else
                                            line.desc = item.Element(ns + "description").Value;

                                    }
                                    else
                                    {
                                        line.desc = item.Element(ns + "description").Value;
                                    }
                                }
                                else
                                {
                                    line.desc = "null";
                                }
                                if (item.Elements(ns + "styleUrl").Any())
                                {

                                    string styleName = item.Element(ns + "styleUrl").Value.ToString();
                                    if (styleName.StartsWith("#m"))
                                        styleName = styleName.Substring(2);
                                    else if (styleName.StartsWith("#"))
                                        styleName = styleName.Substring(1);

                                    for (int i = 0; i < styles.Count(); i++)
                                        if (styles[i].id == styleName)
                                            line.style = styles[i];

                                }
                                else if (tempstyle != null)
                                    line.style = tempstyle;
                                if (line.style == null)
                                    line.style = styles[0];
                                if (line.desc.Contains("NRGalignment:") == false)
                                    lineList.Add(line);
                            }
                        }
                        if(item.Elements(ns + "MultiGeometry").Any())
                        {
                            foreach(XElement l in item.Element(ns + "MultiGeometry").Descendants(ns + "LineString"))
                            {
                                dataTypes.Line line = new dataTypes.Line();
                                List<dataTypes.Coord> coordinates = new List<dataTypes.Coord>();
                                string[] coords = l.Element(ns + "coordinates").Value.Replace("\r\n", "").Replace("\r", "").Replace("\n", "").Replace("\t", "").Split(' ');
                                foreach (string coord in coords)
                                {
                                    try
                                    {
                                        dataTypes.Coord c = new dataTypes.Coord();
                                        string[] tmpCoords = coord.Split(',');
                                        double lat, lng, alt;
                                        double.TryParse(tmpCoords[0], out lng);
                                        double.TryParse(tmpCoords[1], out lat);
                                        double.TryParse(tmpCoords[2], out alt);

                                        c.x = lat;
                                        c.y = lng;
                                        c.z = alt;

                                        coordinates.Add(c);
                                    }
                                    catch(Exception error)
                                    {

                                    }
                                }
                                if (coordinates.Count() >= 2)
                                {
                                    line.coords = coordinates;
                                    if (item.Elements(ns + "description").Any())
                                    {
                                        if (item.Elements(ns + "name").Any())
                                        {
                                            if (item.Element(ns + "name").Value.ToString().Contains("NRGalignment:"))
                                                line.desc = "NRGalignment:";
                                            else
                                                line.desc = item.Element(ns + "description").Value;

                                        }
                                        else
                                        {
                                            line.desc = item.Element(ns + "description").Value;
                                        }
                                    }
                                    else
                                    {
                                        line.desc = "null";
                                    }
                                    if (item.Elements(ns + "styleUrl").Any())
                                    {
                                        var value = item.Element(ns + "styleUrl").Value;
                                        if (value.StartsWith("#m"))
                                            value = value.Substring(2);
                                        else if (value.StartsWith("#"))
                                            value = value.Substring(1);

                                        for (int i = 0; i < styles.Count(); i++)
                                            if (styles[i].id == value)
                                                line.style = styles[i];

                                    }
                                    else if (tempstyle != null)
                                        line.style = tempstyle;
                                    if (line.style == null)
                                        line.style = styles[0];
                                    if (line.desc.Contains("NRGalignment:") == false)
                                        lineList.Add(line);
                                }
                            }
                        }
                    }
                    catch
                    {
                        //Not a line
                    }
                }
                await placeLines(name, lineList);
            }
            catch(Exception error)
            {
                var test = error.Message;
                loadFailure(Path.GetFileName(filename), crossSectionLineList.Contains(Path.GetFileName(filename.Replace(".kml", ""))), mapLineList.Contains(Path.GetFileName(filename.Replace(".kml", ""))));
                DisplayAlert("Error", "Failed to load " + Path.GetFileName(filename), "Ok");
            }
        }

        public async Task loadKmlPins(string filename)
        {
            await refreshUI();
            XNamespace ns = "";

            try
            {
                var doc = XDocument.Load(filename);
                ns = doc.Root.Name.Namespace;

                foreach (XElement item in doc.Descendants(ns + "Placemark"))
                {
                    try
                    {
                        if (item.Elements(ns + "Point").Any())
                        {
                            var pin = new Pin();
                            if (item.Elements(ns + "name").Any())
                                pin.Label = "\"" + item.Element(ns + "name").Value.ToString() + "\"";
                            else
                                pin.Label = "\"\"";

                            if (item.Elements(ns + "description").Any())
                                pin.Label += ",\"" + item.Element(ns + "description").Value.ToString() + "\"";
                            else
                                pin.Label += ",\"\"";

                            if (item.Element(ns + "Point").Elements(ns + "coordinates").Any())
                            {
                                string[] tempData = item.Element(ns + "Point").Element(ns + "coordinates").Value.ToString().Split(',');
                                var pos = new Xamarin.Forms.GoogleMaps.Position(Convert.ToDouble(tempData[1]), Convert.ToDouble(tempData[0]));
                                pin.Position = pos;
                                pin.Address = "";
                                pin.Tag = tempData[2];
                                pin.Flat = true;
                            }
                            Device.BeginInvokeOnMainThread(() =>
                            {
                                mapObject.Pins.Add(pin);
                            });
                            currentPinId++;
                        }
                    }
                    catch
                    {
                        DisplayAlert("Error", "Failed to load the pin file", "Ok");
                    }
                }
                var test = currentPinId;
            }
            catch(Exception e)
            {
                DisplayAlert("Error", e.Message, "Ok");
            }
                    
        }

        public async void pinsToKml(string filename)
        {
            await Task.Run( () =>
            {
                var ns = XNamespace.Get("http://www.opengis.net/kml/2.2");
                var doc = new XDocument();
                doc.Add(new XElement(ns + "kml", new XAttribute(XNamespace.Xmlns + "gx", "http://www.google.com/kml/ext/2.2"), 
                    new XAttribute(XNamespace.Xmlns + "kml", "http://opengis.net/kml/2.2"), 
                    new XAttribute(XNamespace.Xmlns + "atom", "http://www.w3.org/2005/Atom")));
                //doc.Element(ns + "kml").Add(new XAttribute(XNamespace.Xmlns + "gx", "http://www.google.com/kml/ext/2.2"));
                //doc.Element(ns + "kml").Add(new XAttribute(XNamespace.Xmlns + "kml", "http://www.opengis.net/kml/2.2"));
                //doc.Element(ns + "kml").Add(new XAttribute(XNamespace.Xmlns + "atom", "http://www.w3.org/2005/Atom"));
                doc.Element(ns + "kml").Add(new XElement(ns + "Document"));
                doc.Element(ns + "kml").Element(ns + "Document").Add(new XElement(ns + "open", "0"));
                doc.Element(ns + "kml").Element(ns + "Document").Add(new XElement(ns + "name", filename + ".kml"));
                doc.Element(ns + "kml").Element(ns + "Document").Add(new XElement(ns + "StyleMap"));
                doc.Element(ns + "kml").Element(ns + "Document").Element(ns + "StyleMap").Add(new XAttribute("id", "msnffff00ff|0_3|placemark_circle"));
                try
                {
                    var pair1 = new XElement(ns + "Pair", new XElement(ns + "key", "normal"), new XElement(ns + "styleUrl", "#snffff00ff|0_3|placemark_circle"));
                    var pair2 = new XElement(ns + "Pair", new XElement(ns + "key", "highlight"), new XElement(ns + "styleUrl", "#shffff00ff|0_3|placemark_circle"));
                    doc.Element(ns + "kml").Element(ns + "Document").Element(ns + "StyleMap").Add(pair1);
                    doc.Element(ns + "kml").Element(ns + "Document").Element(ns + "StyleMap").Add(pair2);

                    var Style1 = new XElement(ns + "Style", new XElement(ns + "IconStyle", new XElement(ns +"color", "ffff00ff"), new XElement(ns + "colorMode", "normal"), new XElement(ns + "scale", "0.3"),
                        new XElement(ns + "Icon", new XElement(ns +"href", "http://maps.google.com/mapfiles/kml/shapes/placemark_circle.png"))));
                    Style1.Add(new XAttribute("id", "snffff00ff|0_3|placemark_circle"));
                    var Style2 = new XElement(ns + "Style", new XElement(ns + "IconStyle", new XElement(ns + "color", "ffff00ff"), new XElement(ns + "colorMode", "normal"), new XElement(ns + "scale", "0.6"),
                        new XElement(ns + "Icon", new XElement(ns + "href", "http://maps.google.com/mapfiles/kml/shapes/placemark_circle.png"))));
                    Style2.Add(new XAttribute("id", "shffff00ff|0_3|placemark_circle"));

                    doc.Element(ns + "kml").Element(ns + "Document").Add(Style1, Style2);

                    doc.Element(ns + "kml").Element(ns + "Document").Add(new XElement(ns + "Folder", new XElement(ns + "name", filename), new XElement(ns + "open", "0")));
                    doc.Element(ns + "kml").Element(ns + "Document").Element(ns + "Folder").Add(new XElement(ns + "Folder", new XElement(ns + "name", "Points"), new XElement(ns + "open", "0")));
                    foreach (Pin pin in mapObject.Pins)
                    {
                        string note = "";
                        string[] tmp = pin.Label.Replace("\"", "").Split(',');
                        for (int i = 1; i < tmp.Count(); i++)
                        {
                            note += tmp[i];
                        }
                        var placemark = new XElement(ns + "Placemark", new XElement(ns + "name", tmp.First()), new XElement(ns + "description", note), new XElement(ns + "styleUrl", "msnffff00ff|0_3|placemark_circle"),
                            new XElement(ns + "Point", new XElement(ns + "extrude", "0"), new XElement(ns + "tessellate", "1"), new XElement(ns + "altitudeMode", "absolute"), new XElement(ns + "coordinates", pin.Position.Longitude.ToString() + "," + pin.Position.Latitude.ToString() + pin.Tag)));

                        doc.Element(ns + "kml").Element(ns + "Document").Element(ns + "Folder").Element(ns + "Folder").Add(placemark);
                    }
                    var stream = nrgFileRead.reader.stringToStream(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + filename + ".kml", doc.ToString(SaveOptions.None));

                    switch(Device.RuntimePlatform)
                    {
                        case Device.Android:
                            App.ScanFiles.scanFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + filename + ".kml");
                            Acr.UserDialogs.UserDialogs.Instance.Toast("Saved " + Path.GetFileName(filename), TimeSpan.FromSeconds(1));
                            break;
                        case Device.iOS:
                            Acr.UserDialogs.UserDialogs.Instance.Toast("Saved " + Path.GetFileName(filename), TimeSpan.FromSeconds(1));
                            break;

                    }
                }
                catch(Exception e)
                {
                    Device.BeginInvokeOnMainThread(() =>
                    {
                        DisplayAlert("Error", e.Message, "Ok");
                    });
                }
             
            });

        }

        public void placeLine(string coords, Color col, string desc)
        {
            string[] coordinates = coords.Split(' ');
            Polyline line = new Polyline();
            line.StrokeColor = col;
            line.StrokeWidth = 0.6f;
            line.IsClickable = false;
            line.Tag = desc;

            for (int i = 0; i < coordinates.Count(); i++)
            {

                try
                {
                    string[] tempCoord = coordinates[i].Split(',');
                    double lat, lng;
                    double.TryParse(tempCoord[0], out lng);
                    double.TryParse(tempCoord[1], out lat);

                    line.Positions.Add(new Xamarin.Forms.GoogleMaps.Position(lat, lng));
                }
                catch
                {

                }

            }

            mapObject.Polylines.Add(line);

        }

        private async Task refreshUI()
        {
            gridLinesSelector.IsVisible = false;
            gridFileContent.IsVisible = false;
            gridMapPanel.IsVisible = true;
            gridMain.IsVisible = true;
            buttonFile.IsVisible = true;
            buttonPinDropDown.IsVisible = true;
            gridEditPinPanel.IsVisible = false;
            gridPinButtonPanel.IsVisible = false;
            labelAccuracyMain.IsVisible = true;
            buttonNRGIcon.IsVisible = true;
            buttonCHPhoto.IsVisible = true;
            buttonFile.Image = "SettingsIcon.png";
            buttonLandscapeFile.Image = "SettingsIcon.png";
            buttonLandscapeChOff.IsVisible = true;
            buttonLandscapeEN.IsVisible = true;
            buttonLandscapeLL.IsVisible = true;
            buttonLandscapePhoto.IsVisible = true;
            buttonLandscapePcode.IsVisible = true;
            buttonLandscapePhoto.IsVisible = true;
        }

        private async Task placeLines(string name, List<dataTypes.Line> lines)
        {
            var printLines = true;
            foreach (string lineName in dictionaryLoadedLines.Keys)
            {
                if (name == lineName)
                    printLines = false;
            }
            await refreshUI();
            if (mapLineList.Contains(name) && printLines == true)
            {
                int lineCount;
                if (dictionaryLoadedLines.Count() == 0)
                    lineCount = mapObject.Polylines.Count();
                else
                    lineCount = dictionaryLoadedLines.Last().Value.Value;
                dictionaryLoadedLines.Add(name, new KeyValuePair<int, int>(lineCount, lineCount + lines.Count()));
                lastline = lineCount + lines.Count();
                for (int i = 0; i < lines.Count(); i++)
                {
                    Polyline polyline = new Polyline();
                    polyline.StrokeColor = lines[i].style.col;
                    polyline.StrokeWidth = lines[i].style.width;

                    for (int j = 0; j < lines[i].coords.Count(); j++)
                    {
                        polyline.Positions.Add(new Xamarin.Forms.GoogleMaps.Position(lines[i].coords[j].x, lines[i].coords[j].y));
                    }

                    await Task.Run(() =>
                    {
                        Device.BeginInvokeOnMainThread(() =>
                        {
                            mapObject.Polylines.Add(polyline);
                        });
                    });
                    //Device.BeginInvokeOnMainThread(() =>
                    //{
                    //    mapObject.Polylines.Add(polyline);
                    //});
                }
            }
            convertCoords(name, lines);

        }

        public void convertCoords(string name, List<dataTypes.Line> lines)
        {

            foreach (dataTypes.Line line in lines)
            {
                foreach (dataTypes.Coord coord in line.coords)
                {
                    var estNrth = GetENFromLL(coord.x, coord.y);
                    coord.x = estNrth.X;
                    coord.y = estNrth.Y;
                }
            }

            if(dictionaryLines.ContainsKey(name))
            {
                dictionaryLines[name] = lines;
            }
            else
            {
                dictionaryLines.Add(name, lines);
            }    
        }

        private void moveToChOff(string chainage, string offset)
        {
            try
            {
                double ch;
                double off;

                if (chainage.StartsWith("0+"))
                    double.TryParse(chainage.Trim('+').TrimStart('0'), out ch);
                else if (chainage.Contains('+'))
                    double.TryParse(chainage.Replace("+", ""), out ch);
                else
                    double.TryParse(chainage, out ch);

                if (offset.Contains(" Left"))
                {
                    double.TryParse(offset.Replace(" ", "").Replace("L", "").Replace("e", "").Replace("f", "").Replace("t", ""), out off);
                    off *= -1;
                }
                else if (offset.Contains(" Right"))
                    double.TryParse(offset.Replace(" ", "").Replace("R", "").Replace("i", "").Replace("g", "").Replace("h", "").Replace("t", ""), out off);
                else
                    double.TryParse(offset, out off);

                var acoords = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(ch, off);

                var latLng = GetLLFromEN(acoords.X, acoords.Y);

                Xamarin.Forms.GoogleMaps.Position pos = new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y);
                mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(pos, Distance.FromMeters(100)));
                alignmentWorkspace.devicePosition = pos;
                mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPosition(alignmentWorkspace.devicePosition);
                if(!alignmentWorkspace.alwaysUseGPS)
                    alignmentWorkspace.scrollMap = false;
                buttonAutoScroll.Image = "NoScrollBlack.png";

                CrossSettings.Current.AddOrUpdateValue("scrollMap", alignmentWorkspace.scrollMap);
            }
            catch
            {
                if (alignmentWorkspace.currentFolder == null || alignmentWorkspace.Alignments == null)
                    DisplayAlert("Error", "You have not selected an Alignment String", "Ok");
                else
                    DisplayAlert("Error", "That " + labelInternational.Text.ToLower() + " does not exist within the alignment", "Ok");
            }

        }

        private void moveToEN(string easting, string northing)
        {
            try
            {
                double east;
                double nrth;

                double.TryParse(easting.ToString(), out east);
                double.TryParse(northing.ToString(), out nrth);

                var latLng = GetLLFromEN(east, nrth);

                Xamarin.Forms.GoogleMaps.Position pos = new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y);
                mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(pos, Distance.FromMeters(100)));
                alignmentWorkspace.devicePosition = pos;
                mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPosition(alignmentWorkspace.devicePosition);
                if (!alignmentWorkspace.alwaysUseGPS)
                    alignmentWorkspace.scrollMap = false;
                if (alignmentWorkspace.mapType == 0)
                    buttonAutoScroll.Image = "NoScrollBlack.png";
                CrossSettings.Current.AddOrUpdateValue("scrollMap", alignmentWorkspace.scrollMap);
            }
            catch
            {
                DisplayAlert("Error", "Could not find that location", "Ok");
            }
        }

        private void moveToLatLng(string latitude, string longitude)
        {
            try
            {
                double lat;
                double lng;

                double.TryParse(latitude.ToString(), out lat);
                double.TryParse(longitude.ToString(), out lng);

                Xamarin.Forms.GoogleMaps.Position pos = new Xamarin.Forms.GoogleMaps.Position(lat, lng);
                mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(pos, Distance.FromMeters(100)));
                alignmentWorkspace.devicePosition = pos;
                mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPosition(alignmentWorkspace.devicePosition);
                if (!alignmentWorkspace.alwaysUseGPS)
                    alignmentWorkspace.scrollMap = false;
                if (alignmentWorkspace.mapType == 0)
                    buttonAutoScroll.Image = "NoScrollBlack.png";
                CrossSettings.Current.AddOrUpdateValue("scrollMap", alignmentWorkspace.scrollMap);
            }
            catch
            {
                DisplayAlert("Error", "Could not find that location", "Ok");
            }
        }

        private async void moveToPCode(string postcode)
        {
            try
            {
                var geo = CrossGeolocator.Current;
                var location = await App.PostCodeSearch.getLocation(postcode,  entryCountryCode.Text);
                string[] temp = location.Split(',');
                mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(new Xamarin.Forms.GoogleMaps.Position(Convert.ToDouble(temp[0]), Convert.ToDouble(temp[1])), Distance.FromMeters(200D)));
            }
            catch
            {
                DisplayAlert("Error", "Error there was an issue with the connection or the address you entered", "Ok");
            }
        }

        public async void getPCode(double lat, double lng)
        {
            var pos = new Plugin.Geolocator.Abstractions.Position();
            pos.Latitude = lat;
            pos.Longitude = lng;

            var geo = CrossGeolocator.Current;
            var address = await geo.GetAddressesForPositionAsync(pos);
            Device.BeginInvokeOnMainThread(() =>
            {
                labelPcode.Text = address.First().PostalCode.ToString();
                labelLandPcode.Text = address.First().PostalCode.ToString();
            });

        }

        public interface IPostCodeSearch
        {

            Task<string> getLocation(string Postcode, string code);
            Task<string> getPostCode(double lat, double lng);
        }

        public interface IScanFiles
        {
            void scanFile(string filepath);
        }

        public interface ISetupSampleData
        {
            void setupData();
        }

        protected override void OnSizeAllocated(double width, double height)
        {
            base.OnSizeAllocated(width, height);
            if ((this.Width > height) && alignmentWorkspace.landscape == false)
            {
                alignmentWorkspace.landscape = true;
                setLandscape();
                curWidth = this.Width;
                curHeight = this.Height;
            }
            else if ((this.Width < height) && alignmentWorkspace.landscape == true)
            {
                alignmentWorkspace.landscape = false;
                setPortrait();
                curWidth = this.Width;
                curHeight = this.Height;
            }
        }

        private void setLandscape()
        {
            if (gridSearchPanel.IsVisible == true)
            {
                gridSearchPanel.IsVisible = false;
                buttonSearchPanel.Image = "SearchIcon.png";
            }

            if(Device.RuntimePlatform == Device.iOS)
            {
                fullGrid.Padding = new Thickness(0, 0, 0, 0);
            }
            hideEntries();

            mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPositionZoom(new Xamarin.Forms.GoogleMaps.Position(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude), mapObject.CameraPosition.Zoom);

            var curID = 0;

            if (gridChainagePanel.IsVisible == true)
                curID = 1;
            else if (gridENPanel.IsVisible == true)
                curID = 2;
            else if (gridLatLongPanel.IsVisible == true)
                curID = 3;
            else if (gridPcodePanel.IsVisible == true)
                curID = 4;

            gridEditPinPanel.IsVisible = false;
            gridPinButtonPanel.IsVisible = false;
            buttonPinDropDown.Image = "DropDownSmall.png";
            fullGrid.Children.Remove(gridMenu);
            fullGrid.RowDefinitions.Clear();
            fullGrid.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
            fullGrid.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
            fullGrid.ColumnDefinitions.Clear();
            fullGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(fullGrid.Width, GridUnitType.Absolute) });
            fullGrid.Children.Remove(gridLinesSelector);
            fullGrid.Children.Remove(gridSimpleColorPicker);
            fullGrid.Children.Remove(gridColorPicker);

            gridMain.Children.Add(gridFileContent, 1, 0);

            gridColorPicker.ColumnDefinitions.Clear();
            gridColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.2, GridUnitType.Star) });
            gridColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.6, GridUnitType.Star) });
            gridColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.2, GridUnitType.Star) });

            gridSimpleColorPicker.ColumnDefinitions.Clear();
            gridSimpleColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.2, GridUnitType.Star) });
            gridSimpleColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.6, GridUnitType.Star) });
            gridSimpleColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.2, GridUnitType.Star) });

            gridLinesSelector.ColumnDefinitions.Clear();
            gridLinesSelector.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.2, GridUnitType.Star) });
            gridLinesSelector.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.6, GridUnitType.Star) });
            gridLinesSelector.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.2, GridUnitType.Star) });




            gridMain.RowDefinitions.Clear();
            gridMain.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
            gridMain.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
            gridMain.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
            gridMain.ColumnDefinitions.Clear();
            gridMain.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.15, GridUnitType.Star) });
            gridMain.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });

            buttonLandscapeChOff.Opacity = 0.5;
            buttonLandscapeEN.Opacity = 0.5;
            buttonLandscapeLL.Opacity = 0.5;
            buttonLandscapePcode.Opacity = 0.5;

            gridMain.Children.Add(gridLandscapeMenu, 0, 0);
            gridLandscapeMenu.IsVisible = true;
            gridMapPanel.IsVisible = true;

            if (gridFileContent.IsVisible == false)
            {
                gridFileContent.IsVisible = false;
                gridLandscapeChainage.IsVisible = false;
                gridLandscapeEN.IsVisible = false;
                gridLandscapeLL.IsVisible = false;
                gridLandscapePcode.IsVisible = false;
                buttonLandscapeChOff.IsVisible = true;
                buttonLandscapePhoto.IsVisible = true;
                buttonLandscapeEN.IsVisible = true;
                buttonLandscapePhoto.IsVisible = true;
                buttonLandscapeLL.IsVisible = true;
                buttonLandscapePcode.IsVisible = true;
                labelAccuracyMain.IsVisible = true;
                labelLandAccuracy.IsVisible = true;

                buttonLandscapeFile.Image = "SettingsIcon.png";

                switch (curID)
                {
                    case 0:
                        gridLandscapeChainage.IsVisible = true;
                        buttonLandscapeChOff.Opacity = 1;
                        gridMain.Children.Add(gridLandscapeChainage, 1, 0);
                        break;
                    case 1:
                        gridLandscapeChainage.IsVisible = true;
                        buttonLandscapeChOff.Opacity = 1;
                        gridMain.Children.Add(gridLandscapeChainage, 1, 0);
                        break;
                    case 2:
                        gridLandscapeEN.IsVisible = true;
                        buttonLandscapeEN.Opacity = 1;
                        gridMain.Children.Add(gridLandscapeEN, 1, 0);
                        break;
                    case 3:
                        gridLandscapeLL.IsVisible = true;
                        buttonLandscapeLL.Opacity = 1;
                        gridMain.Children.Add(gridLandscapeLL, 1, 0);
                        break;
                    case 4:
                        gridLandscapePcode.IsVisible = true;
                        buttonLandscapePcode.Opacity = 1;
                        gridMain.Children.Add(gridLandscapePcode, 1, 0);
                        break;
                }
            }

            if (gridFileContent.IsVisible == true)
            {
                buttonLandscapeChOff.IsVisible = false;
                buttonLandscapeEN.IsVisible = false;
                buttonLandscapeLL.IsVisible = false;
                buttonLandscapePcode.IsVisible = false;
                gridLandscapeChainage.IsVisible = false;
                gridLandscapeEN.IsVisible = false;
                gridLandscapeLL.IsVisible = false;
                buttonLandscapePhoto.IsVisible = false;
                gridLandscapePcode.IsVisible = false;
                gridFileContent.IsVisible = true;
                buttonLandscapeFile.Image = "Back.png";
                buttonFile.IsVisible = true;
                labelAccuracyMain.IsVisible = false;
                labelLandAccuracy.IsVisible = false;
            }

            gridMain.Children.Add(gridLinesSelector, 1, 0);
            gridMain.Children.Add(gridSimpleColorPicker, 1, 0);
            gridMain.Children.Add(gridColorPicker, 1, 0);

            try
            {
                gridMain.MinimumWidthRequest = fullGrid.Width;
            }
            catch
            {
                DisplayAlert("Error", "failed to set width", "Ok");
            }

            labelLandPcode.Text = labelPcode.Text;
            gridMain.IsVisible = true;


            if (gridCrossection.IsVisible == true)
            {
                try
                {
                    redrawCanvas();
                }
                catch
                {
                    DisplayAlert("Error", "Failed to redraw Canvas (landscape)", "Ok");
                }
            }
        }

        private void setPortrait()
        {
            if(Device.RuntimePlatform == Device.iOS)
            {
                fullGrid.Padding = new Thickness(0, 20, 0, 0);
            }
            hideEntries();

            var panelID = 0;

            if (gridLandscapeChainage.IsVisible == true)
                panelID = 1;
            else if (gridLandscapeEN.IsVisible == true)
                panelID = 2;
            else if (gridLandscapeLL.IsVisible == true)
                panelID = 3;
            else if (gridLandscapePcode.IsVisible == true)
                panelID = 4;

            mapObject.MoveToRegion(MapSpan.FromCenterAndRadius(new Xamarin.Forms.GoogleMaps.Position(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude), Distance.FromMeters(200)));

            fullGrid.RowDefinitions.Clear();
            fullGrid.RowDefinitions.Add(new RowDefinition { Height = new GridLength(.1, GridUnitType.Star) });
            fullGrid.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
            fullGrid.ColumnDefinitions.Clear();
            fullGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(fullGrid.Width, GridUnitType.Absolute) });

            gridMain.Children.Remove(gridLandscapeMenu);
            gridMain.Children.Remove(gridFileContent);
            fullGrid.Children.Add(gridFileContent, 0, 1);
            gridMain.RowDefinitions.Clear();
            gridMain.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
            gridMain.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Auto) });
            gridMain.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Auto) });
            gridMain.ColumnDefinitions.Clear();
            gridMain.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(0, GridUnitType.Absolute) });
            gridMain.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            gridMain.Children.Remove(gridLinesSelector);
            gridMain.Children.Remove(gridColorPicker);
            gridMain.Children.Remove(gridSimpleColorPicker);
            fullGrid.Children.Add(gridMenu, 0, 0);

            gridMain.Children.Remove(gridLandscapeChainage);
            gridMain.Children.Remove(gridLandscapeEN);
            gridMain.Children.Remove(gridLandscapeLL);
            gridMain.Children.Remove(gridLandscapePcode);

            gridColorPicker.ColumnDefinitions.Clear();
            gridColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.1, GridUnitType.Star) });
            gridColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.8, GridUnitType.Star) });
            gridColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.1, GridUnitType.Star) });

            gridSimpleColorPicker.ColumnDefinitions.Clear();
            gridSimpleColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.1, GridUnitType.Star) });
            gridSimpleColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.8, GridUnitType.Star) });
            gridSimpleColorPicker.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.1, GridUnitType.Star) });

            gridLinesSelector.ColumnDefinitions.Clear();
            gridLinesSelector.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.1, GridUnitType.Star) });
            gridLinesSelector.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.8, GridUnitType.Star) });
            gridLinesSelector.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(.1, GridUnitType.Star) });

            fullGrid.Children.Add(gridLinesSelector, 0, 1);
            fullGrid.Children.Add(gridSimpleColorPicker, 0, 1);
            fullGrid.Children.Add(gridColorPicker, 0, 1);

            fullGrid.Children.Add(gridFileContent, 0, 1);


            if (gridFileContent.IsVisible == true)
            {

                gridFileContent.IsVisible = true;
                buttonFile.Image = "Back.png";
                buttonNRGIcon.IsVisible = false;
                buttonCHPhoto.IsVisible = false;
                buttonPinDropDown.IsVisible = false;
                labelAccuracyMain.IsVisible = false;
            }
            else
            {
                buttonPinDropDown.IsVisible = true;
                buttonNRGIcon.IsVisible = true;
                buttonCHPhoto.IsVisible = true;
                buttonFile.Image = "SettingsIcon.png";
                labelAccuracyMain.IsVisible = true;
            }

            gridChainagePanel.IsVisible = false;
            gridENPanel.IsVisible = false;
            gridLatLongPanel.IsVisible = false;
            gridPcodePanel.IsVisible = false;
            buttonChOff.Opacity = 0.5;
            buttonEN.Opacity = 0.5;
            buttonLatLong.Opacity = 0.5;
            buttonPcode.Opacity = 0.5;

            switch (panelID)
            {
                case 0:
                    gridChainagePanel.IsVisible = true;
                    buttonChOff.Opacity = 1;
                    break;
                case 1:
                    gridChainagePanel.IsVisible = true;
                    buttonChOff.Opacity = 1;
                    break;
                case 2:
                    gridENPanel.IsVisible = true;
                    buttonEN.Opacity = 1;
                    break;
                case 3:
                    gridLatLongPanel.IsVisible = true;
                    buttonLatLong.Opacity = 1;
                    break;
                case 4:
                    gridPcodePanel.IsVisible = true;
                    buttonPcode.Opacity = 1;
                    break;
            }
            if(gridCrossection.IsVisible == true)
            {
                try
                {
                    redrawCanvas();
                }
                catch
                {
                    DisplayAlert("Error", "Failed to Redraw Canvas", "Ok");
                }
            }
            else
            {
                labelCurMapStatus.Text = "Enabled";
                mapObject.IsVisible = true;
                gridMapPanel.RowDefinitions.Clear();
                gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                getRadius();
                mapBoxView.IsVisible = true;
                frameScale.IsVisible = true;
                labelScale.IsVisible = true;
                frameScaleLeft.IsVisible = true;
                frameScaleRight.IsVisible = true;
                frameScaleCenter.IsVisible = true;
            }



        }

        private string getDecimalDisplay(double value)
        {
            try
            {
                switch (alignmentWorkspace.decimalDisplay)
                {
                    case 0:
                        return value.ToString("0");
                    case 1:
                        return value.ToString("0.0");
                    case 2:
                        return value.ToString("0.00");
                    case 3:
                        return value.ToString("0.000");
                }
            }
            catch
            {
                DisplayAlert("Error", "error code: 2", "OK");
            }
            return null;
        }

        private void createCrossSection()
        {
            try
            {
                CrossSettings.Current.AddOrUpdateValue("curLat", mapObject.CameraPosition.Target.Latitude);
                CrossSettings.Current.AddOrUpdateValue("curLng", mapObject.CameraPosition.Target.Longitude);
                CrossSettings.Current.AddOrUpdateValue("curRadius", mapObject.VisibleRegion.Radius.Meters);
                alignmentWorkspace.devicePosition = new Xamarin.Forms.GoogleMaps.Position(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPositionZoom(alignmentWorkspace.devicePosition, mapObject.CameraPosition.Zoom);
            }
            catch
            {

            }
            xScaleCanvasView.IsEnabled = true;
            xScaleCanvasView.IsVisible = true;
            yScaleCanvasView.IsEnabled = true;
            yScaleCanvasView.IsVisible = true;
            canvasView.IsEnabled = true;
            canvasView.IsVisible = true;
            gridCrossection.IsVisible = true;
            gridMapPanel.RowDefinitions[0].Height = new GridLength(.5, GridUnitType.Star);
            gridMapPanel.RowDefinitions[1].Height = new GridLength(.5, GridUnitType.Star);

            if (alignmentWorkspace.scale == true)
            {
                enableScale();
            }
            else
                disableScale();

            gridCrossection.IsVisible = true;
            try
            {
                createIntersection();
                //if (toUpdate == false)
                //{
                //    redrawCanvas();
                //}
            }
            catch
            {

            }
            redrawCanvas();
        }

        private void removeCrossSection()
        {
            if(mapObject.VisibleRegion != null)
            {
                CrossSettings.Current.AddOrUpdateValue("curLat", mapObject.CameraPosition.Target.Latitude);
                CrossSettings.Current.AddOrUpdateValue("curLng", mapObject.CameraPosition.Target.Longitude);
                CrossSettings.Current.AddOrUpdateValue("curRadius", mapObject.VisibleRegion.Radius.Meters);
                alignmentWorkspace.devicePosition = new Xamarin.Forms.GoogleMaps.Position(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                mapObject.InitialCameraUpdate = CameraUpdateFactory.NewPositionZoom(alignmentWorkspace.devicePosition, mapObject.CameraPosition.Zoom);
            }


            gridMapPanel.RowDefinitions[0].Height = new GridLength(1, GridUnitType.Star);
            gridMapPanel.RowDefinitions[1].Height = new GridLength(0, GridUnitType.Star);
            gridCrossection.IsVisible = false;
            canvasView.IsVisible = false;
            xScaleCanvasView.IsVisible = false;
            xScaleCanvasView.IsEnabled = false;
            yScaleCanvasView.IsVisible = false;
            yScaleCanvasView.IsEnabled = false;
            canvasView.IsEnabled = false;
        }

        public List<dataTypes.nrgVector2> mergeSort(List<dataTypes.nrgVector2> list)
        {
            if (list.Count <= 1)
                return list;

            var left = new List<dataTypes.nrgVector2>();
            var right = new List<dataTypes.nrgVector2>();

            for (int i = 0; i < list.Count(); i++)
            {
                if (i < list.Count / 2)
                    left.Add(list[i]);
                else
                    right.Add(list[i]);
            }

            left = mergeSort(left);
            right = mergeSort(right);

            return merge(left, right);
        }

        public List<dataTypes.nrgVector2> merge(List<dataTypes.nrgVector2> left, List<dataTypes.nrgVector2> right)
        {
            var result = new List<dataTypes.nrgVector2>();
            while (left.Count != 0 && right.Count != 0)
            {
                if (left.First().X <= right.First().X)
                {
                    result.Add(left.First());
                    left.RemoveAt(0);
                }
                else
                {
                    result.Add(right.First());
                    right.RemoveAt(0);
                }
            }

            while (left.Count != 0)
            {
                result.Add(left.First());
                left.RemoveAt(0);
            }
            while (right.Count != 0)
            {
                result.Add(right.First());
                right.RemoveAt(0);
            }
            return result;


        }

        void createIntersection()
        {
            if (alignmentWorkspace.currentAlignment == "None Found" || alignmentWorkspace.currentAlignment == "None Selected" || alignmentWorkspace.currentAlignment == null || labelCh.Text == "????")
            {
                createIntersectionWithoutAlignment();
            }
            else
            {
                try
                {
                    var lat = mapObject.CameraPosition.Target.Latitude;
                    var lng = mapObject.CameraPosition.Target.Longitude;

                    var estNrth = GetENFromLL(lat, lng);

                    var inVec = new workspace.dataTypes.nrgVector2 { X = estNrth.X, Y = estNrth.Y };
                    var outVec = new workspace.dataTypes.nrgAlignmentVector5();

                    double ch = -999;
                    if (alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(inVec, ref outVec))
                    {
                        ch = outVec.chainage;
                        curOffset = outVec.offset;
                    }

                    double offCenter = 0;
                    if (gridOffsetCenter.IsVisible == true)
                        double.TryParse(entryOffsetCenter.Text, out offCenter);
                    if (ch != -999 && toUpdate == true)
                    {
                        if (alignmentWorkspace.locked == false)
                        {
                            scale.scaleX = 1;
                            scale.scaleY = 1;
                            scale.transX = 0;
                            scale.transY = 0;
                            x = 0;
                            y = 0;
                        }

                        var en1 = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(ch, offCenter - (alignmentWorkspace.distance) / 2);
                        var en2 = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(ch, offCenter + (alignmentWorkspace.distance) / 2);
                        var en3 = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(ch, offCenter);
                        var inCH = new dataTypes.nrgVector2 { X = en3.X, Y = en3.Y };
                        var outCH = new dataTypes.nrgAlignmentVector5();
                        alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(inCH, ref outCH);
                        var ele = outCH.element;

                        var brg = helpers.RPC(en1.X, en1.Y, en2.X, en2.Y);
                        Dictionary<string, List<dataTypes.nrgVector2>> dictionaryIntersect = new Dictionary<string, List<dataTypes.nrgVector2>>();
                        foreach (string item in crossSectionLineList)
                        {
                            if (dictionaryLines.ContainsKey(item))
                            {
                                var lineList = dictionaryLines[item];
                                var intersectList = new List<dataTypes.nrgVector2>();
                                foreach (dataTypes.Line line in lineList)
                                {
                                    //Segments
                                    for (int i = 0; i < line.coords.Count() - 1; i++)
                                    {
                                        var test1 = helpers.functionOnLine3(en1.X, en1.Y, en2.X, en2.Y, line.coords[i].x, line.coords[i].y, 0.001);
                                        var test2 = helpers.functionOnLine3(en1.X, en1.Y, en2.X, en2.Y, line.coords[i + 1].x, line.coords[i + 1].y, 0.001);
                                        if (test1 != test2)
                                        {

                                            line.brg = helpers.RPC(line.coords[i].x, line.coords[i].y, line.coords[i + 1].x, line.coords[i + 1].y);
                                            if (line.coords[i].z != -999 && line.coords[i + 1].z != -999)
                                            {
                                                var intersect = helpers.IntersectTwoBrgLines(line.coords[i].x, line.coords[i].y, line.brg, en1.X, en1.Y, brg);

                                                if (helpers.functionInBox1(line.coords[i].x, line.coords[i].y, line.coords[i + 1].x, line.coords[i + 1].y, intersect.X, intersect.Y) && helpers.functionInBox1(en1.X, en1.Y, en2.X, en2.Y, intersect.X, intersect.Y))
                                                {
                                                    intersect.Z = helpers.FNInterpolateY(0, line.coords[i].z, helpers.FNDistance(line.coords[i].x, line.coords[i].y, line.coords[i + 1].x, line.coords[i + 1].y), line.coords[i + 1].z, helpers.FNDistance(line.coords[i].x, line.coords[i].y, intersect.X, intersect.Y));
                                                    var inVector = new dataTypes.nrgVector2 { X = intersect.X, Y = intersect.Y };
                                                    var outVector = new dataTypes.nrgAlignmentVector5();

                                                    if (alignmentWorkspace.Alignments.horizontal.Compare(inVector, ele, outVector, ele - 1))
                                                    {
                                                        var dataToAdd = new dataTypes.nrgVector2 { X = outVector.offset, Y = intersect.Z };
                                                        intersectList.Add(dataToAdd);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                intersectList = mergeSort(intersectList);
                                if (intersectList.Count() >= 2)
                                    dictionaryIntersect.Add(item, intersectList);
                            }
                        }
                        if (elevationFlag == true)
                        {
                            var data = getElevationData(ch, offCenter);
                            if (data != null)
                                dictionaryIntersect.Add("Google Section", data);
                            elevationFlag = false;
                            toUpdate = false;
                            buttonCrossSectionPause.Image = "Play.png";

                        }

                        if (dictionaryIntersect.Count() >= 1)
                            setIntersectPoints(dictionaryIntersect);
                        else
                            isData = false;
                    }
                }
                catch
                {
                    isData = false;
                }
            }            
        }

        void createIntersectionWithoutAlignment()
        {
            try
            {
                if (mapObject.Circles.Count() != 0)
                {
                    mapObject.Circles.Clear();
                }

                var lat = mapObject.CameraPosition.Target.Latitude;
                var lng = mapObject.CameraPosition.Target.Longitude;
                
                var estNrth = GetENFromLL(lat, lng);
                if (toUpdate == true)
                {
                    if (alignmentWorkspace.locked == false)
                    {
                        scale.scaleX = 1;
                        scale.scaleY = 1;
                        scale.transX = 0;
                        scale.transY = 0;
                        x = 0;
                        y = 0;
                    }

                    dataTypes.nrgVector2 en1;
                    dataTypes.nrgVector2 en2;

                    en1 = helpers.PRC(estNrth.X, estNrth.Y, (helpers.degToRad(alignmentWorkspace.heading) + (Math.PI / 2)), (alignmentWorkspace.distance / 2), 1);
                    en2 = helpers.PRC(estNrth.X, estNrth.Y, (helpers.degToRad(alignmentWorkspace.heading) - (Math.PI / 2)), (alignmentWorkspace.distance / 2), 1);
                    curOffset = 0;
                    var brg = helpers.RPC(en1.X, en1.Y, en2.X, en2.Y);
                    Dictionary<string, List<dataTypes.nrgVector2>> dictionaryIntersect = new Dictionary<string, List<dataTypes.nrgVector2>>();
                    foreach (string item in crossSectionLineList)
                    {
                        if (dictionaryLines.ContainsKey(item))
                        {
                            var lineList = dictionaryLines[item];
                            var intersectList = new List<dataTypes.nrgVector2>();
                            foreach (dataTypes.Line line in lineList)
                            {
                                //Segments
                                for (int i = 0; i < line.coords.Count() - 1; i++)
                                {
                                    var test1 = helpers.functionOnLine3(en1.X, en1.Y, en2.X, en2.Y, line.coords[i].x, line.coords[i].y, 0.001);
                                    var test2 = helpers.functionOnLine3(en1.X, en1.Y, en2.X, en2.Y, line.coords[i + 1].x, line.coords[i + 1].y, 0.001);
                                    if (test1 != test2)
                                    {

                                        line.brg = helpers.RPC(line.coords[i].x, line.coords[i].y, line.coords[i + 1].x, line.coords[i + 1].y);
                                        if (line.coords[i].z != -999 && line.coords[i + 1].z != -999)
                                        {
                                            var intersect = helpers.IntersectTwoBrgLines(line.coords[i].x, line.coords[i].y, line.brg, en1.X, en1.Y, brg);

                                            if (helpers.functionInBox1(line.coords[i].x, line.coords[i].y, line.coords[i + 1].x, line.coords[i + 1].y, intersect.X, intersect.Y) && helpers.functionInBox1(en1.X, en1.Y, en2.X, en2.Y, intersect.X, intersect.Y))
                                            {
                                                intersect.Z = helpers.FNInterpolateY(0, line.coords[i].z, helpers.FNDistance(line.coords[i].x, line.coords[i].y, line.coords[i + 1].x, line.coords[i + 1].y), line.coords[i + 1].z, helpers.FNDistance(line.coords[i].x, line.coords[i].y, intersect.X, intersect.Y));

                                                var dataToAdd = new dataTypes.nrgVector2 { X = intersect.X - estNrth.X, Y = intersect.Z };
                                                intersectList.Add(dataToAdd);
                                            }
                                        }
                                    }
                                }
                            }
                            intersectList = mergeSort(intersectList);
                            if (intersectList.Count() >= 2)
                                dictionaryIntersect.Add(item, intersectList);
                        }
                    }
                    if (elevationFlag == true)
                    {
                        var data = getElevationData(estNrth.X, estNrth.Y);
                        if (data != null)
                            dictionaryIntersect.Add("Google Section", data);
                        elevationFlag = false;
                        toUpdate = false;
                        buttonCrossSectionPause.Image = "Play.png";
                    }

                    if (dictionaryIntersect.Count() >= 1)
                        setIntersectPoints(dictionaryIntersect);
                    else
                        isData = false;
                }
            }
            catch
            {
                isData = false;
            }

        }

        void OnCanvasViewPaintSurface(object sender, SKPaintSurfaceEventArgs e)
        {
            SKImageInfo info = e.Info;
            SKSurface surface = e.Surface;
            SKCanvas canvas = surface.Canvas;


            SKPaint thinBlackLine = new SKPaint
            {
                Style = SKPaintStyle.Stroke,
                Color = SKColors.Black,
                StrokeWidth = 2 / (float)scale.scaleX
            };

            SKPaint curOffsetLine = new SKPaint
            {
                Style = SKPaintStyle.Stroke,
                Color = SKColors.LightBlue,
                StrokeWidth = 2 / (float)scale.scaleX
            };

            canvas.Clear();
            if(isData == true)
            {
                if (crossSection.minW < 0 && crossSection.maxH < 0)
                {
                    var centerpoint = crossSection.w;
                    if (curOffset < 0)
                        canvas.DrawLine(centerpoint - ((float)Math.Abs(curOffset) * crossSection.ppw), crossSection.h, centerpoint - ((float)Math.Abs(curOffset) * crossSection.ppw), 0, curOffsetLine);
                    else if (curOffset >= 0)
                        canvas.DrawLine(centerpoint + ((float)Math.Abs(curOffset) * crossSection.ppw), crossSection.h, centerpoint + ((float)Math.Abs(curOffset) * crossSection.ppw), 0, curOffsetLine);
                }
                else if (crossSection.minW < 0 && crossSection.maxW >= 0)
                {
                    var centerpoint = (float)(Math.Abs(crossSection.minW) * crossSection.ppw);
                    if (curOffset < 0)
                        canvas.DrawLine(centerpoint - ((float)Math.Abs(curOffset) * crossSection.ppw), crossSection.h, centerpoint - ((float)Math.Abs(curOffset) * crossSection.ppw), 0, curOffsetLine);
                    else if (curOffset >= 0)
                        canvas.DrawLine(centerpoint + ((float)Math.Abs(curOffset) * crossSection.ppw), crossSection.h, centerpoint + ((float)Math.Abs(curOffset) * crossSection.ppw), 0, curOffsetLine);
                }
                else if (crossSection.minW >= 0)
                {
                    var centerpoint = 0;
                    if (curOffset < 0)
                        canvas.DrawLine(centerpoint - ((float)Math.Abs(curOffset) * crossSection.ppw), crossSection.h, centerpoint - ((float)Math.Abs(curOffset) * crossSection.ppw), 0, curOffsetLine);
                    else if (curOffset >= 0)
                        canvas.DrawLine(centerpoint + ((float)Math.Abs(curOffset) * crossSection.ppw), crossSection.h, centerpoint + ((float)Math.Abs(curOffset) * crossSection.ppw), 0, curOffsetLine);
                }


                foreach (string key in dictionaryIntersectPoints.Keys)
                {
                    var intersectPoints = dictionaryIntersectPoints[key];
                    thinBlackLine.Color = SKColor.Parse(CrossSettings.Current.GetValueOrDefault("LineColor" + key, "#000000"));
                    if (intersectPoints.Count >= 2)
                    {
                        for (int i = 0; i < intersectPoints.Count() - 1; i++)
                        {
                            canvas.DrawLine((float)intersectPoints[i].X, (float)intersectPoints[i].Y, (float)intersectPoints[i + 1].X, (float)intersectPoints[i + 1].Y, thinBlackLine);

                        }
                    }
                }
               
                canvas.Scale((float)scale.scaleX, (float)scale.scaleY, (float)(info.Width / 2), (float)(info.Height / 2));
                canvas.Translate(new SKPoint((float)scale.transX, (float)scale.transY));

            }
            else
            {
                SKPaint largeBlackText = new SKPaint
                {
                    TextSize = 20f * App.density,
                    Color = SKColors.Black,
                    IsAntialias = true,
                    StrokeWidth = 5,
                    TextScaleX = 1,
                    TextAlign = SKTextAlign.Center
                    
                };

                canvas.DrawText("No data to show", canvasView.CanvasSize.Width/2, (canvasView.CanvasSize.Height / 2) + (20 * App.density), largeBlackText);
            }
            


    
        }

        public bool redrawCanvas()
        {
            if (gridCrossection.IsVisible)
            {
                canvasView.InvalidateSurface();
                if(alignmentWorkspace.scale == true && xrepaint == false && yrepaint == false)
                {
                    xrepaint = true;
                    yrepaint = true;
                    xScaleCanvasView.InvalidateSurface();
                    yScaleCanvasView.InvalidateSurface();
                }
                else
                {
                    if (xrepaint == true)
                        xScaleCanvasView.InvalidateSurface();
                    if (yrepaint == true)
                        yScaleCanvasView.InvalidateSurface();
                }
            }
            return true;
        }

        private void xScaleCanvasView_PaintSurface(object sender, SKPaintSurfaceEventArgs e)
        {
            if(xrepaint == true && crossSection.wd != 0 && alignmentWorkspace.scale == true && isData == true && crossSection.h > 0 && crossSection.w > 0)
            {
                SKImageInfo info = e.Info;
                SKSurface surface = e.Surface;
                SKCanvas canvas = surface.Canvas;

                canvas.Clear();

                canvas.Scale((float)scale.scaleX, 1, (info.Width / 2), info.Height);
                canvas.Translate(new SKPoint((float)scale.transX, 0f));

                SKPaint thinBlackLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.Black,
                    StrokeWidth = 3 / (float)scale.scaleX,
                    StrokeCap = SKStrokeCap.Square
                };

                SKPaint thickBlueLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.Blue,
                    StrokeWidth = 5 / (float)scale.scaleX
                };

                SKPaint thinnerBlackLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.Black,
                    StrokeWidth = 1 / (float)scale.scaleX
                };

                SKPaint curOffsetLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.LightBlue,
                    StrokeWidth = 1 / (float)scale.scaleX
                };

                SKPaint smallBlackText = new SKPaint
                {
                    TextSize = 7.5f * App.density,
                    Color = SKColors.Black,
                    IsAntialias = true,
                    TextAlign = SKTextAlign.Center,
                    StrokeWidth = 2,
                    TextScaleX = 1 / (float)scale.scaleX
                };

                SKPaint thinGrayPaint = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.LightGray,
                    StrokeWidth = 1 
                };

                var segmentScale = 10 * (crossSection.wd / (crossSection.w / crossSection.ppw));
                var scaleXTo = setSegmentScale(crossSection.wd / segmentScale);
                var segDist = (float)(crossSection.ppw * scaleXTo);
                var segments = (float)(Math.Max(Math.Ceiling(crossSection.wd / scaleXTo), 10));

                var h = (float)(xScaleCanvasView.CanvasSize.Height);
                var w = (float)(xScaleCanvasView.CanvasSize.Width);
                var textStart = (float)(12.5f * App.density);
                var lineEnd = (float)(10f * App.density);
                var shortLineEnd = (float)(5f * App.density);

                if(alignmentWorkspace.centeredByOffset == false)
                {
                    if (crossSection.minW < 0 && crossSection.maxW >= 0)
                    {
                        var centerpoint = (float)(Math.Abs(crossSection.minW) * crossSection.ppw);
                        var rightDist = crossSection.maxW * crossSection.ppw;
                        //var segDist = (float)((w / crossSection.wd) * scaleXTo);

                        for (int i = 0; i <= segments; i++)
                        {
                            canvas.DrawLine(centerpoint + (segDist * i), h, centerpoint + (segDist * i), h - lineEnd, thinBlackLine);
                            if (i > 0)
                                canvas.DrawText("" + scaleXTo * i, centerpoint + (segDist * i), h - textStart, smallBlackText);
                            for (int j = 1; j < 10; j++)
                                canvas.DrawLine(centerpoint + ((segDist * i) + (segDist / 10) * j), h, centerpoint + ((segDist * i) + (segDist / 10) * j), h - shortLineEnd, thinnerBlackLine);
                        }

                        for (int i = 0; i <= segments; i++)
                        {
                            canvas.DrawLine(centerpoint - (segDist * i), h, centerpoint - (segDist * i), h - lineEnd, thinBlackLine);
                            if (i > 0)
                                canvas.DrawText("-" + scaleXTo * i, centerpoint - (segDist * i), h - textStart, smallBlackText);
                            for (int j = 1; j < 10; j++)
                                canvas.DrawLine(centerpoint - ((segDist * i) + (segDist / 10) * j), h, centerpoint - ((segDist * i) + (segDist / 10) * j), h - shortLineEnd, thinnerBlackLine);
                        }
                        //var offset = (float)((Math.Round(rightDist / segDist)) + 1);
                        //var offset2 = (float)((Math.Round(centerpoint / segDist)) + 1);
                        var offset = segments + 1;
                        var offset2 = segments + 1;
                        canvas.DrawLine(centerpoint, h, centerpoint, 2, thickBlueLine);
                        canvas.DrawLine(centerpoint + (segDist * offset), h, centerpoint + (segDist * offset), h - lineEnd, thinBlackLine);
                        canvas.DrawLine(centerpoint - (segDist * offset2), h, centerpoint - (segDist * offset2), h - lineEnd, thinBlackLine);
                        canvas.DrawText("" + scaleXTo * offset, centerpoint + (segDist * offset), h - textStart, smallBlackText);
                        canvas.DrawText("" + scaleXTo * -offset2, centerpoint - (segDist * offset2), h - textStart, smallBlackText);
                        canvas.DrawLine(centerpoint + (segDist * offset), 1, centerpoint - (segDist * offset2), 1, thinGrayPaint);
                        xrepaint = false; 

                    }
                    else if (crossSection.minW < 0 && crossSection.maxW < 0)
                    {
                        var centerpoint = w;
                        //var segDist = (float)((w / crossSection.wd) * scaleXTo);

                        for (int i = 0; i <= segments; i++)
                        {
                            canvas.DrawLine(centerpoint - (segDist * i), h, centerpoint - (segDist * i), h - lineEnd, thinBlackLine);
                            if (i > 0)
                                canvas.DrawText("-" + scaleXTo * i, centerpoint - (segDist * i), h - textStart, smallBlackText);
                            for (int j = 1; j < 10; j++)
                                canvas.DrawLine(centerpoint - ((segDist * i) + (segDist / 10) * j), h, centerpoint - ((segDist * i) + (segDist / 10) * j), h - shortLineEnd, thinnerBlackLine);
                        }

                        //var offset = (float)((Math.Round(centerpoint / segDist)) + 1);
                        var offset = segments + 1;
                        canvas.DrawLine(centerpoint, h, centerpoint, 2, thickBlueLine);
                        canvas.DrawLine(centerpoint - (segDist * (float)((Math.Round(centerpoint / segDist)) + 1)), h, centerpoint - (segDist * (float)((Math.Round(centerpoint / segDist)) + 1)), h - lineEnd, thinBlackLine);
                        canvas.DrawLine(centerpoint + (segDist * 1), h, centerpoint + (segDist * 1), h - lineEnd, thinBlackLine);
                        canvas.DrawText("-" + scaleXTo * offset, centerpoint - (segDist * offset), h - textStart, smallBlackText);
                        canvas.DrawLine(centerpoint + (segDist * offset), 1, centerpoint - (segDist * offset), 1, thinGrayPaint);
                        xrepaint = false;
                    }
                    else if (crossSection.minW >= 0)
                    {
                        var centerpoint = 0;
                        var rightDist = crossSection.maxW * crossSection.ppw;
                        //var segDist = (float)((w / crossSection.wd) * scaleXTo);

                        for (int i = 0; i <= segments; i++)
                        {
                            canvas.DrawLine(centerpoint + (segDist * i), h, centerpoint + (segDist * i), h - lineEnd, thinBlackLine);
                            if (i > 0)
                                canvas.DrawText("" + scaleXTo * i, centerpoint + (segDist * i), h - textStart, smallBlackText);
                            for (int j = 1; j < 10; j++)
                                canvas.DrawLine(centerpoint + ((segDist * i) + (segDist / 10) * j), h, centerpoint + ((segDist * i) + (segDist / 10) * j), h - shortLineEnd, thinnerBlackLine);
                        }
                        //var offset = (float)((Math.Round(rightDist / segDist)) + 1);
                        var offset = segments + 1;
                        canvas.DrawLine(centerpoint, h, centerpoint, 2, thickBlueLine);
                        canvas.DrawLine(centerpoint + (segDist * offset), h, centerpoint + (segDist * offset), h - lineEnd, thinBlackLine);
                        canvas.DrawLine(centerpoint - (segDist * 1), h, centerpoint - (segDist * 1), h - lineEnd, thinBlackLine);
                        canvas.DrawText("" + scaleXTo * offset, centerpoint + (segDist * offset), h - textStart, smallBlackText);
                        canvas.DrawLine(centerpoint + (segDist * offset), 1, centerpoint - (segDist * offset), 1, thinGrayPaint);
                        xrepaint = false;
                    }
                }
                else
                {
                    var centerpoint = (float)(w / 2);
                    //var segDist = (float)((w / crossSection.wd) * scaleXTo);

                    for(int i = 0; i<= segments; i++)
                    {
                        canvas.DrawLine(centerpoint + (segDist * i), h, centerpoint + (segDist * i), h - lineEnd, thinBlackLine);
                        if (i > 0 && alignmentWorkspace.offsetCenter + (scaleXTo * i) != 0)
                            canvas.DrawText("" + (alignmentWorkspace.offsetCenter + (scaleXTo * i)), centerpoint + (segDist * i), h - textStart, smallBlackText);
                        for (int j = 1; j < 10; j++)
                            canvas.DrawLine(centerpoint + ((segDist * i) + (segDist / 10) * j), h, centerpoint + ((segDist * i) + (segDist / 10) * j), h - shortLineEnd, thinnerBlackLine);
                    }

                    for(int i = 0; i <= segments; i++)
                    {
                        canvas.DrawLine(centerpoint - (segDist * i), h, centerpoint - (segDist * i), h - lineEnd, thinBlackLine);
                        if (i > 0 && alignmentWorkspace.offsetCenter - (scaleXTo * i) != 0)
                            canvas.DrawText("" + (alignmentWorkspace.offsetCenter - (scaleXTo * i)), centerpoint - (segDist * i), h - textStart, smallBlackText);
                        for (int j = 1; j < 10; j++)
                            canvas.DrawLine(centerpoint - ((segDist * i) + (segDist / 10) * j), h, centerpoint - ((segDist * i) + (segDist / 10) * j), h - shortLineEnd, thinnerBlackLine);
                    }
                    //var offset = (float)((Math.Round(centerpoint / segDist)) + 1);
                    var offset = segments + 1;
                    if(alignmentWorkspace.offsetCenter != 0)
                        canvas.DrawText("" + alignmentWorkspace.offsetCenter, centerpoint, h - textStart, smallBlackText);
                    canvas.DrawLine(centerpoint - (float)(alignmentWorkspace.offsetCenter * crossSection.ppw), h, centerpoint - (float)(alignmentWorkspace.offsetCenter * crossSection.ppw), 2, thickBlueLine);
                    canvas.DrawLine(centerpoint + (segDist * offset), h, centerpoint + (segDist * offset), h - lineEnd, thinBlackLine);
                    canvas.DrawLine(centerpoint - (segDist * offset), h, centerpoint - (segDist * offset), h - lineEnd, thinBlackLine);
                    if(alignmentWorkspace.offsetCenter + (scaleXTo * offset) != 0)
                        canvas.DrawText("" + (alignmentWorkspace.offsetCenter + (scaleXTo * offset)), centerpoint + (segDist * offset), h - textStart, smallBlackText);
                    if(alignmentWorkspace.offsetCenter + (scaleXTo * -offset) != 0)
                        canvas.DrawText("" + (alignmentWorkspace.offsetCenter + (scaleXTo * -offset)), centerpoint - (segDist * offset), h - textStart, smallBlackText);
                    canvas.DrawLine(centerpoint + (segDist * offset), 1, centerpoint - (segDist * offset), 1, thinGrayPaint);
                    xrepaint = false;
                }
            }
        }

        private void colorButtonClicked(object sender, EventArgs e)
        {
            var button = (Button)sender;
            colorFrame2.BackgroundColor = button.BackgroundColor;
            if (curLineID == 0)
                listLineGrid[0].Children[1].BackgroundColor = button.BackgroundColor;
            else
                listLineGrid[curLineID].Children[3].BackgroundColor = button.BackgroundColor;
            Label label = (Label)listLineGrid[curLineID].Children[0];
            gridSimpleColorPicker.IsVisible = false;
            gridLinesList.IsVisible = true;;

        }

        private void collapseAndExpand(object sender, EventArgs e)
        {
            var grid = (Grid)sender;
            var upStack = (StackLayout)grid.Parent;
            var stack = (StackLayout)upStack.Children[0];
            var curstack = (StackLayout)grid.Parent.Parent;
            var button = (Button)stack.Children[1];

            if (button.Image == "DropUp.png")
            {
                if (curstack.Children[1].ClassId == "file")
                    CrossSettings.Current.AddOrUpdateValue("fileExpanded", "0");
                else if (curstack.Children[1].ClassId == "settings")
                    CrossSettings.Current.AddOrUpdateValue("settingsExpanded", "0");
                else if (curstack.Children[1].ClassId == "map")
                    CrossSettings.Current.AddOrUpdateValue("mapExpanded", "0");
                else if (curstack.Children[1].ClassId == "crossSection")
                    CrossSettings.Current.AddOrUpdateValue("crossSectionExpanded", "0");
                else if (curstack.Children[1].ClassId == "location")
                    CrossSettings.Current.AddOrUpdateValue("locationExpanded", "0");
                else if (curstack.Children[1].ClassId == "photo")
                    CrossSettings.Current.AddOrUpdateValue("photoExpanded", "0");
                curstack.Children[1].IsVisible = false;
                
                button.Image = "DropDown.png";
            }
            else if (button.Image == "DropDown.png")
            {
                if (curstack.Children[1].ClassId == "file")
                    CrossSettings.Current.AddOrUpdateValue("fileExpanded", "1");
                else if (curstack.Children[1].ClassId == "settings")
                    CrossSettings.Current.AddOrUpdateValue("settingsExpanded", "1");
                else if (curstack.Children[1].ClassId == "map")
                    CrossSettings.Current.AddOrUpdateValue("mapExpanded", "1");
                else if (curstack.Children[1].ClassId == "crossSection")
                    CrossSettings.Current.AddOrUpdateValue("crossSectionExpanded", "1");
                else if (curstack.Children[1].ClassId == "location")
                    CrossSettings.Current.AddOrUpdateValue("locationExpanded", "1");
                else if (curstack.Children[1].ClassId == "photo")
                    CrossSettings.Current.AddOrUpdateValue("photoExpanded", "1");
                curstack.Children[1].IsVisible = true;

                button.Image = "DropUp.png";
            }

        }

        private void yScaleCanvasView_PaintSurface(object sender, SKPaintSurfaceEventArgs e)
        {
            if(yrepaint == true && crossSection.hd != 0 && alignmentWorkspace.scale == true && isData == true && crossSection.h > 0 && crossSection.w > 0)
            {
                SKImageInfo info = e.Info;
                SKSurface surface = e.Surface;
                SKCanvas canvas = surface.Canvas;

                canvas.Clear();

                canvas.Scale(1, (float)scale.scaleY, info.Width, ((info.Height)/2));
                canvas.Translate(new SKPoint(0f, (float)scale.transY));

                SKPaint thinBlackLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.Black,
                    StrokeWidth = 3 / (float)scale.scaleY,
                    StrokeCap = SKStrokeCap.Square
                };

                SKPaint thickBlueLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.Blue,
                    StrokeWidth = 5 / (float)scale.scaleY
                };

                SKPaint thinnerBlackLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.Black,
                    StrokeWidth = 1 / (float)scale.scaleY
                };

                SKPaint smallBlackText = new SKPaint
                {
                    TextSize = (7.5f*App.density) / (float)scale.scaleY,
                    Color = SKColors.Black,
                    IsAntialias = true,
                    StrokeWidth = 2,
                    TextScaleX = 1 * (float)scale.scaleY 
                };

                SKPaint thinRedLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.Red,
                    StrokeWidth = 1 / (float)scale.scaleY
                };

                SKPaint thinGrayLine = new SKPaint
                {
                    Style = SKPaintStyle.Stroke,
                    Color = SKColors.LightGray,
                    StrokeWidth = 1
                };

                var vert = (float)alignmentWorkspace.vertExag;
                var segmentScale = 5 * (crossSection.hd / (crossSection.h / crossSection.ppw));
                var scaleYTo = setSegmentScale((crossSection.hd / segmentScale));
                var segments = (float)Math.Max((Math.Ceiling(crossSection.hd / scaleYTo) + 2), 5);
                var textStart = (float)(12.5f * App.density);
                var lineEnd = (float)(10f * App.density);
                var shortLineEnd = (float)(5f * App.density);
                var textOffset = (float)((2.5f * App.density) / scale.scaleY);

                if(crossSection.minH < 0 && crossSection.maxH >= 0)
                {
                    var hOffset = (crossSection.h / 2) - (((crossSection.hd * vert) * crossSection.ppw)/2);
                    var centerpoint = hOffset + ((crossSection.maxH * crossSection.ppw) * vert);
                    var segDist = (float)((crossSection.ppw * scaleYTo) * vert);

                    for(int i = 0; i<=segments; i++)
                    {
                        canvas.DrawLine(0, centerpoint + (segDist * i), lineEnd, centerpoint + (segDist * i), thinBlackLine);
                        if (i > 0)
                            canvas.DrawText("-" + scaleYTo * i, textStart, centerpoint + (segDist * i) + textOffset, smallBlackText);
                        for (int j = 2; j < 10; j+=2)
                            canvas.DrawLine(0, centerpoint + ((segDist * i) + (segDist / 10) * j), shortLineEnd, centerpoint + ((segDist * i) + (segDist / 10) * j), thinnerBlackLine);
                    }
                    for(int i = 0; i<=segments; i++)
                    {
                        canvas.DrawLine(0, centerpoint - (segDist * i), lineEnd, centerpoint - (segDist * i), thinBlackLine);
                        if (i > 0)
                            canvas.DrawText("" + scaleYTo * i, textStart, centerpoint - (segDist * i) + textOffset, smallBlackText);
                        for (int j = 2; j < 10; j+=2)
                            canvas.DrawLine(0, centerpoint - ((segDist * i) + (segDist / 10) * j), shortLineEnd, centerpoint - ((segDist * i) + (segDist / 10) * j), thinnerBlackLine);
                    }

                    canvas.DrawLine(0, centerpoint - (segDist * (segments+1)), lineEnd, centerpoint - (segDist * (segments+1)), thinBlackLine);
                    canvas.DrawLine(0, centerpoint + (segDist * (segments+1)), lineEnd, centerpoint + (segDist * (segments+1)), thinBlackLine);
                    canvas.DrawText("0", textStart, centerpoint + textOffset, smallBlackText);
                    canvas.DrawText("" + (scaleYTo * (segments+1)), textStart, centerpoint - (segDist * (segments+1)) + textOffset, smallBlackText);
                    canvas.DrawText("" + (scaleYTo * -(segments+1)), textStart, centerpoint + (segDist * (segments+1)) + textOffset, smallBlackText);
                    canvas.DrawLine(yScaleCanvasView.CanvasSize.Width - 1, centerpoint + (segDist * (segments+1)), yScaleCanvasView.CanvasSize.Width - 1, centerpoint - (segDist * (segments+1)), thinGrayLine);
                    yrepaint = false;
                }
                else if(crossSection.minH < 0 && crossSection.maxH < 0)
                {
                    var hOffset = (crossSection.h / 2) - (((crossSection.hd * vert) * crossSection.ppw) / 2);
                    var roundedMin = setSegmentScale(Math.Abs(crossSection.minH));
                    var difference = (float)(roundedMin - Math.Abs(crossSection.minH));
                    var centerpoint = hOffset + (((Math.Abs(crossSection.minH) - Math.Abs(crossSection.maxH) * crossSection.ppw) + (difference * -1 * crossSection.ppw)) * vert);
                    var segDist = (float)((crossSection.ppw * scaleYTo) * vert);

                    for(int i = 0; i<=segments; i++)
                    {
                        canvas.DrawLine(0, centerpoint - (segDist * i), lineEnd, centerpoint - (segDist * i), thinBlackLine);
                        if (i > 0)
                            canvas.DrawText("" + (roundedMin - (scaleYTo * i)), textStart, centerpoint - (segDist * i) + textOffset, smallBlackText);
                        for (int j = 2; j < 10; j += 2)
                            canvas.DrawLine(0, centerpoint - ((segDist * i) + (segDist / 10) * j), shortLineEnd, centerpoint - ((segDist * i) + (segDist / 10) * j), thinnerBlackLine);
                    }
                    for(int i = 0; i<=segments; i++)
                    {
                        canvas.DrawLine(0, centerpoint + (segDist * i), lineEnd, centerpoint + (segDist * i), thinBlackLine);
                        if (i > 0)
                            canvas.DrawText("" + (roundedMin + (scaleYTo * i)), textStart, centerpoint + (segDist * i) + textOffset,  smallBlackText);
                        for (int j = 2; j < 10; j += 2)
                            canvas.DrawLine(0, centerpoint - ((segDist * i) + (segDist / 10) * j), shortLineEnd, centerpoint - ((segDist * i) + (segDist / 10) * j), thinnerBlackLine);
                    }

                    canvas.DrawLine(0, centerpoint - (segDist * (segments+1)), lineEnd, centerpoint - (segDist * (segments+1)), thinBlackLine);
                    canvas.DrawLine(0, centerpoint + (segDist * (segments+1)), lineEnd, centerpoint + (segDist * (segments+1)), thinBlackLine);
                    canvas.DrawText("" + roundedMin, textStart, centerpoint + textOffset, smallBlackText);
                    canvas.DrawText("" + (roundedMin + (scaleYTo * (segments+1))), textStart, centerpoint - (segDist * (segments+1)) + textOffset, smallBlackText);
                    canvas.DrawText("" + (roundedMin + (scaleYTo * -(segments+1))), textStart, centerpoint + (segDist * (segments+1)) + textOffset, smallBlackText);
                    canvas.DrawLine(yScaleCanvasView.CanvasSize.Width - 1, centerpoint + (segDist * (segments+1)), yScaleCanvasView.CanvasSize.Width - 1, centerpoint - (segDist * (segments+1)), thinGrayLine);
                    yrepaint = false;
                }
                else if(crossSection.minH >= 0)
                {
                    var hOffset = (crossSection.h / 2) - (((crossSection.hd * vert) * crossSection.ppw) / 2);
                    var roundedMin = setSegmentScale(crossSection.minH);
                    var difference = (float)(roundedMin - crossSection.minH);
                    var centerpoint = hOffset + (((crossSection.hd * crossSection.ppw) + (difference * -1 * crossSection.ppw)) * vert);
                    var segDist = (float)((crossSection.ppw * scaleYTo) * vert);
                    
                    for(int i = 0; i<=segments; i++)
                    {
                        canvas.DrawLine(0, centerpoint - (segDist * i), lineEnd, centerpoint - (segDist * i), thinBlackLine);
                        if (i > 0)
                            canvas.DrawText("" + (roundedMin + (scaleYTo * i)), textStart, centerpoint - (segDist * i) + textOffset, smallBlackText);
                        for (int j = 2; j < 10; j += 2)
                            canvas.DrawLine(0, centerpoint - ((segDist * i) + (segDist / 10) * j), shortLineEnd, centerpoint - ((segDist * i) + (segDist / 10) * j), thinnerBlackLine);
                    }
                    for(int i = 0; i<=segments;  i++)
                    {
                        canvas.DrawLine(0, centerpoint + (segDist * i), lineEnd, centerpoint + (segDist * i), thinBlackLine);
                        if (i > 0)
                            canvas.DrawText("" + (roundedMin - (scaleYTo * i)), textStart, centerpoint + (segDist * i) + textOffset, smallBlackText);
                        for (int j = 2; j < 10; j += 2)
                            canvas.DrawLine(0, centerpoint + ((segDist * i) + (segDist / 10) * j), shortLineEnd, centerpoint + ((segDist * i) + (segDist / 10) * j), thinnerBlackLine);
                    }

                    canvas.DrawLine(0, centerpoint - (segDist * (segments+1)), lineEnd, centerpoint - (segDist * (segments+1)), thinBlackLine);
                    canvas.DrawLine(0, centerpoint + (segDist * (segments+1)), lineEnd, centerpoint + (segDist * (segments+1)), thinBlackLine);
                    canvas.DrawText("" + roundedMin, textStart, centerpoint + textOffset, smallBlackText);
                    canvas.DrawText("" + (roundedMin + (scaleYTo * (segments+1))), textStart, centerpoint - (segDist * (segments+1)) + textOffset, smallBlackText);
                    canvas.DrawText("" + (roundedMin + (scaleYTo * -(segments+1))), textStart, centerpoint + (segDist * (segments+1)) + textOffset, smallBlackText);
                    canvas.DrawLine(yScaleCanvasView.CanvasSize.Width - 1, centerpoint + (segDist * (segments+1)), yScaleCanvasView.CanvasSize.Width - 1, centerpoint - (segDist * (segments+1)), thinGrayLine);
                    yrepaint = false;
                }
            }
            
        }

        private double setSegmentScale(double value)
        {
            if (value <= 1.5)
                return 1D;
            else if (value <= 3.5)
                return 2D;
            else if (value <= 7.5)
                return 5D;
            else if (value <= 100)
                return Math.Round(value / 10.0, 0) * 10;
            else if (value <= 125)
                return 100D;
            else if (value <= 175)
                return 150D;
            else if (value <= 250)
                return 200D;
            else if (value <= 350)
                return 300D;
            else if (value <= 450)
                return 400D;
            else if (value <= 625)
                return 500D;
            else if (value <= 875)
                return 750D;
            else
                return 1000D;
        }

        private void setIntersectPoints(Dictionary<string, List<dataTypes.nrgVector2>> dictionaryIntersect)
        {
            var test1 = App.screenWidth;
            var test2 = App.screenHeight;
            var w = Math.Round(canvasView.CanvasSize.Width);
            var h = Math.Round(canvasView.CanvasSize.Height);
            var vert = (float)alignmentWorkspace.vertExag;
            double maxW = dictionaryIntersect.First().Value.Last().X, maxH = dictionaryIntersect.First().Value.First().Y;
            double minW = dictionaryIntersect.First().Value.First().X, minH = dictionaryIntersect.First().Value.First().Y;

            foreach (string key in dictionaryIntersect.Keys)
            {
                var intersectList = dictionaryIntersect[key];

                for (int i = 0; i < intersectList.Count(); i++)
                {
                    if (intersectList[i].Y > maxH)
                        maxH = intersectList[i].Y;
                    if (intersectList[i].Y < minH)
                        minH = intersectList[i].Y;
                    if (intersectList[i].X > maxW)
                        maxW = intersectList[i].X;
                    if (intersectList[i].X < minW)
                        minW = intersectList[i].X;
                }

                intersectList = null;
            }

            double wd = 0;
            double hd = 0;
            double ppw = 1;

            if (minW < 0 && maxW >= 0)
                wd = Math.Abs(minW) + maxW;
            else if (minW < 0 && maxW < 0)
                wd = Math.Abs(minW);
            else if (minW >= 0)
                wd = maxW;

            if (minH < 0 && maxH >= 0)
                hd = Math.Abs(minH) + maxH;
            else if (minH < 0 && maxH < 0)
                hd = Math.Abs(minH) + Math.Abs(maxH);
            else if (minH >= 0)
                hd = maxH - minH;

            if(wd != 0)
            {
                ppw = (float)(w / wd);
                if ((ppw * (hd  * vert)) >= h)
                    ppw = (float)((h / (hd * vert)) * 0.8);
            }
            if (ppw > w / 10)
                ppw = (w / 10);

            crossSection.w = (float)w;
            crossSection.h = (float)h;
            crossSection.minH = (float)minH;
            crossSection.maxH = (float)maxH;
            crossSection.minW = (float)minW;
            crossSection.maxW = (float)maxW;
            crossSection.ppw = (float)ppw;
            crossSection.wd = (float)wd;
            crossSection.hd = (float)hd;

            dictionaryIntersectPoints.Clear();
            if(alignmentWorkspace.centeredByOffset == false)
            {
                foreach (string key in dictionaryIntersect.Keys)
                {
                    var intersectList = dictionaryIntersect[key];
                    var intersectPoints = new List<dataTypes.nrgVector2>();
                    foreach (dataTypes.nrgVector2 pos in intersectList)
                    {
                        double x = 0, y = 0;
                        double hOffset = ((h / 2) - (((hd * vert) * ppw) / 2));

                        if (minW < 0 && maxW < 0)
                            x = (Math.Abs(minW) - Math.Abs(pos.X)) * ppw;
                        else if (minW < 0 && maxW >= 0)
                        {
                            if (pos.X < 0)
                                x = (Math.Abs(minW) - Math.Abs(pos.X)) * ppw;
                            else
                                x = (Math.Abs(minW) + pos.X) * ppw;
                        }
                        else if (minW >= 0)
                            x = pos.X * ppw;

                        if (minH < 0 && maxH < 0)
                            y = hOffset + ((Math.Abs(Math.Abs(crossSection.maxH) - Math.Abs(pos.Y)) * crossSection.ppw) * vert);
                        else if (minH < 0 && maxH >= 0)
                        {
                            if (pos.Y < 0)
                                y = (hOffset + ((hd * vert) * ppw)) - (((Math.Abs(minH) + pos.Y) * ppw) * vert);
                            else
                                y = (hOffset + (maxH * ppw)) - ((pos.Y * ppw) * vert);
                        }
                        else if (minH >= 0)
                            y = hOffset + (((hd * vert) * ppw) - (((pos.Y - minH) * ppw) * vert));

                        intersectPoints.Add(new dataTypes.nrgVector2 { X = x, Y = y });
                    }
                    dictionaryIntersectPoints.Add(key, intersectPoints);
                    intersectList = null;
                    intersectPoints = null;
                    isData = true;

                }
            }
            else
            {
                if(wd != 0)
                {
                    double first = 0;
                    double last = 0;
                    if (minW <= alignmentWorkspace.offsetCenter)
                        first = alignmentWorkspace.offsetCenter - minW;
                    else
                        first = minW - alignmentWorkspace.offsetCenter;

                    if (maxW >= alignmentWorkspace.offsetCenter)
                        last = maxW - alignmentWorkspace.offsetCenter;
                    else
                        last = alignmentWorkspace.offsetCenter - maxW;

                    wd = Math.Max(first, last) * 2;
                    ppw = (float)(w / wd);
                    if ((ppw * (hd * vert)) >= h)
                        ppw = (float)((h / (hd * vert)) * 0.8);

                    if (ppw > (w / 10))
                        ppw = (w / 10);

                    crossSection.ppw = (float)ppw;
                    crossSection.wd = (float)wd;
                }

                foreach(string key in dictionaryIntersect.Keys)
                {
                    var intersectList = dictionaryIntersect[key];
                    var intersectPoints = new List<dataTypes.nrgVector2>();
                    foreach(dataTypes.nrgVector2 pos in intersectList)
                    {
                        double x = 0, y = 0;
                        double hOffset = ((h / 2) - (((hd * vert) * ppw) / 2));

                        if (pos.X <= alignmentWorkspace.offsetCenter)
                            x = (w / 2) - ((alignmentWorkspace.offsetCenter - pos.X) * ppw);
                        else if (pos.X > alignmentWorkspace.offsetCenter)
                            x = (w / 2) + ((pos.X - alignmentWorkspace.offsetCenter) * ppw);

                        if (minH < 0 && maxH < 0)
                            y = hOffset + ((Math.Abs(Math.Abs(crossSection.maxH) - Math.Abs(pos.Y)) * crossSection.ppw) * vert);
                        else if (minH < 0 && maxH >= 0)
                        {
                            if (pos.Y < 0)
                                y = (hOffset + ((hd * vert) * ppw)) - (((Math.Abs(minH) + pos.Y) * ppw) * vert);
                            else
                                y = (hOffset + (maxH * ppw)) - ((pos.Y * ppw) * vert);
                        }
                        else if (minH >= 0)
                            y = hOffset + (((hd * vert) * ppw) - (((pos.Y - minH) * ppw) * vert));
                        intersectPoints.Add(new dataTypes.nrgVector2 { X = x, Y = y });
                    }
                    dictionaryIntersectPoints.Add(key, intersectPoints);
                    intersectList = null;
                    isData = true;
                }
            }

            if (dictionaryIntersectPoints.Count() == 0 || dictionaryIntersectPoints == null)
                isData = false;

            if(alignmentWorkspace.locked == true)
                setNewScale();
            redrawCanvas();
        }

        public string rgbToHex(double r, double g, double b)
        {
            string hexValue = "#";

            var red = toHex(r);
            var green = toHex(g);
            var blue = toHex(b);

            hexValue += (red + green + blue);

            return hexValue;
        }

        public string toHex(double rgb)
        {
            var hex = "";
            hex = ((int)rgb).ToString("X").ToUpper();

            if (hex.Length == 1)
                hex = "0" + hex;
            else if (hex.Length == 0)
                hex = "00";

            return hex;
        }

        void setupColorPicker(Color rgb)
        {
            var r = Math.Round(rgb.R * 255);
            var g = Math.Round(rgb.G * 255);
            var b = Math.Round(rgb.B * 255);

            entryR.Text = r.ToString();
            entryG.Text = g.ToString();
            entryB.Text = b.ToString();

            sliderR.Value = r;
            sliderG.Value = g;
            sliderB.Value = b;

            entryHex.Text = rgbToHex(r, g, b);
            colorFrame.BackgroundColor = Color.FromHex(entryHex.Text);
        }
        
        private void enableScale()
        {
            gridCrossection.IsVisible = false;
            gridCrossection.Children.Clear();
            gridCrossection.RowDefinitions.Clear();
            gridCrossection.ColumnDefinitions.Clear(); 
            gridCrossection.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
            gridCrossection.RowDefinitions.Add(new RowDefinition { Height = new GridLength((10 * App.density), GridUnitType.Absolute) });
            gridCrossection.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength((10 * App.density), GridUnitType.Absolute) });
            gridCrossection.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            gridCrossection.Children.Add(xScaleCanvasView, 1, 1);
            gridCrossection.Children.Add(yScaleCanvasView, 0, 0);
            gridCrossection.Children.Add(canvasView, 1, 0);
            gridCrossection.Children.Add(gridCrossSectionButtons, 1, 0);
            xScaleCanvasView.IsEnabled = true;
            xScaleCanvasView.IsVisible = true;
            yScaleCanvasView.IsEnabled = true;
            yScaleCanvasView.IsVisible = true;
            canvasView.IsEnabled = true;
            canvasView.IsVisible = true;
            gridCrossection.IsVisible = true;
        }

        void disableScale()
        {
            xScaleCanvasView.IsEnabled = false;
            xScaleCanvasView.IsVisible = false;
            yScaleCanvasView.IsEnabled = false;
            yScaleCanvasView.IsVisible = false;
            gridCrossection.Children.Clear();
            gridCrossection.RowDefinitions.Clear();
            gridCrossection.ColumnDefinitions.Clear();
            gridCrossection.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
            gridCrossection.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            gridCrossection.Children.Add(canvasView, 0, 0);
            gridCrossection.Children.Add(gridCrossSectionButtons, 0, 0);
            canvasView.IsEnabled = true;
            canvasView.IsVisible = true;
        }

        void getOldScale()
        {
            var start = crossSection.minW + (((scale.transX) / crossSection.ppw) * -1);
            var end = start + (crossSection.w / crossSection.ppw);

            offsetXToCenterTo = ((end - start) / 2) + start;
        }

        void setNewScale()
        {
            if(alignmentWorkspace.centeredByOffset == true)
            {
                if (alignmentWorkspace.offsetCenter >= offsetXToCenterTo)
                    scale.transX = (((offsetXToCenterTo - alignmentWorkspace.offsetCenter) * crossSection.ppw)) * - 1;
                else if (alignmentWorkspace.offsetCenter < offsetXToCenterTo)
                    scale.transX = (((alignmentWorkspace.offsetCenter - offsetXToCenterTo) * crossSection.ppw));
            }
            else
            {
                var midPoint = crossSection.minW + (crossSection.wd / 2);

                if (midPoint >= offsetXToCenterTo)
                    scale.transX = (((offsetXToCenterTo - midPoint) * crossSection.ppw)) * -1;
                else if (midPoint < offsetXToCenterTo)
                    scale.transX = (((midPoint - offsetXToCenterTo) * crossSection.ppw));
            }
        }

        async void setLines()
        {
            for (int i = 0; i < 1; i++)
            {
                var grid = listLineGrid[i];
                var label = (Label)grid.Children[0];
                var button = (Button)grid.Children[1];

                CrossSettings.Current.AddOrUpdateValue("LineColor" + label.Text, rgbToHex(button.BackgroundColor.R * 255, button.BackgroundColor.G * 255, button.BackgroundColor.B * 255));
            }

            for (int i = 1; i < listLineGrid.Count(); i++)
            {
                var grid = listLineGrid[i];
                var label = (Label)grid.Children[0];
                var button = (Button)grid.Children[1];
                var button2 = (Button)grid.Children[2];
                var button3 = (Button)grid.Children[3];

                if (button.Image == "checkMarkSmall.png" && mapLineList.Contains(label.Text) == false)
                    mapLineList.Add(label.Text);
                else if (button.Image == "" && mapLineList.Contains(label.Text) == true)
                    mapLineList.Remove(label.Text);

                if (button2.Image == "checkMarkSmall.png" && crossSectionLineList.Contains(label.Text) == false)
                    crossSectionLineList.Add(label.Text);
                else if (button2.Image == "" && crossSectionLineList.Contains(label.Text) == true)
                    crossSectionLineList.Remove(label.Text);

                CrossSettings.Current.AddOrUpdateValue("LineColor" + label.Text, rgbToHex(button3.BackgroundColor.R * 255, button3.BackgroundColor.G * 255, button3.BackgroundColor.B * 255));
            }




            for(int i = 0; i<mapLineList.Count(); i++)
            {
                //Checks if file exists and attempts to load if so
                if (dictionaryLines.ContainsKey(mapLineList[i]) == false || dictionaryLoadedLines.ContainsKey(mapLineList[i]) == false && nrgFileRead.reader.fileExists(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + mapLineList[i] + ".kml"))
                    await loadKmlFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + mapLineList[i] + ".kml");
            }

            for(int i = 0; i<crossSectionLineList.Count(); i++)
            {
                //Checks if file exists and attempts to load if so
                if (dictionaryLines.ContainsKey(crossSectionLineList[i]) == false && nrgFileRead.reader.fileExists(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + crossSectionLineList[i] + ".kml") == true)
                    await loadKmlFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + crossSectionLineList[i] + ".kml");
            }

            for (int i = 0; i < dictionaryLines.Count();)
            {
                if (mapLineList.Contains(dictionaryLines.Keys.ElementAt(i)) == false && crossSectionLineList.Contains(dictionaryLines.Keys.ElementAt(i)) == false)
                    dictionaryLines.Remove(dictionaryLines.Keys.ElementAt(i));
                else
                    i++;
            }

            while (linesToRemove.Count() != 0)
            {
                if (crossSectionLineList.Contains(linesToRemove.Last()) == false)
                    dictionaryLines.Remove(linesToRemove.Last());
                for (int i = dictionaryLoadedLines[linesToRemove.Last()].Value - 1; i >= dictionaryLoadedLines[linesToRemove.Last()].Key; i--)
                    mapObject.Polylines.RemoveAt(i);
                var last = dictionaryLoadedLines[linesToRemove.Last()].Value - 1;
                var count = dictionaryLoadedLines[linesToRemove.Last()].Value - dictionaryLoadedLines[linesToRemove.Last()].Key;
                mapLineList.Remove(linesToRemove.Last());
                dictionaryLoadedLines.Remove(linesToRemove.Last());
                for (int i = 0; i < dictionaryLoadedLines.Count(); i++)
                {
                    if (dictionaryLoadedLines[dictionaryLoadedLines.ElementAt(i).Key].Key > last)
                        dictionaryLoadedLines[dictionaryLoadedLines.ElementAt(i).Key] = new KeyValuePair<int, int>(dictionaryLoadedLines[dictionaryLoadedLines.ElementAt(i).Key].Key - count, dictionaryLoadedLines[dictionaryLoadedLines.ElementAt(i).Key].Value - count);

                }
                linesToRemove.Remove(linesToRemove.Last());
                lastline -= count;
            }

            while (crossSectionToRemove.Count() != 0)
            {
                if (mapLineList.Contains(crossSectionToRemove.Last()) == false)
                    dictionaryLines.Remove(crossSectionToRemove.Last());
                dictionaryIntersectPoints.Remove(crossSectionToRemove.Last());
                crossSectionLineList.Remove(crossSectionToRemove.Last());
                crossSectionToRemove.Remove(crossSectionToRemove.Last());
            }

            //if (crossSectionLineList.Count != 0 && alignmentWorkspace.crossSection == false)
            //{
            //    alignmentWorkspace.crossSection = true;
            //    labelCrossSection.Text = "Enabled";
            //    CrossSettings.Current.AddOrUpdateValue("crossSection", alignmentWorkspace.crossSection);
            //    createIntersection();
            //    createCrossSection();
            //}

            if (crossSectionLineList.Count != 0 && alignmentWorkspace.crossSection == true && gridCrossection.IsVisible == false)
            {
                createIntersection();
                if(mapObject.IsVisible == false && alignmentWorkspace.landscape == true)
                {
                    createCrossSection();
                    labelCurMapStatus.Text = "Disabled";
                    mapObject.IsVisible = false;
                    gridMapPanel.RowDefinitions.Clear();
                    gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                    gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                    mapBoxView.IsVisible = false;
                    frameScale.IsVisible = false;
                    labelScale.IsVisible = false;
                    frameScaleLeft.IsVisible = false;
                    frameScaleRight.IsVisible = false;
                    frameScaleCenter.IsVisible = false;

                }
                else
                    createCrossSection();
            }

            if (crossSectionLineList.Count == 0 && alignmentWorkspace.crossSection == false && gridCrossection.IsVisible == true)
            {
                if(mapObject.IsVisible == false && alignmentWorkspace.landscape == false)
                {
                    removeCrossSection();
                    labelCurMapStatus.Text = "Enabled";
                    mapObject.IsVisible = true;
                    gridMapPanel.RowDefinitions.Clear();
                    gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                    gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                    getRadius();
                    mapBoxView.IsVisible = true;
                    frameScale.IsVisible = true;
                    labelScale.IsVisible = true;
                    frameScaleLeft.IsVisible = true;
                    frameScaleRight.IsVisible = true;
                    frameScaleCenter.IsVisible = true;
                }
                else if(mapObject.IsVisible == false && alignmentWorkspace.landscape == true)
                {
                    removeCrossSection();
                    labelCurMapStatus.Text = "Disabled";
                    mapObject.IsVisible = false;
                    gridMapPanel.RowDefinitions.Clear();
                    gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(0, GridUnitType.Absolute) });
                    gridMapPanel.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Star) });
                    mapBoxView.IsVisible = false;
                    frameScale.IsVisible = false;
                    labelScale.IsVisible = false;
                    frameScaleLeft.IsVisible = false;
                    frameScaleRight.IsVisible = false;
                    frameScaleCenter.IsVisible = false;
                }
                else
                {
                    removeCrossSection();
                }

            }


            gridLinesSelector.IsVisible = false;
            if (mapLineList.Count() == 0)
                labelCurLines.Text = "None Selected";
            else if (mapLineList.Count() == 1)
                labelCurLines.Text = mapLineList[0];
            else if (mapLineList.Count() > 1)
                labelCurLines.Text = "Multiple Selected";

            string mapSave = "";
            string crossSave = "";
            foreach (string item in mapLineList)
                mapSave += item + ",";
            foreach (string item in crossSectionLineList)
                crossSave += item + ",";

            CrossSettings.Current.AddOrUpdateValue("mapLineString", mapSave);
            CrossSettings.Current.AddOrUpdateValue("crossLineString", crossSave);
        }

        private List<dataTypes.nrgVector2> getElevationData(double ch, double off)
        {
            double totalDist = 0;
            double segDist = 0;
            double segCount = 0;

            double.TryParse(entryDistance.Text, out totalDist);
            double.TryParse(entryElevation.Text, out segDist);

            if (segDist != 0 && totalDist != 0)
            {
                segCount = Math.Round(totalDist / segDist);
                //Map center for elevation 
                var en1 = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(ch, off);
                var inVec = new dataTypes.nrgVector2() { X = en1.X, Y = en1.Y };
                var outVec = new dataTypes.nrgAlignmentVector5();
                if(alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(inVec, ref outVec))
                {
                    var en2 = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(ch, off + (alignmentWorkspace.distance / 2));
                    var en3 = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(ch, off - (alignmentWorkspace.distance / 2));

                    var ll1 = GetLLFromEN(en2.X, en2.Y);
                    var ll2 = GetLLFromEN(en3.X, en3.Y);

                    string uri = "https://maps.googleapis.com/maps/api/elevation/xml?path=" + ll1.X + "," + ll1.Y + "|" + ll2.X + "," + ll2.Y + "&samples=" + segCount + "&key=AIzaSyBG5pFUIThqFKGq_X-rVnfA68U8pitS-SQ";
                    List<dataTypes.nrgVector2> elevationList = new List<dataTypes.nrgVector2>();
                    try
                    {                                
                        var xDoc = XDocument.Load(uri);
                        foreach(XElement item in xDoc.Descendants("result"))
                        {
                            double lat = Convert.ToDouble(item.Element("location").Element("lat").Value);
                            double lng = Convert.ToDouble(item.Element("location").Element("lng").Value);
                            double ele = Convert.ToDouble(item.Element("elevation").Value);

                            var eastNrth = GetENFromLL(lat, lng);
                            dataTypes.nrgVector2 inVector = new dataTypes.nrgVector2() { X = eastNrth.X, Y = eastNrth.Y };
                            dataTypes.nrgAlignmentVector5 outVector = new dataTypes.nrgAlignmentVector5();
                            alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(inVector, ref outVector);
                            dataTypes.nrgVector2 dataToAdd = new dataTypes.nrgVector2() { X = outVector.offset, Y = ele };
                            elevationList.Add(dataToAdd);
                        }
                        elevationList = mergeSort(elevationList);
                        DisplayAlert("Notice", "The cross-section will be paused while showing elevation data. Press the play button to resume automatic updating", "Ok");
                        return elevationList;
                    }
                    catch
                    {
                        DisplayAlert("Error", "Failed to retrieve elevation data.", "Ok");
                        return null;
                    }
                }
                else
                {
                    DisplayAlert("Error", "Not on alignment", "Ok");
                    return null;
                }            
            }
            return null;


        }

        private void hideEntries()
        {
            if (entryB.IsFocused == true)
                entryB.Unfocus();
            else if (entryCountryCode.IsFocused == true)
                entryCountryCode.Unfocus();
            else if (entryDistance.IsFocused == true)
                entryDistance.Unfocus();
            else if (entryElevation.IsFocused == true)
                entryElevation.Unfocus();
            else if (entryG.IsFocused == true)
                entryG.Unfocus();
            else if (entryHex.IsFocused == true)
                entryHex.Unfocus();
            else if (entryOffsetCenter.IsFocused == true)
                entryOffsetCenter.Unfocus();
            else if (entryPinID.IsFocused == true)
                entryPinID.Unfocus();
            else if (entryPinNote.IsFocused == true)
                entryPinNote.Unfocus();
            else if (entryPinX.IsFocused == true)
                entryPinX.Unfocus();
            else if (entryPinY.IsFocused == true)
                entryPinY.Unfocus();
            else if (entryR.IsFocused == true)
                entryR.Unfocus();
            else if (entryVertExag.IsFocused == true)
                entryVertExag.Unfocus();
        }

        private void setUpChOffPin()
        {
            if (gridEditPinPanel.IsVisible == true)
            {
                switch (labelCurPinType.Text)
                {
                    case "Station":
                        break;
                    case "Meterage":
                        break;
                    case "Chainage":
                        break;
                    case "Easting Northing":
                        dataTypes.nrgAlignmentVector5 outVec = new dataTypes.nrgAlignmentVector5();
                        alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(new dataTypes.nrgVector2 { X = Convert.ToDouble(entryPinX.Text), Y = Convert.ToDouble(entryPinY.Text) }, ref outVec);
                        entryPinX.Text = outVec.chainage.ToString("0.000000");
                        entryPinY.Text = outVec.offset.ToString("0.000000");
                        break;
                    case "Latitude Longitude":
                        dataTypes.nrgAlignmentVector5 outVector = new dataTypes.nrgAlignmentVector5();
                        var estNrth = GetENFromLL(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                        alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(new dataTypes.nrgVector2 { X = estNrth.X, Y = estNrth.Y }, ref outVector);
                        entryPinX.Text = outVector.chainage.ToString("0.000000");
                        entryPinY.Text = outVector.offset.ToString("0.000000");
                        break;
                }
                labelCurPinType.Text = alignmentWorkspace.internationalDisplay;
                titlePinX.Text = alignmentWorkspace.internationalDisplay;
                titlePinY.Text = "Offset";

            }
        }

        private void setUpENPin()
        {
            if(gridEditPinPanel.IsVisible == true)
            {
                switch(labelCurPinType.Text)
                {
                    case "Station":
                        if(alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Found" && alignmentWorkspace.currentAlignment != "None Selected")
                        {
                            var estNrth = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                            entryPinX.Text = estNrth.X.ToString("0.000000");
                            entryPinY.Text = estNrth.Y.ToString("0.000000");
                        }
                        else
                        {
                            var estNrth = GetENFromLL(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                            entryPinX.Text = estNrth.X.ToString("0.000000");
                            entryPinY.Text = estNrth.Y.ToString("0.000000");
                        }
                        break;
                    case "Meterage":
                        if(alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Found" && alignmentWorkspace.currentAlignment != "None Selected")
                        {
                            var EN = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                            entryPinX.Text = EN.X.ToString("0.000000");
                            entryPinY.Text = EN.Y.ToString("0.000000");
                        }
                        else
                        {
                            var EN = GetENFromLL(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                            entryPinX.Text = EN.X.ToString("0.000000");
                            entryPinY.Text = EN.Y.ToString("0.000000");
                        }
                        break;
                    case "Chainage":
                        if(alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Found" && alignmentWorkspace.currentAlignment != "None Selected")
                        {
                            var en = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                            entryPinX.Text = en.X.ToString("0.000000");
                            entryPinY.Text = en.Y.ToString("0.000000");
                        }
                        else
                        {
                            var en = GetENFromLL(mapObject.CameraPosition.Target.Latitude, mapObject.CameraPosition.Target.Longitude);
                            entryPinX.Text = en.X.ToString("0.000000");
                            entryPinY.Text = en.Y.ToString("0.000000");
                        }
                        break;
                    case "Easting Northing":
                        break;
                    case "Latitude Longitude":
                        var estNorth = GetENFromLL(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                        entryPinX.Text = estNorth.X.ToString("0.000000");
                        entryPinY.Text = estNorth.Y.ToString("0.000000");
                        break;
                }
                titlePinX.Text = "Easting";
                titlePinY.Text = "Northing";
                labelCurPinType.Text = "Easting Northing";
            }
        }

        private void setUpLLPin()
        {
            if(gridEditPinPanel.IsVisible == true)
            {
                switch(labelCurPinType.Text)
                {
                    case "Station":
                        if(alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Found" && alignmentWorkspace.currentAlignment != "None Selected")
                        {
                            var en = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                            var ll = GetLLFromEN(en.X, en.Y);
                            entryPinX.Text = ll.X.ToString("0.000000");
                            entryPinY.Text = ll.Y.ToString("0.000000");
                        }
                        else
                        {
                            entryPinX.Text = mapObject.CameraPosition.Target.Latitude.ToString("0.000000");
                            entryPinY.Text = mapObject.CameraPosition.Target.Longitude.ToString("0.000000");
                        }
                        break;
                    case "Meterage":
                        if(alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Found" && alignmentWorkspace.currentAlignment != "None Selected")
                        {
                            var en2 = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                            var ll2 = GetLLFromEN(en2.X, en2.Y);
                            entryPinX.Text = ll2.X.ToString("0.000000");
                            entryPinY.Text = ll2.Y.ToString("0.000000");
                        }
                        else
                        {
                            entryPinX.Text = mapObject.CameraPosition.Target.Latitude.ToString("0.000000");
                            entryPinY.Text = mapObject.CameraPosition.Target.Longitude.ToString("0.000000");
                        }
                        break;
                    case "Chainage":
                        if(alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Found" && alignmentWorkspace.currentAlignment != "None Selected")
                        {
                            var en3 = alignmentWorkspace.Alignments.horizontal.CoordsAtChAndOffset(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                            var ll3 = GetLLFromEN(en3.X, en3.Y);
                            entryPinX.Text = ll3.X.ToString("0.000000");
                            entryPinY.Text = ll3.Y.ToString("0.000000");
                        }
                        else
                        {
                            entryPinX.Text = mapObject.CameraPosition.Target.Latitude.ToString("0.000000");
                            entryPinY.Text = mapObject.CameraPosition.Target.Longitude.ToString("0.000000");
                        }
                        break;
                    case "Easting Northing":
                        var ll4 = GetLLFromEN(Convert.ToDouble(entryPinX.Text), Convert.ToDouble(entryPinY.Text));
                        entryPinX.Text = ll4.X.ToString("0.000000");
                        entryPinY.Text = ll4.Y.ToString("0.000000");
                        break;
                    case "Latitude Longitude":
                        break;
                }
                titlePinX.Text = "Latitude";
                titlePinY.Text = "Longitude";
                labelCurPinType.Text = "Latitude Longitude";
            }
        }

        private async Task changePinAddres(string format)
        {
            var address = mapObject.SelectedPin.Address;
            var pin = mapObject.SelectedPin;
            var lat = pin.Position.Latitude;
            var lng = pin.Position.Longitude;

            pin.IsVisible = false;
            mapObject.SelectedPin = null;
            switch(format)
            {
                case "Chainage":
                    var estNrth = GetENFromLL(lat, lng);
                    var outVec = new dataTypes.nrgAlignmentVector5();
                    if (alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(new dataTypes.nrgVector2() { X = estNrth.X, Y = estNrth.Y }, ref outVec) && (alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Found" && alignmentWorkspace.currentAlignment != "None Selected"))
                        address = getDecimalDisplay(outVec.chainage) + "," + getDecimalDisplay(outVec.offset);
                    else
                        address = "????" + "," + "????";
                    break;
                case "Easting Northing":
                    var eastNrth = GetENFromLL(lat, lng);
                    address = getDecimalDisplay(eastNrth.X) + "," + getDecimalDisplay(eastNrth.Y);
                    break;
                case "Latitude Longitude":
                    address = lat.ToString("0.000000") + "," + lng.ToString("0.000000");
                    break;
            }
            pin.Address = address;

            Device.BeginInvokeOnMainThread(() =>
            {
                pin.IsVisible = true;
                mapObject.SelectedPin = pin;
            });
        }

        public void FinalizeAlignment(alignments.alignment alignment)
        {
            var stringName = alignment.stringName;
            var filesList = nrgFileRead.reader.listFiles(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/");
            bool exists = false;
            foreach(string item in filesList)
            {
                if (Path.GetFileName(item).ToString().ToUpper() == (alignment.stringName + ".NST").ToUpper())
                    exists = true;
            }

            if(exists)
            {
                Device.BeginInvokeOnMainThread(async () =>
                { 
                    var action = await DisplayActionSheet("a file named " + stringName + " already exists would you like to overwrite?", "Cancel", "Overwrite");
                    switch (action)
                    {
                        case "Overwrite":
                            nrgFileRead.reader.deleteFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + stringName + ".nst");
                            alignment.saveNST(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + stringName + ".nst");
                            break;
                    }
                });
            }
            else
            {
                alignment.saveNST(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + alignment.stringName + ".nst");
            }

            switch(Device.RuntimePlatform)
            {
                case Device.Android:
                    App.ScanFiles.scanFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + alignment.stringName + ".nst");
                    break;
            }
        }

        private void loadAsciiPins(string filename)
        {
            var lineList = nrgFileRead.reader.loadAsciiToStringList(filename, false);
            mapObject.Pins.Clear();
            currentPinId = 0;
            foreach(string item in lineList)
            {
                string[] tmp = item.Split(',');
                var latLng = GetLLFromEN(Convert.ToDouble(tmp[1]), Convert.ToDouble(tmp[2]));
                var pin = new Pin()
                {
                    Label = tmp[0] + "," + tmp[4],
                    Address = "",
                    Position = new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y),
                    Tag = tmp[3]
                };

                mapObject.Pins.Add(pin);
                currentPinId++;
            }

        }

        private void hidePickers()
        {
            pickerCenter.Unfocus();
            pickerDecimalPlace.Unfocus();
            PickerScaleFactor.Unfocus();
            pickerFolder.Unfocus();
            pickerFontSize.Unfocus();
            pickerFormat.Unfocus();
            pickerImport.Unfocus();
            pickerInternational.Unfocus();
            pickerLines.Unfocus();
            pickerMapType.Unfocus();
            pickerNst.Unfocus();
            pickerOffset.Unfocus();
            pickerPins.Unfocus();
            pickerPinSaveOptions.Unfocus();
            pickerTransform.Unfocus();
            pickerUnits.Unfocus();
        }

        private void loadFailure(string lineName, bool crossSection, bool map)
        {
            try
            {
                lineName = lineName.Replace(".kml", "");
                for (int i =0; i<listLineGrid.Count(); i++)
                {
                    var label = (Label)listLineGrid[i].Children[0];

                    if (label.Text == lineName)
                    {
                        if (map == true && mapLineList.Contains(lineName))
                        {
                            mapLineList.Remove(lineName);
                            var button = (Button)listLineGrid[i].Children[1];
                            button.Image = "EmptyBox.png";
                        }
                        if (crossSection == true && crossSectionLineList.Contains(lineName))
                        {
                            crossSectionLineList.Remove(lineName);
                            var button = (Button)listLineGrid[i].Children[2];
                            button.Image = "EmptyBox.png";
                        }
                    }
                }

                string mapSave = "";
                string crossSave = "";
                foreach (string item in mapLineList)
                    mapSave += item + ",";
                foreach (string item in crossSectionLineList)
                    crossSave += item + ",";

                CrossSettings.Current.AddOrUpdateValue("mapLineString", mapSave);
                CrossSettings.Current.AddOrUpdateValue("crossLineString", crossSave);
            }
            catch (Exception e)
            {
                DisplayAlert("Error", e.Message, "Ok");
            }
        }

        private dataTypes.nrgVector2 GetLLFromEN(double e, double n)
        {
            var newCoord = new dataTypes.nrgVector2();
            if (!NewTransformation)
            {
                //Get old transformation
                alignmentWorkspace.transformation.siteScale = alignmentWorkspace.scaleFactor;
                alignmentWorkspace.transformation.siteShiftE = alignmentWorkspace.siteShiftE;
                alignmentWorkspace.transformation.siteShiftN = alignmentWorkspace.siteShiftN;
                newCoord = helpers.LatLongAltFromENL(e, n, 0, 0, 0, 0, alignmentWorkspace.transformation);
                return newCoord;
            }
            else
            {
                if (App.ScaleFactorMethod == ScaleFactor.Multiply)
                {
                    e = ((e * (1 / alignmentWorkspace.scaleFactor)) + alignmentWorkspace.siteShiftE);
                    n = ((n * (1 / alignmentWorkspace.scaleFactor)) + alignmentWorkspace.siteShiftN);
                }
                else if (App.ScaleFactorMethod == ScaleFactor.Divide)
                {
                    e = (e + alignmentWorkspace.siteShiftE) * alignmentWorkspace.scaleFactor;
                    n = (n + alignmentWorkspace.siteShiftN) * alignmentWorkspace.scaleFactor;
                }

                if(App.DefaultUnits != App.SavedUnits)
                {
                    if(App.DefaultUnits == Unit.Metre)
                    {
                        if(App.SavedUnits == Unit.Foot)
                        {
                            //Convert feet to metres
                            e *= 0.3048;
                            n *= 0.3048;
                        }
                        else if(App.SavedUnits == Unit.USFoot)
                        {
                            //Convert usFeet to metres
                            e *= 0.304800609601219;
                            n *= 0.304800609601219;
                        }
                    }
                    else if(App.DefaultUnits == Unit.Foot)
                    {
                        if(App.SavedUnits == Unit.Metre)
                        {
                            //Convert metres to feet
                            e *= (1 / 0.3048);
                            n *= (1 / 0.3048);
                        }
                        else if(App.SavedUnits == Unit.USFoot)
                        {
                            //Convert usfeet to feet
                            e *= 1.000002;
                            n *= 1.000002;
                        }
                    }
                    else if(App.DefaultUnits == Unit.USFoot)
                    {
                        if(App.SavedUnits == Unit.Metre)
                        {
                            //Convert metres to usfeet
                            e *= (1 / 0.304800609601219);
                            n *= (1 / 0.304800609601219);
                        }
                        else if(App.SavedUnits == Unit.Foot)
                        {
                            //Convert feet to usfeet
                            e *= 0.999998;
                            n *= 0.999998;
                        }
                    } 
                }

                double[] coords = new double[2] { e, n };
                double[] coordz = new double[2] { 0, 0 };
                DotSpatial.Projections.Reproject.ReprojectPoints(coords, coordz, projTarget, projSource, 0, coords.Length - 1);
                newCoord.X = coords[1];
                newCoord.Y = coords[0];
                return newCoord;
            }
        }

        private dataTypes.nrgVector2 GetENFromLL(double lat, double lng)
        {
            var newCoord = new dataTypes.nrgVector2();
            if (!NewTransformation)
            {
                //Get old transformation
                alignmentWorkspace.transformation.siteScale = alignmentWorkspace.scaleFactor;
                alignmentWorkspace.transformation.siteShiftE = alignmentWorkspace.siteShiftE;
                alignmentWorkspace.transformation.siteShiftN = alignmentWorkspace.siteShiftN;
                newCoord = helpers.ENLFromLatLongAlt(lat, lng, 0, 0, 0, 0, alignmentWorkspace.transformation);
            }
            else
            {
                double[] coords = new double[2] { lng, lat };
                double[] coordz = new double[2] { 0, 0 };
                DotSpatial.Projections.Reproject.ReprojectPoints(coords, coordz, projSource, projTarget, 0, coords.Length - 1);
                newCoord.X = coords[0];
                newCoord.Y = coords[1];

                if (App.DefaultUnits != App.SavedUnits)
                {
                    if (App.DefaultUnits == Unit.Metre)
                    {
                        if (App.SavedUnits == Unit.Foot)
                        {
                            //Convert metres to feet
                            newCoord.X *= (1 / 0.3048);
                            newCoord.Y *= (1 / 0.3048);
                        }
                        else if (App.SavedUnits == Unit.USFoot)
                        {
                            //Convert metres to usfeet
                            newCoord.X *= (1 / 0.304800609601219);
                            newCoord.Y *= (1 / 0.304800609601219);
                        }
                    }
                    else if (App.DefaultUnits == Unit.Foot)
                    {
                        if (App.SavedUnits == Unit.Metre)
                        {
                            //Convert feet to metres
                            newCoord.X *= 0.3048;
                            newCoord.Y *= 0.3048;
                        }
                        else if (App.SavedUnits == Unit.USFoot)
                        {
                            //Convert feet to usfeet
                            newCoord.X *= 0.999998;
                            newCoord.Y *= 0.999998;
                        }
                    }
                    else if (App.DefaultUnits == Unit.USFoot)
                    {
                        if (App.SavedUnits == Unit.Metre)
                        {
                            //Convert usFeet to metres
                            newCoord.X *= 0.304800609601219;
                            newCoord.Y *= 0.304800609601219;
                        }
                        else if (App.SavedUnits == Unit.Foot)
                        {
                            //Convert usfeet to feet
                            newCoord.X *= 1.000002;
                            newCoord.Y *= 1.000002;
                        }
                    }
                }

                if (App.ScaleFactorMethod == ScaleFactor.Multiply)
                {
                    newCoord.X = ((newCoord.X - alignmentWorkspace.siteShiftE) / (1 / alignmentWorkspace.scaleFactor));
                    newCoord.Y = ((newCoord.Y - alignmentWorkspace.siteShiftN) / (1 / alignmentWorkspace.scaleFactor));
                }
                else if (App.ScaleFactorMethod == ScaleFactor.Divide)
                {
                    newCoord.X = (newCoord.X / alignmentWorkspace.scaleFactor) - alignmentWorkspace.siteShiftE;
                    newCoord.Y = (newCoord.Y / alignmentWorkspace.scaleFactor) - alignmentWorkspace.siteShiftN;
                }
            }

            return newCoord;
        }

        private void UpdateLLDisplay(double lat, double lng)
        {
            Task.Run(() =>
            {
                var x = lat.ToString("0.000000");
                var y = lng.ToString("0.000000");
                Device.BeginInvokeOnMainThread(() =>
                {
                    labelLat.Text = x;
                    labelLong.Text = y;
                    labelLandLat.Text = x;
                    LabelLandLng.Text = y;
                });
            });
        }

        private void UpdateENDisplay(double lat, double lng)
        {
            Task.Run(() =>
            {
                var en = GetENFromLL(lat, lng);
                var x = getDecimalDisplay(en.X);
                var y = getDecimalDisplay(en.Y);
                Device.BeginInvokeOnMainThread(() =>
                {
                    labelEast.Text = x;
                    labelNorth.Text = y;
                    labelLandEast.Text = x;
                    labelLandNorth.Text = y;
                });
            });
        }

        private void UpdateChOffDisplay(double lat, double lng)
        {
            Task.Run(() =>
            {
                var ch = "????";
                var off = "????";

                if (alignmentWorkspace.currentAlignment != null && alignmentWorkspace.currentAlignment != "None Found")
                {
                    var en = GetENFromLL(lat, lng);
                    var outVec = new dataTypes.nrgAlignmentVector5();
                    var chExists = alignmentWorkspace.Alignments.horizontal.getChainageAndOffset(en, ref outVec);

                    try
                    {
                        if(chExists)
                        {
                            if (alignmentWorkspace.chainageFormat == true)
                            {
                                if (alignmentWorkspace.chainageFormatDesc == "Format1")
                                    ch = setChainage(getDecimalDisplay(outVec.chainage));
                                else
                                    ch = setChainageFormat2(getDecimalDisplay(outVec.chainage));
                            }
                            else
                                ch = getDecimalDisplay(outVec.chainage);

                            if (alignmentWorkspace.offsetDisplay == "-/+")
                                off = getDecimalDisplay(outVec.offset);
                            else if (outVec.offset < 0)
                                off = getDecimalDisplay(Math.Abs(outVec.offset)) + " Left";
                            else
                                off = getDecimalDisplay(Math.Abs(outVec.offset)) + " Right";
                        }
                    }
                    catch
                    {
                        //Couldn't calc
                        ch = "????";
                        off = "????";
                    }
                }
                Device.BeginInvokeOnMainThread(() =>
                {
                    labelCh.Text = ch;
                    labelOff.Text = off;
                    labelLandCh.Text = ch;
                    labelLandOff.Text = off;
                });
            });
        }

        private void UpdateSiteShiftScale(double shiftE, double shiftN, double siteScale)
        {
            alignmentWorkspace.siteShiftE = shiftE;
            alignmentWorkspace.siteShiftN = shiftN;
            alignmentWorkspace.scaleFactor = siteScale;

            entryScaleFactor.Text = siteScale.ToString();
            entryShiftE.Text = shiftE.ToString();
            entryShiftN.Text = shiftN.ToString();
        }

        private void SaveTransformationFile(string filename)
        {
            string text = "";
            if(!NewTransformation)
            {
                if (alignmentWorkspace.transformation.name != null)
                    if (alignmentWorkspace.transformation.name != "")
                        text += "Name=" + alignmentWorkspace.transformation.name + "\r\n";

                text += "WGS84_A=" + alignmentWorkspace.transformation.WGS84_A + "\r\n";
                text += "WGS84_B=" + alignmentWorkspace.transformation.WGS84_B + "\r\n";
                text += "P_A=" + alignmentWorkspace.transformation.P_A + "\r\n";
                text += "P_B=" + alignmentWorkspace.transformation.P_B + "\r\n";
                text += "P_FO=" + alignmentWorkspace.transformation.P_FO + "\r\n";
                text += "P_EO=" + alignmentWorkspace.transformation.P_EO + "\r\n";
                text += "P_NO=" + alignmentWorkspace.transformation.P_NO + "\r\n";
                text += "PHIO=" + alignmentWorkspace.transformation.PHIO + "\r\n";
                text += "LAMO=" + alignmentWorkspace.transformation.LAMO + "\r\n";
                text += "p_translation_parallel_to_X=" + alignmentWorkspace.transformation.p_translation_parallel_to_X + "\r\n";
                text += "p_translation_parallel_to_Y=" + alignmentWorkspace.transformation.p_translation_parallel_to_Y + "\r\n";
                text += "p_translation_parallel_to_Z=" + alignmentWorkspace.transformation.p_translation_parallel_to_Z + "\r\n";
                text += "p_scale_change=" + alignmentWorkspace.transformation.p_scale_change + "\r\n";
                text += "p_rotation_about_X=" + alignmentWorkspace.transformation.p_rotation_about_X + "\r\n";
                text += "p_rotation_about_Y=" + alignmentWorkspace.transformation.p_rotation_about_Y + "\r\n";
                text += "p_rotation_about_Z=" + alignmentWorkspace.transformation.p_rotation_about_Z + "\r\n";
                text += "site_shift_e=" + alignmentWorkspace.transformation.siteShiftE + "\r\n";
                text += "site_shift_n=" + alignmentWorkspace.transformation.siteShiftN + "\r\n";
                text += "site_scale=" + alignmentWorkspace.transformation.siteScale + "\r\n";
                text += "ScaleFactorMethod=" + (int)App.ScaleFactorMethod + "\r\n";
                text += "current transformation=1";
                if (nrgFileRead.reader.fileExists(filename))
                    nrgFileRead.reader.deleteFile(filename);
                nrgFileRead.reader.stringToStream(filename, text);
            }
            else
            {
                text += "Name=" + App.transformationString + "\r\n";
                text += App.transformationBody + "\r\n";
                text += "site_shift_e=" + alignmentWorkspace.siteShiftE + "\r\n";
                text += "site_shift_n=" + alignmentWorkspace.siteShiftN + "\r\n";
                text += "site_scale=" + alignmentWorkspace.scaleFactor + "\r\n";
                text += "SavedUnits=" + (int)App.SavedUnits + "\r\n";
                text += "ScaleFactorMethod=" + (int)App.ScaleFactorMethod + "\r\n";
                if (nrgFileRead.reader.fileExists(filename))
                    nrgFileRead.reader.deleteFile(filename);
                nrgFileRead.reader.stringToStream(filename, text);
            }
        }

        private void ReadTransformation(string filename)
        {
            if(!nrgFileRead.reader.fileExists(filename))
                return;

            var lines = nrgFileRead.reader.loadAsciiToStringList(filename, false);

            //If its a proj string
            foreach(var line in lines)
            {
                if(line.Contains("+proj="))
                {
                    CreateProjTransformation(lines);
                    return;
                }
            }

            //Otherwise its a .dat or invalid
            CreateDatTransformation(lines, filename); 
        }

        private void CreateProjTransformation(List<string> lines)
        {
            try
            {
                var projString = "";
                var name = "";
                double shiftE = 0, shiftN = 0, scale = 1;
                App.SavedUnits = Unit.None;
                App.ScaleFactorMethod = ScaleFactor.Multiply;
                foreach (var line in lines)
                {
                    if (line.Contains("+proj="))
                    {
                        projString = line;
                    }
                    else if (line.Contains("="))
                    {
                        string[] temp = line.Split('=');
                        if (temp.Count() >= 2)
                        {
                            double val = 0;
                            switch (temp[0])
                            {
                                case "Name":
                                    name = temp[1];
                                    break;
                                case "site_shift_e":
                                    val = 0;
                                    double.TryParse(temp[1], out val);
                                    shiftE = val;
                                    break;
                                case "site_shift_n":
                                    val = 0;
                                    double.TryParse(temp[1], out val);
                                    shiftN = val;
                                    break;
                                case "site_scale":
                                    val = 1;
                                    double.TryParse(temp[1], out val);
                                    if (val == 0)
                                        val = 1;
                                    scale = val;
                                    break;
                                case "SavedUnits":
                                    val = 1;
                                    double.TryParse(temp[1], out val);
                                    App.SavedUnits = (Unit)val;
                                    break;
                                case "ScaleFactorMethod":
                                    val = 0;
                                    double.TryParse(temp[1], out val);
                                    App.ScaleFactorMethod = (ScaleFactor)val;
                                    break;
                            }
                        }
                    }
                }

                if (name == "")
                {
                    name = "Custom Transformation";
                }

                if (projString.Contains("+proj="))
                {
                    if (projString.Contains("+units=m"))
                        App.DefaultUnits = Unit.Metre;
                    else if (projString.Contains("+units=ft"))
                        App.DefaultUnits = Unit.Foot;
                    else if (projString.Contains("+units=us-ft"))
                        App.DefaultUnits = Unit.USFoot;
                    else
                        App.DefaultUnits = Unit.None;
                          
                    alignmentWorkspace.currentTransformation = "More...";
                    CrossSettings.Current.AddOrUpdateValue("currentTransformation", alignmentWorkspace.currentTransformation);
                    App.transformationString = name;
                    App.transformationBody = projString;
                    CrossSettings.Current.AddOrUpdateValue("transformationString", App.transformationString);
                    CrossSettings.Current.AddOrUpdateValue("transformationBody", App.transformationBody);
                    LoadNewTransformation();
                    labelCurTransformation.Text = App.transformationString;
                    NewTransformation = true;
                    updateToNewTransformation();
                    UpdateSiteShiftScale(shiftE, shiftN, scale);
                }

                UpdateTransformUnits();
            }
            catch
            {
                UpdateTransformUnits();
            }
        }

        private void CreateDatTransformation(List<string> lines, string filename)
        {
            try
            {
                var ts = new transformationSettings();
                App.SavedUnits = Unit.None;
                App.DefaultUnits = Unit.None;
                App.ScaleFactorMethod = ScaleFactor.Multiply;
                int count = 0;
                double value = 0;
                foreach (var line in lines)
                {
                    if (!line.Contains("="))
                        continue;

                    string[] temp = line.Split('=');

                    if (temp.Count() < 2)
                        continue;

                    if (temp[0] == "Name")
                        ts.name = temp[1];

                    if (!double.TryParse(temp[1], out value))
                        continue;

                    switch (temp[0])
                    {
                        case "WGS84_A":
                            count += 1;
                            ts.WGS84_A = value;
                            break;
                        case "WGS84_B":
                            count += 1;
                            ts.WGS84_B = value;
                            break;
                        case "P_A":
                            count += 1;
                            ts.P_A = value;
                            break;
                        case "P_B":
                            count += 1;
                            ts.P_B = value;
                            break;
                        case "P_FO":
                            count += 1;
                            ts.P_FO = value;
                            break;
                        case "P_EO":
                            count += 1;
                            ts.P_EO = value;
                            break;
                        case "P_NO":
                            count += 1;
                            ts.P_NO = value;
                            break;
                        case "PHIO":
                            count += 1;
                            ts.PHIO = value;
                            break;
                        case "LAMO":
                            count += 1;
                            ts.LAMO = value;
                            break;
                        case "p_translation_parallel_to_X":
                            count += 1;
                            ts.p_translation_parallel_to_X = value;
                            break;
                        case "p_translation_parallel_to_Y":
                            count += 1;
                            ts.p_translation_parallel_to_Y = value;
                            break;
                        case "p_translation_parallel_to_Z":
                            count += 1;
                            ts.p_translation_parallel_to_Z = value;
                            break;
                        case "p_scale_change":
                            count += 1;
                            ts.p_scale_change = value;
                            break;
                        case "p_rotation_about_X":
                            count += 1;
                            ts.p_rotation_about_X = value;
                            break;
                        case "p_rotation_about_Y":
                            count += 1;
                            ts.p_rotation_about_Y = value;
                            break;
                        case "p_rotation_about_Z":
                            count += 1;
                            ts.p_rotation_about_Z = value;
                            break;
                        case "site_shift_e":
                            ts.siteShiftE = value;
                            break;
                        case "site_shift_n":
                            ts.siteShiftN = value;
                            break;
                        case "site_scale":
                            if (value == 0)
                                value = 1;
                            ts.siteScale = value;
                            break;
                        case "ScaleFactorMethod":
                            App.ScaleFactorMethod = (ScaleFactor)value;
                            break;
                    }
                }

                if (count >= 16)
                {
                    if (ts.name == "")
                        ts.name = Path.GetFileNameWithoutExtension(filename);
                    alignmentWorkspace.transformation = ts;
                    NewTransformation = false;
                    UpdateSiteShiftScale(ts.siteShiftE, ts.siteShiftN, ts.siteScale);
                    alignmentWorkspace.currentTransformation = ts.name;
                    CrossSettings.Current.AddOrUpdateValue("currentTransformation", alignmentWorkspace.currentTransformation);
                    labelCurTransformation.Text = alignmentWorkspace.currentTransformation;
                    updateToNewTransformation();
                }

                UpdateTransformUnits();
            }
            catch
            {
                UpdateTransformUnits();
            }
        }

        private void UpdateTransformUnits()
        {
            if (App.DefaultUnits != Unit.None && App.SavedUnits == Unit.None)
                App.SavedUnits = App.DefaultUnits;

            //IF default = none hide panel
            if (App.DefaultUnits == Unit.None)
                gridTransformUnits.IsVisible = false;
            else
                gridTransformUnits.IsVisible = true;

            switch(App.SavedUnits)
            {
                case Unit.None: default:
                    labelTransformUnits.Text = "None";
                    break;
                case Unit.Metre:
                    labelTransformUnits.Text = "Meters";
                    break;
                case Unit.Foot:
                    labelTransformUnits.Text = "Feet";
                    break;
                case Unit.USFoot:
                    labelTransformUnits.Text = "US Survey Feet";
                    break;
            }         

            switch(App.ScaleFactorMethod)
            {
                case ScaleFactor.Multiply: default:
                    labelScaleMethod.Text = "Multiply";
                    break;
                case ScaleFactor.Divide:
                    labelScaleMethod.Text = "Divide";
                    break;
            }

            //Update scale factor method
        }

        private void RedrawAlignment()
        {
            if (nrgFileRead.reader.fileExists(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + labelCurAlignment.Text) == false)
            {
                DisplayAlert("Error", "Failed to load alignment: could not find alignment file", "Ok");
            }
            else
            {
                try
                {
                    alignmentWorkspace.Alignments = new alignments.alignment();
                    alignmentWorkspace.currentAlignment = labelCurAlignment.Text;
                    CrossSettings.Current.AddOrUpdateValue("currentAlignment", alignmentWorkspace.currentAlignment);
                    if (labelCurAlignment.Text.ToUpper().EndsWith(".NST"))
                        alignmentWorkspace.Alignments.loadNST(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + labelCurAlignment.Text);
                    else if (labelCurAlignment.Text.ToUpper().EndsWith(".NAF"))
                        alignmentWorkspace.Alignments.loadFile(alignmentWorkspace.datapath + alignmentWorkspace.currentFolder + "/" + labelCurAlignment.Text);
                    alignmentWorkspace.workspaceBounds = alignmentWorkspace.Alignments.CalcHzExtents();
                    double maxChain = alignmentWorkspace.Alignments.horizontal.elements.OrderBy(x => x.Chainage).Last().Chainage;
                    double minChain = alignmentWorkspace.Alignments.horizontal.elements.OrderBy(x => x.Chainage).First().Chainage;

                    int chainInt = (int)((maxChain - minChain) / 500);

                    if (chainInt < 1)
                        chainInt = 1;

                    alignmentWorkspace.Alignments.calcDrawCoords(alignmentWorkspace.Alignments.coords, chainInt);
                    Polyline polyline2 = null;
                    polyline2 = new Polyline();

                    foreach (workspace.dataTypes.nrgVector3 point in alignmentWorkspace.Alignments.coords)
                    {
                        var latLng = GetLLFromEN(point.X, point.Y);
                        polyline2.Positions.Add(new Xamarin.Forms.GoogleMaps.Position(latLng.X, latLng.Y));
                    }

                    polyline2.IsClickable = true;
                    polyline2.StrokeColor = Color.Blue;
                    polyline2.StrokeWidth = 5f;
                    polyline2.Tag = "Alignment";
                    lastline = mapObject.Polylines.Count();
                    updateChainageAndOffset();


                    for (int i = 0; i < mapObject.Polylines.Count; i++)
                    {
                        if (mapObject.Polylines[i].Tag != null)
                        {
                            mapObject.Polylines[i] = polyline2;
                            break;
                        }
                    }
                }
                catch (Exception e)
                {
                    DisplayAlert("Load Failure", e.Message, "Ok");
                }
            }
        }
    }
}

